module Make (Instruction : Sparc.S with type 'a Reloc.relocatable = 'a) = 
  struct 
    
    module RP = Rtl.Private
    
    module B = Bits
    
    module BO = Bitops
    
    let defaultOfInstruction rtl = 
      (match rtl with
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sx", [8; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg1)]), 
                _), 8)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1 13) -> 
           Instruction.ldsb (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg1))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("sx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              8)]), 32))] -> 
           Instruction.ldsb (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sx", [8; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg2)]), 
                _), 8)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2 13) -> 
           Instruction.ldsb (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
                RP.Const (RP.Bits arg3), _), 8)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3 13) -> 
           Instruction.ldsb (Instruction.absolutea (Bits.U.to_native arg3)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("sx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              8)]), 32))] -> 
           Instruction.ldsb (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.App (("sx", [8; 32]), 
            [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32), _), 8)]), 
          32))] -> 
           Instruction.ldsb (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sx", [16; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg4)]), 
                _), 16)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg4 13) -> 
           Instruction.ldsh (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg4))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("sx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              16)]), 32))] -> 
           Instruction.ldsh (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sx", [16; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg5)]), 
                _), 16)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg5 13) -> 
           Instruction.ldsh (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg5)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 
                16, RP.Const (RP.Bits arg6), _), 16)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg6 13) -> 
           Instruction.ldsh (Instruction.absolutea (Bits.U.to_native arg6)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("sx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              16)]), 32))] -> 
           Instruction.ldsh (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.App (("sx", [16; 32]), 
            [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32), _), 16)]), 
          32))] -> 
           Instruction.ldsh (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("zx", [8; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg7)]), 
                _), 8)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg7 13) -> 
           Instruction.ldub (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg7))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("zx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              8)]), 32))] -> 
           Instruction.ldub (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("zx", [8; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg8)]), 
                _), 8)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg8 13) -> 
           Instruction.ldub (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg8)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("zx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
                RP.Const (RP.Bits arg9), _), 8)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg9 13) -> 
           Instruction.ldub (Instruction.absolutea (Bits.U.to_native arg9)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("zx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              8)]), 32))] -> 
           Instruction.ldub (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.App (("zx", [8; 32]), 
            [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32), _), 8)]), 
          32))] -> 
           Instruction.ldub (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("zx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 
                16, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                    RP.Const (RP.Bits arg10)]), _), 16)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg10 13) -> 
           Instruction.lduh (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg10))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("zx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              16)]), 32))] -> 
           Instruction.lduh (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("zx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 
                16, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                    RP.Const (RP.Bits arg11)]), _), 16)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg11 13) -> 
           Instruction.lduh (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg11)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("zx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 
                16, RP.Const (RP.Bits arg12), _), 16)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg12 13) -> 
           Instruction.lduh (Instruction.absolutea (Bits.U.to_native arg12)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("zx", [16; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              16)]), 32))] -> 
           Instruction.lduh (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.App (("zx", [16; 32]), 
            [RP.Fetch (RP.Cell ('m', Rtl.Identity, 16, RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32), _), 16)]), 
          32))] -> 
           Instruction.lduh (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg13)]), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg13 13) -> 
           Instruction.ld (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg13))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 32), 
          32))] -> 
           Instruction.ld (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg14)]), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg14 13) -> 
           Instruction.ld (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg14)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg15), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg15 13) -> 
           Instruction.ld (Instruction.absolutea (Bits.U.to_native arg15)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 32), 
          32))] -> 
           Instruction.ld (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), 32), 32))] -> 
           Instruction.ld (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("zx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
                RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs1'), _), 32); 
                    RP.Const (RP.Bits arg18)]), _), 8)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg16)]), 
              _), RP.Const (RP.Bits arg17), 8))] 
      when Base.to_bool (rs1 = rs1' && (Bitops.fits_signed arg18 13 && 
          Bitops.eq arg17 (Bits.U.of_int 255 8))) -> 
           Instruction.ldstub (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg18))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("zx", [8; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)]), _), 
                8)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              RP.Const (RP.Bits arg19), 8))] 
      when Base.to_bool 
          (rs1 = rs1' && rs2 = rs2' && Bitops.eq arg19 (Bits.U.of_int 255 8)) -> 
           Instruction.ldstub (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("zx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
                RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs1'), _), 32); 
                    RP.Const (RP.Bits arg22)]), _), 8)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg20)]), 
              _), RP.Const (RP.Bits arg21), 8))] 
      when Base.to_bool (rs1 = rs1' && (Bitops.fits_signed arg22 13 && 
          Bitops.eq arg21 (Bits.U.of_int 255 8))) -> 
           Instruction.ldstub (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg22)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("zx", [8; 32]), [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, 
                RP.Const (RP.Bits arg25), _), 8)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 8, 
              RP.Const (RP.Bits arg23), _), RP.Const (RP.Bits arg24), 8))] 
      when Base.to_bool (Bitops.fits_signed arg25 13 && Bitops.eq arg24 
            (Bits.U.of_int 255 8)) -> 
           Instruction.ldstub (Instruction.absolutea (Bits.U.to_native arg25)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("zx", [8; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.App (("add", [32]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)]), _), 
                8)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 8, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              RP.Const (RP.Bits arg26), 8))] 
      when Base.to_bool 
          (rs1 = rs1' && rs2 = rs2' && Bitops.eq arg26 (Bits.U.of_int 255 8)) -> 
           Instruction.ldstub (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("zx", [8; 32]), 
              [RP.Fetch (RP.Cell ('m', Rtl.Identity, 8, RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32), _), 8)]), 
            32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('m', Rtl.Identity, 8, RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32), _), 
              RP.Const (RP.Bits arg27), 8))] 
      when Base.to_bool (rs1 = rs1' && Bitops.eq arg27 (Bits.U.of_int 255 8)) -> 
           Instruction.ldstub (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd'), _), 
            RP.Fetch (RP.Cell ('m', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg29)]), 
              _), 32), 32)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg28)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32), 32))] 
      when Base.to_bool 
          (rd = rd' && rs1 = rs1' && Bitops.fits_signed arg29 13) -> 
           Instruction.swap_ (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg29))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd'), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2'), _), 32)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
                _), 32), 32))] 
      when Base.to_bool (rd = rd' && rs1 = rs1' && rs2 = rs2') -> 
           Instruction.swap_ (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd'), _), 
            RP.Fetch (RP.Cell ('m', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg31)]), 
              _), 32), 32)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg30)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32), 32))] 
      when Base.to_bool 
          (rd = rd' && rs1 = rs1' && Bitops.fits_signed arg31 13) -> 
           Instruction.swap_ (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg31)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd'), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg33), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg32), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32), 32))] 
      when Base.to_bool (rd = rd' && Bitops.fits_signed arg33 13) -> 
           Instruction.swap_ (Instruction.absolutea (Bits.U.to_native arg33)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd'), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2'), _), 32)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
                _), 32), 32))] 
      when Base.to_bool (rd = rd' && rs1 = rs1' && rs2 = rs2') -> 
           Instruction.swap_ (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd'), _), 
            RP.Fetch (RP.Cell ('m', Rtl.Identity, 32, RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32), _), 32), 
            32)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32), 32))] 
      when Base.to_bool (rd = rd' && rs1 = rs1') -> 
           Instruction.swap_ (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            64, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg34)]), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg34 13) -> 
           Instruction.ldd (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg34))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          64, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 64), 
          64))] -> 
           Instruction.ldd (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            64, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg35)]), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg35 13) -> 
           Instruction.ldd (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg35)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            64, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.Const (RP.Bits arg36), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg36 13) -> 
           Instruction.ldd (Instruction.absolutea (Bits.U.to_native arg36)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          64, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 64), 
          64))] -> 
           Instruction.ldd (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          64, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), 64), 64))] -> 
           Instruction.ldd (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            32, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg37)]), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg37 13) -> 
           Instruction.ldf (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg37))) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
          32, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 32), 
          32))] -> 
           Instruction.ldf (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            32, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg38)]), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg38 13) -> 
           Instruction.ldf (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg38)) (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            32, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg39), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg39 13) -> 
           Instruction.ldf (Instruction.absolutea (Bits.U.to_native arg39)) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
          32, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 32), 
          32))] -> 
           Instruction.ldf (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
          32, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), 32), 32))] -> 
           Instruction.ldf (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            64, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg40)]), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg40 13) -> 
           Instruction.lddf (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg40))) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
          64, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 64), 
          64))] -> 
           Instruction.lddf (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            64, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg41)]), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg41 13) -> 
           Instruction.lddf (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg41)) (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            64, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.Const (RP.Bits arg42), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg42 13) -> 
           Instruction.lddf (Instruction.absolutea (Bits.U.to_native arg42)) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
          64, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 64), 
          64))] -> 
           Instruction.lddf (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
          64, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), 64), 64))] -> 
           Instruction.lddf (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
            32, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg43)]), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg43 13) -> 
           Instruction.ldc (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg43))) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
          32, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 32), 
          32))] -> 
           Instruction.ldc (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
            32, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg44)]), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg44 13) -> 
           Instruction.ldc (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg44)) (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
            32, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg45), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg45 13) -> 
           Instruction.ldc (Instruction.absolutea (Bits.U.to_native arg45)) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
          32, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 32), 
          32))] -> 
           Instruction.ldc (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
          32, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), 32), 32))] -> 
           Instruction.ldc (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
            64, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg46)]), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg46 13) -> 
           Instruction.lddc (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg46))) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
          64, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 64), 
          64))] -> 
           Instruction.lddc (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
            64, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg47)]), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg47 13) -> 
           Instruction.lddc (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg47)) (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
            64, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 64, RP.Const (RP.Bits arg48), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg48 13) -> 
           Instruction.lddc (Instruction.absolutea (Bits.U.to_native arg48)) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
          64, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 64), 
          64))] -> 
           Instruction.lddc (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('c', Rtl.Identity, 
          64, RP.Const (RP.Bits cd), _), RP.Fetch (RP.Cell ('m', Rtl.Identity, 
            64, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), 64), 64))] -> 
           Instruction.lddc (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native cd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            8, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg49)]), _), RP.App (("bitExtract", 
              [32; 32; 8]), [RP.Const (RP.Bits arg50); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 8))] 
      when Base.to_bool (Bitops.fits_signed arg49 13 && Bitops.eq arg50 
            (Bits.U.of_int 0 32)) -> 
           Instruction.stb (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg49))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            8, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.App (("bitExtract", [32; 32; 8]), 
              [RP.Const (RP.Bits arg51); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rd), _), 32)]), 8))] 
      when Base.to_bool (Bitops.eq arg51 (Bits.U.of_int 0 32)) -> 
           Instruction.stb (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            8, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg52)]), _), RP.App (("bitExtract", 
              [32; 32; 8]), [RP.Const (RP.Bits arg53); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 8))] 
      when Base.to_bool (Bitops.fits_signed arg52 13 && Bitops.eq arg53 
            (Bits.U.of_int 0 32)) -> 
           Instruction.stb (Bits.U.to_native rd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg52)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            8, RP.Const (RP.Bits arg54), _), RP.App (("bitExtract", 
              [32; 32; 8]), [RP.Const (RP.Bits arg55); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 8))] 
      when Base.to_bool (Bitops.fits_signed arg54 13 && Bitops.eq arg55 
            (Bits.U.of_int 0 32)) -> 
           Instruction.stb (Bits.U.to_native rd) 
             (Instruction.absolutea (Bits.U.to_native arg54)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            8, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.App (("bitExtract", [32; 32; 8]), 
              [RP.Const (RP.Bits arg56); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rd), _), 32)]), 8))] 
      when Base.to_bool (Bitops.eq arg56 (Bits.U.of_int 0 32)) -> 
           Instruction.stb (Bits.U.to_native rd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            8, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), RP.App (("bitExtract", 
              [32; 32; 8]), [RP.Const (RP.Bits arg57); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 8))] 
      when Base.to_bool (Bitops.eq arg57 (Bits.U.of_int 0 32)) -> 
           Instruction.stb (Bits.U.to_native rd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            16, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg58)]), _), RP.App (("bitExtract", 
              [32; 32; 16]), [RP.Const (RP.Bits arg59); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 16))] 
      when Base.to_bool (Bitops.fits_signed arg58 13 && Bitops.eq arg59 
            (Bits.U.of_int 0 32)) -> 
           Instruction.sth (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg58))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            16, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.App (("bitExtract", [32; 32; 16]), 
              [RP.Const (RP.Bits arg60); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rd), _), 32)]), 16))] 
      when Base.to_bool (Bitops.eq arg60 (Bits.U.of_int 0 32)) -> 
           Instruction.sth (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            16, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg61)]), _), RP.App (("bitExtract", 
              [32; 32; 16]), [RP.Const (RP.Bits arg62); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 16))] 
      when Base.to_bool (Bitops.fits_signed arg61 13 && Bitops.eq arg62 
            (Bits.U.of_int 0 32)) -> 
           Instruction.sth (Bits.U.to_native rd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg61)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            16, RP.Const (RP.Bits arg63), _), RP.App (("bitExtract", 
              [32; 32; 16]), [RP.Const (RP.Bits arg64); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 16))] 
      when Base.to_bool (Bitops.fits_signed arg63 13 && Bitops.eq arg64 
            (Bits.U.of_int 0 32)) -> 
           Instruction.sth (Bits.U.to_native rd) 
             (Instruction.absolutea (Bits.U.to_native arg63)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            16, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.App (("bitExtract", [32; 32; 16]), 
              [RP.Const (RP.Bits arg65); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rd), _), 32)]), 16))] 
      when Base.to_bool (Bitops.eq arg65 (Bits.U.of_int 0 32)) -> 
           Instruction.sth (Bits.U.to_native rd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            16, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), RP.App (("bitExtract", 
              [32; 32; 16]), [RP.Const (RP.Bits arg66); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32)]), 16))] 
      when Base.to_bool (Bitops.eq arg66 (Bits.U.of_int 0 32)) -> 
           Instruction.sth (Bits.U.to_native rd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg67)]), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg67 13) -> 
           Instruction.st (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg67))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            32), 32))] -> 
           Instruction.st (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg68)]), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg68 13) -> 
           Instruction.st (Bits.U.to_native rd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg68)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.Const (RP.Bits arg69), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg69 13) -> 
           Instruction.st (Bits.U.to_native rd) 
             (Instruction.absolutea (Bits.U.to_native arg69)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            32), 32))] -> 
           Instruction.st (Bits.U.to_native rd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
            _), 32), _), RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
            RP.Const (RP.Bits rd), _), 32), 32))] -> 
           Instruction.st (Bits.U.to_native rd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg70)]), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 64, RP.Const (RP.Bits rd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg70 13) -> 
           Instruction.std (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg70))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('r', Rtl.Identity, 64, RP.Const (RP.Bits rd), _), 
            64), 64))] -> 
           Instruction.std (Bits.U.to_native rd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg71)]), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 64, RP.Const (RP.Bits rd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg71 13) -> 
           Instruction.std (Bits.U.to_native rd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg71)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.Const (RP.Bits arg72), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 64, RP.Const (RP.Bits rd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg72 13) -> 
           Instruction.std (Bits.U.to_native rd) 
             (Instruction.absolutea (Bits.U.to_native arg72)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('r', Rtl.Identity, 64, RP.Const (RP.Bits rd), _), 
            64), 64))] -> 
           Instruction.std (Bits.U.to_native rd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
            _), 32), _), RP.Fetch (RP.Cell ('r', Rtl.Identity, 64, 
            RP.Const (RP.Bits rd), _), 64), 64))] -> 
           Instruction.std (Bits.U.to_native rd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg73)]), _), RP.Fetch (RP.Cell ('f', 
              Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg73 13) -> 
           Instruction.stf (Bits.U.to_native fd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg73))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            32), 32))] -> 
           Instruction.stf (Bits.U.to_native fd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg74)]), _), RP.Fetch (RP.Cell ('f', 
              Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg74 13) -> 
           Instruction.stf (Bits.U.to_native fd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg74)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.Const (RP.Bits arg75), _), RP.Fetch (RP.Cell ('f', 
              Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg75 13) -> 
           Instruction.stf (Bits.U.to_native fd) 
             (Instruction.absolutea (Bits.U.to_native arg75)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            32), 32))] -> 
           Instruction.stf (Bits.U.to_native fd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
            _), 32), _), RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
            RP.Const (RP.Bits fd), _), 32), 32))] -> 
           Instruction.stf (Bits.U.to_native fd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg76)]), _), RP.Fetch (RP.Cell ('f', 
              Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg76 13) -> 
           Instruction.stdf (Bits.U.to_native fd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg76))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            64), 64))] -> 
           Instruction.stdf (Bits.U.to_native fd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg77)]), _), RP.Fetch (RP.Cell ('f', 
              Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg77 13) -> 
           Instruction.stdf (Bits.U.to_native fd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg77)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.Const (RP.Bits arg78), _), RP.Fetch (RP.Cell ('f', 
              Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg78 13) -> 
           Instruction.stdf (Bits.U.to_native fd) 
             (Instruction.absolutea (Bits.U.to_native arg78)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            64), 64))] -> 
           Instruction.stdf (Bits.U.to_native fd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
            _), 32), _), RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
            RP.Const (RP.Bits fd), _), 64), 64))] -> 
           Instruction.stdf (Bits.U.to_native fd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg79)]), _), RP.Fetch (RP.Cell ('c', 
              Rtl.Identity, 32, RP.Const (RP.Bits cd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg79 13) -> 
           Instruction.stc (Bits.U.to_native cd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg79))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('c', Rtl.Identity, 32, RP.Const (RP.Bits cd), _), 
            32), 32))] -> 
           Instruction.stc (Bits.U.to_native cd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg80)]), _), RP.Fetch (RP.Cell ('c', 
              Rtl.Identity, 32, RP.Const (RP.Bits cd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg80 13) -> 
           Instruction.stc (Bits.U.to_native cd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg80)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.Const (RP.Bits arg81), _), RP.Fetch (RP.Cell ('c', 
              Rtl.Identity, 32, RP.Const (RP.Bits cd), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg81 13) -> 
           Instruction.stc (Bits.U.to_native cd) 
             (Instruction.absolutea (Bits.U.to_native arg81)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('c', Rtl.Identity, 32, RP.Const (RP.Bits cd), _), 
            32), 32))] -> 
           Instruction.stc (Bits.U.to_native cd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
            _), 32), _), RP.Fetch (RP.Cell ('c', Rtl.Identity, 32, 
            RP.Const (RP.Bits cd), _), 32), 32))] -> 
           Instruction.stc (Bits.U.to_native cd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg82)]), _), RP.Fetch (RP.Cell ('c', 
              Rtl.Identity, 64, RP.Const (RP.Bits cd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg82 13) -> 
           Instruction.stdc (Bits.U.to_native cd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg82))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('c', Rtl.Identity, 64, RP.Const (RP.Bits cd), _), 
            64), 64))] -> 
           Instruction.stdc (Bits.U.to_native cd) 
             (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg83)]), _), RP.Fetch (RP.Cell ('c', 
              Rtl.Identity, 64, RP.Const (RP.Bits cd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg83 13) -> 
           Instruction.stdc (Bits.U.to_native cd) (Instruction.dispa 
                 (Bits.U.to_native rs1) (Bits.U.to_native arg83)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            64, RP.Const (RP.Bits arg84), _), RP.Fetch (RP.Cell ('c', 
              Rtl.Identity, 64, RP.Const (RP.Bits cd), _), 64), 64))] 
      when Base.to_bool (Bitops.fits_signed arg84 13) -> 
           Instruction.stdc (Bits.U.to_native cd) 
             (Instruction.absolutea (Bits.U.to_native arg84)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
          RP.Fetch (RP.Cell ('c', Rtl.Identity, 64, RP.Const (RP.Bits cd), _), 
            64), 64))] -> 
           Instruction.stdc (Bits.U.to_native cd) (Instruction.indexa 
                 (Bits.U.to_native rs1) (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
          64, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
            _), 32), _), RP.Fetch (RP.Cell ('c', Rtl.Identity, 64, 
            RP.Const (RP.Bits cd), _), 64), 64))] -> 
           Instruction.stdc (Bits.U.to_native cd) 
             (Instruction.indirecta (Bits.U.to_native rs1)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('F', Rtl.Identity, 
            32, RP.Const (RP.Bits arg85), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg86)]), _), 32), 32))] 
      when Base.to_bool 
          (Bitops.fits_signed arg86 13 && Bitops.eq arg85 (Bits.U.of_int 0 1)) -> 
           Instruction.ldfsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg86))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('F', Rtl.Identity, 
            32, RP.Const (RP.Bits arg87), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2), _), 32)]), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg87 (Bits.U.of_int 0 1)) -> 
           Instruction.ldfsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('F', Rtl.Identity, 
            32, RP.Const (RP.Bits arg88), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg89)]), _), 32), 32))] 
      when Base.to_bool 
          (Bitops.fits_signed arg89 13 && Bitops.eq arg88 (Bits.U.of_int 0 1)) -> 
           Instruction.ldfsr (Instruction.dispa (Bits.U.to_native rs1) 
               (Bits.U.to_native arg89)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('F', Rtl.Identity, 
            32, RP.Const (RP.Bits arg90), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg91), _), 32), 32))] 
      when Base.to_bool 
          (Bitops.fits_signed arg91 13 && Bitops.eq arg90 (Bits.U.of_int 0 1)) -> 
           Instruction.ldfsr (Instruction.absolutea (Bits.U.to_native arg91)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('F', Rtl.Identity, 
            32, RP.Const (RP.Bits arg92), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2), _), 32)]), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg92 (Bits.U.of_int 0 1)) -> 
           Instruction.ldfsr (Instruction.indexa (Bits.U.to_native rs1) 
               (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('F', Rtl.Identity, 
            32, RP.Const (RP.Bits arg93), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg93 (Bits.U.of_int 0 1)) -> 
           Instruction.ldfsr (Instruction.indirecta (Bits.U.to_native rs1)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('C', Rtl.Identity, 
            32, RP.Const (RP.Bits arg94), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg95)]), _), 32), 32))] 
      when Base.to_bool 
          (Bitops.fits_signed arg95 13 && Bitops.eq arg94 (Bits.U.of_int 0 1)) -> 
           Instruction.ldcsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg95))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('C', Rtl.Identity, 
            32, RP.Const (RP.Bits arg96), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2), _), 32)]), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg96 (Bits.U.of_int 0 1)) -> 
           Instruction.ldcsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('C', Rtl.Identity, 
            32, RP.Const (RP.Bits arg97), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg98)]), _), 32), 32))] 
      when Base.to_bool 
          (Bitops.fits_signed arg98 13 && Bitops.eq arg97 (Bits.U.of_int 0 1)) -> 
           Instruction.ldcsr (Instruction.dispa (Bits.U.to_native rs1) 
               (Bits.U.to_native arg98)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('C', Rtl.Identity, 
            32, RP.Const (RP.Bits arg99), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg100), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg100 13 && Bitops.eq arg99 
            (Bits.U.of_int 0 1)) -> 
           Instruction.ldcsr (Instruction.absolutea (Bits.U.to_native arg100)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('C', Rtl.Identity, 
            32, RP.Const (RP.Bits arg101), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2), _), 32)]), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg101 (Bits.U.of_int 0 1)) -> 
           Instruction.ldcsr (Instruction.indexa (Bits.U.to_native rs1) 
               (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('C', Rtl.Identity, 
            32, RP.Const (RP.Bits arg102), _), RP.Fetch (RP.Cell ('m', 
              Rtl.Identity, 32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg102 (Bits.U.of_int 0 1)) -> 
           Instruction.ldcsr (Instruction.indirecta (Bits.U.to_native rs1)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg103)]), _), RP.Fetch (RP.Cell ('F', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg104), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg103 13 && Bitops.eq arg104 
            (Bits.U.of_int 0 1)) -> 
           Instruction.stfsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg103))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.Fetch (RP.Cell ('F', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg105), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg105 (Bits.U.of_int 0 1)) -> 
           Instruction.stfsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg106)]), _), RP.Fetch (RP.Cell ('F', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg107), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg106 13 && Bitops.eq arg107 
            (Bits.U.of_int 0 1)) -> 
           Instruction.stfsr (Instruction.dispa (Bits.U.to_native rs1) 
               (Bits.U.to_native arg106)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.Const (RP.Bits arg108), _), RP.Fetch (RP.Cell ('F', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg109), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg108 13 && Bitops.eq arg109 
            (Bits.U.of_int 0 1)) -> 
           Instruction.stfsr (Instruction.absolutea (Bits.U.to_native arg108)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.Fetch (RP.Cell ('F', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg110), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg110 (Bits.U.of_int 0 1)) -> 
           Instruction.stfsr (Instruction.indexa (Bits.U.to_native rs1) 
               (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), RP.Fetch (RP.Cell ('F', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg111), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg111 (Bits.U.of_int 0 1)) -> 
           Instruction.stfsr (Instruction.indirecta (Bits.U.to_native rs1)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg112)]), _), RP.Fetch (RP.Cell ('C', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg113), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg112 13 && Bitops.eq arg113 
            (Bits.U.of_int 0 1)) -> 
           Instruction.stcsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg112))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.Fetch (RP.Cell ('C', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg114), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg114 (Bits.U.of_int 0 1)) -> 
           Instruction.stcsr (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); 
                RP.Const (RP.Bits arg115)]), _), RP.Fetch (RP.Cell ('C', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg116), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg115 13 && Bitops.eq arg116 
            (Bits.U.of_int 0 1)) -> 
           Instruction.stcsr (Instruction.dispa (Bits.U.to_native rs1) 
               (Bits.U.to_native arg115)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.Const (RP.Bits arg117), _), RP.Fetch (RP.Cell ('C', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg118), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg117 13 && Bitops.eq arg118 
            (Bits.U.of_int 0 1)) -> 
           Instruction.stcsr (Instruction.absolutea (Bits.U.to_native arg117)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), _), 
            RP.Fetch (RP.Cell ('C', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg119), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg119 (Bits.U.of_int 0 1)) -> 
           Instruction.stcsr (Instruction.indexa (Bits.U.to_native rs1) 
               (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('m', Rtl.Identity, 
            32, RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32), _), RP.Fetch (RP.Cell ('C', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg120), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg120 (Bits.U.of_int 0 1)) -> 
           Instruction.stcsr (Instruction.indirecta (Bits.U.to_native rs1)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg121), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg121 (Bits.U.of_int 3 3)) -> 
           Instruction.rdy (Bits.U.to_native rd) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg122), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg122 (Bits.U.of_int 0 3)) -> 
           Instruction.rdpsr (Bits.U.to_native rd) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg123), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg123 (Bits.U.of_int 1 3)) -> 
           Instruction.rdwim (Bits.U.to_native rd) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg124), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg124 (Bits.U.of_int 2 3)) -> 
           Instruction.rdtbr (Bits.U.to_native rd) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg125), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg126)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg126 13 && Bitops.eq arg125 
            (Bits.U.of_int 3 3)) -> 
           Instruction.wry (Bits.U.to_native rs1) 
             (Instruction.imode (Bits.U.to_native arg126)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg127), 
            _), RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg127 (Bits.U.of_int 3 3)) -> 
           Instruction.wry (Bits.U.to_native rs1) 
             (Instruction.rmode (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg128), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg129)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg129 13 && Bitops.eq arg128 
            (Bits.U.of_int 0 3)) -> 
           Instruction.wrpsr (Bits.U.to_native rs1) 
             (Instruction.imode (Bits.U.to_native arg129)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg130), 
            _), RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg130 (Bits.U.of_int 0 3)) -> 
           Instruction.wrpsr (Bits.U.to_native rs1) 
             (Instruction.rmode (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg131), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg132)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg132 13 && Bitops.eq arg131 
            (Bits.U.of_int 1 3)) -> 
           Instruction.wrwim (Bits.U.to_native rs1) 
             (Instruction.imode (Bits.U.to_native arg132)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg133), 
            _), RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg133 (Bits.U.of_int 1 3)) -> 
           Instruction.wrwim (Bits.U.to_native rs1) 
             (Instruction.rmode (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg134), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg135)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg135 13 && Bitops.eq arg134 
            (Bits.U.of_int 2 3)) -> 
           Instruction.wrtbr (Bits.U.to_native rs1) 
             (Instruction.imode (Bits.U.to_native arg135)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg136), 
            _), RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg136 (Bits.U.of_int 2 3)) -> 
           Instruction.wrtbr (Bits.U.to_native rs1) 
             (Instruction.rmode (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('b', Rtl.Identity, 
            1, RP.Const (RP.Bits arg137), _), RP.Const (RP.Bits arg138), 1))] 
      when Base.to_bool (Bitops.eq arg137 (Bits.U.of_int 0 1) && 
          Bitops.eq arg138 (Bits.U.of_int 1 1)) -> Instruction.stbar () 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg139)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg139 13) -> 
           Instruction.and_ (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg139)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] -> 
           Instruction.and_ (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.Const (RP.Bits arg160)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg156)]); RP.Const (RP.Bits arg157)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg158), _)), 
              RP.Const (RP.Bits arg159), 1)); (RP.App (("ge", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Const (RP.Bits arg152)]); 
              RP.Const (RP.Bits arg153)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg154), _)), 
              RP.Const (RP.Bits arg155), 1)); (RP.App (("eq", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg148)]); 
              RP.Const (RP.Bits arg149)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg150), _)), 
              RP.Const (RP.Bits arg151), 1)); (RP.App (("ne", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg144)]); 
              RP.Const (RP.Bits arg145)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg146), _)), 
              RP.Const (RP.Bits arg147), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg142), _)), RP.Const (RP.Bits arg143), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg140), _)), 
              RP.Const (RP.Bits arg141), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && 
          rs1 = rs1' && (Bitops.fits_signed arg160 13 && Bitops.eq arg157 
            (Bits.U.of_int 0 32) && Bitops.eq arg158 (Bits.U.of_int 0 3) && 
          Bitops.eq arg159 (Bits.U.of_int 1 1) && Bitops.eq arg153 (Bits.U.of_int 0 
              32) && Bitops.eq arg154 (Bits.U.of_int 0 3) && Bitops.eq arg155 
            (Bits.U.of_int 0 1) && Bitops.eq arg149 (Bits.U.of_int 0 32) && 
          Bitops.eq arg150 (Bits.U.of_int 0 3) && Bitops.eq arg151 
            (Bits.U.of_int 1 1) && Bitops.eq arg145 (Bits.U.of_int 0 32) && 
          Bitops.eq arg146 (Bits.U.of_int 0 3) && Bitops.eq arg147 
            (Bits.U.of_int 0 1) && Bitops.eq arg142 (Bits.U.of_int 0 3) && 
          Bitops.eq arg143 (Bits.U.of_int 0 1) && Bitops.eq arg140 
            (Bits.U.of_int 0 3) && Bitops.eq arg141 (Bits.U.of_int 0 1))) -> 
           Instruction.andcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg160)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''''), _), 32)]), 
            32)); (RP.App (("lt", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)]); 
              RP.Const (RP.Bits arg174)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg175), _)), 
              RP.Const (RP.Bits arg176), 1)); (RP.App (("ge", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''), _), 32)]); 
              RP.Const (RP.Bits arg171)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg172), _)), 
              RP.Const (RP.Bits arg173), 1)); (RP.App (("eq", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)]); 
              RP.Const (RP.Bits arg168)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg169), _)), 
              RP.Const (RP.Bits arg170), 1)); (RP.App (("ne", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]); 
              RP.Const (RP.Bits arg165)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg166), _)), 
              RP.Const (RP.Bits arg167), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg163), _)), RP.Const (RP.Bits arg164), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg161), _)), 
              RP.Const (RP.Bits arg162), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs2 = rs2'''' && rs1 = rs1''' && 
          rs2 = rs2''' && rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && 
          rs2 = rs2' && (Bitops.eq arg174 (Bits.U.of_int 0 32) && Bitops.eq 
            arg175 (Bits.U.of_int 0 3) && Bitops.eq arg176 (Bits.U.of_int 1 1) && 
          Bitops.eq arg171 (Bits.U.of_int 0 32) && Bitops.eq arg172 (Bits.U.of_int 
              0 3) && Bitops.eq arg173 (Bits.U.of_int 0 1) && Bitops.eq arg168 
            (Bits.U.of_int 0 32) && Bitops.eq arg169 (Bits.U.of_int 0 3) && 
          Bitops.eq arg170 (Bits.U.of_int 1 1) && Bitops.eq arg165 (Bits.U.of_int 0 
              32) && Bitops.eq arg166 (Bits.U.of_int 0 3) && Bitops.eq arg167 
            (Bits.U.of_int 0 1) && Bitops.eq arg163 (Bits.U.of_int 0 3) && 
          Bitops.eq arg164 (Bits.U.of_int 0 1) && Bitops.eq arg161 
            (Bits.U.of_int 0 3) && Bitops.eq arg162 (Bits.U.of_int 0 1))) -> 
           Instruction.andcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg177)]), 
            32))] 
      when false -> 
           Sledlib.fail ["could not solve equations: arg177 = com(sx(simm13))"] 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.App (("and", [32]), 
            [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
              _), 32); RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)])]), 
          32))] -> 
           Instruction.andn (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.Const (RP.Bits arg198)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg194)]); RP.Const (RP.Bits arg195)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg196), _)), 
              RP.Const (RP.Bits arg197), 1)); (RP.App (("ge", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Const (RP.Bits arg190)]); 
              RP.Const (RP.Bits arg191)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg192), _)), 
              RP.Const (RP.Bits arg193), 1)); (RP.App (("eq", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg186)]); 
              RP.Const (RP.Bits arg187)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg188), _)), 
              RP.Const (RP.Bits arg189), 1)); (RP.App (("ne", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg182)]); 
              RP.Const (RP.Bits arg183)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg184), _)), 
              RP.Const (RP.Bits arg185), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg180), _)), RP.Const (RP.Bits arg181), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg178), _)), 
              RP.Const (RP.Bits arg179), 1))] 
      when rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
        false -> 
           Sledlib.fail 
             ["could not solve equations: arg182 = com(sx(simm13)), arg186 = com(sx(simm13)), arg190 = com(sx(simm13)), arg194 = com(sx(simm13)), arg198 = com(sx(simm13))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2''''), _), 32)])]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.Const (RP.Bits arg212)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg213), _)), 
              RP.Const (RP.Bits arg214), 1)); (RP.App (("ge", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2''), _), 32)])]); 
              RP.Const (RP.Bits arg209)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg210), _)), 
              RP.Const (RP.Bits arg211), 1)); (RP.App (("eq", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2'), _), 32)])]); 
              RP.Const (RP.Bits arg206)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg207), _)), 
              RP.Const (RP.Bits arg208), 1)); (RP.App (("ne", [32]), 
            [RP.App (("and", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2), _), 32)])]); 
              RP.Const (RP.Bits arg203)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg204), _)), 
              RP.Const (RP.Bits arg205), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg201), _)), RP.Const (RP.Bits arg202), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg199), _)), 
              RP.Const (RP.Bits arg200), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs2 = rs2'''' && rs1 = rs1''' && 
          rs2 = rs2''' && rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && 
          rs2 = rs2' && (Bitops.eq arg212 (Bits.U.of_int 0 32) && Bitops.eq 
            arg213 (Bits.U.of_int 0 3) && Bitops.eq arg214 (Bits.U.of_int 1 1) && 
          Bitops.eq arg209 (Bits.U.of_int 0 32) && Bitops.eq arg210 (Bits.U.of_int 
              0 3) && Bitops.eq arg211 (Bits.U.of_int 0 1) && Bitops.eq arg206 
            (Bits.U.of_int 0 32) && Bitops.eq arg207 (Bits.U.of_int 0 3) && 
          Bitops.eq arg208 (Bits.U.of_int 1 1) && Bitops.eq arg203 (Bits.U.of_int 0 
              32) && Bitops.eq arg204 (Bits.U.of_int 0 3) && Bitops.eq arg205 
            (Bits.U.of_int 0 1) && Bitops.eq arg201 (Bits.U.of_int 0 3) && 
          Bitops.eq arg202 (Bits.U.of_int 0 1) && Bitops.eq arg199 
            (Bits.U.of_int 0 3) && Bitops.eq arg200 (Bits.U.of_int 0 1))) -> 
           Instruction.andncc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg215)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg215 13) -> 
           Instruction.or_ (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg215)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] -> 
           Instruction.or_ (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.Const (RP.Bits arg236)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg232)]); RP.Const (RP.Bits arg233)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg234), _)), 
              RP.Const (RP.Bits arg235), 1)); (RP.App (("ge", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Const (RP.Bits arg228)]); 
              RP.Const (RP.Bits arg229)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg230), _)), 
              RP.Const (RP.Bits arg231), 1)); (RP.App (("eq", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg224)]); 
              RP.Const (RP.Bits arg225)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg226), _)), 
              RP.Const (RP.Bits arg227), 1)); (RP.App (("ne", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg220)]); 
              RP.Const (RP.Bits arg221)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg222), _)), 
              RP.Const (RP.Bits arg223), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg218), _)), RP.Const (RP.Bits arg219), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg216), _)), 
              RP.Const (RP.Bits arg217), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && 
          rs1 = rs1' && (Bitops.fits_signed arg236 13 && Bitops.eq arg233 
            (Bits.U.of_int 0 32) && Bitops.eq arg234 (Bits.U.of_int 0 3) && 
          Bitops.eq arg235 (Bits.U.of_int 1 1) && Bitops.eq arg229 (Bits.U.of_int 0 
              32) && Bitops.eq arg230 (Bits.U.of_int 0 3) && Bitops.eq arg231 
            (Bits.U.of_int 0 1) && Bitops.eq arg225 (Bits.U.of_int 0 32) && 
          Bitops.eq arg226 (Bits.U.of_int 0 3) && Bitops.eq arg227 
            (Bits.U.of_int 1 1) && Bitops.eq arg221 (Bits.U.of_int 0 32) && 
          Bitops.eq arg222 (Bits.U.of_int 0 3) && Bitops.eq arg223 
            (Bits.U.of_int 0 1) && Bitops.eq arg218 (Bits.U.of_int 0 3) && 
          Bitops.eq arg219 (Bits.U.of_int 0 1) && Bitops.eq arg216 
            (Bits.U.of_int 0 3) && Bitops.eq arg217 (Bits.U.of_int 0 1))) -> 
           Instruction.orcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg236)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''''), _), 32)]), 
            32)); (RP.App (("lt", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)]); 
              RP.Const (RP.Bits arg250)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg251), _)), 
              RP.Const (RP.Bits arg252), 1)); (RP.App (("ge", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''), _), 32)]); 
              RP.Const (RP.Bits arg247)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg248), _)), 
              RP.Const (RP.Bits arg249), 1)); (RP.App (("eq", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)]); 
              RP.Const (RP.Bits arg244)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg245), _)), 
              RP.Const (RP.Bits arg246), 1)); (RP.App (("ne", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]); 
              RP.Const (RP.Bits arg241)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg242), _)), 
              RP.Const (RP.Bits arg243), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg239), _)), RP.Const (RP.Bits arg240), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg237), _)), 
              RP.Const (RP.Bits arg238), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs2 = rs2'''' && rs1 = rs1''' && 
          rs2 = rs2''' && rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && 
          rs2 = rs2' && (Bitops.eq arg250 (Bits.U.of_int 0 32) && Bitops.eq 
            arg251 (Bits.U.of_int 0 3) && Bitops.eq arg252 (Bits.U.of_int 1 1) && 
          Bitops.eq arg247 (Bits.U.of_int 0 32) && Bitops.eq arg248 (Bits.U.of_int 
              0 3) && Bitops.eq arg249 (Bits.U.of_int 0 1) && Bitops.eq arg244 
            (Bits.U.of_int 0 32) && Bitops.eq arg245 (Bits.U.of_int 0 3) && 
          Bitops.eq arg246 (Bits.U.of_int 1 1) && Bitops.eq arg241 (Bits.U.of_int 0 
              32) && Bitops.eq arg242 (Bits.U.of_int 0 3) && Bitops.eq arg243 
            (Bits.U.of_int 0 1) && Bitops.eq arg239 (Bits.U.of_int 0 3) && 
          Bitops.eq arg240 (Bits.U.of_int 0 1) && Bitops.eq arg237 
            (Bits.U.of_int 0 3) && Bitops.eq arg238 (Bits.U.of_int 0 1))) -> 
           Instruction.orcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg253)]), 
            32))] 
      when false -> 
           Sledlib.fail ["could not solve equations: arg253 = com(sx(simm13))"] 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.App (("or", [32]), 
            [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
              _), 32); RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)])]), 
          32))] -> 
           Instruction.orn (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.Const (RP.Bits arg274)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg270)]); RP.Const (RP.Bits arg271)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg272), _)), 
              RP.Const (RP.Bits arg273), 1)); (RP.App (("ge", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Const (RP.Bits arg266)]); 
              RP.Const (RP.Bits arg267)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg268), _)), 
              RP.Const (RP.Bits arg269), 1)); (RP.App (("eq", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg262)]); 
              RP.Const (RP.Bits arg263)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg264), _)), 
              RP.Const (RP.Bits arg265), 1)); (RP.App (("ne", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg258)]); 
              RP.Const (RP.Bits arg259)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg260), _)), 
              RP.Const (RP.Bits arg261), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg256), _)), RP.Const (RP.Bits arg257), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg254), _)), 
              RP.Const (RP.Bits arg255), 1))] 
      when rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
        false -> 
           Sledlib.fail 
             ["could not solve equations: arg258 = com(sx(simm13)), arg262 = com(sx(simm13)), arg266 = com(sx(simm13)), arg270 = com(sx(simm13)), arg274 = com(sx(simm13))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2''''), _), 32)])]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.Const (RP.Bits arg288)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg289), _)), 
              RP.Const (RP.Bits arg290), 1)); (RP.App (("ge", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2''), _), 32)])]); 
              RP.Const (RP.Bits arg285)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg286), _)), 
              RP.Const (RP.Bits arg287), 1)); (RP.App (("eq", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2'), _), 32)])]); 
              RP.Const (RP.Bits arg282)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg283), _)), 
              RP.Const (RP.Bits arg284), 1)); (RP.App (("ne", [32]), 
            [RP.App (("or", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2), _), 32)])]); 
              RP.Const (RP.Bits arg279)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg280), _)), 
              RP.Const (RP.Bits arg281), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg277), _)), RP.Const (RP.Bits arg278), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg275), _)), 
              RP.Const (RP.Bits arg276), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs2 = rs2'''' && rs1 = rs1''' && 
          rs2 = rs2''' && rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && 
          rs2 = rs2' && (Bitops.eq arg288 (Bits.U.of_int 0 32) && Bitops.eq 
            arg289 (Bits.U.of_int 0 3) && Bitops.eq arg290 (Bits.U.of_int 1 1) && 
          Bitops.eq arg285 (Bits.U.of_int 0 32) && Bitops.eq arg286 (Bits.U.of_int 
              0 3) && Bitops.eq arg287 (Bits.U.of_int 0 1) && Bitops.eq arg282 
            (Bits.U.of_int 0 32) && Bitops.eq arg283 (Bits.U.of_int 0 3) && 
          Bitops.eq arg284 (Bits.U.of_int 1 1) && Bitops.eq arg279 (Bits.U.of_int 0 
              32) && Bitops.eq arg280 (Bits.U.of_int 0 3) && Bitops.eq arg281 
            (Bits.U.of_int 0 1) && Bitops.eq arg277 (Bits.U.of_int 0 3) && 
          Bitops.eq arg278 (Bits.U.of_int 0 1) && Bitops.eq arg275 
            (Bits.U.of_int 0 3) && Bitops.eq arg276 (Bits.U.of_int 0 1))) -> 
           Instruction.orncc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg291)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg291 13) -> 
           Instruction.xor (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg291)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] -> 
           Instruction.xor (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.Const (RP.Bits arg312)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg308)]); RP.Const (RP.Bits arg309)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg310), _)), 
              RP.Const (RP.Bits arg311), 1)); (RP.App (("ge", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Const (RP.Bits arg304)]); 
              RP.Const (RP.Bits arg305)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg306), _)), 
              RP.Const (RP.Bits arg307), 1)); (RP.App (("eq", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg300)]); 
              RP.Const (RP.Bits arg301)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg302), _)), 
              RP.Const (RP.Bits arg303), 1)); (RP.App (("ne", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg296)]); 
              RP.Const (RP.Bits arg297)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg298), _)), 
              RP.Const (RP.Bits arg299), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg294), _)), RP.Const (RP.Bits arg295), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg292), _)), 
              RP.Const (RP.Bits arg293), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && 
          rs1 = rs1' && (Bitops.fits_signed arg312 13 && Bitops.eq arg309 
            (Bits.U.of_int 0 32) && Bitops.eq arg310 (Bits.U.of_int 0 3) && 
          Bitops.eq arg311 (Bits.U.of_int 1 1) && Bitops.eq arg305 (Bits.U.of_int 0 
              32) && Bitops.eq arg306 (Bits.U.of_int 0 3) && Bitops.eq arg307 
            (Bits.U.of_int 0 1) && Bitops.eq arg301 (Bits.U.of_int 0 32) && 
          Bitops.eq arg302 (Bits.U.of_int 0 3) && Bitops.eq arg303 
            (Bits.U.of_int 1 1) && Bitops.eq arg297 (Bits.U.of_int 0 32) && 
          Bitops.eq arg298 (Bits.U.of_int 0 3) && Bitops.eq arg299 
            (Bits.U.of_int 0 1) && Bitops.eq arg294 (Bits.U.of_int 0 3) && 
          Bitops.eq arg295 (Bits.U.of_int 0 1) && Bitops.eq arg292 
            (Bits.U.of_int 0 3) && Bitops.eq arg293 (Bits.U.of_int 0 1))) -> 
           Instruction.xorcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg312)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''''), _), 32)]), 
            32)); (RP.App (("lt", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)]); 
              RP.Const (RP.Bits arg326)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg327), _)), 
              RP.Const (RP.Bits arg328), 1)); (RP.App (("ge", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''), _), 32)]); 
              RP.Const (RP.Bits arg323)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg324), _)), 
              RP.Const (RP.Bits arg325), 1)); (RP.App (("eq", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)]); 
              RP.Const (RP.Bits arg320)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg321), _)), 
              RP.Const (RP.Bits arg322), 1)); (RP.App (("ne", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]); 
              RP.Const (RP.Bits arg317)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg318), _)), 
              RP.Const (RP.Bits arg319), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg315), _)), RP.Const (RP.Bits arg316), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg313), _)), 
              RP.Const (RP.Bits arg314), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs2 = rs2'''' && rs1 = rs1''' && 
          rs2 = rs2''' && rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && 
          rs2 = rs2' && (Bitops.eq arg326 (Bits.U.of_int 0 32) && Bitops.eq 
            arg327 (Bits.U.of_int 0 3) && Bitops.eq arg328 (Bits.U.of_int 1 1) && 
          Bitops.eq arg323 (Bits.U.of_int 0 32) && Bitops.eq arg324 (Bits.U.of_int 
              0 3) && Bitops.eq arg325 (Bits.U.of_int 0 1) && Bitops.eq arg320 
            (Bits.U.of_int 0 32) && Bitops.eq arg321 (Bits.U.of_int 0 3) && 
          Bitops.eq arg322 (Bits.U.of_int 1 1) && Bitops.eq arg317 (Bits.U.of_int 0 
              32) && Bitops.eq arg318 (Bits.U.of_int 0 3) && Bitops.eq arg319 
            (Bits.U.of_int 0 1) && Bitops.eq arg315 (Bits.U.of_int 0 3) && 
          Bitops.eq arg316 (Bits.U.of_int 0 1) && Bitops.eq arg313 
            (Bits.U.of_int 0 3) && Bitops.eq arg314 (Bits.U.of_int 0 1))) -> 
           Instruction.xorcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg329)]), 
            32))] 
      when false -> 
           Sledlib.fail ["could not solve equations: arg329 = com(sx(simm13))"] 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
          32, RP.Const (RP.Bits rd), _), RP.App (("xor", [32]), 
            [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rs1), 
              _), 32); RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)])]), 
          32))] -> 
           Instruction.xnor (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.Const (RP.Bits arg350)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg346)]); RP.Const (RP.Bits arg347)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg348), _)), 
              RP.Const (RP.Bits arg349), 1)); (RP.App (("ge", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Const (RP.Bits arg342)]); 
              RP.Const (RP.Bits arg343)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg344), _)), 
              RP.Const (RP.Bits arg345), 1)); (RP.App (("eq", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg338)]); 
              RP.Const (RP.Bits arg339)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg340), _)), 
              RP.Const (RP.Bits arg341), 1)); (RP.App (("ne", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg334)]); 
              RP.Const (RP.Bits arg335)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg336), _)), 
              RP.Const (RP.Bits arg337), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg332), _)), RP.Const (RP.Bits arg333), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg330), _)), 
              RP.Const (RP.Bits arg331), 1))] 
      when rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
        false -> 
           Sledlib.fail 
             ["could not solve equations: arg334 = com(sx(simm13)), arg338 = com(sx(simm13)), arg342 = com(sx(simm13)), arg346 = com(sx(simm13)), arg350 = com(sx(simm13))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2''''), _), 32)])]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.Const (RP.Bits arg364)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg365), _)), 
              RP.Const (RP.Bits arg366), 1)); (RP.App (("ge", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2''), _), 32)])]); 
              RP.Const (RP.Bits arg361)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg362), _)), 
              RP.Const (RP.Bits arg363), 1)); (RP.App (("eq", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2'), _), 32)])]); 
              RP.Const (RP.Bits arg358)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg359), _)), 
              RP.Const (RP.Bits arg360), 1)); (RP.App (("ne", [32]), 
            [RP.App (("xor", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); 
                RP.App (("com", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                    32, RP.Const (RP.Bits rs2), _), 32)])]); 
              RP.Const (RP.Bits arg355)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg356), _)), 
              RP.Const (RP.Bits arg357), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg353), _)), RP.Const (RP.Bits arg354), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg351), _)), 
              RP.Const (RP.Bits arg352), 1))] 
      when Base.to_bool (rs1 = rs1'''' && rs2 = rs2'''' && rs1 = rs1''' && 
          rs2 = rs2''' && rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && 
          rs2 = rs2' && (Bitops.eq arg364 (Bits.U.of_int 0 32) && Bitops.eq 
            arg365 (Bits.U.of_int 0 3) && Bitops.eq arg366 (Bits.U.of_int 1 1) && 
          Bitops.eq arg361 (Bits.U.of_int 0 32) && Bitops.eq arg362 (Bits.U.of_int 
              0 3) && Bitops.eq arg363 (Bits.U.of_int 0 1) && Bitops.eq arg358 
            (Bits.U.of_int 0 32) && Bitops.eq arg359 (Bits.U.of_int 0 3) && 
          Bitops.eq arg360 (Bits.U.of_int 1 1) && Bitops.eq arg355 (Bits.U.of_int 0 
              32) && Bitops.eq arg356 (Bits.U.of_int 0 3) && Bitops.eq arg357 
            (Bits.U.of_int 0 1) && Bitops.eq arg353 (Bits.U.of_int 0 3) && 
          Bitops.eq arg354 (Bits.U.of_int 0 1) && Bitops.eq arg351 
            (Bits.U.of_int 0 3) && Bitops.eq arg352 (Bits.U.of_int 0 1))) -> 
           Instruction.xnorcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("shl", [32; 5]), 
              [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg367)]), 
            32))] 
      when false -> 
           Sledlib.fail 
             ["could not solve equations: arg367 = bitExtract(0, sx(simm13))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("shl", [32; 5]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.App (("bitExtract", 
                  [32; 32; 5]), [RP.Const (RP.Bits arg368); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2), _), 32)])]), 32))] 
      when Base.to_bool (Bitops.eq arg368 (Bits.U.of_int 0 32)) -> 
           Instruction.sll (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("shrl", [32; 5]), 
              [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg369)]), 
            32))] 
      when false -> 
           Sledlib.fail 
             ["could not solve equations: arg369 = bitExtract(0, sx(simm13))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("shrl", [32; 5]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.App (("bitExtract", 
                  [32; 32; 5]), [RP.Const (RP.Bits arg370); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2), _), 32)])]), 32))] 
      when Base.to_bool (Bitops.eq arg370 (Bits.U.of_int 0 32)) -> 
           Instruction.srl (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("shra", [32; 5]), 
              [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg371)]), 
            32))] 
      when false -> 
           Sledlib.fail 
             ["could not solve equations: arg371 = bitExtract(0, sx(simm13))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("shra", [32; 5]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.App (("bitExtract", 
                  [32; 32; 5]), [RP.Const (RP.Bits arg372); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2), _), 32)])]), 32))] 
      when Base.to_bool (Bitops.eq arg372 (Bits.U.of_int 0 32)) -> 
           Instruction.sra (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg373)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg373 13) -> 
           Instruction.add (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg373)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] -> 
           Instruction.add (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Const (RP.Bits arg411)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Const (RP.Bits arg407)]); RP.Const (RP.Bits arg408)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg409), _)), 
              RP.Const (RP.Bits arg410), 1)); (RP.App (("ge", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Const (RP.Bits arg403)]); RP.Const (RP.Bits arg404)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg405), _)), 
              RP.Const (RP.Bits arg406), 1)); (RP.App (("eq", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Const (RP.Bits arg399)]); RP.Const (RP.Bits arg400)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg401), _)), 
              RP.Const (RP.Bits arg402), 1)); (RP.App (("ne", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Const (RP.Bits arg395)]); RP.Const (RP.Bits arg396)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg397), _)), 
              RP.Const (RP.Bits arg398), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg391); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg392)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg390); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg389); RP.App (("add", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Const (RP.Bits arg388)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg393), _)), 
              RP.Const (RP.Bits arg394), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg384); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg385)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg383); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg382); 
                      RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Const (RP.Bits arg381)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg386), _)), RP.Const (RP.Bits arg387), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg377); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg378)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg379), _)), 
              RP.Const (RP.Bits arg380), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg374), _)), 
              RP.App (("carry", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg376); RP.Const (RP.Bits arg375)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg411 13 && Bitops.eq arg408 (Bits.U.of_int 0 
              32) && Bitops.eq arg409 (Bits.U.of_int 0 3) && Bitops.eq arg410 
            (Bits.U.of_int 1 1) && Bitops.eq arg404 (Bits.U.of_int 0 32) && 
          Bitops.eq arg405 (Bits.U.of_int 0 3) && Bitops.eq arg406 (Bits.U.of_int 0 
              1) && Bitops.eq arg400 (Bits.U.of_int 0 32) && Bitops.eq arg401 
            (Bits.U.of_int 0 3) && Bitops.eq arg402 (Bits.U.of_int 1 1) && 
          Bitops.eq arg396 (Bits.U.of_int 0 32) && Bitops.eq arg397 (Bits.U.of_int 
              0 3) && Bitops.eq arg398 (Bits.U.of_int 0 1) && Bitops.eq arg391 
            (Bits.U.of_int 31 32) && Bitops.eq arg390 (Bits.U.of_int 31 32) && 
          Bitops.eq arg389 (Bits.U.of_int 31 32) && Bitops.eq arg393 (Bits.U.of_int 
              0 3) && Bitops.eq arg394 (Bits.U.of_int 1 1) && Bitops.eq arg384 
            (Bits.U.of_int 31 32) && Bitops.eq arg383 (Bits.U.of_int 31 32) && 
          Bitops.eq arg382 (Bits.U.of_int 31 32) && Bitops.eq arg386 (Bits.U.of_int 
              0 3) && Bitops.eq arg387 (Bits.U.of_int 0 1) && Bitops.eq arg377 
            (Bits.U.of_int 31 32) && Bitops.eq arg379 (Bits.U.of_int 0 3) && 
          Bitops.eq arg380 (Bits.U.of_int 0 1) && Bitops.eq arg374 
            (Bits.U.of_int 0 3) && Bitops.eq arg375 (Bits.U.of_int 0 1))) -> 
           Instruction.addcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg411)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''''), _), 32)]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''''), _), 32)]); 
              RP.Const (RP.Bits arg439)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg440), _)), 
              RP.Const (RP.Bits arg441), 1)); (RP.App (("ge", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''), _), 32)]); 
              RP.Const (RP.Bits arg436)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg437), _)), 
              RP.Const (RP.Bits arg438), 1)); (RP.App (("eq", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''), _), 32)]); 
              RP.Const (RP.Bits arg433)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg434), _)), 
              RP.Const (RP.Bits arg435), 1)); (RP.App (("ne", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''), _), 32)]); 
              RP.Const (RP.Bits arg430)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg431), _)), 
              RP.Const (RP.Bits arg432), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg427); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg426); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg425); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg424); RP.App (("add", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg428), _)), 
              RP.Const (RP.Bits arg429), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg421); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg420); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg419); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg418); 
                      RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg422), _)), RP.Const (RP.Bits arg423), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg415); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg414); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg416), _)), RP.Const (RP.Bits arg417), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg412), _)), 
              RP.App (("carry", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Const (RP.Bits arg413)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg439 (Bits.U.of_int 0 32) && Bitops.eq arg440 (Bits.U.of_int 0 3) && 
          Bitops.eq arg441 (Bits.U.of_int 1 1) && Bitops.eq arg436 (Bits.U.of_int 0 
              32) && Bitops.eq arg437 (Bits.U.of_int 0 3) && Bitops.eq arg438 
            (Bits.U.of_int 0 1) && Bitops.eq arg433 (Bits.U.of_int 0 32) && 
          Bitops.eq arg434 (Bits.U.of_int 0 3) && Bitops.eq arg435 (Bits.U.of_int 1 
              1) && Bitops.eq arg430 (Bits.U.of_int 0 32) && Bitops.eq arg431 
            (Bits.U.of_int 0 3) && Bitops.eq arg432 (Bits.U.of_int 0 1) && 
          Bitops.eq arg427 (Bits.U.of_int 31 32) && Bitops.eq arg426 (Bits.U.of_int 
              31 32) && Bitops.eq arg425 (Bits.U.of_int 31 32) && Bitops.eq 
            arg424 (Bits.U.of_int 31 32) && Bitops.eq arg428 (Bits.U.of_int 0 3) && 
          Bitops.eq arg429 (Bits.U.of_int 1 1) && Bitops.eq arg421 
            (Bits.U.of_int 31 32) && Bitops.eq arg420 (Bits.U.of_int 31 32) && 
          Bitops.eq arg419 (Bits.U.of_int 31 32) && Bitops.eq arg418 (Bits.U.of_int 
              31 32) && Bitops.eq arg422 (Bits.U.of_int 0 3) && Bitops.eq arg423 
            (Bits.U.of_int 0 1) && Bitops.eq arg415 (Bits.U.of_int 31 32) && 
          Bitops.eq arg414 (Bits.U.of_int 31 32) && Bitops.eq arg416 (Bits.U.of_int 
              0 3) && Bitops.eq arg417 (Bits.U.of_int 0 1) && Bitops.eq arg412 
            (Bits.U.of_int 0 3) && Bitops.eq arg413 (Bits.U.of_int 0 1))) -> 
           Instruction.addcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg443)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg442), _)), 1)])]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg443 13 && Bitops.eq arg442 
            (Bits.U.of_int 0 3)) -> 
           Instruction.addx (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg443)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]); 
                RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg444), _)), 1)])]), 32))] 
      when Base.to_bool (Bitops.eq arg444 (Bits.U.of_int 0 3)) -> 
           Instruction.addx (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                  RP.Const (RP.Bits arg489)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg488), _)), 1)])]), 32)); 
          (RP.App (("lt", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                  RP.Const (RP.Bits arg484)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg483), _)), 1)])]); 
              RP.Const (RP.Bits arg485)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg486), _)), 
              RP.Const (RP.Bits arg487), 1)); 
          (RP.App (("ge", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''), _), 32); 
                  RP.Const (RP.Bits arg479)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg478), _)), 1)])]); 
              RP.Const (RP.Bits arg480)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg481), _)), 
              RP.Const (RP.Bits arg482), 1)); 
          (RP.App (("eq", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''), _), 32); 
                  RP.Const (RP.Bits arg474)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg473), _)), 1)])]); 
              RP.Const (RP.Bits arg475)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg476), _)), 
              RP.Const (RP.Bits arg477), 1)); 
          (RP.App (("ne", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''), _), 32); 
                  RP.Const (RP.Bits arg469)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg468), _)), 1)])]); 
              RP.Const (RP.Bits arg470)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg471), _)), 
              RP.Const (RP.Bits arg472), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg464); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg465)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg463); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg462); RP.App (("add", [32]), 
                        [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 
                            32); RP.Const (RP.Bits arg461)]); RP.App (("zx", 
                            [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                                Rtl.Identity, 32, RP.Const (RP.Bits arg460), 
                                _)), 1)])])])])]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg466), _)), 
              RP.Const (RP.Bits arg467), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg456); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg457)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg455); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg454); RP.App (("add", [32]), 
                        [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 
                            32); RP.Const (RP.Bits arg453)]); 
                          RP.App (("zx", [1; 32]), [RP.Fetch (RP.Slice (1, 20, 
                              RP.Cell ('i', Rtl.Identity, 32, 
                                RP.Const (RP.Bits arg452), _)), 1)])])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg458), _)), RP.Const (RP.Bits arg459), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg448); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg449)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg450), _)), 
              RP.Const (RP.Bits arg451), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg445), _)), RP.App (("carry", [32]), 
                [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg447); 
                  RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg446), _)), 1)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg489 13 && Bitops.eq arg488 (Bits.U.of_int 0 
              3) && Bitops.eq arg483 (Bits.U.of_int 0 3) && Bitops.eq arg485 
            (Bits.U.of_int 0 32) && Bitops.eq arg486 (Bits.U.of_int 0 3) && 
          Bitops.eq arg487 (Bits.U.of_int 1 1) && Bitops.eq arg478 (Bits.U.of_int 0 
              3) && Bitops.eq arg480 (Bits.U.of_int 0 32) && Bitops.eq arg481 
            (Bits.U.of_int 0 3) && Bitops.eq arg482 (Bits.U.of_int 0 1) && 
          Bitops.eq arg473 (Bits.U.of_int 0 3) && Bitops.eq arg475 (Bits.U.of_int 0 
              32) && Bitops.eq arg476 (Bits.U.of_int 0 3) && Bitops.eq arg477 
            (Bits.U.of_int 1 1) && Bitops.eq arg468 (Bits.U.of_int 0 3) && 
          Bitops.eq arg470 (Bits.U.of_int 0 32) && Bitops.eq arg471 (Bits.U.of_int 
              0 3) && Bitops.eq arg472 (Bits.U.of_int 0 1) && Bitops.eq arg464 
            (Bits.U.of_int 31 32) && Bitops.eq arg463 (Bits.U.of_int 31 32) && 
          Bitops.eq arg462 (Bits.U.of_int 31 32) && Bitops.eq arg460 (Bits.U.of_int 
              0 3) && Bitops.eq arg466 (Bits.U.of_int 0 3) && Bitops.eq arg467 
            (Bits.U.of_int 1 1) && Bitops.eq arg456 (Bits.U.of_int 31 32) && 
          Bitops.eq arg455 (Bits.U.of_int 31 32) && Bitops.eq arg454 (Bits.U.of_int 
              31 32) && Bitops.eq arg452 (Bits.U.of_int 0 3) && Bitops.eq arg458 
            (Bits.U.of_int 0 3) && Bitops.eq arg459 (Bits.U.of_int 0 1) && 
          Bitops.eq arg448 (Bits.U.of_int 31 32) && Bitops.eq arg450 (Bits.U.of_int 
              0 3) && Bitops.eq arg451 (Bits.U.of_int 0 1) && Bitops.eq arg445 
            (Bits.U.of_int 0 3) && Bitops.eq arg446 (Bits.U.of_int 0 3))) -> 
           Instruction.addxcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg489)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2''''''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg526), _)), 1)])]), 
            32)); (RP.App (("lt", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2'''''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg522), _)), 1)])]); 
              RP.Const (RP.Bits arg523)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg524), _)), 
              RP.Const (RP.Bits arg525), 1)); 
          (RP.App (("ge", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2''''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg518), _)), 1)])]); 
              RP.Const (RP.Bits arg519)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg520), _)), 
              RP.Const (RP.Bits arg521), 1)); 
          (RP.App (("eq", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2'''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg514), _)), 1)])]); 
              RP.Const (RP.Bits arg515)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg516), _)), 
              RP.Const (RP.Bits arg517), 1)); 
          (RP.App (("ne", [32]), [RP.App (("add", [32]), 
              [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg510), _)), 1)])]); 
              RP.Const (RP.Bits arg511)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg512), _)), 
              RP.Const (RP.Bits arg513), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg507); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg506); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg505); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg504); RP.App (("add", [32]), 
                        [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 
                            32); RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                              RP.Const (RP.Bits rs2''''), _), 32)]); 
                          RP.App (("zx", [1; 32]), [RP.Fetch (RP.Slice (1, 20, 
                              RP.Cell ('i', Rtl.Identity, 32, 
                                RP.Const (RP.Bits arg503), _)), 1)])])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg508), _)), 
              RP.Const (RP.Bits arg509), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg500); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg499); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg498); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg497); RP.App (("add", [32]), 
                        [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 
                            32); RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                              RP.Const (RP.Bits rs2''), _), 32)]); 
                          RP.App (("zx", [1; 32]), [RP.Fetch (RP.Slice (1, 20, 
                              RP.Cell ('i', Rtl.Identity, 32, 
                                RP.Const (RP.Bits arg496), _)), 1)])])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg501), _)), RP.Const (RP.Bits arg502), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg493); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg492); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg494), _)), RP.Const (RP.Bits arg495), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg490), _)), 
              RP.App (("carry", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg491), _)), 1)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg526 (Bits.U.of_int 0 3) && Bitops.eq arg522 (Bits.U.of_int 0 3) && 
          Bitops.eq arg523 (Bits.U.of_int 0 32) && Bitops.eq arg524 (Bits.U.of_int 
              0 3) && Bitops.eq arg525 (Bits.U.of_int 1 1) && Bitops.eq arg518 
            (Bits.U.of_int 0 3) && Bitops.eq arg519 (Bits.U.of_int 0 32) && 
          Bitops.eq arg520 (Bits.U.of_int 0 3) && Bitops.eq arg521 (Bits.U.of_int 0 
              1) && Bitops.eq arg514 (Bits.U.of_int 0 3) && Bitops.eq arg515 
            (Bits.U.of_int 0 32) && Bitops.eq arg516 (Bits.U.of_int 0 3) && 
          Bitops.eq arg517 (Bits.U.of_int 1 1) && Bitops.eq arg510 (Bits.U.of_int 0 
              3) && Bitops.eq arg511 (Bits.U.of_int 0 32) && Bitops.eq arg512 
            (Bits.U.of_int 0 3) && Bitops.eq arg513 (Bits.U.of_int 0 1) && 
          Bitops.eq arg507 (Bits.U.of_int 31 32) && Bitops.eq arg506 (Bits.U.of_int 
              31 32) && Bitops.eq arg505 (Bits.U.of_int 31 32) && Bitops.eq 
            arg504 (Bits.U.of_int 31 32) && Bitops.eq arg503 (Bits.U.of_int 0 3) && 
          Bitops.eq arg508 (Bits.U.of_int 0 3) && Bitops.eq arg509 (Bits.U.of_int 1 
              1) && Bitops.eq arg500 (Bits.U.of_int 31 32) && Bitops.eq arg499 
            (Bits.U.of_int 31 32) && Bitops.eq arg498 (Bits.U.of_int 31 32) && 
          Bitops.eq arg497 (Bits.U.of_int 31 32) && Bitops.eq arg496 (Bits.U.of_int 
              0 3) && Bitops.eq arg501 (Bits.U.of_int 0 3) && Bitops.eq arg502 
            (Bits.U.of_int 0 1) && Bitops.eq arg493 (Bits.U.of_int 31 32) && 
          Bitops.eq arg492 (Bits.U.of_int 31 32) && Bitops.eq arg494 (Bits.U.of_int 
              0 3) && Bitops.eq arg495 (Bits.U.of_int 0 1) && Bitops.eq arg490 
            (Bits.U.of_int 0 3) && Bitops.eq arg491 (Bits.U.of_int 0 3))) -> 
           Instruction.addxcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Const (RP.Bits arg564)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Const (RP.Bits arg560)]); RP.Const (RP.Bits arg561)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg562), _)), 
              RP.Const (RP.Bits arg563), 1)); (RP.App (("ge", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Const (RP.Bits arg556)]); RP.Const (RP.Bits arg557)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg558), _)), 
              RP.Const (RP.Bits arg559), 1)); (RP.App (("eq", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Const (RP.Bits arg552)]); RP.Const (RP.Bits arg553)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg554), _)), 
              RP.Const (RP.Bits arg555), 1)); (RP.App (("ne", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Const (RP.Bits arg548)]); RP.Const (RP.Bits arg549)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg550), _)), 
              RP.Const (RP.Bits arg551), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg544); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg545)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg543); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg542); RP.App (("add", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Const (RP.Bits arg541)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg546), _)), 
              RP.Const (RP.Bits arg547), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg537); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg538)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg536); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg535); 
                      RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Const (RP.Bits arg534)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg539), _)), RP.Const (RP.Bits arg540), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg530); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg531)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg532), _)), 
              RP.Const (RP.Bits arg533), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg527), _)), 
              RP.App (("carry", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg529); RP.Const (RP.Bits arg528)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg564 13 && Bitops.eq arg561 (Bits.U.of_int 0 
              32) && Bitops.eq arg562 (Bits.U.of_int 0 3) && Bitops.eq arg563 
            (Bits.U.of_int 1 1) && Bitops.eq arg557 (Bits.U.of_int 0 32) && 
          Bitops.eq arg558 (Bits.U.of_int 0 3) && Bitops.eq arg559 (Bits.U.of_int 0 
              1) && Bitops.eq arg553 (Bits.U.of_int 0 32) && Bitops.eq arg554 
            (Bits.U.of_int 0 3) && Bitops.eq arg555 (Bits.U.of_int 1 1) && 
          Bitops.eq arg549 (Bits.U.of_int 0 32) && Bitops.eq arg550 (Bits.U.of_int 
              0 3) && Bitops.eq arg551 (Bits.U.of_int 0 1) && Bitops.eq arg544 
            (Bits.U.of_int 31 32) && Bitops.eq arg543 (Bits.U.of_int 31 32) && 
          Bitops.eq arg542 (Bits.U.of_int 31 32) && Bitops.eq arg546 (Bits.U.of_int 
              0 3) && Bitops.eq arg547 (Bits.U.of_int 1 1) && Bitops.eq arg537 
            (Bits.U.of_int 31 32) && Bitops.eq arg536 (Bits.U.of_int 31 32) && 
          Bitops.eq arg535 (Bits.U.of_int 31 32) && Bitops.eq arg539 (Bits.U.of_int 
              0 3) && Bitops.eq arg540 (Bits.U.of_int 0 1) && Bitops.eq arg530 
            (Bits.U.of_int 31 32) && Bitops.eq arg532 (Bits.U.of_int 0 3) && 
          Bitops.eq arg533 (Bits.U.of_int 0 1) && Bitops.eq arg527 
            (Bits.U.of_int 0 3) && Bitops.eq arg528 (Bits.U.of_int 0 1))) -> 
           Instruction.taddcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg564)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''''), _), 32)]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''''), _), 32)]); 
              RP.Const (RP.Bits arg592)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg593), _)), 
              RP.Const (RP.Bits arg594), 1)); (RP.App (("ge", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''), _), 32)]); 
              RP.Const (RP.Bits arg589)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg590), _)), 
              RP.Const (RP.Bits arg591), 1)); (RP.App (("eq", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''), _), 32)]); 
              RP.Const (RP.Bits arg586)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg587), _)), 
              RP.Const (RP.Bits arg588), 1)); (RP.App (("ne", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''), _), 32)]); 
              RP.Const (RP.Bits arg583)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg584), _)), 
              RP.Const (RP.Bits arg585), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg580); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg579); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg578); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg577); RP.App (("add", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg581), _)), 
              RP.Const (RP.Bits arg582), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg574); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg573); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg572); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg571); 
                      RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg575), _)), RP.Const (RP.Bits arg576), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg568); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg567); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg569), _)), RP.Const (RP.Bits arg570), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg565), _)), 
              RP.App (("carry", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Const (RP.Bits arg566)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg592 (Bits.U.of_int 0 32) && Bitops.eq arg593 (Bits.U.of_int 0 3) && 
          Bitops.eq arg594 (Bits.U.of_int 1 1) && Bitops.eq arg589 (Bits.U.of_int 0 
              32) && Bitops.eq arg590 (Bits.U.of_int 0 3) && Bitops.eq arg591 
            (Bits.U.of_int 0 1) && Bitops.eq arg586 (Bits.U.of_int 0 32) && 
          Bitops.eq arg587 (Bits.U.of_int 0 3) && Bitops.eq arg588 (Bits.U.of_int 1 
              1) && Bitops.eq arg583 (Bits.U.of_int 0 32) && Bitops.eq arg584 
            (Bits.U.of_int 0 3) && Bitops.eq arg585 (Bits.U.of_int 0 1) && 
          Bitops.eq arg580 (Bits.U.of_int 31 32) && Bitops.eq arg579 (Bits.U.of_int 
              31 32) && Bitops.eq arg578 (Bits.U.of_int 31 32) && Bitops.eq 
            arg577 (Bits.U.of_int 31 32) && Bitops.eq arg581 (Bits.U.of_int 0 3) && 
          Bitops.eq arg582 (Bits.U.of_int 1 1) && Bitops.eq arg574 
            (Bits.U.of_int 31 32) && Bitops.eq arg573 (Bits.U.of_int 31 32) && 
          Bitops.eq arg572 (Bits.U.of_int 31 32) && Bitops.eq arg571 (Bits.U.of_int 
              31 32) && Bitops.eq arg575 (Bits.U.of_int 0 3) && Bitops.eq arg576 
            (Bits.U.of_int 0 1) && Bitops.eq arg568 (Bits.U.of_int 31 32) && 
          Bitops.eq arg567 (Bits.U.of_int 31 32) && Bitops.eq arg569 (Bits.U.of_int 
              0 3) && Bitops.eq arg570 (Bits.U.of_int 0 1) && Bitops.eq arg565 
            (Bits.U.of_int 0 3) && Bitops.eq arg566 (Bits.U.of_int 0 1))) -> 
           Instruction.taddcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Const (RP.Bits arg632)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Const (RP.Bits arg628)]); RP.Const (RP.Bits arg629)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg630), _)), 
              RP.Const (RP.Bits arg631), 1)); (RP.App (("ge", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Const (RP.Bits arg624)]); RP.Const (RP.Bits arg625)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg626), _)), 
              RP.Const (RP.Bits arg627), 1)); (RP.App (("eq", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Const (RP.Bits arg620)]); RP.Const (RP.Bits arg621)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg622), _)), 
              RP.Const (RP.Bits arg623), 1)); (RP.App (("ne", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Const (RP.Bits arg616)]); RP.Const (RP.Bits arg617)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg618), _)), 
              RP.Const (RP.Bits arg619), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg612); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg613)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg611); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg610); RP.App (("add", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Const (RP.Bits arg609)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg614), _)), 
              RP.Const (RP.Bits arg615), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg605); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg606)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg604); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg603); 
                      RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Const (RP.Bits arg602)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg607), _)), RP.Const (RP.Bits arg608), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg598); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg599)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg600), _)), 
              RP.Const (RP.Bits arg601), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg595), _)), 
              RP.App (("carry", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg597); RP.Const (RP.Bits arg596)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg632 13 && Bitops.eq arg629 (Bits.U.of_int 0 
              32) && Bitops.eq arg630 (Bits.U.of_int 0 3) && Bitops.eq arg631 
            (Bits.U.of_int 1 1) && Bitops.eq arg625 (Bits.U.of_int 0 32) && 
          Bitops.eq arg626 (Bits.U.of_int 0 3) && Bitops.eq arg627 (Bits.U.of_int 0 
              1) && Bitops.eq arg621 (Bits.U.of_int 0 32) && Bitops.eq arg622 
            (Bits.U.of_int 0 3) && Bitops.eq arg623 (Bits.U.of_int 1 1) && 
          Bitops.eq arg617 (Bits.U.of_int 0 32) && Bitops.eq arg618 (Bits.U.of_int 
              0 3) && Bitops.eq arg619 (Bits.U.of_int 0 1) && Bitops.eq arg612 
            (Bits.U.of_int 31 32) && Bitops.eq arg611 (Bits.U.of_int 31 32) && 
          Bitops.eq arg610 (Bits.U.of_int 31 32) && Bitops.eq arg614 (Bits.U.of_int 
              0 3) && Bitops.eq arg615 (Bits.U.of_int 1 1) && Bitops.eq arg605 
            (Bits.U.of_int 31 32) && Bitops.eq arg604 (Bits.U.of_int 31 32) && 
          Bitops.eq arg603 (Bits.U.of_int 31 32) && Bitops.eq arg607 (Bits.U.of_int 
              0 3) && Bitops.eq arg608 (Bits.U.of_int 0 1) && Bitops.eq arg598 
            (Bits.U.of_int 31 32) && Bitops.eq arg600 (Bits.U.of_int 0 3) && 
          Bitops.eq arg601 (Bits.U.of_int 0 1) && Bitops.eq arg595 
            (Bits.U.of_int 0 3) && Bitops.eq arg596 (Bits.U.of_int 0 1))) -> 
           Instruction.taddcctv (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg632)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''''), _), 32)]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''''), _), 32)]); 
              RP.Const (RP.Bits arg660)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg661), _)), 
              RP.Const (RP.Bits arg662), 1)); (RP.App (("ge", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''), _), 32)]); 
              RP.Const (RP.Bits arg657)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg658), _)), 
              RP.Const (RP.Bits arg659), 1)); (RP.App (("eq", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''), _), 32)]); 
              RP.Const (RP.Bits arg654)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg655), _)), 
              RP.Const (RP.Bits arg656), 1)); (RP.App (("ne", [32]), 
            [RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''), _), 32)]); 
              RP.Const (RP.Bits arg651)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg652), _)), 
              RP.Const (RP.Bits arg653), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg648); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg647); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg646); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg645); RP.App (("add", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg649), _)), 
              RP.Const (RP.Bits arg650), 1)); (RP.App (("conjoin", []), 
            [RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg642); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg641); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg640); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg639); 
                      RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg643), _)), RP.Const (RP.Bits arg644), 1)); 
          (RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg636); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg635); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg637), _)), RP.Const (RP.Bits arg638), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg633), _)), 
              RP.App (("carry", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Const (RP.Bits arg634)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg660 (Bits.U.of_int 0 32) && Bitops.eq arg661 (Bits.U.of_int 0 3) && 
          Bitops.eq arg662 (Bits.U.of_int 1 1) && Bitops.eq arg657 (Bits.U.of_int 0 
              32) && Bitops.eq arg658 (Bits.U.of_int 0 3) && Bitops.eq arg659 
            (Bits.U.of_int 0 1) && Bitops.eq arg654 (Bits.U.of_int 0 32) && 
          Bitops.eq arg655 (Bits.U.of_int 0 3) && Bitops.eq arg656 (Bits.U.of_int 1 
              1) && Bitops.eq arg651 (Bits.U.of_int 0 32) && Bitops.eq arg652 
            (Bits.U.of_int 0 3) && Bitops.eq arg653 (Bits.U.of_int 0 1) && 
          Bitops.eq arg648 (Bits.U.of_int 31 32) && Bitops.eq arg647 (Bits.U.of_int 
              31 32) && Bitops.eq arg646 (Bits.U.of_int 31 32) && Bitops.eq 
            arg645 (Bits.U.of_int 31 32) && Bitops.eq arg649 (Bits.U.of_int 0 3) && 
          Bitops.eq arg650 (Bits.U.of_int 1 1) && Bitops.eq arg642 
            (Bits.U.of_int 31 32) && Bitops.eq arg641 (Bits.U.of_int 31 32) && 
          Bitops.eq arg640 (Bits.U.of_int 31 32) && Bitops.eq arg639 (Bits.U.of_int 
              31 32) && Bitops.eq arg643 (Bits.U.of_int 0 3) && Bitops.eq arg644 
            (Bits.U.of_int 0 1) && Bitops.eq arg636 (Bits.U.of_int 31 32) && 
          Bitops.eq arg635 (Bits.U.of_int 31 32) && Bitops.eq arg637 (Bits.U.of_int 
              0 3) && Bitops.eq arg638 (Bits.U.of_int 0 1) && Bitops.eq arg633 
            (Bits.U.of_int 0 3) && Bitops.eq arg634 (Bits.U.of_int 0 1))) -> 
           Instruction.taddcctv (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg663)]), 
            32))] 
      when Base.to_bool (Bitops.fits_signed arg663 13) -> 
           Instruction.sub (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg663)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
          RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] -> 
           Instruction.sub (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Const (RP.Bits arg701)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Const (RP.Bits arg697)]); RP.Const (RP.Bits arg698)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg699), _)), 
              RP.Const (RP.Bits arg700), 1)); (RP.App (("ge", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Const (RP.Bits arg693)]); RP.Const (RP.Bits arg694)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg695), _)), 
              RP.Const (RP.Bits arg696), 1)); (RP.App (("eq", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Const (RP.Bits arg689)]); RP.Const (RP.Bits arg690)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg691), _)), 
              RP.Const (RP.Bits arg692), 1)); (RP.App (("ne", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Const (RP.Bits arg685)]); RP.Const (RP.Bits arg686)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg687), _)), 
              RP.Const (RP.Bits arg688), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg681); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg682)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg680); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg679); RP.App (("sub", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Const (RP.Bits arg678)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg683), _)), 
              RP.Const (RP.Bits arg684), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg674); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg675)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg673); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg672); 
                      RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Const (RP.Bits arg671)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg676), _)), RP.Const (RP.Bits arg677), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg667); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg668)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg669), _)), 
              RP.Const (RP.Bits arg670), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg664), _)), 
              RP.App (("borrow", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg666); RP.Const (RP.Bits arg665)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg701 13 && Bitops.eq arg698 (Bits.U.of_int 0 
              32) && Bitops.eq arg699 (Bits.U.of_int 0 3) && Bitops.eq arg700 
            (Bits.U.of_int 1 1) && Bitops.eq arg694 (Bits.U.of_int 0 32) && 
          Bitops.eq arg695 (Bits.U.of_int 0 3) && Bitops.eq arg696 (Bits.U.of_int 0 
              1) && Bitops.eq arg690 (Bits.U.of_int 0 32) && Bitops.eq arg691 
            (Bits.U.of_int 0 3) && Bitops.eq arg692 (Bits.U.of_int 1 1) && 
          Bitops.eq arg686 (Bits.U.of_int 0 32) && Bitops.eq arg687 (Bits.U.of_int 
              0 3) && Bitops.eq arg688 (Bits.U.of_int 0 1) && Bitops.eq arg681 
            (Bits.U.of_int 31 32) && Bitops.eq arg680 (Bits.U.of_int 31 32) && 
          Bitops.eq arg679 (Bits.U.of_int 31 32) && Bitops.eq arg683 (Bits.U.of_int 
              0 3) && Bitops.eq arg684 (Bits.U.of_int 1 1) && Bitops.eq arg674 
            (Bits.U.of_int 31 32) && Bitops.eq arg673 (Bits.U.of_int 31 32) && 
          Bitops.eq arg672 (Bits.U.of_int 31 32) && Bitops.eq arg676 (Bits.U.of_int 
              0 3) && Bitops.eq arg677 (Bits.U.of_int 0 1) && Bitops.eq arg667 
            (Bits.U.of_int 31 32) && Bitops.eq arg669 (Bits.U.of_int 0 3) && 
          Bitops.eq arg670 (Bits.U.of_int 0 1) && Bitops.eq arg664 
            (Bits.U.of_int 0 3) && Bitops.eq arg665 (Bits.U.of_int 0 1))) -> 
           Instruction.subcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg701)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''''), _), 32)]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''''), _), 32)]); 
              RP.Const (RP.Bits arg729)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg730), _)), 
              RP.Const (RP.Bits arg731), 1)); (RP.App (("ge", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''), _), 32)]); 
              RP.Const (RP.Bits arg726)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg727), _)), 
              RP.Const (RP.Bits arg728), 1)); (RP.App (("eq", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''), _), 32)]); 
              RP.Const (RP.Bits arg723)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg724), _)), 
              RP.Const (RP.Bits arg725), 1)); (RP.App (("ne", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''), _), 32)]); 
              RP.Const (RP.Bits arg720)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg721), _)), 
              RP.Const (RP.Bits arg722), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg717); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg716); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg715); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg714); RP.App (("sub", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg718), _)), 
              RP.Const (RP.Bits arg719), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg711); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg710); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg709); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg708); 
                      RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg712), _)), RP.Const (RP.Bits arg713), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg705); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg704); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg706), _)), RP.Const (RP.Bits arg707), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg702), _)), 
              RP.App (("borrow", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Const (RP.Bits arg703)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg729 (Bits.U.of_int 0 32) && Bitops.eq arg730 (Bits.U.of_int 0 3) && 
          Bitops.eq arg731 (Bits.U.of_int 1 1) && Bitops.eq arg726 (Bits.U.of_int 0 
              32) && Bitops.eq arg727 (Bits.U.of_int 0 3) && Bitops.eq arg728 
            (Bits.U.of_int 0 1) && Bitops.eq arg723 (Bits.U.of_int 0 32) && 
          Bitops.eq arg724 (Bits.U.of_int 0 3) && Bitops.eq arg725 (Bits.U.of_int 1 
              1) && Bitops.eq arg720 (Bits.U.of_int 0 32) && Bitops.eq arg721 
            (Bits.U.of_int 0 3) && Bitops.eq arg722 (Bits.U.of_int 0 1) && 
          Bitops.eq arg717 (Bits.U.of_int 31 32) && Bitops.eq arg716 (Bits.U.of_int 
              31 32) && Bitops.eq arg715 (Bits.U.of_int 31 32) && Bitops.eq 
            arg714 (Bits.U.of_int 31 32) && Bitops.eq arg718 (Bits.U.of_int 0 3) && 
          Bitops.eq arg719 (Bits.U.of_int 1 1) && Bitops.eq arg711 
            (Bits.U.of_int 31 32) && Bitops.eq arg710 (Bits.U.of_int 31 32) && 
          Bitops.eq arg709 (Bits.U.of_int 31 32) && Bitops.eq arg708 (Bits.U.of_int 
              31 32) && Bitops.eq arg712 (Bits.U.of_int 0 3) && Bitops.eq arg713 
            (Bits.U.of_int 0 1) && Bitops.eq arg705 (Bits.U.of_int 31 32) && 
          Bitops.eq arg704 (Bits.U.of_int 31 32) && Bitops.eq arg706 (Bits.U.of_int 
              0 3) && Bitops.eq arg707 (Bits.U.of_int 0 1) && Bitops.eq arg702 
            (Bits.U.of_int 0 3) && Bitops.eq arg703 (Bits.U.of_int 0 1))) -> 
           Instruction.subcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg733)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg732), _)), 1)])]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg733 13 && Bitops.eq arg732 
            (Bits.U.of_int 0 3)) -> 
           Instruction.subx (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg733)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]); 
                RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg734), _)), 1)])]), 32))] 
      when Base.to_bool (Bitops.eq arg734 (Bits.U.of_int 0 3)) -> 
           Instruction.subx (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                  RP.Const (RP.Bits arg779)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg778), _)), 1)])]), 32)); 
          (RP.App (("lt", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                  RP.Const (RP.Bits arg774)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg773), _)), 1)])]); 
              RP.Const (RP.Bits arg775)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg776), _)), 
              RP.Const (RP.Bits arg777), 1)); 
          (RP.App (("ge", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''), _), 32); 
                  RP.Const (RP.Bits arg769)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg768), _)), 1)])]); 
              RP.Const (RP.Bits arg770)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg771), _)), 
              RP.Const (RP.Bits arg772), 1)); 
          (RP.App (("eq", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''), _), 32); 
                  RP.Const (RP.Bits arg764)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg763), _)), 1)])]); 
              RP.Const (RP.Bits arg765)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg766), _)), 
              RP.Const (RP.Bits arg767), 1)); 
          (RP.App (("ne", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''), _), 32); 
                  RP.Const (RP.Bits arg759)]); RP.App (("zx", [1; 32]), 
                  [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg758), _)), 1)])]); 
              RP.Const (RP.Bits arg760)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg761), _)), 
              RP.Const (RP.Bits arg762), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg754); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg755)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg753); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg752); RP.App (("sub", [32]), 
                        [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 
                            32); RP.Const (RP.Bits arg751)]); RP.App (("zx", 
                            [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                                Rtl.Identity, 32, RP.Const (RP.Bits arg750), 
                                _)), 1)])])])])]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg756), _)), 
              RP.Const (RP.Bits arg757), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg746); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg747)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg745); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg744); RP.App (("sub", [32]), 
                        [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 
                            32); RP.Const (RP.Bits arg743)]); 
                          RP.App (("zx", [1; 32]), [RP.Fetch (RP.Slice (1, 20, 
                              RP.Cell ('i', Rtl.Identity, 32, 
                                RP.Const (RP.Bits arg742), _)), 1)])])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg748), _)), RP.Const (RP.Bits arg749), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg738); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg739)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg740), _)), 
              RP.Const (RP.Bits arg741), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg735), _)), RP.App (("borrow", [32]), 
                [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg737); 
                  RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg736), _)), 1)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg779 13 && Bitops.eq arg778 (Bits.U.of_int 0 
              3) && Bitops.eq arg773 (Bits.U.of_int 0 3) && Bitops.eq arg775 
            (Bits.U.of_int 0 32) && Bitops.eq arg776 (Bits.U.of_int 0 3) && 
          Bitops.eq arg777 (Bits.U.of_int 1 1) && Bitops.eq arg768 (Bits.U.of_int 0 
              3) && Bitops.eq arg770 (Bits.U.of_int 0 32) && Bitops.eq arg771 
            (Bits.U.of_int 0 3) && Bitops.eq arg772 (Bits.U.of_int 0 1) && 
          Bitops.eq arg763 (Bits.U.of_int 0 3) && Bitops.eq arg765 (Bits.U.of_int 0 
              32) && Bitops.eq arg766 (Bits.U.of_int 0 3) && Bitops.eq arg767 
            (Bits.U.of_int 1 1) && Bitops.eq arg758 (Bits.U.of_int 0 3) && 
          Bitops.eq arg760 (Bits.U.of_int 0 32) && Bitops.eq arg761 (Bits.U.of_int 
              0 3) && Bitops.eq arg762 (Bits.U.of_int 0 1) && Bitops.eq arg754 
            (Bits.U.of_int 31 32) && Bitops.eq arg753 (Bits.U.of_int 31 32) && 
          Bitops.eq arg752 (Bits.U.of_int 31 32) && Bitops.eq arg750 (Bits.U.of_int 
              0 3) && Bitops.eq arg756 (Bits.U.of_int 0 3) && Bitops.eq arg757 
            (Bits.U.of_int 1 1) && Bitops.eq arg746 (Bits.U.of_int 31 32) && 
          Bitops.eq arg745 (Bits.U.of_int 31 32) && Bitops.eq arg744 (Bits.U.of_int 
              31 32) && Bitops.eq arg742 (Bits.U.of_int 0 3) && Bitops.eq arg748 
            (Bits.U.of_int 0 3) && Bitops.eq arg749 (Bits.U.of_int 0 1) && 
          Bitops.eq arg738 (Bits.U.of_int 31 32) && Bitops.eq arg740 (Bits.U.of_int 
              0 3) && Bitops.eq arg741 (Bits.U.of_int 0 1) && Bitops.eq arg735 
            (Bits.U.of_int 0 3) && Bitops.eq arg736 (Bits.U.of_int 0 3))) -> 
           Instruction.subxcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg779)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2''''''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg816), _)), 1)])]), 
            32)); (RP.App (("lt", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2'''''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg812), _)), 1)])]); 
              RP.Const (RP.Bits arg813)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg814), _)), 
              RP.Const (RP.Bits arg815), 1)); 
          (RP.App (("ge", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2''''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg808), _)), 1)])]); 
              RP.Const (RP.Bits arg809)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg810), _)), 
              RP.Const (RP.Bits arg811), 1)); 
          (RP.App (("eq", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2'''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg804), _)), 1)])]); 
              RP.Const (RP.Bits arg805)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg806), _)), 
              RP.Const (RP.Bits arg807), 1)); 
          (RP.App (("ne", [32]), [RP.App (("sub", [32]), 
              [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1''''''''), _), 32); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2''''''), _), 32)]); RP.App (("zx", 
                  [1; 32]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg800), _)), 1)])]); 
              RP.Const (RP.Bits arg801)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg802), _)), 
              RP.Const (RP.Bits arg803), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg797); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg796); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg795); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg794); RP.App (("sub", [32]), 
                        [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 
                            32); RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                              RP.Const (RP.Bits rs2''''), _), 32)]); 
                          RP.App (("zx", [1; 32]), [RP.Fetch (RP.Slice (1, 20, 
                              RP.Cell ('i', Rtl.Identity, 32, 
                                RP.Const (RP.Bits arg793), _)), 1)])])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg798), _)), 
              RP.Const (RP.Bits arg799), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg790); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg789); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg788); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg787); RP.App (("sub", [32]), 
                        [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                            Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 
                            32); RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                              RP.Const (RP.Bits rs2''), _), 32)]); 
                          RP.App (("zx", [1; 32]), [RP.Fetch (RP.Slice (1, 20, 
                              RP.Cell ('i', Rtl.Identity, 32, 
                                RP.Const (RP.Bits arg786), _)), 1)])])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg791), _)), RP.Const (RP.Bits arg792), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg783); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg782); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg784), _)), RP.Const (RP.Bits arg785), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg780), _)), 
              RP.App (("borrow", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg781), _)), 1)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg816 (Bits.U.of_int 0 3) && Bitops.eq arg812 (Bits.U.of_int 0 3) && 
          Bitops.eq arg813 (Bits.U.of_int 0 32) && Bitops.eq arg814 (Bits.U.of_int 
              0 3) && Bitops.eq arg815 (Bits.U.of_int 1 1) && Bitops.eq arg808 
            (Bits.U.of_int 0 3) && Bitops.eq arg809 (Bits.U.of_int 0 32) && 
          Bitops.eq arg810 (Bits.U.of_int 0 3) && Bitops.eq arg811 (Bits.U.of_int 0 
              1) && Bitops.eq arg804 (Bits.U.of_int 0 3) && Bitops.eq arg805 
            (Bits.U.of_int 0 32) && Bitops.eq arg806 (Bits.U.of_int 0 3) && 
          Bitops.eq arg807 (Bits.U.of_int 1 1) && Bitops.eq arg800 (Bits.U.of_int 0 
              3) && Bitops.eq arg801 (Bits.U.of_int 0 32) && Bitops.eq arg802 
            (Bits.U.of_int 0 3) && Bitops.eq arg803 (Bits.U.of_int 0 1) && 
          Bitops.eq arg797 (Bits.U.of_int 31 32) && Bitops.eq arg796 (Bits.U.of_int 
              31 32) && Bitops.eq arg795 (Bits.U.of_int 31 32) && Bitops.eq 
            arg794 (Bits.U.of_int 31 32) && Bitops.eq arg793 (Bits.U.of_int 0 3) && 
          Bitops.eq arg798 (Bits.U.of_int 0 3) && Bitops.eq arg799 (Bits.U.of_int 1 
              1) && Bitops.eq arg790 (Bits.U.of_int 31 32) && Bitops.eq arg789 
            (Bits.U.of_int 31 32) && Bitops.eq arg788 (Bits.U.of_int 31 32) && 
          Bitops.eq arg787 (Bits.U.of_int 31 32) && Bitops.eq arg786 (Bits.U.of_int 
              0 3) && Bitops.eq arg791 (Bits.U.of_int 0 3) && Bitops.eq arg792 
            (Bits.U.of_int 0 1) && Bitops.eq arg783 (Bits.U.of_int 31 32) && 
          Bitops.eq arg782 (Bits.U.of_int 31 32) && Bitops.eq arg784 (Bits.U.of_int 
              0 3) && Bitops.eq arg785 (Bits.U.of_int 0 1) && Bitops.eq arg780 
            (Bits.U.of_int 0 3) && Bitops.eq arg781 (Bits.U.of_int 0 3))) -> 
           Instruction.subxcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Const (RP.Bits arg854)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Const (RP.Bits arg850)]); RP.Const (RP.Bits arg851)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg852), _)), 
              RP.Const (RP.Bits arg853), 1)); (RP.App (("ge", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Const (RP.Bits arg846)]); RP.Const (RP.Bits arg847)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg848), _)), 
              RP.Const (RP.Bits arg849), 1)); (RP.App (("eq", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Const (RP.Bits arg842)]); RP.Const (RP.Bits arg843)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg844), _)), 
              RP.Const (RP.Bits arg845), 1)); (RP.App (("ne", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Const (RP.Bits arg838)]); RP.Const (RP.Bits arg839)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg840), _)), 
              RP.Const (RP.Bits arg841), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg834); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg835)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg833); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg832); RP.App (("sub", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Const (RP.Bits arg831)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg836), _)), 
              RP.Const (RP.Bits arg837), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg827); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg828)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg826); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg825); 
                      RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Const (RP.Bits arg824)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg829), _)), RP.Const (RP.Bits arg830), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg820); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg821)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg822), _)), 
              RP.Const (RP.Bits arg823), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg817), _)), 
              RP.App (("borrow", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg819); RP.Const (RP.Bits arg818)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg854 13 && Bitops.eq arg851 (Bits.U.of_int 0 
              32) && Bitops.eq arg852 (Bits.U.of_int 0 3) && Bitops.eq arg853 
            (Bits.U.of_int 1 1) && Bitops.eq arg847 (Bits.U.of_int 0 32) && 
          Bitops.eq arg848 (Bits.U.of_int 0 3) && Bitops.eq arg849 (Bits.U.of_int 0 
              1) && Bitops.eq arg843 (Bits.U.of_int 0 32) && Bitops.eq arg844 
            (Bits.U.of_int 0 3) && Bitops.eq arg845 (Bits.U.of_int 1 1) && 
          Bitops.eq arg839 (Bits.U.of_int 0 32) && Bitops.eq arg840 (Bits.U.of_int 
              0 3) && Bitops.eq arg841 (Bits.U.of_int 0 1) && Bitops.eq arg834 
            (Bits.U.of_int 31 32) && Bitops.eq arg833 (Bits.U.of_int 31 32) && 
          Bitops.eq arg832 (Bits.U.of_int 31 32) && Bitops.eq arg836 (Bits.U.of_int 
              0 3) && Bitops.eq arg837 (Bits.U.of_int 1 1) && Bitops.eq arg827 
            (Bits.U.of_int 31 32) && Bitops.eq arg826 (Bits.U.of_int 31 32) && 
          Bitops.eq arg825 (Bits.U.of_int 31 32) && Bitops.eq arg829 (Bits.U.of_int 
              0 3) && Bitops.eq arg830 (Bits.U.of_int 0 1) && Bitops.eq arg820 
            (Bits.U.of_int 31 32) && Bitops.eq arg822 (Bits.U.of_int 0 3) && 
          Bitops.eq arg823 (Bits.U.of_int 0 1) && Bitops.eq arg817 
            (Bits.U.of_int 0 3) && Bitops.eq arg818 (Bits.U.of_int 0 1))) -> 
           Instruction.tsubcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg854)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''''), _), 32)]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''''), _), 32)]); 
              RP.Const (RP.Bits arg882)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg883), _)), 
              RP.Const (RP.Bits arg884), 1)); (RP.App (("ge", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''), _), 32)]); 
              RP.Const (RP.Bits arg879)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg880), _)), 
              RP.Const (RP.Bits arg881), 1)); (RP.App (("eq", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''), _), 32)]); 
              RP.Const (RP.Bits arg876)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg877), _)), 
              RP.Const (RP.Bits arg878), 1)); (RP.App (("ne", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''), _), 32)]); 
              RP.Const (RP.Bits arg873)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg874), _)), 
              RP.Const (RP.Bits arg875), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg870); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg869); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg868); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg867); RP.App (("sub", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg871), _)), 
              RP.Const (RP.Bits arg872), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg864); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg863); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg862); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg861); 
                      RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg865), _)), RP.Const (RP.Bits arg866), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg858); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg857); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg859), _)), RP.Const (RP.Bits arg860), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg855), _)), 
              RP.App (("borrow", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Const (RP.Bits arg856)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg882 (Bits.U.of_int 0 32) && Bitops.eq arg883 (Bits.U.of_int 0 3) && 
          Bitops.eq arg884 (Bits.U.of_int 1 1) && Bitops.eq arg879 (Bits.U.of_int 0 
              32) && Bitops.eq arg880 (Bits.U.of_int 0 3) && Bitops.eq arg881 
            (Bits.U.of_int 0 1) && Bitops.eq arg876 (Bits.U.of_int 0 32) && 
          Bitops.eq arg877 (Bits.U.of_int 0 3) && Bitops.eq arg878 (Bits.U.of_int 1 
              1) && Bitops.eq arg873 (Bits.U.of_int 0 32) && Bitops.eq arg874 
            (Bits.U.of_int 0 3) && Bitops.eq arg875 (Bits.U.of_int 0 1) && 
          Bitops.eq arg870 (Bits.U.of_int 31 32) && Bitops.eq arg869 (Bits.U.of_int 
              31 32) && Bitops.eq arg868 (Bits.U.of_int 31 32) && Bitops.eq 
            arg867 (Bits.U.of_int 31 32) && Bitops.eq arg871 (Bits.U.of_int 0 3) && 
          Bitops.eq arg872 (Bits.U.of_int 1 1) && Bitops.eq arg864 
            (Bits.U.of_int 31 32) && Bitops.eq arg863 (Bits.U.of_int 31 32) && 
          Bitops.eq arg862 (Bits.U.of_int 31 32) && Bitops.eq arg861 (Bits.U.of_int 
              31 32) && Bitops.eq arg865 (Bits.U.of_int 0 3) && Bitops.eq arg866 
            (Bits.U.of_int 0 1) && Bitops.eq arg858 (Bits.U.of_int 31 32) && 
          Bitops.eq arg857 (Bits.U.of_int 31 32) && Bitops.eq arg859 (Bits.U.of_int 
              0 3) && Bitops.eq arg860 (Bits.U.of_int 0 1) && Bitops.eq arg855 
            (Bits.U.of_int 0 3) && Bitops.eq arg856 (Bits.U.of_int 0 1))) -> 
           Instruction.tsubcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Const (RP.Bits arg922)]), 32)); (RP.App (("lt", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Const (RP.Bits arg918)]); RP.Const (RP.Bits arg919)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg920), _)), 
              RP.Const (RP.Bits arg921), 1)); (RP.App (("ge", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Const (RP.Bits arg914)]); RP.Const (RP.Bits arg915)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg916), _)), 
              RP.Const (RP.Bits arg917), 1)); (RP.App (("eq", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Const (RP.Bits arg910)]); RP.Const (RP.Bits arg911)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg912), _)), 
              RP.Const (RP.Bits arg913), 1)); (RP.App (("ne", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Const (RP.Bits arg906)]); RP.Const (RP.Bits arg907)]), 
            RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg908), _)), 
              RP.Const (RP.Bits arg909), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg902); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.Const (RP.Bits arg903)]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg901); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg900); RP.App (("sub", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Const (RP.Bits arg899)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg904), _)), 
              RP.Const (RP.Bits arg905), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg895); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.Const (RP.Bits arg896)]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg894); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg893); 
                      RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Const (RP.Bits arg892)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg897), _)), RP.Const (RP.Bits arg898), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg888); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.Const (RP.Bits arg889)]), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg890), _)), 
              RP.Const (RP.Bits arg891), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg885), _)), 
              RP.App (("borrow", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg887); RP.Const (RP.Bits arg886)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs1 = rs1''''''''''' && 
          rs1 = rs1'''''''''' && rs1 = rs1''''''''' && rs1 = rs1'''''''' && 
          rs1 = rs1''''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && rs1 = rs1' && 
          (Bitops.fits_signed arg922 13 && Bitops.eq arg919 (Bits.U.of_int 0 
              32) && Bitops.eq arg920 (Bits.U.of_int 0 3) && Bitops.eq arg921 
            (Bits.U.of_int 1 1) && Bitops.eq arg915 (Bits.U.of_int 0 32) && 
          Bitops.eq arg916 (Bits.U.of_int 0 3) && Bitops.eq arg917 (Bits.U.of_int 0 
              1) && Bitops.eq arg911 (Bits.U.of_int 0 32) && Bitops.eq arg912 
            (Bits.U.of_int 0 3) && Bitops.eq arg913 (Bits.U.of_int 1 1) && 
          Bitops.eq arg907 (Bits.U.of_int 0 32) && Bitops.eq arg908 (Bits.U.of_int 
              0 3) && Bitops.eq arg909 (Bits.U.of_int 0 1) && Bitops.eq arg902 
            (Bits.U.of_int 31 32) && Bitops.eq arg901 (Bits.U.of_int 31 32) && 
          Bitops.eq arg900 (Bits.U.of_int 31 32) && Bitops.eq arg904 (Bits.U.of_int 
              0 3) && Bitops.eq arg905 (Bits.U.of_int 1 1) && Bitops.eq arg895 
            (Bits.U.of_int 31 32) && Bitops.eq arg894 (Bits.U.of_int 31 32) && 
          Bitops.eq arg893 (Bits.U.of_int 31 32) && Bitops.eq arg897 (Bits.U.of_int 
              0 3) && Bitops.eq arg898 (Bits.U.of_int 0 1) && Bitops.eq arg888 
            (Bits.U.of_int 31 32) && Bitops.eq arg890 (Bits.U.of_int 0 3) && 
          Bitops.eq arg891 (Bits.U.of_int 0 1) && Bitops.eq arg885 
            (Bits.U.of_int 0 3) && Bitops.eq arg886 (Bits.U.of_int 0 1))) -> 
           Instruction.tsubcctv (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg922)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
            RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''''), _), 32)]), 32)); 
          (RP.App (("lt", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''''), _), 32)]); 
              RP.Const (RP.Bits arg950)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg951), _)), 
              RP.Const (RP.Bits arg952), 1)); (RP.App (("ge", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''''), _), 32)]); 
              RP.Const (RP.Bits arg947)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg948), _)), 
              RP.Const (RP.Bits arg949), 1)); (RP.App (("eq", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2'''''''), _), 32)]); 
              RP.Const (RP.Bits arg944)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg945), _)), 
              RP.Const (RP.Bits arg946), 1)); (RP.App (("ne", [32]), 
            [RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''''''''), _), 32); 
                RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs2''''''), _), 32)]); 
              RP.Const (RP.Bits arg941)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg942), _)), 
              RP.Const (RP.Bits arg943), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg938); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg937); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]); 
              RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg936); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''''), _), 
                      32)]); RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg935); RP.App (("sub", [32]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg939), _)), 
              RP.Const (RP.Bits arg940), 1)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg932); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32)]); 
                RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg931); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)])]); 
              RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
                  [RP.Const (RP.Bits arg930); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32)]); 
                  RP.App (("bitExtract", [32; 32; 1]), 
                    [RP.Const (RP.Bits arg929); 
                      RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1''), _), 32); 
                          RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                            RP.Const (RP.Bits rs2''), _), 32)])])])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg933), _)), RP.Const (RP.Bits arg934), 1)); 
          (RP.App (("eq", [1]), [RP.App (("bitExtract", [32; 32; 1]), 
              [RP.Const (RP.Bits arg926); RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1'), _), 32)]); 
              RP.App (("bitExtract", [32; 32; 1]), 
                [RP.Const (RP.Bits arg925); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg927), _)), RP.Const (RP.Bits arg928), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg923), _)), 
              RP.App (("borrow", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32); 
                  RP.Const (RP.Bits arg924)]), 1))] 
      when Base.to_bool (rs1 = rs1'''''''''''' && rs2 = rs2'''''''''' && 
          rs1 = rs1''''''''''' && rs2 = rs2''''''''' && rs1 = rs1'''''''''' && 
          rs2 = rs2'''''''' && rs1 = rs1''''''''' && rs2 = rs2''''''' && 
          rs1 = rs1'''''''' && rs2 = rs2'''''' && rs1 = rs1''''''' && 
          rs2 = rs2''''' && rs1 = rs1'''''' && rs1 = rs1''''' && 
          rs2 = rs2'''' && rs1 = rs1'''' && rs2 = rs2''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq 
            arg950 (Bits.U.of_int 0 32) && Bitops.eq arg951 (Bits.U.of_int 0 3) && 
          Bitops.eq arg952 (Bits.U.of_int 1 1) && Bitops.eq arg947 (Bits.U.of_int 0 
              32) && Bitops.eq arg948 (Bits.U.of_int 0 3) && Bitops.eq arg949 
            (Bits.U.of_int 0 1) && Bitops.eq arg944 (Bits.U.of_int 0 32) && 
          Bitops.eq arg945 (Bits.U.of_int 0 3) && Bitops.eq arg946 (Bits.U.of_int 1 
              1) && Bitops.eq arg941 (Bits.U.of_int 0 32) && Bitops.eq arg942 
            (Bits.U.of_int 0 3) && Bitops.eq arg943 (Bits.U.of_int 0 1) && 
          Bitops.eq arg938 (Bits.U.of_int 31 32) && Bitops.eq arg937 (Bits.U.of_int 
              31 32) && Bitops.eq arg936 (Bits.U.of_int 31 32) && Bitops.eq 
            arg935 (Bits.U.of_int 31 32) && Bitops.eq arg939 (Bits.U.of_int 0 3) && 
          Bitops.eq arg940 (Bits.U.of_int 1 1) && Bitops.eq arg932 
            (Bits.U.of_int 31 32) && Bitops.eq arg931 (Bits.U.of_int 31 32) && 
          Bitops.eq arg930 (Bits.U.of_int 31 32) && Bitops.eq arg929 (Bits.U.of_int 
              31 32) && Bitops.eq arg933 (Bits.U.of_int 0 3) && Bitops.eq arg934 
            (Bits.U.of_int 0 1) && Bitops.eq arg926 (Bits.U.of_int 31 32) && 
          Bitops.eq arg925 (Bits.U.of_int 31 32) && Bitops.eq arg927 (Bits.U.of_int 
              0 3) && Bitops.eq arg928 (Bits.U.of_int 0 1) && Bitops.eq arg923 
            (Bits.U.of_int 0 3) && Bitops.eq arg924 (Bits.U.of_int 0 1))) -> 
           Instruction.tsubcctv (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg955), _), 
            RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg957); 
                RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32); 
                    RP.Const (RP.Bits arg956)])]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), 
              RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg954); 
                  RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                      RP.Const (RP.Bits arg953)])]), 32))] 
      when Base.to_bool (rs1 = rs1' && (Bitops.fits_signed arg956 13 && 
          Bitops.eq arg955 (Bits.U.of_int 3 3) && Bitops.eq arg957 
            (Bits.U.of_int 32 32) && Bitops.eq arg954 (Bits.U.of_int 0 32))) -> 
           Instruction.umul (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg956)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg959), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg960); RP.App (("mulu", [32; 64]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            32)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
              RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg958); 
                  RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                      RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2), _), 32)])]), 32))] 
      when Base.to_bool (rs1 = rs1' && rs2 = rs2' && 
          (Bitops.eq arg959 (Bits.U.of_int 3 3) && Bitops.eq arg960 
            (Bits.U.of_int 32 32) && Bitops.eq arg958 (Bits.U.of_int 0 32))) -> 
           Instruction.umul (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg963), _), 
            RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg965); 
                RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32); 
                    RP.Const (RP.Bits arg964)])]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), 
              RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg962); 
                  RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                      RP.Const (RP.Bits arg961)])]), 32))] 
      when Base.to_bool (rs1 = rs1' && (Bitops.fits_signed arg964 13 && 
          Bitops.eq arg963 (Bits.U.of_int 3 3) && Bitops.eq arg965 
            (Bits.U.of_int 32 32) && Bitops.eq arg962 (Bits.U.of_int 0 32))) -> 
           Instruction.smul (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg964)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg967), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg968); RP.App (("mul", [32; 64]), 
                  [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)])]), 
            32)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits rd), _), 
              RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg966); 
                  RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32); 
                      RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2), _), 32)])]), 32))] 
      when Base.to_bool (rs1 = rs1' && rs2 = rs2' && 
          (Bitops.eq arg967 (Bits.U.of_int 3 3) && Bitops.eq arg968 
            (Bits.U.of_int 32 32) && Bitops.eq arg966 (Bits.U.of_int 0 32))) -> 
           Instruction.smul (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg991), _), 
            RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg993); 
                RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 32); 
                    RP.Const (RP.Bits arg992)])]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), 
              RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg990); 
                  RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32); 
                      RP.Const (RP.Bits arg989)])]), 32)); (RP.App (("lt", 
            [64]), [RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg985)]); RP.Const (RP.Bits arg986)]), 
            RP.Store (RP.Slice (1, 23, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg987), _)), RP.Const (RP.Bits arg988), 1)); 
          (RP.App (("ge", [64]), [RP.App (("mulu", [32; 64]), 
              [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1''), _), 32); RP.Const (RP.Bits arg981)]); 
              RP.Const (RP.Bits arg982)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg983), _)), 
              RP.Const (RP.Bits arg984), 1)); (RP.App (("eq", [64]), 
            [RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1'), _), 32); 
                RP.Const (RP.Bits arg977)]); RP.Const (RP.Bits arg978)]), 
            RP.Store (RP.Slice (1, 22, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg979), _)), RP.Const (RP.Bits arg980), 1)); 
          (RP.App (("ne", [64]), [RP.App (("mulu", [32; 64]), 
              [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg973)]); 
              RP.Const (RP.Bits arg974)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg975), _)), 
              RP.Const (RP.Bits arg976), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg971), _)), RP.Const (RP.Bits arg972), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg969), _)), 
              RP.Const (RP.Bits arg970), 1))] 
      when Base.to_bool (rs1 = rs1''''' && rs1 = rs1'''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs1 = rs1' && (Bitops.fits_signed arg992 13 && 
          Bitops.eq arg991 (Bits.U.of_int 3 3) && Bitops.eq arg993 (Bits.U.of_int 
              32 32) && Bitops.eq arg990 (Bits.U.of_int 0 32) && Bitops.eq arg986 
            (Bits.U.of_int 0 64) && Bitops.eq arg987 (Bits.U.of_int 0 3) && 
          Bitops.eq arg988 (Bits.U.of_int 1 1) && Bitops.eq arg982 (Bits.U.of_int 0 
              64) && Bitops.eq arg983 (Bits.U.of_int 0 3) && Bitops.eq arg984 
            (Bits.U.of_int 0 1) && Bitops.eq arg978 (Bits.U.of_int 0 64) && 
          Bitops.eq arg979 (Bits.U.of_int 0 3) && Bitops.eq arg980 
            (Bits.U.of_int 1 1) && Bitops.eq arg974 (Bits.U.of_int 0 64) && 
          Bitops.eq arg975 (Bits.U.of_int 0 3) && Bitops.eq arg976 
            (Bits.U.of_int 0 1) && Bitops.eq arg971 (Bits.U.of_int 0 3) && 
          Bitops.eq arg972 (Bits.U.of_int 0 1) && Bitops.eq arg969 
            (Bits.U.of_int 0 3) && Bitops.eq arg970 (Bits.U.of_int 0 1))) -> 
           Instruction.umulcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg992)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1011), _), 
            RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1012); 
                RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 32); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 32]), 
                [RP.Const (RP.Bits arg1010); 
                  RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32); 
                      RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2''''), _), 32)])]), 32)); 
          (RP.App (("lt", [64]), 
            [RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1'''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)]); 
              RP.Const (RP.Bits arg1007)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1008), _)), 
              RP.Const (RP.Bits arg1009), 1)); (RP.App (("ge", [64]), 
            [RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''), _), 32)]); 
              RP.Const (RP.Bits arg1004)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1005), _)), 
              RP.Const (RP.Bits arg1006), 1)); (RP.App (("eq", [64]), 
            [RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)]); 
              RP.Const (RP.Bits arg1001)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1002), _)), 
              RP.Const (RP.Bits arg1003), 1)); (RP.App (("ne", [64]), 
            [RP.App (("mulu", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]); 
              RP.Const (RP.Bits arg998)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg999), _)), 
              RP.Const (RP.Bits arg1000), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg996), _)), RP.Const (RP.Bits arg997), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg994), _)), 
              RP.Const (RP.Bits arg995), 1))] 
      when Base.to_bool (rs1 = rs1''''' && rs2 = rs2''''' && rs1 = rs1'''' && 
          rs2 = rs2'''' && rs1 = rs1''' && rs2 = rs2''' && rs1 = rs1'' && 
          rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq arg1011 
            (Bits.U.of_int 3 3) && Bitops.eq arg1012 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1010 (Bits.U.of_int 0 32) && Bitops.eq arg1007 
            (Bits.U.of_int 0 64) && Bitops.eq arg1008 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1009 (Bits.U.of_int 1 1) && Bitops.eq arg1004 (Bits.U.of_int 
              0 64) && Bitops.eq arg1005 (Bits.U.of_int 0 3) && Bitops.eq arg1006 
            (Bits.U.of_int 0 1) && Bitops.eq arg1001 (Bits.U.of_int 0 64) && 
          Bitops.eq arg1002 (Bits.U.of_int 0 3) && Bitops.eq arg1003 
            (Bits.U.of_int 1 1) && Bitops.eq arg998 (Bits.U.of_int 0 64) && 
          Bitops.eq arg999 (Bits.U.of_int 0 3) && Bitops.eq arg1000 
            (Bits.U.of_int 0 1) && Bitops.eq arg996 (Bits.U.of_int 0 3) && 
          Bitops.eq arg997 (Bits.U.of_int 0 1) && Bitops.eq arg994 
            (Bits.U.of_int 0 3) && Bitops.eq arg995 (Bits.U.of_int 0 1))) -> 
           Instruction.umulcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1035), _), 
            RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1037); 
                RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 32); 
                    RP.Const (RP.Bits arg1036)])]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 32]), 
                [RP.Const (RP.Bits arg1034); 
                  RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32); 
                      RP.Const (RP.Bits arg1033)])]), 32)); (RP.App (("lt", 
            [64]), [RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1'''), _), 32); 
                RP.Const (RP.Bits arg1029)]); RP.Const (RP.Bits arg1030)]), 
            RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1031), _)), 
              RP.Const (RP.Bits arg1032), 1)); (RP.App (("ge", [64]), 
            [RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1''), _), 32); 
                RP.Const (RP.Bits arg1025)]); RP.Const (RP.Bits arg1026)]), 
            RP.Store (RP.Slice (1, 23, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1027), _)), RP.Const (RP.Bits arg1028), 
              1)); (RP.App (("eq", [64]), [RP.App (("mul", [32; 64]), 
              [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1'), _), 32); RP.Const (RP.Bits arg1021)]); 
              RP.Const (RP.Bits arg1022)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1023), _)), 
              RP.Const (RP.Bits arg1024), 1)); 
          (RP.App (("ne", [64]), [RP.App (("mul", [32; 64]), 
              [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg1017)]); 
              RP.Const (RP.Bits arg1018)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1019), _)), 
              RP.Const (RP.Bits arg1020), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1015), _)), RP.Const (RP.Bits arg1016), 
              1)); (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1013), _)), 
              RP.Const (RP.Bits arg1014), 1))] 
      when Base.to_bool (rs1 = rs1''''' && rs1 = rs1'''' && rs1 = rs1''' && 
          rs1 = rs1'' && rs1 = rs1' && (Bitops.fits_signed arg1036 13 && 
          Bitops.eq arg1035 (Bits.U.of_int 3 3) && Bitops.eq arg1037 (Bits.U.of_int 
              32 32) && Bitops.eq arg1034 (Bits.U.of_int 0 32) && Bitops.eq 
            arg1030 (Bits.U.of_int 0 64) && Bitops.eq arg1031 (Bits.U.of_int 0 
              3) && Bitops.eq arg1032 (Bits.U.of_int 1 1) && Bitops.eq arg1026 
            (Bits.U.of_int 0 64) && Bitops.eq arg1027 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1028 (Bits.U.of_int 0 1) && Bitops.eq arg1022 (Bits.U.of_int 
              0 64) && Bitops.eq arg1023 (Bits.U.of_int 0 3) && Bitops.eq arg1024 
            (Bits.U.of_int 1 1) && Bitops.eq arg1018 (Bits.U.of_int 0 64) && 
          Bitops.eq arg1019 (Bits.U.of_int 0 3) && Bitops.eq arg1020 
            (Bits.U.of_int 0 1) && Bitops.eq arg1015 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1016 (Bits.U.of_int 0 1) && Bitops.eq arg1013 
            (Bits.U.of_int 0 3) && Bitops.eq arg1014 (Bits.U.of_int 0 1))) -> 
           Instruction.smulcc (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1036)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1055), _), 
            RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1056); 
                RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs1'''''), _), 32); 
                    RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                      RP.Const (RP.Bits rs2'''''), _), 32)])]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 32]), 
                [RP.Const (RP.Bits arg1054); 
                  RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                      Rtl.Identity, 32, RP.Const (RP.Bits rs1''''), _), 32); 
                      RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2''''), _), 32)])]), 32)); 
          (RP.App (("lt", [64]), 
            [RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1'''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'''), _), 32)]); 
              RP.Const (RP.Bits arg1051)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1052), _)), 
              RP.Const (RP.Bits arg1053), 1)); (RP.App (("ge", [64]), 
            [RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1''), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2''), _), 32)]); 
              RP.Const (RP.Bits arg1048)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1049), _)), 
              RP.Const (RP.Bits arg1050), 1)); (RP.App (("eq", [64]), 
            [RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1'), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2'), _), 32)]); 
              RP.Const (RP.Bits arg1045)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1046), _)), 
              RP.Const (RP.Bits arg1047), 1)); (RP.App (("ne", [64]), 
            [RP.App (("mul", [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]); 
              RP.Const (RP.Bits arg1042)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1043), _)), 
              RP.Const (RP.Bits arg1044), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1040), _)), RP.Const (RP.Bits arg1041), 
              1)); (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1038), _)), 
              RP.Const (RP.Bits arg1039), 1))] 
      when Base.to_bool (rs1 = rs1''''' && rs2 = rs2''''' && rs1 = rs1'''' && 
          rs2 = rs2'''' && rs1 = rs1''' && rs2 = rs2''' && rs1 = rs1'' && 
          rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq arg1055 
            (Bits.U.of_int 3 3) && Bitops.eq arg1056 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1054 (Bits.U.of_int 0 32) && Bitops.eq arg1051 
            (Bits.U.of_int 0 64) && Bitops.eq arg1052 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1053 (Bits.U.of_int 1 1) && Bitops.eq arg1048 (Bits.U.of_int 
              0 64) && Bitops.eq arg1049 (Bits.U.of_int 0 3) && Bitops.eq arg1050 
            (Bits.U.of_int 0 1) && Bitops.eq arg1045 (Bits.U.of_int 0 64) && 
          Bitops.eq arg1046 (Bits.U.of_int 0 3) && Bitops.eq arg1047 
            (Bits.U.of_int 1 1) && Bitops.eq arg1042 (Bits.U.of_int 0 64) && 
          Bitops.eq arg1043 (Bits.U.of_int 0 3) && Bitops.eq arg1044 
            (Bits.U.of_int 0 1) && Bitops.eq arg1040 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1041 (Bits.U.of_int 0 1) && Bitops.eq arg1038 
            (Bits.U.of_int 0 3) && Bitops.eq arg1039 (Bits.U.of_int 0 1))) -> 
           Instruction.smulcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1060); RP.App (("divu", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1058); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1057), _), 32)]); 
                    RP.Const (RP.Bits arg1059)])]), 32))] 
      when false -> 
           Sledlib.fail ["could not solve equations: arg1059 = zx(sx(simm13))"] 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1063); RP.App (("divu", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1062); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1061), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2), _), 32)])])]), 32))] 
      when Base.to_bool 
          (Bitops.eq arg1063 (Bits.U.of_int 0 32) && Bitops.eq arg1062 
            (Bits.U.of_int 32 32) && Bitops.eq arg1061 (Bits.U.of_int 3 3)) -> 
           Instruction.udiv (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1067); RP.App (("quot", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1065); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1064), _), 32)]); 
                    RP.Const (RP.Bits arg1066)])]), 32))] 
      when false -> 
           Sledlib.fail ["could not solve equations: arg1066 = zx(sx(simm13))"] 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1070); RP.App (("quot", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1069); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1068), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2), _), 32)])])]), 32))] 
      when Base.to_bool 
          (Bitops.eq arg1070 (Bits.U.of_int 0 32) && Bitops.eq arg1069 
            (Bits.U.of_int 32 32) && Bitops.eq arg1068 (Bits.U.of_int 3 3)) -> 
           Instruction.sdiv (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1108); RP.App (("divu", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1106); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1105), _), 32)]); 
                    RP.Const (RP.Bits arg1107)])]), 32)); 
          (RP.App (("lt", [32]), [RP.App (("bitExtract", [64; 32; 32]), 
              [RP.Const (RP.Bits arg1101); RP.App (("divu", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1099); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1098), _), 32)]); 
                    RP.Const (RP.Bits arg1100)])]); 
              RP.Const (RP.Bits arg1102)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1103), _)), 
              RP.Const (RP.Bits arg1104), 1)); (RP.App (("ge", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1094); 
                RP.App (("divu", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1092); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1091), _), 32)]); 
                    RP.Const (RP.Bits arg1093)])]); 
              RP.Const (RP.Bits arg1095)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1096), _)), 
              RP.Const (RP.Bits arg1097), 1)); (RP.App (("eq", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1087); 
                RP.App (("divu", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1085); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1084), _), 32)]); 
                    RP.Const (RP.Bits arg1086)])]); 
              RP.Const (RP.Bits arg1088)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1089), _)), 
              RP.Const (RP.Bits arg1090), 1)); (RP.App (("ne", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1080); 
                RP.App (("divu", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1078); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1077), _), 32)]); 
                    RP.Const (RP.Bits arg1079)])]); 
              RP.Const (RP.Bits arg1081)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1082), _)), 
              RP.Const (RP.Bits arg1083), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1073), _)), 
              RP.App (("sparc_udiv_overflow", []), [RP.App (("bitInsert", 
                  [32; 64; 32]), [RP.Const (RP.Bits arg1075); 
                    RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                        Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                    RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg1074), _), 32)]); 
                  RP.Const (RP.Bits arg1076)]), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1071), _)), 
              RP.Const (RP.Bits arg1072), 1))] 
      when rs1 = rs1''''' && rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && 
        rs1 = rs1' && false -> 
           Sledlib.fail 
             ["could not solve equations: arg1079 = zx(sx(arg1076)), arg1086 = zx(sx(arg1076)), arg1093 = zx(sx(arg1076)), arg1100 = zx(sx(arg1076)), arg1107 = zx(sx(arg1076))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1140); RP.App (("divu", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1139); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1138), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2'''''), _), 32)])])]), 32)); 
          (RP.App (("lt", [32]), [RP.App (("bitExtract", [64; 32; 32]), 
              [RP.Const (RP.Bits arg1134); RP.App (("divu", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1133); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1132), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2''''), _), 32)])])]); 
              RP.Const (RP.Bits arg1135)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1136), _)), 
              RP.Const (RP.Bits arg1137), 1)); (RP.App (("ge", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1128); 
                RP.App (("divu", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1127); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1126), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2'''), _), 32)])])]); 
              RP.Const (RP.Bits arg1129)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1130), _)), 
              RP.Const (RP.Bits arg1131), 1)); (RP.App (("eq", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1122); 
                RP.App (("divu", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1121); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1120), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2''), _), 32)])])]); 
              RP.Const (RP.Bits arg1123)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1124), _)), 
              RP.Const (RP.Bits arg1125), 1)); (RP.App (("ne", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1116); 
                RP.App (("divu", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1115); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1114), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2'), _), 32)])])]); 
              RP.Const (RP.Bits arg1117)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1118), _)), 
              RP.Const (RP.Bits arg1119), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1111), _)), 
              RP.App (("sparc_udiv_overflow", []), [RP.App (("bitInsert", 
                  [32; 64; 32]), [RP.Const (RP.Bits arg1113); 
                    RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                        Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                    RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg1112), _), 32)]); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2), _), 32)]), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1109), _)), 
              RP.Const (RP.Bits arg1110), 1))] 
      when Base.to_bool (rs1 = rs1''''' && rs2 = rs2''''' && rs1 = rs1'''' && 
          rs2 = rs2'''' && rs1 = rs1''' && rs2 = rs2''' && rs1 = rs1'' && 
          rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq arg1140 
            (Bits.U.of_int 0 32) && Bitops.eq arg1139 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1138 (Bits.U.of_int 3 3) && Bitops.eq arg1134 
            (Bits.U.of_int 0 32) && Bitops.eq arg1133 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1132 (Bits.U.of_int 3 3) && Bitops.eq arg1135 
            (Bits.U.of_int 0 32) && Bitops.eq arg1136 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1137 (Bits.U.of_int 1 1) && Bitops.eq arg1128 
            (Bits.U.of_int 0 32) && Bitops.eq arg1127 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1126 (Bits.U.of_int 3 3) && Bitops.eq arg1129 
            (Bits.U.of_int 0 32) && Bitops.eq arg1130 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1131 (Bits.U.of_int 0 1) && Bitops.eq arg1122 
            (Bits.U.of_int 0 32) && Bitops.eq arg1121 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1120 (Bits.U.of_int 3 3) && Bitops.eq arg1123 
            (Bits.U.of_int 0 32) && Bitops.eq arg1124 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1125 (Bits.U.of_int 1 1) && Bitops.eq arg1116 
            (Bits.U.of_int 0 32) && Bitops.eq arg1115 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1114 (Bits.U.of_int 3 3) && Bitops.eq arg1117 
            (Bits.U.of_int 0 32) && Bitops.eq arg1118 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1119 (Bits.U.of_int 0 1) && Bitops.eq arg1111 
            (Bits.U.of_int 0 3) && Bitops.eq arg1113 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1112 (Bits.U.of_int 3 3) && Bitops.eq arg1109 
            (Bits.U.of_int 0 3) && Bitops.eq arg1110 (Bits.U.of_int 0 1))) -> 
           Instruction.udivcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1178); RP.App (("quot", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1176); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1175), _), 32)]); 
                    RP.Const (RP.Bits arg1177)])]), 32)); 
          (RP.App (("lt", [32]), [RP.App (("bitExtract", [64; 32; 32]), 
              [RP.Const (RP.Bits arg1171); RP.App (("quot", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1169); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1168), _), 32)]); 
                    RP.Const (RP.Bits arg1170)])]); 
              RP.Const (RP.Bits arg1172)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1173), _)), 
              RP.Const (RP.Bits arg1174), 1)); (RP.App (("ge", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1164); 
                RP.App (("quot", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1162); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1161), _), 32)]); 
                    RP.Const (RP.Bits arg1163)])]); 
              RP.Const (RP.Bits arg1165)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1166), _)), 
              RP.Const (RP.Bits arg1167), 1)); (RP.App (("eq", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1157); 
                RP.App (("quot", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1155); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1154), _), 32)]); 
                    RP.Const (RP.Bits arg1156)])]); 
              RP.Const (RP.Bits arg1158)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1159), _)), 
              RP.Const (RP.Bits arg1160), 1)); (RP.App (("ne", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1150); 
                RP.App (("quot", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1148); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1147), _), 32)]); 
                    RP.Const (RP.Bits arg1149)])]); 
              RP.Const (RP.Bits arg1151)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1152), _)), 
              RP.Const (RP.Bits arg1153), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1143), _)), 
              RP.App (("sparc_sdiv_overflow", []), [RP.App (("bitInsert", 
                  [32; 64; 32]), [RP.Const (RP.Bits arg1145); 
                    RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                        Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                    RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg1144), _), 32)]); 
                  RP.Const (RP.Bits arg1146)]), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1141), _)), 
              RP.Const (RP.Bits arg1142), 1))] 
      when rs1 = rs1''''' && rs1 = rs1'''' && rs1 = rs1''' && rs1 = rs1'' && 
        rs1 = rs1' && false -> 
           Sledlib.fail 
             ["could not solve equations: arg1149 = zx(sx(arg1146)), arg1156 = zx(sx(arg1146)), arg1163 = zx(sx(arg1146)), arg1170 = zx(sx(arg1146)), arg1177 = zx(sx(arg1146))"] 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.App (("bitExtract", [64; 32; 
                32]), [RP.Const (RP.Bits arg1210); RP.App (("quot", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1209); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1208), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2'''''), _), 32)])])]), 32)); 
          (RP.App (("lt", [32]), [RP.App (("bitExtract", [64; 32; 32]), 
              [RP.Const (RP.Bits arg1204); RP.App (("quot", [64; 32]), 
                  [RP.App (("bitInsert", [32; 64; 32]), 
                    [RP.Const (RP.Bits arg1203); RP.App (("zx", [32; 64]), 
                        [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1202), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2''''), _), 32)])])]); 
              RP.Const (RP.Bits arg1205)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1206), _)), 
              RP.Const (RP.Bits arg1207), 1)); (RP.App (("ge", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1198); 
                RP.App (("quot", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1197); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1'''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1196), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2'''), _), 32)])])]); 
              RP.Const (RP.Bits arg1199)]), RP.Store (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1200), _)), 
              RP.Const (RP.Bits arg1201), 1)); (RP.App (("eq", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1192); 
                RP.App (("quot", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1191); RP.App (("zx", 
                        [32; 64]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                          RP.Const (RP.Bits rs1''), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1190), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2''), _), 32)])])]); 
              RP.Const (RP.Bits arg1193)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1194), _)), 
              RP.Const (RP.Bits arg1195), 1)); (RP.App (("ne", [32]), 
            [RP.App (("bitExtract", [64; 32; 32]), [RP.Const (RP.Bits arg1186); 
                RP.App (("quot", [64; 32]), [RP.App (("bitInsert", 
                    [32; 64; 32]), [RP.Const (RP.Bits arg1185); 
                      RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                          Rtl.Identity, 32, RP.Const (RP.Bits rs1'), _), 32)]); 
                      RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg1184), _), 32)]); RP.App (("zx", 
                      [32; 32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                        RP.Const (RP.Bits rs2'), _), 32)])])]); 
              RP.Const (RP.Bits arg1187)]), RP.Store (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1188), _)), 
              RP.Const (RP.Bits arg1189), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1181), _)), 
              RP.App (("sparc_sdiv_overflow", []), [RP.App (("bitInsert", 
                  [32; 64; 32]), [RP.Const (RP.Bits arg1183); 
                    RP.App (("zx", [32; 64]), [RP.Fetch (RP.Cell ('r', 
                        Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32)]); 
                    RP.Fetch (RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg1182), _), 32)]); 
                  RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                    RP.Const (RP.Bits rs2), _), 32)]), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg1179), _)), 
              RP.Const (RP.Bits arg1180), 1))] 
      when Base.to_bool (rs1 = rs1''''' && rs2 = rs2''''' && rs1 = rs1'''' && 
          rs2 = rs2'''' && rs1 = rs1''' && rs2 = rs2''' && rs1 = rs1'' && 
          rs2 = rs2'' && rs1 = rs1' && rs2 = rs2' && (Bitops.eq arg1210 
            (Bits.U.of_int 0 32) && Bitops.eq arg1209 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1208 (Bits.U.of_int 3 3) && Bitops.eq arg1204 
            (Bits.U.of_int 0 32) && Bitops.eq arg1203 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1202 (Bits.U.of_int 3 3) && Bitops.eq arg1205 
            (Bits.U.of_int 0 32) && Bitops.eq arg1206 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1207 (Bits.U.of_int 1 1) && Bitops.eq arg1198 
            (Bits.U.of_int 0 32) && Bitops.eq arg1197 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1196 (Bits.U.of_int 3 3) && Bitops.eq arg1199 
            (Bits.U.of_int 0 32) && Bitops.eq arg1200 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1201 (Bits.U.of_int 0 1) && Bitops.eq arg1192 
            (Bits.U.of_int 0 32) && Bitops.eq arg1191 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1190 (Bits.U.of_int 3 3) && Bitops.eq arg1193 
            (Bits.U.of_int 0 32) && Bitops.eq arg1194 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1195 (Bits.U.of_int 1 1) && Bitops.eq arg1186 
            (Bits.U.of_int 0 32) && Bitops.eq arg1185 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1184 (Bits.U.of_int 3 3) && Bitops.eq arg1187 
            (Bits.U.of_int 0 32) && Bitops.eq arg1188 (Bits.U.of_int 0 3) && 
          Bitops.eq arg1189 (Bits.U.of_int 0 1) && Bitops.eq arg1181 
            (Bits.U.of_int 0 3) && Bitops.eq arg1183 (Bits.U.of_int 32 32) && 
          Bitops.eq arg1182 (Bits.U.of_int 3 3) && Bitops.eq arg1179 
            (Bits.U.of_int 0 3) && Bitops.eq arg1180 (Bits.U.of_int 0 1))) -> 
           Instruction.sdivcc (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1276), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1277), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1274), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1275), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1272), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1273), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1270), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1271), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1268), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1269), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1266), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1267), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1264), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1265), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1262), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1263), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1259), _), 32); 
                  RP.Const (RP.Bits arg1260)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1261), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1256), _), 32); 
                  RP.Const (RP.Bits arg1257)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1258), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1253), _), 32); 
                  RP.Const (RP.Bits arg1254)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1255), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1250), _), 32); 
                  RP.Const (RP.Bits arg1251)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1252), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1247), _), 32); 
                  RP.Const (RP.Bits arg1248)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1249), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1244), _), 32); 
                  RP.Const (RP.Bits arg1245)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1246), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1241), _), 32); 
                  RP.Const (RP.Bits arg1242)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1243), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1238), _), 32); 
                  RP.Const (RP.Bits arg1239)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1240), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1236), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1237), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1233), _), 32); 
                  RP.Const (RP.Bits arg1234)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1235), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1230), _), 32); 
                  RP.Const (RP.Bits arg1231)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1232), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1227), _), 32); 
                  RP.Const (RP.Bits arg1228)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1229), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1224), _), 32); 
                  RP.Const (RP.Bits arg1225)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1226), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1221), _), 32); 
                  RP.Const (RP.Bits arg1222)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1223), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1218), _), 32); 
                  RP.Const (RP.Bits arg1219)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1220), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1215), _), 32); 
                  RP.Const (RP.Bits arg1216)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1217), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1212), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1213), _), 32); 
                  RP.Const (RP.Bits arg1214)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1211)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1211 13 && Bitops.eq arg1276 
            (Bits.U.of_int 24 5) && Bitops.eq arg1277 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1274 (Bits.U.of_int 25 5) && Bitops.eq arg1275 
            (Bits.U.of_int 9 5) && Bitops.eq arg1272 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1273 (Bits.U.of_int 10 5) && Bitops.eq arg1270 
            (Bits.U.of_int 27 5) && Bitops.eq arg1271 (Bits.U.of_int 11 5) && 
          Bitops.eq arg1268 (Bits.U.of_int 28 5) && Bitops.eq arg1269 
            (Bits.U.of_int 12 5) && Bitops.eq arg1266 (Bits.U.of_int 29 5) && 
          Bitops.eq arg1267 (Bits.U.of_int 13 5) && Bitops.eq arg1264 
            (Bits.U.of_int 30 5) && Bitops.eq arg1265 (Bits.U.of_int 14 5) && 
          Bitops.eq arg1262 (Bits.U.of_int 31 5) && Bitops.eq arg1263 
            (Bits.U.of_int 15 5) && Bitops.eq arg1259 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1260 (Bits.U.of_int 8 32) && Bitops.eq arg1261 
            (Bits.U.of_int 16 5) && Bitops.eq arg1256 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1257 (Bits.U.of_int 9 32) && Bitops.eq arg1258 
            (Bits.U.of_int 17 5) && Bitops.eq arg1253 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1254 (Bits.U.of_int 10 32) && Bitops.eq arg1255 
            (Bits.U.of_int 18 5) && Bitops.eq arg1250 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1251 (Bits.U.of_int 11 32) && Bitops.eq arg1252 
            (Bits.U.of_int 19 5) && Bitops.eq arg1247 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1248 (Bits.U.of_int 12 32) && Bitops.eq arg1249 
            (Bits.U.of_int 20 5) && Bitops.eq arg1244 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1245 (Bits.U.of_int 13 32) && Bitops.eq arg1246 
            (Bits.U.of_int 21 5) && Bitops.eq arg1241 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1242 (Bits.U.of_int 14 32) && Bitops.eq arg1243 
            (Bits.U.of_int 22 5) && Bitops.eq arg1238 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1239 (Bits.U.of_int 15 32) && Bitops.eq arg1240 
            (Bits.U.of_int 23 5) && Bitops.eq arg1236 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1237 (Bits.U.of_int 24 5) && Bitops.eq arg1233 
            (Bits.U.of_int 0 1) && Bitops.eq arg1234 (Bits.U.of_int 1 32) && 
          Bitops.eq arg1235 (Bits.U.of_int 25 5) && Bitops.eq arg1230 
            (Bits.U.of_int 0 1) && Bitops.eq arg1231 (Bits.U.of_int 2 32) && 
          Bitops.eq arg1232 (Bits.U.of_int 26 5) && Bitops.eq arg1227 
            (Bits.U.of_int 0 1) && Bitops.eq arg1228 (Bits.U.of_int 3 32) && 
          Bitops.eq arg1229 (Bits.U.of_int 27 5) && Bitops.eq arg1224 
            (Bits.U.of_int 0 1) && Bitops.eq arg1225 (Bits.U.of_int 4 32) && 
          Bitops.eq arg1226 (Bits.U.of_int 28 5) && Bitops.eq arg1221 
            (Bits.U.of_int 0 1) && Bitops.eq arg1222 (Bits.U.of_int 5 32) && 
          Bitops.eq arg1223 (Bits.U.of_int 29 5) && Bitops.eq arg1218 
            (Bits.U.of_int 0 1) && Bitops.eq arg1219 (Bits.U.of_int 6 32) && 
          Bitops.eq arg1220 (Bits.U.of_int 30 5) && Bitops.eq arg1215 
            (Bits.U.of_int 0 1) && Bitops.eq arg1216 (Bits.U.of_int 7 32) && 
          Bitops.eq arg1217 (Bits.U.of_int 31 5) && Bitops.eq arg1212 
            (Bits.U.of_int 0 1) && Bitops.eq arg1213 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1214 (Bits.U.of_int 16 32) && Bitops.ne rd 
            (Bits.U.of_int 24 5) && Bitops.ne rd (Bits.U.of_int 25 5) && Bitops.ne 
            rd (Bits.U.of_int 26 5) && Bitops.ne rd (Bits.U.of_int 27 5) && 
          Bitops.ne rd (Bits.U.of_int 28 5) && Bitops.ne rd (Bits.U.of_int 29 5) && 
          Bitops.ne rd (Bits.U.of_int 30 5) && Bitops.ne rd 
            (Bits.U.of_int 31 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1211)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1341), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1342), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1339), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1340), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1337), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1338), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1335), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1336), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1333), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1334), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1331), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1332), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1329), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1330), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1326), _), 32); 
                  RP.Const (RP.Bits arg1327)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1328), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1323), _), 32); 
                  RP.Const (RP.Bits arg1324)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1325), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1320), _), 32); 
                  RP.Const (RP.Bits arg1321)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1322), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1317), _), 32); 
                  RP.Const (RP.Bits arg1318)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1319), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1314), _), 32); 
                  RP.Const (RP.Bits arg1315)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1316), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1311), _), 32); 
                  RP.Const (RP.Bits arg1312)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1313), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1308), _), 32); 
                  RP.Const (RP.Bits arg1309)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1310), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1305), _), 32); 
                  RP.Const (RP.Bits arg1306)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1307), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1303), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1304), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1300), _), 32); 
                  RP.Const (RP.Bits arg1301)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1302), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1297), _), 32); 
                  RP.Const (RP.Bits arg1298)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1299), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1294), _), 32); 
                  RP.Const (RP.Bits arg1295)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1296), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1291), _), 32); 
                  RP.Const (RP.Bits arg1292)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1293), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1288), _), 32); 
                  RP.Const (RP.Bits arg1289)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1290), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1285), _), 32); 
                  RP.Const (RP.Bits arg1286)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1287), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1282), _), 32); 
                  RP.Const (RP.Bits arg1283)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1284), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1279), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1280), _), 32); 
                  RP.Const (RP.Bits arg1281)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1278)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1278 13 && Bitops.eq arg1341 
            (Bits.U.of_int 24 5) && Bitops.eq arg1342 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1339 (Bits.U.of_int 25 5) && Bitops.eq arg1340 
            (Bits.U.of_int 9 5) && Bitops.eq arg1337 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1338 (Bits.U.of_int 10 5) && Bitops.eq arg1335 
            (Bits.U.of_int 27 5) && Bitops.eq arg1336 (Bits.U.of_int 11 5) && 
          Bitops.eq arg1333 (Bits.U.of_int 28 5) && Bitops.eq arg1334 
            (Bits.U.of_int 12 5) && Bitops.eq arg1331 (Bits.U.of_int 29 5) && 
          Bitops.eq arg1332 (Bits.U.of_int 13 5) && Bitops.eq arg1329 
            (Bits.U.of_int 30 5) && Bitops.eq arg1330 (Bits.U.of_int 14 5) && 
          Bitops.eq arg1326 (Bits.U.of_int 0 1) && Bitops.eq arg1327 
            (Bits.U.of_int 8 32) && Bitops.eq arg1328 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1323 (Bits.U.of_int 0 1) && Bitops.eq arg1324 
            (Bits.U.of_int 9 32) && Bitops.eq arg1325 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1320 (Bits.U.of_int 0 1) && Bitops.eq arg1321 
            (Bits.U.of_int 10 32) && Bitops.eq arg1322 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1317 (Bits.U.of_int 0 1) && Bitops.eq arg1318 
            (Bits.U.of_int 11 32) && Bitops.eq arg1319 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1314 (Bits.U.of_int 0 1) && Bitops.eq arg1315 
            (Bits.U.of_int 12 32) && Bitops.eq arg1316 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1311 (Bits.U.of_int 0 1) && Bitops.eq arg1312 
            (Bits.U.of_int 13 32) && Bitops.eq arg1313 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1308 (Bits.U.of_int 0 1) && Bitops.eq arg1309 
            (Bits.U.of_int 14 32) && Bitops.eq arg1310 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1305 (Bits.U.of_int 0 1) && Bitops.eq arg1306 
            (Bits.U.of_int 15 32) && Bitops.eq arg1307 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1303 (Bits.U.of_int 0 1) && Bitops.eq arg1304 
            (Bits.U.of_int 24 5) && Bitops.eq arg1300 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1301 (Bits.U.of_int 1 32) && Bitops.eq arg1302 
            (Bits.U.of_int 25 5) && Bitops.eq arg1297 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1298 (Bits.U.of_int 2 32) && Bitops.eq arg1299 
            (Bits.U.of_int 26 5) && Bitops.eq arg1294 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1295 (Bits.U.of_int 3 32) && Bitops.eq arg1296 
            (Bits.U.of_int 27 5) && Bitops.eq arg1291 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1292 (Bits.U.of_int 4 32) && Bitops.eq arg1293 
            (Bits.U.of_int 28 5) && Bitops.eq arg1288 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1289 (Bits.U.of_int 5 32) && Bitops.eq arg1290 
            (Bits.U.of_int 29 5) && Bitops.eq arg1285 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1286 (Bits.U.of_int 6 32) && Bitops.eq arg1287 
            (Bits.U.of_int 30 5) && Bitops.eq arg1282 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1283 (Bits.U.of_int 7 32) && Bitops.eq arg1284 
            (Bits.U.of_int 31 5) && Bitops.eq arg1279 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1280 (Bits.U.of_int 0 1) && Bitops.eq arg1281 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 31 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1278)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1406), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1407), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1404), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1405), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1402), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1403), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1400), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1401), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1398), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1399), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1396), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1397), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1394), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1395), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1391), _), 32); 
                  RP.Const (RP.Bits arg1392)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1393), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1388), _), 32); 
                  RP.Const (RP.Bits arg1389)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1390), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1385), _), 32); 
                  RP.Const (RP.Bits arg1386)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1387), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1382), _), 32); 
                  RP.Const (RP.Bits arg1383)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1384), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1379), _), 32); 
                  RP.Const (RP.Bits arg1380)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1381), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1376), _), 32); 
                  RP.Const (RP.Bits arg1377)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1378), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1373), _), 32); 
                  RP.Const (RP.Bits arg1374)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1375), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1370), _), 32); 
                  RP.Const (RP.Bits arg1371)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1372), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1368), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1369), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1365), _), 32); 
                  RP.Const (RP.Bits arg1366)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1367), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1362), _), 32); 
                  RP.Const (RP.Bits arg1363)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1364), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1359), _), 32); 
                  RP.Const (RP.Bits arg1360)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1361), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1356), _), 32); 
                  RP.Const (RP.Bits arg1357)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1358), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1353), _), 32); 
                  RP.Const (RP.Bits arg1354)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1355), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1350), _), 32); 
                  RP.Const (RP.Bits arg1351)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1352), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1347), _), 32); 
                  RP.Const (RP.Bits arg1348)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1349), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1344), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1345), _), 32); 
                  RP.Const (RP.Bits arg1346)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1343)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1343 13 && Bitops.eq arg1406 
            (Bits.U.of_int 24 5) && Bitops.eq arg1407 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1404 (Bits.U.of_int 25 5) && Bitops.eq arg1405 
            (Bits.U.of_int 9 5) && Bitops.eq arg1402 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1403 (Bits.U.of_int 10 5) && Bitops.eq arg1400 
            (Bits.U.of_int 27 5) && Bitops.eq arg1401 (Bits.U.of_int 11 5) && 
          Bitops.eq arg1398 (Bits.U.of_int 28 5) && Bitops.eq arg1399 
            (Bits.U.of_int 12 5) && Bitops.eq arg1396 (Bits.U.of_int 29 5) && 
          Bitops.eq arg1397 (Bits.U.of_int 13 5) && Bitops.eq arg1394 
            (Bits.U.of_int 31 5) && Bitops.eq arg1395 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1391 (Bits.U.of_int 0 1) && Bitops.eq arg1392 
            (Bits.U.of_int 8 32) && Bitops.eq arg1393 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1388 (Bits.U.of_int 0 1) && Bitops.eq arg1389 
            (Bits.U.of_int 9 32) && Bitops.eq arg1390 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1385 (Bits.U.of_int 0 1) && Bitops.eq arg1386 
            (Bits.U.of_int 10 32) && Bitops.eq arg1387 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1382 (Bits.U.of_int 0 1) && Bitops.eq arg1383 
            (Bits.U.of_int 11 32) && Bitops.eq arg1384 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1379 (Bits.U.of_int 0 1) && Bitops.eq arg1380 
            (Bits.U.of_int 12 32) && Bitops.eq arg1381 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1376 (Bits.U.of_int 0 1) && Bitops.eq arg1377 
            (Bits.U.of_int 13 32) && Bitops.eq arg1378 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1373 (Bits.U.of_int 0 1) && Bitops.eq arg1374 
            (Bits.U.of_int 14 32) && Bitops.eq arg1375 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1370 (Bits.U.of_int 0 1) && Bitops.eq arg1371 
            (Bits.U.of_int 15 32) && Bitops.eq arg1372 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1368 (Bits.U.of_int 0 1) && Bitops.eq arg1369 
            (Bits.U.of_int 24 5) && Bitops.eq arg1365 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1366 (Bits.U.of_int 1 32) && Bitops.eq arg1367 
            (Bits.U.of_int 25 5) && Bitops.eq arg1362 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1363 (Bits.U.of_int 2 32) && Bitops.eq arg1364 
            (Bits.U.of_int 26 5) && Bitops.eq arg1359 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1360 (Bits.U.of_int 3 32) && Bitops.eq arg1361 
            (Bits.U.of_int 27 5) && Bitops.eq arg1356 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1357 (Bits.U.of_int 4 32) && Bitops.eq arg1358 
            (Bits.U.of_int 28 5) && Bitops.eq arg1353 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1354 (Bits.U.of_int 5 32) && Bitops.eq arg1355 
            (Bits.U.of_int 29 5) && Bitops.eq arg1350 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1351 (Bits.U.of_int 6 32) && Bitops.eq arg1352 
            (Bits.U.of_int 30 5) && Bitops.eq arg1347 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1348 (Bits.U.of_int 7 32) && Bitops.eq arg1349 
            (Bits.U.of_int 31 5) && Bitops.eq arg1344 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1345 (Bits.U.of_int 0 1) && Bitops.eq arg1346 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 30 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1343)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1471), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1472), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1469), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1470), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1467), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1468), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1465), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1466), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1463), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1464), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1461), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1462), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1459), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1460), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1456), _), 32); 
                  RP.Const (RP.Bits arg1457)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1458), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1453), _), 32); 
                  RP.Const (RP.Bits arg1454)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1455), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1450), _), 32); 
                  RP.Const (RP.Bits arg1451)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1452), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1447), _), 32); 
                  RP.Const (RP.Bits arg1448)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1449), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1444), _), 32); 
                  RP.Const (RP.Bits arg1445)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1446), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1441), _), 32); 
                  RP.Const (RP.Bits arg1442)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1443), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1438), _), 32); 
                  RP.Const (RP.Bits arg1439)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1440), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1435), _), 32); 
                  RP.Const (RP.Bits arg1436)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1437), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1433), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1434), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1430), _), 32); 
                  RP.Const (RP.Bits arg1431)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1432), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1427), _), 32); 
                  RP.Const (RP.Bits arg1428)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1429), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1424), _), 32); 
                  RP.Const (RP.Bits arg1425)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1426), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1421), _), 32); 
                  RP.Const (RP.Bits arg1422)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1423), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1418), _), 32); 
                  RP.Const (RP.Bits arg1419)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1420), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1415), _), 32); 
                  RP.Const (RP.Bits arg1416)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1417), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1412), _), 32); 
                  RP.Const (RP.Bits arg1413)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1414), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1409), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1410), _), 32); 
                  RP.Const (RP.Bits arg1411)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1408)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1408 13 && Bitops.eq arg1471 
            (Bits.U.of_int 24 5) && Bitops.eq arg1472 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1469 (Bits.U.of_int 25 5) && Bitops.eq arg1470 
            (Bits.U.of_int 9 5) && Bitops.eq arg1467 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1468 (Bits.U.of_int 10 5) && Bitops.eq arg1465 
            (Bits.U.of_int 27 5) && Bitops.eq arg1466 (Bits.U.of_int 11 5) && 
          Bitops.eq arg1463 (Bits.U.of_int 28 5) && Bitops.eq arg1464 
            (Bits.U.of_int 12 5) && Bitops.eq arg1461 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1462 (Bits.U.of_int 14 5) && Bitops.eq arg1459 
            (Bits.U.of_int 31 5) && Bitops.eq arg1460 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1456 (Bits.U.of_int 0 1) && Bitops.eq arg1457 
            (Bits.U.of_int 8 32) && Bitops.eq arg1458 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1453 (Bits.U.of_int 0 1) && Bitops.eq arg1454 
            (Bits.U.of_int 9 32) && Bitops.eq arg1455 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1450 (Bits.U.of_int 0 1) && Bitops.eq arg1451 
            (Bits.U.of_int 10 32) && Bitops.eq arg1452 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1447 (Bits.U.of_int 0 1) && Bitops.eq arg1448 
            (Bits.U.of_int 11 32) && Bitops.eq arg1449 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1444 (Bits.U.of_int 0 1) && Bitops.eq arg1445 
            (Bits.U.of_int 12 32) && Bitops.eq arg1446 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1441 (Bits.U.of_int 0 1) && Bitops.eq arg1442 
            (Bits.U.of_int 13 32) && Bitops.eq arg1443 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1438 (Bits.U.of_int 0 1) && Bitops.eq arg1439 
            (Bits.U.of_int 14 32) && Bitops.eq arg1440 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1435 (Bits.U.of_int 0 1) && Bitops.eq arg1436 
            (Bits.U.of_int 15 32) && Bitops.eq arg1437 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1433 (Bits.U.of_int 0 1) && Bitops.eq arg1434 
            (Bits.U.of_int 24 5) && Bitops.eq arg1430 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1431 (Bits.U.of_int 1 32) && Bitops.eq arg1432 
            (Bits.U.of_int 25 5) && Bitops.eq arg1427 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1428 (Bits.U.of_int 2 32) && Bitops.eq arg1429 
            (Bits.U.of_int 26 5) && Bitops.eq arg1424 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1425 (Bits.U.of_int 3 32) && Bitops.eq arg1426 
            (Bits.U.of_int 27 5) && Bitops.eq arg1421 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1422 (Bits.U.of_int 4 32) && Bitops.eq arg1423 
            (Bits.U.of_int 28 5) && Bitops.eq arg1418 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1419 (Bits.U.of_int 5 32) && Bitops.eq arg1420 
            (Bits.U.of_int 29 5) && Bitops.eq arg1415 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1416 (Bits.U.of_int 6 32) && Bitops.eq arg1417 
            (Bits.U.of_int 30 5) && Bitops.eq arg1412 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1413 (Bits.U.of_int 7 32) && Bitops.eq arg1414 
            (Bits.U.of_int 31 5) && Bitops.eq arg1409 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1410 (Bits.U.of_int 0 1) && Bitops.eq arg1411 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 29 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1408)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1536), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1537), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1534), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1535), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1532), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1533), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1530), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1531), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1528), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1529), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1526), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1527), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1524), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1525), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1521), _), 32); 
                  RP.Const (RP.Bits arg1522)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1523), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1518), _), 32); 
                  RP.Const (RP.Bits arg1519)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1520), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1515), _), 32); 
                  RP.Const (RP.Bits arg1516)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1517), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1512), _), 32); 
                  RP.Const (RP.Bits arg1513)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1514), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1509), _), 32); 
                  RP.Const (RP.Bits arg1510)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1511), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1506), _), 32); 
                  RP.Const (RP.Bits arg1507)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1508), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1503), _), 32); 
                  RP.Const (RP.Bits arg1504)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1505), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1500), _), 32); 
                  RP.Const (RP.Bits arg1501)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1502), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1498), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1499), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1495), _), 32); 
                  RP.Const (RP.Bits arg1496)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1497), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1492), _), 32); 
                  RP.Const (RP.Bits arg1493)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1494), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1489), _), 32); 
                  RP.Const (RP.Bits arg1490)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1491), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1486), _), 32); 
                  RP.Const (RP.Bits arg1487)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1488), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1483), _), 32); 
                  RP.Const (RP.Bits arg1484)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1485), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1480), _), 32); 
                  RP.Const (RP.Bits arg1481)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1482), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1477), _), 32); 
                  RP.Const (RP.Bits arg1478)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1479), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1474), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1475), _), 32); 
                  RP.Const (RP.Bits arg1476)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1473)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1473 13 && Bitops.eq arg1536 
            (Bits.U.of_int 24 5) && Bitops.eq arg1537 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1534 (Bits.U.of_int 25 5) && Bitops.eq arg1535 
            (Bits.U.of_int 9 5) && Bitops.eq arg1532 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1533 (Bits.U.of_int 10 5) && Bitops.eq arg1530 
            (Bits.U.of_int 27 5) && Bitops.eq arg1531 (Bits.U.of_int 11 5) && 
          Bitops.eq arg1528 (Bits.U.of_int 29 5) && Bitops.eq arg1529 
            (Bits.U.of_int 13 5) && Bitops.eq arg1526 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1527 (Bits.U.of_int 14 5) && Bitops.eq arg1524 
            (Bits.U.of_int 31 5) && Bitops.eq arg1525 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1521 (Bits.U.of_int 0 1) && Bitops.eq arg1522 
            (Bits.U.of_int 8 32) && Bitops.eq arg1523 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1518 (Bits.U.of_int 0 1) && Bitops.eq arg1519 
            (Bits.U.of_int 9 32) && Bitops.eq arg1520 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1515 (Bits.U.of_int 0 1) && Bitops.eq arg1516 
            (Bits.U.of_int 10 32) && Bitops.eq arg1517 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1512 (Bits.U.of_int 0 1) && Bitops.eq arg1513 
            (Bits.U.of_int 11 32) && Bitops.eq arg1514 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1509 (Bits.U.of_int 0 1) && Bitops.eq arg1510 
            (Bits.U.of_int 12 32) && Bitops.eq arg1511 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1506 (Bits.U.of_int 0 1) && Bitops.eq arg1507 
            (Bits.U.of_int 13 32) && Bitops.eq arg1508 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1503 (Bits.U.of_int 0 1) && Bitops.eq arg1504 
            (Bits.U.of_int 14 32) && Bitops.eq arg1505 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1500 (Bits.U.of_int 0 1) && Bitops.eq arg1501 
            (Bits.U.of_int 15 32) && Bitops.eq arg1502 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1498 (Bits.U.of_int 0 1) && Bitops.eq arg1499 
            (Bits.U.of_int 24 5) && Bitops.eq arg1495 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1496 (Bits.U.of_int 1 32) && Bitops.eq arg1497 
            (Bits.U.of_int 25 5) && Bitops.eq arg1492 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1493 (Bits.U.of_int 2 32) && Bitops.eq arg1494 
            (Bits.U.of_int 26 5) && Bitops.eq arg1489 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1490 (Bits.U.of_int 3 32) && Bitops.eq arg1491 
            (Bits.U.of_int 27 5) && Bitops.eq arg1486 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1487 (Bits.U.of_int 4 32) && Bitops.eq arg1488 
            (Bits.U.of_int 28 5) && Bitops.eq arg1483 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1484 (Bits.U.of_int 5 32) && Bitops.eq arg1485 
            (Bits.U.of_int 29 5) && Bitops.eq arg1480 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1481 (Bits.U.of_int 6 32) && Bitops.eq arg1482 
            (Bits.U.of_int 30 5) && Bitops.eq arg1477 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1478 (Bits.U.of_int 7 32) && Bitops.eq arg1479 
            (Bits.U.of_int 31 5) && Bitops.eq arg1474 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1475 (Bits.U.of_int 0 1) && Bitops.eq arg1476 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 28 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1473)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1601), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1602), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1599), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1600), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1597), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1598), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1595), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1596), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1593), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1594), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1591), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1592), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1589), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1590), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1586), _), 32); 
                  RP.Const (RP.Bits arg1587)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1588), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1583), _), 32); 
                  RP.Const (RP.Bits arg1584)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1585), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1580), _), 32); 
                  RP.Const (RP.Bits arg1581)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1582), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1577), _), 32); 
                  RP.Const (RP.Bits arg1578)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1579), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1574), _), 32); 
                  RP.Const (RP.Bits arg1575)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1576), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1571), _), 32); 
                  RP.Const (RP.Bits arg1572)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1573), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1568), _), 32); 
                  RP.Const (RP.Bits arg1569)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1570), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1565), _), 32); 
                  RP.Const (RP.Bits arg1566)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1567), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1563), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1564), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1560), _), 32); 
                  RP.Const (RP.Bits arg1561)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1562), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1557), _), 32); 
                  RP.Const (RP.Bits arg1558)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1559), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1554), _), 32); 
                  RP.Const (RP.Bits arg1555)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1556), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1551), _), 32); 
                  RP.Const (RP.Bits arg1552)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1553), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1548), _), 32); 
                  RP.Const (RP.Bits arg1549)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1550), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1545), _), 32); 
                  RP.Const (RP.Bits arg1546)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1547), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1542), _), 32); 
                  RP.Const (RP.Bits arg1543)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1544), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1539), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1540), _), 32); 
                  RP.Const (RP.Bits arg1541)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1538)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1538 13 && Bitops.eq arg1601 
            (Bits.U.of_int 24 5) && Bitops.eq arg1602 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1599 (Bits.U.of_int 25 5) && Bitops.eq arg1600 
            (Bits.U.of_int 9 5) && Bitops.eq arg1597 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1598 (Bits.U.of_int 10 5) && Bitops.eq arg1595 
            (Bits.U.of_int 28 5) && Bitops.eq arg1596 (Bits.U.of_int 12 5) && 
          Bitops.eq arg1593 (Bits.U.of_int 29 5) && Bitops.eq arg1594 
            (Bits.U.of_int 13 5) && Bitops.eq arg1591 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1592 (Bits.U.of_int 14 5) && Bitops.eq arg1589 
            (Bits.U.of_int 31 5) && Bitops.eq arg1590 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1586 (Bits.U.of_int 0 1) && Bitops.eq arg1587 
            (Bits.U.of_int 8 32) && Bitops.eq arg1588 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1583 (Bits.U.of_int 0 1) && Bitops.eq arg1584 
            (Bits.U.of_int 9 32) && Bitops.eq arg1585 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1580 (Bits.U.of_int 0 1) && Bitops.eq arg1581 
            (Bits.U.of_int 10 32) && Bitops.eq arg1582 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1577 (Bits.U.of_int 0 1) && Bitops.eq arg1578 
            (Bits.U.of_int 11 32) && Bitops.eq arg1579 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1574 (Bits.U.of_int 0 1) && Bitops.eq arg1575 
            (Bits.U.of_int 12 32) && Bitops.eq arg1576 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1571 (Bits.U.of_int 0 1) && Bitops.eq arg1572 
            (Bits.U.of_int 13 32) && Bitops.eq arg1573 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1568 (Bits.U.of_int 0 1) && Bitops.eq arg1569 
            (Bits.U.of_int 14 32) && Bitops.eq arg1570 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1565 (Bits.U.of_int 0 1) && Bitops.eq arg1566 
            (Bits.U.of_int 15 32) && Bitops.eq arg1567 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1563 (Bits.U.of_int 0 1) && Bitops.eq arg1564 
            (Bits.U.of_int 24 5) && Bitops.eq arg1560 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1561 (Bits.U.of_int 1 32) && Bitops.eq arg1562 
            (Bits.U.of_int 25 5) && Bitops.eq arg1557 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1558 (Bits.U.of_int 2 32) && Bitops.eq arg1559 
            (Bits.U.of_int 26 5) && Bitops.eq arg1554 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1555 (Bits.U.of_int 3 32) && Bitops.eq arg1556 
            (Bits.U.of_int 27 5) && Bitops.eq arg1551 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1552 (Bits.U.of_int 4 32) && Bitops.eq arg1553 
            (Bits.U.of_int 28 5) && Bitops.eq arg1548 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1549 (Bits.U.of_int 5 32) && Bitops.eq arg1550 
            (Bits.U.of_int 29 5) && Bitops.eq arg1545 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1546 (Bits.U.of_int 6 32) && Bitops.eq arg1547 
            (Bits.U.of_int 30 5) && Bitops.eq arg1542 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1543 (Bits.U.of_int 7 32) && Bitops.eq arg1544 
            (Bits.U.of_int 31 5) && Bitops.eq arg1539 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1540 (Bits.U.of_int 0 1) && Bitops.eq arg1541 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 27 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1538)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1666), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1667), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1664), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1665), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1662), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1663), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1660), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1661), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1658), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1659), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1656), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1657), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1654), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1655), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1651), _), 32); 
                  RP.Const (RP.Bits arg1652)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1653), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1648), _), 32); 
                  RP.Const (RP.Bits arg1649)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1650), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1645), _), 32); 
                  RP.Const (RP.Bits arg1646)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1647), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1642), _), 32); 
                  RP.Const (RP.Bits arg1643)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1644), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1639), _), 32); 
                  RP.Const (RP.Bits arg1640)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1641), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1636), _), 32); 
                  RP.Const (RP.Bits arg1637)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1638), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1633), _), 32); 
                  RP.Const (RP.Bits arg1634)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1635), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1630), _), 32); 
                  RP.Const (RP.Bits arg1631)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1632), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1628), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1629), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1625), _), 32); 
                  RP.Const (RP.Bits arg1626)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1627), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1622), _), 32); 
                  RP.Const (RP.Bits arg1623)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1624), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1619), _), 32); 
                  RP.Const (RP.Bits arg1620)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1621), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1616), _), 32); 
                  RP.Const (RP.Bits arg1617)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1618), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1613), _), 32); 
                  RP.Const (RP.Bits arg1614)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1615), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1610), _), 32); 
                  RP.Const (RP.Bits arg1611)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1612), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1607), _), 32); 
                  RP.Const (RP.Bits arg1608)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1609), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1604), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1605), _), 32); 
                  RP.Const (RP.Bits arg1606)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1603)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1603 13 && Bitops.eq arg1666 
            (Bits.U.of_int 24 5) && Bitops.eq arg1667 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1664 (Bits.U.of_int 25 5) && Bitops.eq arg1665 
            (Bits.U.of_int 9 5) && Bitops.eq arg1662 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1663 (Bits.U.of_int 11 5) && Bitops.eq arg1660 
            (Bits.U.of_int 28 5) && Bitops.eq arg1661 (Bits.U.of_int 12 5) && 
          Bitops.eq arg1658 (Bits.U.of_int 29 5) && Bitops.eq arg1659 
            (Bits.U.of_int 13 5) && Bitops.eq arg1656 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1657 (Bits.U.of_int 14 5) && Bitops.eq arg1654 
            (Bits.U.of_int 31 5) && Bitops.eq arg1655 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1651 (Bits.U.of_int 0 1) && Bitops.eq arg1652 
            (Bits.U.of_int 8 32) && Bitops.eq arg1653 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1648 (Bits.U.of_int 0 1) && Bitops.eq arg1649 
            (Bits.U.of_int 9 32) && Bitops.eq arg1650 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1645 (Bits.U.of_int 0 1) && Bitops.eq arg1646 
            (Bits.U.of_int 10 32) && Bitops.eq arg1647 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1642 (Bits.U.of_int 0 1) && Bitops.eq arg1643 
            (Bits.U.of_int 11 32) && Bitops.eq arg1644 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1639 (Bits.U.of_int 0 1) && Bitops.eq arg1640 
            (Bits.U.of_int 12 32) && Bitops.eq arg1641 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1636 (Bits.U.of_int 0 1) && Bitops.eq arg1637 
            (Bits.U.of_int 13 32) && Bitops.eq arg1638 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1633 (Bits.U.of_int 0 1) && Bitops.eq arg1634 
            (Bits.U.of_int 14 32) && Bitops.eq arg1635 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1630 (Bits.U.of_int 0 1) && Bitops.eq arg1631 
            (Bits.U.of_int 15 32) && Bitops.eq arg1632 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1628 (Bits.U.of_int 0 1) && Bitops.eq arg1629 
            (Bits.U.of_int 24 5) && Bitops.eq arg1625 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1626 (Bits.U.of_int 1 32) && Bitops.eq arg1627 
            (Bits.U.of_int 25 5) && Bitops.eq arg1622 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1623 (Bits.U.of_int 2 32) && Bitops.eq arg1624 
            (Bits.U.of_int 26 5) && Bitops.eq arg1619 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1620 (Bits.U.of_int 3 32) && Bitops.eq arg1621 
            (Bits.U.of_int 27 5) && Bitops.eq arg1616 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1617 (Bits.U.of_int 4 32) && Bitops.eq arg1618 
            (Bits.U.of_int 28 5) && Bitops.eq arg1613 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1614 (Bits.U.of_int 5 32) && Bitops.eq arg1615 
            (Bits.U.of_int 29 5) && Bitops.eq arg1610 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1611 (Bits.U.of_int 6 32) && Bitops.eq arg1612 
            (Bits.U.of_int 30 5) && Bitops.eq arg1607 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1608 (Bits.U.of_int 7 32) && Bitops.eq arg1609 
            (Bits.U.of_int 31 5) && Bitops.eq arg1604 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1605 (Bits.U.of_int 0 1) && Bitops.eq arg1606 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 26 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1603)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1731), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1732), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1729), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1730), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1727), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1728), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1725), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1726), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1723), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1724), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1721), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1722), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1719), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1720), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1716), _), 32); 
                  RP.Const (RP.Bits arg1717)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1718), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1713), _), 32); 
                  RP.Const (RP.Bits arg1714)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1715), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1710), _), 32); 
                  RP.Const (RP.Bits arg1711)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1712), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1707), _), 32); 
                  RP.Const (RP.Bits arg1708)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1709), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1704), _), 32); 
                  RP.Const (RP.Bits arg1705)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1706), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1701), _), 32); 
                  RP.Const (RP.Bits arg1702)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1703), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1698), _), 32); 
                  RP.Const (RP.Bits arg1699)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1700), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1695), _), 32); 
                  RP.Const (RP.Bits arg1696)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1697), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1693), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1694), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1690), _), 32); 
                  RP.Const (RP.Bits arg1691)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1692), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1687), _), 32); 
                  RP.Const (RP.Bits arg1688)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1689), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1684), _), 32); 
                  RP.Const (RP.Bits arg1685)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1686), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1681), _), 32); 
                  RP.Const (RP.Bits arg1682)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1683), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1678), _), 32); 
                  RP.Const (RP.Bits arg1679)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1680), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1675), _), 32); 
                  RP.Const (RP.Bits arg1676)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1677), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1672), _), 32); 
                  RP.Const (RP.Bits arg1673)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1674), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1669), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1670), _), 32); 
                  RP.Const (RP.Bits arg1671)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1668)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1668 13 && Bitops.eq arg1731 
            (Bits.U.of_int 24 5) && Bitops.eq arg1732 (Bits.U.of_int 8 5) && 
          Bitops.eq arg1729 (Bits.U.of_int 26 5) && Bitops.eq arg1730 
            (Bits.U.of_int 10 5) && Bitops.eq arg1727 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1728 (Bits.U.of_int 11 5) && Bitops.eq arg1725 
            (Bits.U.of_int 28 5) && Bitops.eq arg1726 (Bits.U.of_int 12 5) && 
          Bitops.eq arg1723 (Bits.U.of_int 29 5) && Bitops.eq arg1724 
            (Bits.U.of_int 13 5) && Bitops.eq arg1721 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1722 (Bits.U.of_int 14 5) && Bitops.eq arg1719 
            (Bits.U.of_int 31 5) && Bitops.eq arg1720 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1716 (Bits.U.of_int 0 1) && Bitops.eq arg1717 
            (Bits.U.of_int 8 32) && Bitops.eq arg1718 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1713 (Bits.U.of_int 0 1) && Bitops.eq arg1714 
            (Bits.U.of_int 9 32) && Bitops.eq arg1715 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1710 (Bits.U.of_int 0 1) && Bitops.eq arg1711 
            (Bits.U.of_int 10 32) && Bitops.eq arg1712 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1707 (Bits.U.of_int 0 1) && Bitops.eq arg1708 
            (Bits.U.of_int 11 32) && Bitops.eq arg1709 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1704 (Bits.U.of_int 0 1) && Bitops.eq arg1705 
            (Bits.U.of_int 12 32) && Bitops.eq arg1706 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1701 (Bits.U.of_int 0 1) && Bitops.eq arg1702 
            (Bits.U.of_int 13 32) && Bitops.eq arg1703 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1698 (Bits.U.of_int 0 1) && Bitops.eq arg1699 
            (Bits.U.of_int 14 32) && Bitops.eq arg1700 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1695 (Bits.U.of_int 0 1) && Bitops.eq arg1696 
            (Bits.U.of_int 15 32) && Bitops.eq arg1697 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1693 (Bits.U.of_int 0 1) && Bitops.eq arg1694 
            (Bits.U.of_int 24 5) && Bitops.eq arg1690 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1691 (Bits.U.of_int 1 32) && Bitops.eq arg1692 
            (Bits.U.of_int 25 5) && Bitops.eq arg1687 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1688 (Bits.U.of_int 2 32) && Bitops.eq arg1689 
            (Bits.U.of_int 26 5) && Bitops.eq arg1684 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1685 (Bits.U.of_int 3 32) && Bitops.eq arg1686 
            (Bits.U.of_int 27 5) && Bitops.eq arg1681 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1682 (Bits.U.of_int 4 32) && Bitops.eq arg1683 
            (Bits.U.of_int 28 5) && Bitops.eq arg1678 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1679 (Bits.U.of_int 5 32) && Bitops.eq arg1680 
            (Bits.U.of_int 29 5) && Bitops.eq arg1675 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1676 (Bits.U.of_int 6 32) && Bitops.eq arg1677 
            (Bits.U.of_int 30 5) && Bitops.eq arg1672 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1673 (Bits.U.of_int 7 32) && Bitops.eq arg1674 
            (Bits.U.of_int 31 5) && Bitops.eq arg1669 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1670 (Bits.U.of_int 0 1) && Bitops.eq arg1671 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 25 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1668)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1796), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1797), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1794), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1795), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1792), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1793), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1790), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1791), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1788), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1789), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1786), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1787), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1784), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1785), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1781), _), 32); 
                  RP.Const (RP.Bits arg1782)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1783), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1778), _), 32); 
                  RP.Const (RP.Bits arg1779)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1780), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1775), _), 32); 
                  RP.Const (RP.Bits arg1776)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1777), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1772), _), 32); 
                  RP.Const (RP.Bits arg1773)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1774), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1769), _), 32); 
                  RP.Const (RP.Bits arg1770)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1771), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1766), _), 32); 
                  RP.Const (RP.Bits arg1767)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1768), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1763), _), 32); 
                  RP.Const (RP.Bits arg1764)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1765), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1760), _), 32); 
                  RP.Const (RP.Bits arg1761)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1762), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1758), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1759), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1755), _), 32); 
                  RP.Const (RP.Bits arg1756)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1757), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1752), _), 32); 
                  RP.Const (RP.Bits arg1753)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1754), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1749), _), 32); 
                  RP.Const (RP.Bits arg1750)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1751), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1746), _), 32); 
                  RP.Const (RP.Bits arg1747)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1748), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1743), _), 32); 
                  RP.Const (RP.Bits arg1744)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1745), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1740), _), 32); 
                  RP.Const (RP.Bits arg1741)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1742), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1737), _), 32); 
                  RP.Const (RP.Bits arg1738)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1739), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1734), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1735), _), 32); 
                  RP.Const (RP.Bits arg1736)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg1733)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg1733 13 && Bitops.eq arg1796 
            (Bits.U.of_int 25 5) && Bitops.eq arg1797 (Bits.U.of_int 9 5) && 
          Bitops.eq arg1794 (Bits.U.of_int 26 5) && Bitops.eq arg1795 
            (Bits.U.of_int 10 5) && Bitops.eq arg1792 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1793 (Bits.U.of_int 11 5) && Bitops.eq arg1790 
            (Bits.U.of_int 28 5) && Bitops.eq arg1791 (Bits.U.of_int 12 5) && 
          Bitops.eq arg1788 (Bits.U.of_int 29 5) && Bitops.eq arg1789 
            (Bits.U.of_int 13 5) && Bitops.eq arg1786 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1787 (Bits.U.of_int 14 5) && Bitops.eq arg1784 
            (Bits.U.of_int 31 5) && Bitops.eq arg1785 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1781 (Bits.U.of_int 0 1) && Bitops.eq arg1782 
            (Bits.U.of_int 8 32) && Bitops.eq arg1783 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1778 (Bits.U.of_int 0 1) && Bitops.eq arg1779 
            (Bits.U.of_int 9 32) && Bitops.eq arg1780 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1775 (Bits.U.of_int 0 1) && Bitops.eq arg1776 
            (Bits.U.of_int 10 32) && Bitops.eq arg1777 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1772 (Bits.U.of_int 0 1) && Bitops.eq arg1773 
            (Bits.U.of_int 11 32) && Bitops.eq arg1774 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1769 (Bits.U.of_int 0 1) && Bitops.eq arg1770 
            (Bits.U.of_int 12 32) && Bitops.eq arg1771 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1766 (Bits.U.of_int 0 1) && Bitops.eq arg1767 
            (Bits.U.of_int 13 32) && Bitops.eq arg1768 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1763 (Bits.U.of_int 0 1) && Bitops.eq arg1764 
            (Bits.U.of_int 14 32) && Bitops.eq arg1765 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1760 (Bits.U.of_int 0 1) && Bitops.eq arg1761 
            (Bits.U.of_int 15 32) && Bitops.eq arg1762 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1758 (Bits.U.of_int 0 1) && Bitops.eq arg1759 
            (Bits.U.of_int 24 5) && Bitops.eq arg1755 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1756 (Bits.U.of_int 1 32) && Bitops.eq arg1757 
            (Bits.U.of_int 25 5) && Bitops.eq arg1752 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1753 (Bits.U.of_int 2 32) && Bitops.eq arg1754 
            (Bits.U.of_int 26 5) && Bitops.eq arg1749 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1750 (Bits.U.of_int 3 32) && Bitops.eq arg1751 
            (Bits.U.of_int 27 5) && Bitops.eq arg1746 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1747 (Bits.U.of_int 4 32) && Bitops.eq arg1748 
            (Bits.U.of_int 28 5) && Bitops.eq arg1743 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1744 (Bits.U.of_int 5 32) && Bitops.eq arg1745 
            (Bits.U.of_int 29 5) && Bitops.eq arg1740 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1741 (Bits.U.of_int 6 32) && Bitops.eq arg1742 
            (Bits.U.of_int 30 5) && Bitops.eq arg1737 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1738 (Bits.U.of_int 7 32) && Bitops.eq arg1739 
            (Bits.U.of_int 31 5) && Bitops.eq arg1734 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1735 (Bits.U.of_int 0 1) && Bitops.eq arg1736 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 24 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg1733)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1862), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1863), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1860), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1861), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1858), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1859), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1856), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1857), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1854), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1855), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1852), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1853), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1850), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1851), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1848), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1849), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1845), _), 32); 
                  RP.Const (RP.Bits arg1846)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1847), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1842), _), 32); 
                  RP.Const (RP.Bits arg1843)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1844), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1839), _), 32); 
                  RP.Const (RP.Bits arg1840)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1841), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1836), _), 32); 
                  RP.Const (RP.Bits arg1837)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1838), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1833), _), 32); 
                  RP.Const (RP.Bits arg1834)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1835), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1830), _), 32); 
                  RP.Const (RP.Bits arg1831)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1832), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1827), _), 32); 
                  RP.Const (RP.Bits arg1828)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1829), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1824), _), 32); 
                  RP.Const (RP.Bits arg1825)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1826), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1822), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1823), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1819), _), 32); 
                  RP.Const (RP.Bits arg1820)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1821), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1816), _), 32); 
                  RP.Const (RP.Bits arg1817)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1818), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1813), _), 32); 
                  RP.Const (RP.Bits arg1814)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1815), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1810), _), 32); 
                  RP.Const (RP.Bits arg1811)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1812), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1807), _), 32); 
                  RP.Const (RP.Bits arg1808)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1809), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1804), _), 32); 
                  RP.Const (RP.Bits arg1805)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1806), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1801), _), 32); 
                  RP.Const (RP.Bits arg1802)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1803), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1798), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1799), _), 32); 
                  RP.Const (RP.Bits arg1800)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg1862 (Bits.U.of_int 24 5) && Bitops.eq 
            arg1863 (Bits.U.of_int 8 5) && Bitops.eq arg1860 
            (Bits.U.of_int 25 5) && Bitops.eq arg1861 (Bits.U.of_int 9 5) && 
          Bitops.eq arg1858 (Bits.U.of_int 26 5) && Bitops.eq arg1859 
            (Bits.U.of_int 10 5) && Bitops.eq arg1856 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1857 (Bits.U.of_int 11 5) && Bitops.eq arg1854 
            (Bits.U.of_int 28 5) && Bitops.eq arg1855 (Bits.U.of_int 12 5) && 
          Bitops.eq arg1852 (Bits.U.of_int 29 5) && Bitops.eq arg1853 
            (Bits.U.of_int 13 5) && Bitops.eq arg1850 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1851 (Bits.U.of_int 14 5) && Bitops.eq arg1848 
            (Bits.U.of_int 31 5) && Bitops.eq arg1849 (Bits.U.of_int 15 5) && 
          Bitops.eq arg1845 (Bits.U.of_int 0 1) && Bitops.eq arg1846 
            (Bits.U.of_int 8 32) && Bitops.eq arg1847 (Bits.U.of_int 16 5) && 
          Bitops.eq arg1842 (Bits.U.of_int 0 1) && Bitops.eq arg1843 
            (Bits.U.of_int 9 32) && Bitops.eq arg1844 (Bits.U.of_int 17 5) && 
          Bitops.eq arg1839 (Bits.U.of_int 0 1) && Bitops.eq arg1840 
            (Bits.U.of_int 10 32) && Bitops.eq arg1841 (Bits.U.of_int 18 5) && 
          Bitops.eq arg1836 (Bits.U.of_int 0 1) && Bitops.eq arg1837 
            (Bits.U.of_int 11 32) && Bitops.eq arg1838 (Bits.U.of_int 19 5) && 
          Bitops.eq arg1833 (Bits.U.of_int 0 1) && Bitops.eq arg1834 
            (Bits.U.of_int 12 32) && Bitops.eq arg1835 (Bits.U.of_int 20 5) && 
          Bitops.eq arg1830 (Bits.U.of_int 0 1) && Bitops.eq arg1831 
            (Bits.U.of_int 13 32) && Bitops.eq arg1832 (Bits.U.of_int 21 5) && 
          Bitops.eq arg1827 (Bits.U.of_int 0 1) && Bitops.eq arg1828 
            (Bits.U.of_int 14 32) && Bitops.eq arg1829 (Bits.U.of_int 22 5) && 
          Bitops.eq arg1824 (Bits.U.of_int 0 1) && Bitops.eq arg1825 
            (Bits.U.of_int 15 32) && Bitops.eq arg1826 (Bits.U.of_int 23 5) && 
          Bitops.eq arg1822 (Bits.U.of_int 0 1) && Bitops.eq arg1823 
            (Bits.U.of_int 24 5) && Bitops.eq arg1819 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1820 (Bits.U.of_int 1 32) && Bitops.eq arg1821 
            (Bits.U.of_int 25 5) && Bitops.eq arg1816 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1817 (Bits.U.of_int 2 32) && Bitops.eq arg1818 
            (Bits.U.of_int 26 5) && Bitops.eq arg1813 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1814 (Bits.U.of_int 3 32) && Bitops.eq arg1815 
            (Bits.U.of_int 27 5) && Bitops.eq arg1810 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1811 (Bits.U.of_int 4 32) && Bitops.eq arg1812 
            (Bits.U.of_int 28 5) && Bitops.eq arg1807 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1808 (Bits.U.of_int 5 32) && Bitops.eq arg1809 
            (Bits.U.of_int 29 5) && Bitops.eq arg1804 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1805 (Bits.U.of_int 6 32) && Bitops.eq arg1806 
            (Bits.U.of_int 30 5) && Bitops.eq arg1801 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1802 (Bits.U.of_int 7 32) && Bitops.eq arg1803 
            (Bits.U.of_int 31 5) && Bitops.eq arg1798 (Bits.U.of_int 0 1) && 
          Bitops.eq arg1799 (Bits.U.of_int 0 1) && Bitops.eq arg1800 
            (Bits.U.of_int 16 32) && Bitops.ne rd (Bits.U.of_int 24 5) && Bitops.ne 
            rd (Bits.U.of_int 25 5) && Bitops.ne rd (Bits.U.of_int 26 5) && 
          Bitops.ne rd (Bits.U.of_int 27 5) && Bitops.ne rd (Bits.U.of_int 28 5) && 
          Bitops.ne rd (Bits.U.of_int 29 5) && Bitops.ne rd (Bits.U.of_int 30 5) && 
          Bitops.ne rd (Bits.U.of_int 31 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1926), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1927), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1924), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1925), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1922), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1923), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1920), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1921), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1918), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1919), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1916), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1917), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1914), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1915), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1911), _), 32); 
                  RP.Const (RP.Bits arg1912)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1913), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1908), _), 32); 
                  RP.Const (RP.Bits arg1909)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1910), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1905), _), 32); 
                  RP.Const (RP.Bits arg1906)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1907), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1902), _), 32); 
                  RP.Const (RP.Bits arg1903)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1904), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1899), _), 32); 
                  RP.Const (RP.Bits arg1900)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1901), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1896), _), 32); 
                  RP.Const (RP.Bits arg1897)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1898), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1893), _), 32); 
                  RP.Const (RP.Bits arg1894)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1895), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1890), _), 32); 
                  RP.Const (RP.Bits arg1891)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1892), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1888), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1889), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1885), _), 32); 
                  RP.Const (RP.Bits arg1886)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1887), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1882), _), 32); 
                  RP.Const (RP.Bits arg1883)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1884), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1879), _), 32); 
                  RP.Const (RP.Bits arg1880)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1881), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1876), _), 32); 
                  RP.Const (RP.Bits arg1877)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1878), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1873), _), 32); 
                  RP.Const (RP.Bits arg1874)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1875), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1870), _), 32); 
                  RP.Const (RP.Bits arg1871)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1872), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1867), _), 32); 
                  RP.Const (RP.Bits arg1868)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1869), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1864), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1865), _), 32); 
                  RP.Const (RP.Bits arg1866)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg1926 (Bits.U.of_int 24 5) && Bitops.eq 
            arg1927 (Bits.U.of_int 8 5) && Bitops.eq arg1924 
            (Bits.U.of_int 25 5) && Bitops.eq arg1925 (Bits.U.of_int 9 5) && 
          Bitops.eq arg1922 (Bits.U.of_int 26 5) && Bitops.eq arg1923 
            (Bits.U.of_int 10 5) && Bitops.eq arg1920 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1921 (Bits.U.of_int 11 5) && Bitops.eq arg1918 
            (Bits.U.of_int 28 5) && Bitops.eq arg1919 (Bits.U.of_int 12 5) && 
          Bitops.eq arg1916 (Bits.U.of_int 29 5) && Bitops.eq arg1917 
            (Bits.U.of_int 13 5) && Bitops.eq arg1914 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1915 (Bits.U.of_int 14 5) && Bitops.eq arg1911 
            (Bits.U.of_int 0 1) && Bitops.eq arg1912 (Bits.U.of_int 8 32) && 
          Bitops.eq arg1913 (Bits.U.of_int 16 5) && Bitops.eq arg1908 
            (Bits.U.of_int 0 1) && Bitops.eq arg1909 (Bits.U.of_int 9 32) && 
          Bitops.eq arg1910 (Bits.U.of_int 17 5) && Bitops.eq arg1905 
            (Bits.U.of_int 0 1) && Bitops.eq arg1906 (Bits.U.of_int 10 32) && 
          Bitops.eq arg1907 (Bits.U.of_int 18 5) && Bitops.eq arg1902 
            (Bits.U.of_int 0 1) && Bitops.eq arg1903 (Bits.U.of_int 11 32) && 
          Bitops.eq arg1904 (Bits.U.of_int 19 5) && Bitops.eq arg1899 
            (Bits.U.of_int 0 1) && Bitops.eq arg1900 (Bits.U.of_int 12 32) && 
          Bitops.eq arg1901 (Bits.U.of_int 20 5) && Bitops.eq arg1896 
            (Bits.U.of_int 0 1) && Bitops.eq arg1897 (Bits.U.of_int 13 32) && 
          Bitops.eq arg1898 (Bits.U.of_int 21 5) && Bitops.eq arg1893 
            (Bits.U.of_int 0 1) && Bitops.eq arg1894 (Bits.U.of_int 14 32) && 
          Bitops.eq arg1895 (Bits.U.of_int 22 5) && Bitops.eq arg1890 
            (Bits.U.of_int 0 1) && Bitops.eq arg1891 (Bits.U.of_int 15 32) && 
          Bitops.eq arg1892 (Bits.U.of_int 23 5) && Bitops.eq arg1888 
            (Bits.U.of_int 0 1) && Bitops.eq arg1889 (Bits.U.of_int 24 5) && 
          Bitops.eq arg1885 (Bits.U.of_int 0 1) && Bitops.eq arg1886 
            (Bits.U.of_int 1 32) && Bitops.eq arg1887 (Bits.U.of_int 25 5) && 
          Bitops.eq arg1882 (Bits.U.of_int 0 1) && Bitops.eq arg1883 
            (Bits.U.of_int 2 32) && Bitops.eq arg1884 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1879 (Bits.U.of_int 0 1) && Bitops.eq arg1880 
            (Bits.U.of_int 3 32) && Bitops.eq arg1881 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1876 (Bits.U.of_int 0 1) && Bitops.eq arg1877 
            (Bits.U.of_int 4 32) && Bitops.eq arg1878 (Bits.U.of_int 28 5) && 
          Bitops.eq arg1873 (Bits.U.of_int 0 1) && Bitops.eq arg1874 
            (Bits.U.of_int 5 32) && Bitops.eq arg1875 (Bits.U.of_int 29 5) && 
          Bitops.eq arg1870 (Bits.U.of_int 0 1) && Bitops.eq arg1871 
            (Bits.U.of_int 6 32) && Bitops.eq arg1872 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1867 (Bits.U.of_int 0 1) && Bitops.eq arg1868 
            (Bits.U.of_int 7 32) && Bitops.eq arg1869 (Bits.U.of_int 31 5) && 
          Bitops.eq arg1864 (Bits.U.of_int 0 1) && Bitops.eq arg1865 
            (Bits.U.of_int 0 1) && Bitops.eq arg1866 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 31 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg1990), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg1991), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1988), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1989), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1986), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1987), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1984), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1985), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1982), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1983), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1980), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1981), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1978), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1979), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1975), _), 32); 
                  RP.Const (RP.Bits arg1976)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1977), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1972), _), 32); 
                  RP.Const (RP.Bits arg1973)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1974), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1969), _), 32); 
                  RP.Const (RP.Bits arg1970)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1971), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1966), _), 32); 
                  RP.Const (RP.Bits arg1967)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1968), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1963), _), 32); 
                  RP.Const (RP.Bits arg1964)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1965), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1960), _), 32); 
                  RP.Const (RP.Bits arg1961)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1962), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1957), _), 32); 
                  RP.Const (RP.Bits arg1958)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1959), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1954), _), 32); 
                  RP.Const (RP.Bits arg1955)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1956), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1952), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg1953), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1949), _), 32); 
                  RP.Const (RP.Bits arg1950)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1951), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1946), _), 32); 
                  RP.Const (RP.Bits arg1947)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1948), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1943), _), 32); 
                  RP.Const (RP.Bits arg1944)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1945), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1940), _), 32); 
                  RP.Const (RP.Bits arg1941)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1942), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1937), _), 32); 
                  RP.Const (RP.Bits arg1938)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1939), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1934), _), 32); 
                  RP.Const (RP.Bits arg1935)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1936), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1931), _), 32); 
                  RP.Const (RP.Bits arg1932)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1933), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1928), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1929), _), 32); 
                  RP.Const (RP.Bits arg1930)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg1990 (Bits.U.of_int 24 5) && Bitops.eq 
            arg1991 (Bits.U.of_int 8 5) && Bitops.eq arg1988 
            (Bits.U.of_int 25 5) && Bitops.eq arg1989 (Bits.U.of_int 9 5) && 
          Bitops.eq arg1986 (Bits.U.of_int 26 5) && Bitops.eq arg1987 
            (Bits.U.of_int 10 5) && Bitops.eq arg1984 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1985 (Bits.U.of_int 11 5) && Bitops.eq arg1982 
            (Bits.U.of_int 28 5) && Bitops.eq arg1983 (Bits.U.of_int 12 5) && 
          Bitops.eq arg1980 (Bits.U.of_int 29 5) && Bitops.eq arg1981 
            (Bits.U.of_int 13 5) && Bitops.eq arg1978 (Bits.U.of_int 31 5) && 
          Bitops.eq arg1979 (Bits.U.of_int 15 5) && Bitops.eq arg1975 
            (Bits.U.of_int 0 1) && Bitops.eq arg1976 (Bits.U.of_int 8 32) && 
          Bitops.eq arg1977 (Bits.U.of_int 16 5) && Bitops.eq arg1972 
            (Bits.U.of_int 0 1) && Bitops.eq arg1973 (Bits.U.of_int 9 32) && 
          Bitops.eq arg1974 (Bits.U.of_int 17 5) && Bitops.eq arg1969 
            (Bits.U.of_int 0 1) && Bitops.eq arg1970 (Bits.U.of_int 10 32) && 
          Bitops.eq arg1971 (Bits.U.of_int 18 5) && Bitops.eq arg1966 
            (Bits.U.of_int 0 1) && Bitops.eq arg1967 (Bits.U.of_int 11 32) && 
          Bitops.eq arg1968 (Bits.U.of_int 19 5) && Bitops.eq arg1963 
            (Bits.U.of_int 0 1) && Bitops.eq arg1964 (Bits.U.of_int 12 32) && 
          Bitops.eq arg1965 (Bits.U.of_int 20 5) && Bitops.eq arg1960 
            (Bits.U.of_int 0 1) && Bitops.eq arg1961 (Bits.U.of_int 13 32) && 
          Bitops.eq arg1962 (Bits.U.of_int 21 5) && Bitops.eq arg1957 
            (Bits.U.of_int 0 1) && Bitops.eq arg1958 (Bits.U.of_int 14 32) && 
          Bitops.eq arg1959 (Bits.U.of_int 22 5) && Bitops.eq arg1954 
            (Bits.U.of_int 0 1) && Bitops.eq arg1955 (Bits.U.of_int 15 32) && 
          Bitops.eq arg1956 (Bits.U.of_int 23 5) && Bitops.eq arg1952 
            (Bits.U.of_int 0 1) && Bitops.eq arg1953 (Bits.U.of_int 24 5) && 
          Bitops.eq arg1949 (Bits.U.of_int 0 1) && Bitops.eq arg1950 
            (Bits.U.of_int 1 32) && Bitops.eq arg1951 (Bits.U.of_int 25 5) && 
          Bitops.eq arg1946 (Bits.U.of_int 0 1) && Bitops.eq arg1947 
            (Bits.U.of_int 2 32) && Bitops.eq arg1948 (Bits.U.of_int 26 5) && 
          Bitops.eq arg1943 (Bits.U.of_int 0 1) && Bitops.eq arg1944 
            (Bits.U.of_int 3 32) && Bitops.eq arg1945 (Bits.U.of_int 27 5) && 
          Bitops.eq arg1940 (Bits.U.of_int 0 1) && Bitops.eq arg1941 
            (Bits.U.of_int 4 32) && Bitops.eq arg1942 (Bits.U.of_int 28 5) && 
          Bitops.eq arg1937 (Bits.U.of_int 0 1) && Bitops.eq arg1938 
            (Bits.U.of_int 5 32) && Bitops.eq arg1939 (Bits.U.of_int 29 5) && 
          Bitops.eq arg1934 (Bits.U.of_int 0 1) && Bitops.eq arg1935 
            (Bits.U.of_int 6 32) && Bitops.eq arg1936 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1931 (Bits.U.of_int 0 1) && Bitops.eq arg1932 
            (Bits.U.of_int 7 32) && Bitops.eq arg1933 (Bits.U.of_int 31 5) && 
          Bitops.eq arg1928 (Bits.U.of_int 0 1) && Bitops.eq arg1929 
            (Bits.U.of_int 0 1) && Bitops.eq arg1930 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 30 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2054), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2055), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2052), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2053), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2050), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2051), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2048), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2049), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2046), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2047), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2044), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2045), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2042), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2043), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2039), _), 32); 
                  RP.Const (RP.Bits arg2040)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2041), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2036), _), 32); 
                  RP.Const (RP.Bits arg2037)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2038), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2033), _), 32); 
                  RP.Const (RP.Bits arg2034)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2035), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2030), _), 32); 
                  RP.Const (RP.Bits arg2031)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2032), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2027), _), 32); 
                  RP.Const (RP.Bits arg2028)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2029), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2024), _), 32); 
                  RP.Const (RP.Bits arg2025)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2026), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2021), _), 32); 
                  RP.Const (RP.Bits arg2022)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2023), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2018), _), 32); 
                  RP.Const (RP.Bits arg2019)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2020), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2016), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg2017), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2013), _), 32); 
                  RP.Const (RP.Bits arg2014)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2015), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2010), _), 32); 
                  RP.Const (RP.Bits arg2011)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2012), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2007), _), 32); 
                  RP.Const (RP.Bits arg2008)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2009), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2004), _), 32); 
                  RP.Const (RP.Bits arg2005)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2006), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2001), _), 32); 
                  RP.Const (RP.Bits arg2002)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2003), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1998), _), 32); 
                  RP.Const (RP.Bits arg1999)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2000), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1995), _), 32); 
                  RP.Const (RP.Bits arg1996)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg1997), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg1992), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg1993), _), 32); 
                  RP.Const (RP.Bits arg1994)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg2054 (Bits.U.of_int 24 5) && Bitops.eq 
            arg2055 (Bits.U.of_int 8 5) && Bitops.eq arg2052 
            (Bits.U.of_int 25 5) && Bitops.eq arg2053 (Bits.U.of_int 9 5) && 
          Bitops.eq arg2050 (Bits.U.of_int 26 5) && Bitops.eq arg2051 
            (Bits.U.of_int 10 5) && Bitops.eq arg2048 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2049 (Bits.U.of_int 11 5) && Bitops.eq arg2046 
            (Bits.U.of_int 28 5) && Bitops.eq arg2047 (Bits.U.of_int 12 5) && 
          Bitops.eq arg2044 (Bits.U.of_int 30 5) && Bitops.eq arg2045 
            (Bits.U.of_int 14 5) && Bitops.eq arg2042 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2043 (Bits.U.of_int 15 5) && Bitops.eq arg2039 
            (Bits.U.of_int 0 1) && Bitops.eq arg2040 (Bits.U.of_int 8 32) && 
          Bitops.eq arg2041 (Bits.U.of_int 16 5) && Bitops.eq arg2036 
            (Bits.U.of_int 0 1) && Bitops.eq arg2037 (Bits.U.of_int 9 32) && 
          Bitops.eq arg2038 (Bits.U.of_int 17 5) && Bitops.eq arg2033 
            (Bits.U.of_int 0 1) && Bitops.eq arg2034 (Bits.U.of_int 10 32) && 
          Bitops.eq arg2035 (Bits.U.of_int 18 5) && Bitops.eq arg2030 
            (Bits.U.of_int 0 1) && Bitops.eq arg2031 (Bits.U.of_int 11 32) && 
          Bitops.eq arg2032 (Bits.U.of_int 19 5) && Bitops.eq arg2027 
            (Bits.U.of_int 0 1) && Bitops.eq arg2028 (Bits.U.of_int 12 32) && 
          Bitops.eq arg2029 (Bits.U.of_int 20 5) && Bitops.eq arg2024 
            (Bits.U.of_int 0 1) && Bitops.eq arg2025 (Bits.U.of_int 13 32) && 
          Bitops.eq arg2026 (Bits.U.of_int 21 5) && Bitops.eq arg2021 
            (Bits.U.of_int 0 1) && Bitops.eq arg2022 (Bits.U.of_int 14 32) && 
          Bitops.eq arg2023 (Bits.U.of_int 22 5) && Bitops.eq arg2018 
            (Bits.U.of_int 0 1) && Bitops.eq arg2019 (Bits.U.of_int 15 32) && 
          Bitops.eq arg2020 (Bits.U.of_int 23 5) && Bitops.eq arg2016 
            (Bits.U.of_int 0 1) && Bitops.eq arg2017 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2013 (Bits.U.of_int 0 1) && Bitops.eq arg2014 
            (Bits.U.of_int 1 32) && Bitops.eq arg2015 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2010 (Bits.U.of_int 0 1) && Bitops.eq arg2011 
            (Bits.U.of_int 2 32) && Bitops.eq arg2012 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2007 (Bits.U.of_int 0 1) && Bitops.eq arg2008 
            (Bits.U.of_int 3 32) && Bitops.eq arg2009 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2004 (Bits.U.of_int 0 1) && Bitops.eq arg2005 
            (Bits.U.of_int 4 32) && Bitops.eq arg2006 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2001 (Bits.U.of_int 0 1) && Bitops.eq arg2002 
            (Bits.U.of_int 5 32) && Bitops.eq arg2003 (Bits.U.of_int 29 5) && 
          Bitops.eq arg1998 (Bits.U.of_int 0 1) && Bitops.eq arg1999 
            (Bits.U.of_int 6 32) && Bitops.eq arg2000 (Bits.U.of_int 30 5) && 
          Bitops.eq arg1995 (Bits.U.of_int 0 1) && Bitops.eq arg1996 
            (Bits.U.of_int 7 32) && Bitops.eq arg1997 (Bits.U.of_int 31 5) && 
          Bitops.eq arg1992 (Bits.U.of_int 0 1) && Bitops.eq arg1993 
            (Bits.U.of_int 0 1) && Bitops.eq arg1994 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 29 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2118), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2119), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2116), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2117), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2114), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2115), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2112), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2113), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2110), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2111), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2108), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2109), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2106), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2107), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2103), _), 32); 
                  RP.Const (RP.Bits arg2104)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2105), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2100), _), 32); 
                  RP.Const (RP.Bits arg2101)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2102), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2097), _), 32); 
                  RP.Const (RP.Bits arg2098)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2099), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2094), _), 32); 
                  RP.Const (RP.Bits arg2095)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2096), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2091), _), 32); 
                  RP.Const (RP.Bits arg2092)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2093), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2088), _), 32); 
                  RP.Const (RP.Bits arg2089)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2090), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2085), _), 32); 
                  RP.Const (RP.Bits arg2086)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2087), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2082), _), 32); 
                  RP.Const (RP.Bits arg2083)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2084), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2080), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg2081), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2077), _), 32); 
                  RP.Const (RP.Bits arg2078)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2079), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2074), _), 32); 
                  RP.Const (RP.Bits arg2075)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2076), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2071), _), 32); 
                  RP.Const (RP.Bits arg2072)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2073), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2068), _), 32); 
                  RP.Const (RP.Bits arg2069)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2070), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2065), _), 32); 
                  RP.Const (RP.Bits arg2066)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2067), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2062), _), 32); 
                  RP.Const (RP.Bits arg2063)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2064), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2059), _), 32); 
                  RP.Const (RP.Bits arg2060)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2061), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2056), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2057), _), 32); 
                  RP.Const (RP.Bits arg2058)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg2118 (Bits.U.of_int 24 5) && Bitops.eq 
            arg2119 (Bits.U.of_int 8 5) && Bitops.eq arg2116 
            (Bits.U.of_int 25 5) && Bitops.eq arg2117 (Bits.U.of_int 9 5) && 
          Bitops.eq arg2114 (Bits.U.of_int 26 5) && Bitops.eq arg2115 
            (Bits.U.of_int 10 5) && Bitops.eq arg2112 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2113 (Bits.U.of_int 11 5) && Bitops.eq arg2110 
            (Bits.U.of_int 29 5) && Bitops.eq arg2111 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2108 (Bits.U.of_int 30 5) && Bitops.eq arg2109 
            (Bits.U.of_int 14 5) && Bitops.eq arg2106 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2107 (Bits.U.of_int 15 5) && Bitops.eq arg2103 
            (Bits.U.of_int 0 1) && Bitops.eq arg2104 (Bits.U.of_int 8 32) && 
          Bitops.eq arg2105 (Bits.U.of_int 16 5) && Bitops.eq arg2100 
            (Bits.U.of_int 0 1) && Bitops.eq arg2101 (Bits.U.of_int 9 32) && 
          Bitops.eq arg2102 (Bits.U.of_int 17 5) && Bitops.eq arg2097 
            (Bits.U.of_int 0 1) && Bitops.eq arg2098 (Bits.U.of_int 10 32) && 
          Bitops.eq arg2099 (Bits.U.of_int 18 5) && Bitops.eq arg2094 
            (Bits.U.of_int 0 1) && Bitops.eq arg2095 (Bits.U.of_int 11 32) && 
          Bitops.eq arg2096 (Bits.U.of_int 19 5) && Bitops.eq arg2091 
            (Bits.U.of_int 0 1) && Bitops.eq arg2092 (Bits.U.of_int 12 32) && 
          Bitops.eq arg2093 (Bits.U.of_int 20 5) && Bitops.eq arg2088 
            (Bits.U.of_int 0 1) && Bitops.eq arg2089 (Bits.U.of_int 13 32) && 
          Bitops.eq arg2090 (Bits.U.of_int 21 5) && Bitops.eq arg2085 
            (Bits.U.of_int 0 1) && Bitops.eq arg2086 (Bits.U.of_int 14 32) && 
          Bitops.eq arg2087 (Bits.U.of_int 22 5) && Bitops.eq arg2082 
            (Bits.U.of_int 0 1) && Bitops.eq arg2083 (Bits.U.of_int 15 32) && 
          Bitops.eq arg2084 (Bits.U.of_int 23 5) && Bitops.eq arg2080 
            (Bits.U.of_int 0 1) && Bitops.eq arg2081 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2077 (Bits.U.of_int 0 1) && Bitops.eq arg2078 
            (Bits.U.of_int 1 32) && Bitops.eq arg2079 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2074 (Bits.U.of_int 0 1) && Bitops.eq arg2075 
            (Bits.U.of_int 2 32) && Bitops.eq arg2076 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2071 (Bits.U.of_int 0 1) && Bitops.eq arg2072 
            (Bits.U.of_int 3 32) && Bitops.eq arg2073 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2068 (Bits.U.of_int 0 1) && Bitops.eq arg2069 
            (Bits.U.of_int 4 32) && Bitops.eq arg2070 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2065 (Bits.U.of_int 0 1) && Bitops.eq arg2066 
            (Bits.U.of_int 5 32) && Bitops.eq arg2067 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2062 (Bits.U.of_int 0 1) && Bitops.eq arg2063 
            (Bits.U.of_int 6 32) && Bitops.eq arg2064 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2059 (Bits.U.of_int 0 1) && Bitops.eq arg2060 
            (Bits.U.of_int 7 32) && Bitops.eq arg2061 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2056 (Bits.U.of_int 0 1) && Bitops.eq arg2057 
            (Bits.U.of_int 0 1) && Bitops.eq arg2058 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 28 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2182), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2183), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2180), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2181), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2178), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2179), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2176), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2177), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2174), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2175), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2172), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2173), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2170), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2171), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2167), _), 32); 
                  RP.Const (RP.Bits arg2168)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2169), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2164), _), 32); 
                  RP.Const (RP.Bits arg2165)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2166), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2161), _), 32); 
                  RP.Const (RP.Bits arg2162)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2163), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2158), _), 32); 
                  RP.Const (RP.Bits arg2159)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2160), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2155), _), 32); 
                  RP.Const (RP.Bits arg2156)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2157), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2152), _), 32); 
                  RP.Const (RP.Bits arg2153)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2154), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2149), _), 32); 
                  RP.Const (RP.Bits arg2150)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2151), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2146), _), 32); 
                  RP.Const (RP.Bits arg2147)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2148), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2144), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg2145), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2141), _), 32); 
                  RP.Const (RP.Bits arg2142)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2143), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2138), _), 32); 
                  RP.Const (RP.Bits arg2139)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2140), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2135), _), 32); 
                  RP.Const (RP.Bits arg2136)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2137), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2132), _), 32); 
                  RP.Const (RP.Bits arg2133)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2134), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2129), _), 32); 
                  RP.Const (RP.Bits arg2130)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2131), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2126), _), 32); 
                  RP.Const (RP.Bits arg2127)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2128), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2123), _), 32); 
                  RP.Const (RP.Bits arg2124)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2125), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2120), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2121), _), 32); 
                  RP.Const (RP.Bits arg2122)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg2182 (Bits.U.of_int 24 5) && Bitops.eq 
            arg2183 (Bits.U.of_int 8 5) && Bitops.eq arg2180 
            (Bits.U.of_int 25 5) && Bitops.eq arg2181 (Bits.U.of_int 9 5) && 
          Bitops.eq arg2178 (Bits.U.of_int 26 5) && Bitops.eq arg2179 
            (Bits.U.of_int 10 5) && Bitops.eq arg2176 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2177 (Bits.U.of_int 12 5) && Bitops.eq arg2174 
            (Bits.U.of_int 29 5) && Bitops.eq arg2175 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2172 (Bits.U.of_int 30 5) && Bitops.eq arg2173 
            (Bits.U.of_int 14 5) && Bitops.eq arg2170 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2171 (Bits.U.of_int 15 5) && Bitops.eq arg2167 
            (Bits.U.of_int 0 1) && Bitops.eq arg2168 (Bits.U.of_int 8 32) && 
          Bitops.eq arg2169 (Bits.U.of_int 16 5) && Bitops.eq arg2164 
            (Bits.U.of_int 0 1) && Bitops.eq arg2165 (Bits.U.of_int 9 32) && 
          Bitops.eq arg2166 (Bits.U.of_int 17 5) && Bitops.eq arg2161 
            (Bits.U.of_int 0 1) && Bitops.eq arg2162 (Bits.U.of_int 10 32) && 
          Bitops.eq arg2163 (Bits.U.of_int 18 5) && Bitops.eq arg2158 
            (Bits.U.of_int 0 1) && Bitops.eq arg2159 (Bits.U.of_int 11 32) && 
          Bitops.eq arg2160 (Bits.U.of_int 19 5) && Bitops.eq arg2155 
            (Bits.U.of_int 0 1) && Bitops.eq arg2156 (Bits.U.of_int 12 32) && 
          Bitops.eq arg2157 (Bits.U.of_int 20 5) && Bitops.eq arg2152 
            (Bits.U.of_int 0 1) && Bitops.eq arg2153 (Bits.U.of_int 13 32) && 
          Bitops.eq arg2154 (Bits.U.of_int 21 5) && Bitops.eq arg2149 
            (Bits.U.of_int 0 1) && Bitops.eq arg2150 (Bits.U.of_int 14 32) && 
          Bitops.eq arg2151 (Bits.U.of_int 22 5) && Bitops.eq arg2146 
            (Bits.U.of_int 0 1) && Bitops.eq arg2147 (Bits.U.of_int 15 32) && 
          Bitops.eq arg2148 (Bits.U.of_int 23 5) && Bitops.eq arg2144 
            (Bits.U.of_int 0 1) && Bitops.eq arg2145 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2141 (Bits.U.of_int 0 1) && Bitops.eq arg2142 
            (Bits.U.of_int 1 32) && Bitops.eq arg2143 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2138 (Bits.U.of_int 0 1) && Bitops.eq arg2139 
            (Bits.U.of_int 2 32) && Bitops.eq arg2140 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2135 (Bits.U.of_int 0 1) && Bitops.eq arg2136 
            (Bits.U.of_int 3 32) && Bitops.eq arg2137 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2132 (Bits.U.of_int 0 1) && Bitops.eq arg2133 
            (Bits.U.of_int 4 32) && Bitops.eq arg2134 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2129 (Bits.U.of_int 0 1) && Bitops.eq arg2130 
            (Bits.U.of_int 5 32) && Bitops.eq arg2131 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2126 (Bits.U.of_int 0 1) && Bitops.eq arg2127 
            (Bits.U.of_int 6 32) && Bitops.eq arg2128 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2123 (Bits.U.of_int 0 1) && Bitops.eq arg2124 
            (Bits.U.of_int 7 32) && Bitops.eq arg2125 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2120 (Bits.U.of_int 0 1) && Bitops.eq arg2121 
            (Bits.U.of_int 0 1) && Bitops.eq arg2122 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 27 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2246), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2247), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2244), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2245), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2242), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2243), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2240), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2241), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2238), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2239), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2236), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2237), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2234), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2235), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2231), _), 32); 
                  RP.Const (RP.Bits arg2232)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2233), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2228), _), 32); 
                  RP.Const (RP.Bits arg2229)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2230), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2225), _), 32); 
                  RP.Const (RP.Bits arg2226)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2227), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2222), _), 32); 
                  RP.Const (RP.Bits arg2223)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2224), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2219), _), 32); 
                  RP.Const (RP.Bits arg2220)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2221), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2216), _), 32); 
                  RP.Const (RP.Bits arg2217)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2218), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2213), _), 32); 
                  RP.Const (RP.Bits arg2214)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2215), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2210), _), 32); 
                  RP.Const (RP.Bits arg2211)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2212), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2208), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg2209), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2205), _), 32); 
                  RP.Const (RP.Bits arg2206)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2207), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2202), _), 32); 
                  RP.Const (RP.Bits arg2203)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2204), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2199), _), 32); 
                  RP.Const (RP.Bits arg2200)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2201), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2196), _), 32); 
                  RP.Const (RP.Bits arg2197)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2198), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2193), _), 32); 
                  RP.Const (RP.Bits arg2194)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2195), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2190), _), 32); 
                  RP.Const (RP.Bits arg2191)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2192), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2187), _), 32); 
                  RP.Const (RP.Bits arg2188)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2189), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2184), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2185), _), 32); 
                  RP.Const (RP.Bits arg2186)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg2246 (Bits.U.of_int 24 5) && Bitops.eq 
            arg2247 (Bits.U.of_int 8 5) && Bitops.eq arg2244 
            (Bits.U.of_int 25 5) && Bitops.eq arg2245 (Bits.U.of_int 9 5) && 
          Bitops.eq arg2242 (Bits.U.of_int 27 5) && Bitops.eq arg2243 
            (Bits.U.of_int 11 5) && Bitops.eq arg2240 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2241 (Bits.U.of_int 12 5) && Bitops.eq arg2238 
            (Bits.U.of_int 29 5) && Bitops.eq arg2239 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2236 (Bits.U.of_int 30 5) && Bitops.eq arg2237 
            (Bits.U.of_int 14 5) && Bitops.eq arg2234 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2235 (Bits.U.of_int 15 5) && Bitops.eq arg2231 
            (Bits.U.of_int 0 1) && Bitops.eq arg2232 (Bits.U.of_int 8 32) && 
          Bitops.eq arg2233 (Bits.U.of_int 16 5) && Bitops.eq arg2228 
            (Bits.U.of_int 0 1) && Bitops.eq arg2229 (Bits.U.of_int 9 32) && 
          Bitops.eq arg2230 (Bits.U.of_int 17 5) && Bitops.eq arg2225 
            (Bits.U.of_int 0 1) && Bitops.eq arg2226 (Bits.U.of_int 10 32) && 
          Bitops.eq arg2227 (Bits.U.of_int 18 5) && Bitops.eq arg2222 
            (Bits.U.of_int 0 1) && Bitops.eq arg2223 (Bits.U.of_int 11 32) && 
          Bitops.eq arg2224 (Bits.U.of_int 19 5) && Bitops.eq arg2219 
            (Bits.U.of_int 0 1) && Bitops.eq arg2220 (Bits.U.of_int 12 32) && 
          Bitops.eq arg2221 (Bits.U.of_int 20 5) && Bitops.eq arg2216 
            (Bits.U.of_int 0 1) && Bitops.eq arg2217 (Bits.U.of_int 13 32) && 
          Bitops.eq arg2218 (Bits.U.of_int 21 5) && Bitops.eq arg2213 
            (Bits.U.of_int 0 1) && Bitops.eq arg2214 (Bits.U.of_int 14 32) && 
          Bitops.eq arg2215 (Bits.U.of_int 22 5) && Bitops.eq arg2210 
            (Bits.U.of_int 0 1) && Bitops.eq arg2211 (Bits.U.of_int 15 32) && 
          Bitops.eq arg2212 (Bits.U.of_int 23 5) && Bitops.eq arg2208 
            (Bits.U.of_int 0 1) && Bitops.eq arg2209 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2205 (Bits.U.of_int 0 1) && Bitops.eq arg2206 
            (Bits.U.of_int 1 32) && Bitops.eq arg2207 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2202 (Bits.U.of_int 0 1) && Bitops.eq arg2203 
            (Bits.U.of_int 2 32) && Bitops.eq arg2204 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2199 (Bits.U.of_int 0 1) && Bitops.eq arg2200 
            (Bits.U.of_int 3 32) && Bitops.eq arg2201 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2196 (Bits.U.of_int 0 1) && Bitops.eq arg2197 
            (Bits.U.of_int 4 32) && Bitops.eq arg2198 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2193 (Bits.U.of_int 0 1) && Bitops.eq arg2194 
            (Bits.U.of_int 5 32) && Bitops.eq arg2195 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2190 (Bits.U.of_int 0 1) && Bitops.eq arg2191 
            (Bits.U.of_int 6 32) && Bitops.eq arg2192 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2187 (Bits.U.of_int 0 1) && Bitops.eq arg2188 
            (Bits.U.of_int 7 32) && Bitops.eq arg2189 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2184 (Bits.U.of_int 0 1) && Bitops.eq arg2185 
            (Bits.U.of_int 0 1) && Bitops.eq arg2186 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 26 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2310), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2311), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2308), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2309), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2306), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2307), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2304), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2305), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2302), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2303), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2300), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2301), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2298), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2299), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2295), _), 32); 
                  RP.Const (RP.Bits arg2296)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2297), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2292), _), 32); 
                  RP.Const (RP.Bits arg2293)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2294), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2289), _), 32); 
                  RP.Const (RP.Bits arg2290)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2291), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2286), _), 32); 
                  RP.Const (RP.Bits arg2287)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2288), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2283), _), 32); 
                  RP.Const (RP.Bits arg2284)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2285), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2280), _), 32); 
                  RP.Const (RP.Bits arg2281)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2282), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2277), _), 32); 
                  RP.Const (RP.Bits arg2278)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2279), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2274), _), 32); 
                  RP.Const (RP.Bits arg2275)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2276), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2272), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg2273), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2269), _), 32); 
                  RP.Const (RP.Bits arg2270)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2271), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2266), _), 32); 
                  RP.Const (RP.Bits arg2267)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2268), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2263), _), 32); 
                  RP.Const (RP.Bits arg2264)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2265), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2260), _), 32); 
                  RP.Const (RP.Bits arg2261)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2262), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2257), _), 32); 
                  RP.Const (RP.Bits arg2258)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2259), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2254), _), 32); 
                  RP.Const (RP.Bits arg2255)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2256), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2251), _), 32); 
                  RP.Const (RP.Bits arg2252)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2253), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2248), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2249), _), 32); 
                  RP.Const (RP.Bits arg2250)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg2310 (Bits.U.of_int 24 5) && Bitops.eq 
            arg2311 (Bits.U.of_int 8 5) && Bitops.eq arg2308 
            (Bits.U.of_int 26 5) && Bitops.eq arg2309 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2306 (Bits.U.of_int 27 5) && Bitops.eq arg2307 
            (Bits.U.of_int 11 5) && Bitops.eq arg2304 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2305 (Bits.U.of_int 12 5) && Bitops.eq arg2302 
            (Bits.U.of_int 29 5) && Bitops.eq arg2303 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2300 (Bits.U.of_int 30 5) && Bitops.eq arg2301 
            (Bits.U.of_int 14 5) && Bitops.eq arg2298 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2299 (Bits.U.of_int 15 5) && Bitops.eq arg2295 
            (Bits.U.of_int 0 1) && Bitops.eq arg2296 (Bits.U.of_int 8 32) && 
          Bitops.eq arg2297 (Bits.U.of_int 16 5) && Bitops.eq arg2292 
            (Bits.U.of_int 0 1) && Bitops.eq arg2293 (Bits.U.of_int 9 32) && 
          Bitops.eq arg2294 (Bits.U.of_int 17 5) && Bitops.eq arg2289 
            (Bits.U.of_int 0 1) && Bitops.eq arg2290 (Bits.U.of_int 10 32) && 
          Bitops.eq arg2291 (Bits.U.of_int 18 5) && Bitops.eq arg2286 
            (Bits.U.of_int 0 1) && Bitops.eq arg2287 (Bits.U.of_int 11 32) && 
          Bitops.eq arg2288 (Bits.U.of_int 19 5) && Bitops.eq arg2283 
            (Bits.U.of_int 0 1) && Bitops.eq arg2284 (Bits.U.of_int 12 32) && 
          Bitops.eq arg2285 (Bits.U.of_int 20 5) && Bitops.eq arg2280 
            (Bits.U.of_int 0 1) && Bitops.eq arg2281 (Bits.U.of_int 13 32) && 
          Bitops.eq arg2282 (Bits.U.of_int 21 5) && Bitops.eq arg2277 
            (Bits.U.of_int 0 1) && Bitops.eq arg2278 (Bits.U.of_int 14 32) && 
          Bitops.eq arg2279 (Bits.U.of_int 22 5) && Bitops.eq arg2274 
            (Bits.U.of_int 0 1) && Bitops.eq arg2275 (Bits.U.of_int 15 32) && 
          Bitops.eq arg2276 (Bits.U.of_int 23 5) && Bitops.eq arg2272 
            (Bits.U.of_int 0 1) && Bitops.eq arg2273 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2269 (Bits.U.of_int 0 1) && Bitops.eq arg2270 
            (Bits.U.of_int 1 32) && Bitops.eq arg2271 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2266 (Bits.U.of_int 0 1) && Bitops.eq arg2267 
            (Bits.U.of_int 2 32) && Bitops.eq arg2268 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2263 (Bits.U.of_int 0 1) && Bitops.eq arg2264 
            (Bits.U.of_int 3 32) && Bitops.eq arg2265 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2260 (Bits.U.of_int 0 1) && Bitops.eq arg2261 
            (Bits.U.of_int 4 32) && Bitops.eq arg2262 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2257 (Bits.U.of_int 0 1) && Bitops.eq arg2258 
            (Bits.U.of_int 5 32) && Bitops.eq arg2259 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2254 (Bits.U.of_int 0 1) && Bitops.eq arg2255 
            (Bits.U.of_int 6 32) && Bitops.eq arg2256 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2251 (Bits.U.of_int 0 1) && Bitops.eq arg2252 
            (Bits.U.of_int 7 32) && Bitops.eq arg2253 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2248 (Bits.U.of_int 0 1) && Bitops.eq arg2249 
            (Bits.U.of_int 0 1) && Bitops.eq arg2250 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 25 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2374), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2375), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2372), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2373), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2370), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2371), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2368), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2369), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2366), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2367), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2364), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2365), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2362), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2363), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2359), _), 32); 
                  RP.Const (RP.Bits arg2360)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2361), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2356), _), 32); 
                  RP.Const (RP.Bits arg2357)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2358), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2353), _), 32); 
                  RP.Const (RP.Bits arg2354)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2355), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2350), _), 32); 
                  RP.Const (RP.Bits arg2351)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2352), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2347), _), 32); 
                  RP.Const (RP.Bits arg2348)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2349), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2344), _), 32); 
                  RP.Const (RP.Bits arg2345)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2346), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2341), _), 32); 
                  RP.Const (RP.Bits arg2342)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2343), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2338), _), 32); 
                  RP.Const (RP.Bits arg2339)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2340), _), 32), 32)); 
          (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('w', Rtl.Identity, 32, RP.Fetch (RP.Cell ('W', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2336), _), 32), _), 
              RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg2337), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2333), _), 32); 
                  RP.Const (RP.Bits arg2334)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2335), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2330), _), 32); 
                  RP.Const (RP.Bits arg2331)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2332), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2327), _), 32); 
                  RP.Const (RP.Bits arg2328)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2329), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2324), _), 32); 
                  RP.Const (RP.Bits arg2325)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2326), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2321), _), 32); 
                  RP.Const (RP.Bits arg2322)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2323), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2318), _), 32); 
                  RP.Const (RP.Bits arg2319)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2320), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('w', Rtl.Identity, 32, 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2315), _), 32); 
                  RP.Const (RP.Bits arg2316)]), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2317), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2312), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2313), _), 32); 
                  RP.Const (RP.Bits arg2314)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg2374 (Bits.U.of_int 25 5) && Bitops.eq 
            arg2375 (Bits.U.of_int 9 5) && Bitops.eq arg2372 
            (Bits.U.of_int 26 5) && Bitops.eq arg2373 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2370 (Bits.U.of_int 27 5) && Bitops.eq arg2371 
            (Bits.U.of_int 11 5) && Bitops.eq arg2368 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2369 (Bits.U.of_int 12 5) && Bitops.eq arg2366 
            (Bits.U.of_int 29 5) && Bitops.eq arg2367 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2364 (Bits.U.of_int 30 5) && Bitops.eq arg2365 
            (Bits.U.of_int 14 5) && Bitops.eq arg2362 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2363 (Bits.U.of_int 15 5) && Bitops.eq arg2359 
            (Bits.U.of_int 0 1) && Bitops.eq arg2360 (Bits.U.of_int 8 32) && 
          Bitops.eq arg2361 (Bits.U.of_int 16 5) && Bitops.eq arg2356 
            (Bits.U.of_int 0 1) && Bitops.eq arg2357 (Bits.U.of_int 9 32) && 
          Bitops.eq arg2358 (Bits.U.of_int 17 5) && Bitops.eq arg2353 
            (Bits.U.of_int 0 1) && Bitops.eq arg2354 (Bits.U.of_int 10 32) && 
          Bitops.eq arg2355 (Bits.U.of_int 18 5) && Bitops.eq arg2350 
            (Bits.U.of_int 0 1) && Bitops.eq arg2351 (Bits.U.of_int 11 32) && 
          Bitops.eq arg2352 (Bits.U.of_int 19 5) && Bitops.eq arg2347 
            (Bits.U.of_int 0 1) && Bitops.eq arg2348 (Bits.U.of_int 12 32) && 
          Bitops.eq arg2349 (Bits.U.of_int 20 5) && Bitops.eq arg2344 
            (Bits.U.of_int 0 1) && Bitops.eq arg2345 (Bits.U.of_int 13 32) && 
          Bitops.eq arg2346 (Bits.U.of_int 21 5) && Bitops.eq arg2341 
            (Bits.U.of_int 0 1) && Bitops.eq arg2342 (Bits.U.of_int 14 32) && 
          Bitops.eq arg2343 (Bits.U.of_int 22 5) && Bitops.eq arg2338 
            (Bits.U.of_int 0 1) && Bitops.eq arg2339 (Bits.U.of_int 15 32) && 
          Bitops.eq arg2340 (Bits.U.of_int 23 5) && Bitops.eq arg2336 
            (Bits.U.of_int 0 1) && Bitops.eq arg2337 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2333 (Bits.U.of_int 0 1) && Bitops.eq arg2334 
            (Bits.U.of_int 1 32) && Bitops.eq arg2335 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2330 (Bits.U.of_int 0 1) && Bitops.eq arg2331 
            (Bits.U.of_int 2 32) && Bitops.eq arg2332 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2327 (Bits.U.of_int 0 1) && Bitops.eq arg2328 
            (Bits.U.of_int 3 32) && Bitops.eq arg2329 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2324 (Bits.U.of_int 0 1) && Bitops.eq arg2325 
            (Bits.U.of_int 4 32) && Bitops.eq arg2326 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2321 (Bits.U.of_int 0 1) && Bitops.eq arg2322 
            (Bits.U.of_int 5 32) && Bitops.eq arg2323 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2318 (Bits.U.of_int 0 1) && Bitops.eq arg2319 
            (Bits.U.of_int 6 32) && Bitops.eq arg2320 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2315 (Bits.U.of_int 0 1) && Bitops.eq arg2316 
            (Bits.U.of_int 7 32) && Bitops.eq arg2317 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2312 (Bits.U.of_int 0 1) && Bitops.eq arg2313 
            (Bits.U.of_int 0 1) && Bitops.eq arg2314 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 24 5)) -> 
           Instruction.save (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2442), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2443), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2440), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2441), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2438), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2439), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2436), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2437), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2434), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2435), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2432), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2433), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2430), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2431), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2428), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2429), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2425), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2426), _), 32); 
                    RP.Const (RP.Bits arg2427)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2422), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2423), _), 32); 
                    RP.Const (RP.Bits arg2424)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2419), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2420), _), 32); 
                    RP.Const (RP.Bits arg2421)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2416), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2417), _), 32); 
                    RP.Const (RP.Bits arg2418)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2413), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2414), _), 32); 
                    RP.Const (RP.Bits arg2415)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2410), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2411), _), 32); 
                    RP.Const (RP.Bits arg2412)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2407), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2408), _), 32); 
                    RP.Const (RP.Bits arg2409)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2404), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2405), _), 32); 
                    RP.Const (RP.Bits arg2406)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2401), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2402), _), 32); 
                    RP.Const (RP.Bits arg2403)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2398), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2399), _), 32); 
                    RP.Const (RP.Bits arg2400)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2395), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2396), _), 32); 
                    RP.Const (RP.Bits arg2397)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2392), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2393), _), 32); 
                    RP.Const (RP.Bits arg2394)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2389), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2390), _), 32); 
                    RP.Const (RP.Bits arg2391)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2386), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2387), _), 32); 
                    RP.Const (RP.Bits arg2388)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2383), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2384), _), 32); 
                    RP.Const (RP.Bits arg2385)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2380), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2381), _), 32); 
                    RP.Const (RP.Bits arg2382)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2377), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2378), _), 32); 
                  RP.Const (RP.Bits arg2379)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2376)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2376 13 && Bitops.eq arg2442 
            (Bits.U.of_int 8 5) && Bitops.eq arg2443 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2440 (Bits.U.of_int 9 5) && Bitops.eq arg2441 
            (Bits.U.of_int 25 5) && Bitops.eq arg2438 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2439 (Bits.U.of_int 26 5) && Bitops.eq arg2436 
            (Bits.U.of_int 11 5) && Bitops.eq arg2437 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2434 (Bits.U.of_int 12 5) && Bitops.eq arg2435 
            (Bits.U.of_int 28 5) && Bitops.eq arg2432 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2433 (Bits.U.of_int 29 5) && Bitops.eq arg2430 
            (Bits.U.of_int 14 5) && Bitops.eq arg2431 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2428 (Bits.U.of_int 15 5) && Bitops.eq arg2429 
            (Bits.U.of_int 31 5) && Bitops.eq arg2425 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2426 (Bits.U.of_int 0 1) && Bitops.eq arg2427 
            (Bits.U.of_int 8 32) && Bitops.eq arg2422 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2423 (Bits.U.of_int 0 1) && Bitops.eq arg2424 
            (Bits.U.of_int 7 32) && Bitops.eq arg2419 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2420 (Bits.U.of_int 0 1) && Bitops.eq arg2421 
            (Bits.U.of_int 6 32) && Bitops.eq arg2416 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2417 (Bits.U.of_int 0 1) && Bitops.eq arg2418 
            (Bits.U.of_int 5 32) && Bitops.eq arg2413 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2414 (Bits.U.of_int 0 1) && Bitops.eq arg2415 
            (Bits.U.of_int 4 32) && Bitops.eq arg2410 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2411 (Bits.U.of_int 0 1) && Bitops.eq arg2412 
            (Bits.U.of_int 3 32) && Bitops.eq arg2407 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2408 (Bits.U.of_int 0 1) && Bitops.eq arg2409 
            (Bits.U.of_int 2 32) && Bitops.eq arg2404 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2405 (Bits.U.of_int 0 1) && Bitops.eq arg2406 
            (Bits.U.of_int 1 32) && Bitops.eq arg2401 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2402 (Bits.U.of_int 0 1) && Bitops.eq arg2403 
            (Bits.U.of_int 16 32) && Bitops.eq arg2398 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2399 (Bits.U.of_int 0 1) && Bitops.eq arg2400 
            (Bits.U.of_int 15 32) && Bitops.eq arg2395 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2396 (Bits.U.of_int 0 1) && Bitops.eq arg2397 
            (Bits.U.of_int 14 32) && Bitops.eq arg2392 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2393 (Bits.U.of_int 0 1) && Bitops.eq arg2394 
            (Bits.U.of_int 13 32) && Bitops.eq arg2389 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2390 (Bits.U.of_int 0 1) && Bitops.eq arg2391 
            (Bits.U.of_int 12 32) && Bitops.eq arg2386 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2387 (Bits.U.of_int 0 1) && Bitops.eq arg2388 
            (Bits.U.of_int 11 32) && Bitops.eq arg2383 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2384 (Bits.U.of_int 0 1) && Bitops.eq arg2385 
            (Bits.U.of_int 10 32) && Bitops.eq arg2380 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2381 (Bits.U.of_int 0 1) && Bitops.eq arg2382 
            (Bits.U.of_int 9 32) && Bitops.eq arg2377 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2378 (Bits.U.of_int 0 1) && Bitops.eq arg2379 
            (Bits.U.of_int 16 32) && Bitops.ne rd (Bits.U.of_int 8 5) && Bitops.ne 
            rd (Bits.U.of_int 9 5) && Bitops.ne rd (Bits.U.of_int 10 5) && 
          Bitops.ne rd (Bits.U.of_int 11 5) && Bitops.ne rd (Bits.U.of_int 12 5) && 
          Bitops.ne rd (Bits.U.of_int 13 5) && Bitops.ne rd (Bits.U.of_int 14 5) && 
          Bitops.ne rd (Bits.U.of_int 15 5) && Bitops.ne rd (Bits.U.of_int 16 5) && 
          Bitops.ne rd (Bits.U.of_int 17 5) && Bitops.ne rd (Bits.U.of_int 18 5) && 
          Bitops.ne rd (Bits.U.of_int 19 5) && Bitops.ne rd (Bits.U.of_int 20 5) && 
          Bitops.ne rd (Bits.U.of_int 21 5) && Bitops.ne rd (Bits.U.of_int 22 5) && 
          Bitops.ne rd (Bits.U.of_int 23 5) && Bitops.ne rd (Bits.U.of_int 24 5) && 
          Bitops.ne rd (Bits.U.of_int 25 5) && Bitops.ne rd (Bits.U.of_int 26 5) && 
          Bitops.ne rd (Bits.U.of_int 27 5) && Bitops.ne rd (Bits.U.of_int 28 5) && 
          Bitops.ne rd (Bits.U.of_int 29 5) && Bitops.ne rd (Bits.U.of_int 30 5) && 
          Bitops.ne rd (Bits.U.of_int 31 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2376)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2507), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2508), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2505), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2506), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2503), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2504), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2501), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2502), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2499), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2500), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2497), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2498), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2495), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2496), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2493), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2494), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2490), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2491), _), 32); 
                    RP.Const (RP.Bits arg2492)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2487), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2488), _), 32); 
                    RP.Const (RP.Bits arg2489)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2484), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2485), _), 32); 
                    RP.Const (RP.Bits arg2486)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2481), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2482), _), 32); 
                    RP.Const (RP.Bits arg2483)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2478), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2479), _), 32); 
                    RP.Const (RP.Bits arg2480)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2475), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2476), _), 32); 
                    RP.Const (RP.Bits arg2477)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2472), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2473), _), 32); 
                    RP.Const (RP.Bits arg2474)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2469), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2470), _), 32); 
                    RP.Const (RP.Bits arg2471)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2466), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2467), _), 32); 
                    RP.Const (RP.Bits arg2468)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2463), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2464), _), 32); 
                    RP.Const (RP.Bits arg2465)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2460), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2461), _), 32); 
                    RP.Const (RP.Bits arg2462)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2457), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2458), _), 32); 
                    RP.Const (RP.Bits arg2459)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2454), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2455), _), 32); 
                    RP.Const (RP.Bits arg2456)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2451), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2452), _), 32); 
                    RP.Const (RP.Bits arg2453)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2448), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2449), _), 32); 
                    RP.Const (RP.Bits arg2450)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2445), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2446), _), 32); 
                  RP.Const (RP.Bits arg2447)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2444)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2444 13 && Bitops.eq arg2507 
            (Bits.U.of_int 8 5) && Bitops.eq arg2508 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2505 (Bits.U.of_int 9 5) && Bitops.eq arg2506 
            (Bits.U.of_int 25 5) && Bitops.eq arg2503 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2504 (Bits.U.of_int 26 5) && Bitops.eq arg2501 
            (Bits.U.of_int 11 5) && Bitops.eq arg2502 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2499 (Bits.U.of_int 12 5) && Bitops.eq arg2500 
            (Bits.U.of_int 28 5) && Bitops.eq arg2497 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2498 (Bits.U.of_int 29 5) && Bitops.eq arg2495 
            (Bits.U.of_int 14 5) && Bitops.eq arg2496 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2493 (Bits.U.of_int 15 5) && Bitops.eq arg2494 
            (Bits.U.of_int 31 5) && Bitops.eq arg2490 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2491 (Bits.U.of_int 0 1) && Bitops.eq arg2492 
            (Bits.U.of_int 8 32) && Bitops.eq arg2487 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2488 (Bits.U.of_int 0 1) && Bitops.eq arg2489 
            (Bits.U.of_int 7 32) && Bitops.eq arg2484 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2485 (Bits.U.of_int 0 1) && Bitops.eq arg2486 
            (Bits.U.of_int 6 32) && Bitops.eq arg2481 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2482 (Bits.U.of_int 0 1) && Bitops.eq arg2483 
            (Bits.U.of_int 5 32) && Bitops.eq arg2478 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2479 (Bits.U.of_int 0 1) && Bitops.eq arg2480 
            (Bits.U.of_int 4 32) && Bitops.eq arg2475 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2476 (Bits.U.of_int 0 1) && Bitops.eq arg2477 
            (Bits.U.of_int 3 32) && Bitops.eq arg2472 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2473 (Bits.U.of_int 0 1) && Bitops.eq arg2474 
            (Bits.U.of_int 2 32) && Bitops.eq arg2469 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2470 (Bits.U.of_int 0 1) && Bitops.eq arg2471 
            (Bits.U.of_int 1 32) && Bitops.eq arg2466 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2467 (Bits.U.of_int 0 1) && Bitops.eq arg2468 
            (Bits.U.of_int 16 32) && Bitops.eq arg2463 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2464 (Bits.U.of_int 0 1) && Bitops.eq arg2465 
            (Bits.U.of_int 15 32) && Bitops.eq arg2460 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2461 (Bits.U.of_int 0 1) && Bitops.eq arg2462 
            (Bits.U.of_int 14 32) && Bitops.eq arg2457 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2458 (Bits.U.of_int 0 1) && Bitops.eq arg2459 
            (Bits.U.of_int 13 32) && Bitops.eq arg2454 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2455 (Bits.U.of_int 0 1) && Bitops.eq arg2456 
            (Bits.U.of_int 12 32) && Bitops.eq arg2451 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2452 (Bits.U.of_int 0 1) && Bitops.eq arg2453 
            (Bits.U.of_int 11 32) && Bitops.eq arg2448 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2449 (Bits.U.of_int 0 1) && Bitops.eq arg2450 
            (Bits.U.of_int 10 32) && Bitops.eq arg2445 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2446 (Bits.U.of_int 0 1) && Bitops.eq arg2447 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 31 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2444)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2572), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2573), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2570), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2571), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2568), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2569), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2566), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2567), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2564), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2565), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2562), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2563), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2560), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2561), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2558), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2559), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2555), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2556), _), 32); 
                    RP.Const (RP.Bits arg2557)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2552), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2553), _), 32); 
                    RP.Const (RP.Bits arg2554)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2549), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2550), _), 32); 
                    RP.Const (RP.Bits arg2551)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2546), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2547), _), 32); 
                    RP.Const (RP.Bits arg2548)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2543), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2544), _), 32); 
                    RP.Const (RP.Bits arg2545)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2540), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2541), _), 32); 
                    RP.Const (RP.Bits arg2542)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2537), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2538), _), 32); 
                    RP.Const (RP.Bits arg2539)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2534), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2535), _), 32); 
                    RP.Const (RP.Bits arg2536)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2531), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2532), _), 32); 
                    RP.Const (RP.Bits arg2533)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2528), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2529), _), 32); 
                    RP.Const (RP.Bits arg2530)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2525), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2526), _), 32); 
                    RP.Const (RP.Bits arg2527)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2522), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2523), _), 32); 
                    RP.Const (RP.Bits arg2524)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2519), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2520), _), 32); 
                    RP.Const (RP.Bits arg2521)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2516), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2517), _), 32); 
                    RP.Const (RP.Bits arg2518)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2513), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2514), _), 32); 
                    RP.Const (RP.Bits arg2515)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2510), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2511), _), 32); 
                  RP.Const (RP.Bits arg2512)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2509)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2509 13 && Bitops.eq arg2572 
            (Bits.U.of_int 8 5) && Bitops.eq arg2573 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2570 (Bits.U.of_int 9 5) && Bitops.eq arg2571 
            (Bits.U.of_int 25 5) && Bitops.eq arg2568 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2569 (Bits.U.of_int 26 5) && Bitops.eq arg2566 
            (Bits.U.of_int 11 5) && Bitops.eq arg2567 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2564 (Bits.U.of_int 12 5) && Bitops.eq arg2565 
            (Bits.U.of_int 28 5) && Bitops.eq arg2562 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2563 (Bits.U.of_int 29 5) && Bitops.eq arg2560 
            (Bits.U.of_int 14 5) && Bitops.eq arg2561 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2558 (Bits.U.of_int 15 5) && Bitops.eq arg2559 
            (Bits.U.of_int 31 5) && Bitops.eq arg2555 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2556 (Bits.U.of_int 0 1) && Bitops.eq arg2557 
            (Bits.U.of_int 8 32) && Bitops.eq arg2552 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2553 (Bits.U.of_int 0 1) && Bitops.eq arg2554 
            (Bits.U.of_int 7 32) && Bitops.eq arg2549 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2550 (Bits.U.of_int 0 1) && Bitops.eq arg2551 
            (Bits.U.of_int 6 32) && Bitops.eq arg2546 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2547 (Bits.U.of_int 0 1) && Bitops.eq arg2548 
            (Bits.U.of_int 5 32) && Bitops.eq arg2543 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2544 (Bits.U.of_int 0 1) && Bitops.eq arg2545 
            (Bits.U.of_int 4 32) && Bitops.eq arg2540 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2541 (Bits.U.of_int 0 1) && Bitops.eq arg2542 
            (Bits.U.of_int 3 32) && Bitops.eq arg2537 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2538 (Bits.U.of_int 0 1) && Bitops.eq arg2539 
            (Bits.U.of_int 2 32) && Bitops.eq arg2534 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2535 (Bits.U.of_int 0 1) && Bitops.eq arg2536 
            (Bits.U.of_int 1 32) && Bitops.eq arg2531 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2532 (Bits.U.of_int 0 1) && Bitops.eq arg2533 
            (Bits.U.of_int 16 32) && Bitops.eq arg2528 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2529 (Bits.U.of_int 0 1) && Bitops.eq arg2530 
            (Bits.U.of_int 15 32) && Bitops.eq arg2525 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2526 (Bits.U.of_int 0 1) && Bitops.eq arg2527 
            (Bits.U.of_int 14 32) && Bitops.eq arg2522 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2523 (Bits.U.of_int 0 1) && Bitops.eq arg2524 
            (Bits.U.of_int 13 32) && Bitops.eq arg2519 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2520 (Bits.U.of_int 0 1) && Bitops.eq arg2521 
            (Bits.U.of_int 12 32) && Bitops.eq arg2516 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2517 (Bits.U.of_int 0 1) && Bitops.eq arg2518 
            (Bits.U.of_int 11 32) && Bitops.eq arg2513 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2514 (Bits.U.of_int 0 1) && Bitops.eq arg2515 
            (Bits.U.of_int 9 32) && Bitops.eq arg2510 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2511 (Bits.U.of_int 0 1) && Bitops.eq arg2512 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 30 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2509)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2637), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2638), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2635), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2636), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2633), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2634), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2631), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2632), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2629), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2630), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2627), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2628), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2625), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2626), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2623), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2624), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2620), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2621), _), 32); 
                    RP.Const (RP.Bits arg2622)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2617), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2618), _), 32); 
                    RP.Const (RP.Bits arg2619)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2614), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2615), _), 32); 
                    RP.Const (RP.Bits arg2616)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2611), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2612), _), 32); 
                    RP.Const (RP.Bits arg2613)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2608), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2609), _), 32); 
                    RP.Const (RP.Bits arg2610)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2605), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2606), _), 32); 
                    RP.Const (RP.Bits arg2607)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2602), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2603), _), 32); 
                    RP.Const (RP.Bits arg2604)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2599), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2600), _), 32); 
                    RP.Const (RP.Bits arg2601)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2596), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2597), _), 32); 
                    RP.Const (RP.Bits arg2598)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2593), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2594), _), 32); 
                    RP.Const (RP.Bits arg2595)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2590), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2591), _), 32); 
                    RP.Const (RP.Bits arg2592)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2587), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2588), _), 32); 
                    RP.Const (RP.Bits arg2589)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2584), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2585), _), 32); 
                    RP.Const (RP.Bits arg2586)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2581), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2582), _), 32); 
                    RP.Const (RP.Bits arg2583)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2578), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2579), _), 32); 
                    RP.Const (RP.Bits arg2580)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2575), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2576), _), 32); 
                  RP.Const (RP.Bits arg2577)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2574)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2574 13 && Bitops.eq arg2637 
            (Bits.U.of_int 8 5) && Bitops.eq arg2638 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2635 (Bits.U.of_int 9 5) && Bitops.eq arg2636 
            (Bits.U.of_int 25 5) && Bitops.eq arg2633 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2634 (Bits.U.of_int 26 5) && Bitops.eq arg2631 
            (Bits.U.of_int 11 5) && Bitops.eq arg2632 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2629 (Bits.U.of_int 12 5) && Bitops.eq arg2630 
            (Bits.U.of_int 28 5) && Bitops.eq arg2627 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2628 (Bits.U.of_int 29 5) && Bitops.eq arg2625 
            (Bits.U.of_int 14 5) && Bitops.eq arg2626 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2623 (Bits.U.of_int 15 5) && Bitops.eq arg2624 
            (Bits.U.of_int 31 5) && Bitops.eq arg2620 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2621 (Bits.U.of_int 0 1) && Bitops.eq arg2622 
            (Bits.U.of_int 8 32) && Bitops.eq arg2617 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2618 (Bits.U.of_int 0 1) && Bitops.eq arg2619 
            (Bits.U.of_int 7 32) && Bitops.eq arg2614 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2615 (Bits.U.of_int 0 1) && Bitops.eq arg2616 
            (Bits.U.of_int 6 32) && Bitops.eq arg2611 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2612 (Bits.U.of_int 0 1) && Bitops.eq arg2613 
            (Bits.U.of_int 5 32) && Bitops.eq arg2608 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2609 (Bits.U.of_int 0 1) && Bitops.eq arg2610 
            (Bits.U.of_int 4 32) && Bitops.eq arg2605 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2606 (Bits.U.of_int 0 1) && Bitops.eq arg2607 
            (Bits.U.of_int 3 32) && Bitops.eq arg2602 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2603 (Bits.U.of_int 0 1) && Bitops.eq arg2604 
            (Bits.U.of_int 2 32) && Bitops.eq arg2599 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2600 (Bits.U.of_int 0 1) && Bitops.eq arg2601 
            (Bits.U.of_int 1 32) && Bitops.eq arg2596 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2597 (Bits.U.of_int 0 1) && Bitops.eq arg2598 
            (Bits.U.of_int 16 32) && Bitops.eq arg2593 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2594 (Bits.U.of_int 0 1) && Bitops.eq arg2595 
            (Bits.U.of_int 15 32) && Bitops.eq arg2590 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2591 (Bits.U.of_int 0 1) && Bitops.eq arg2592 
            (Bits.U.of_int 14 32) && Bitops.eq arg2587 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2588 (Bits.U.of_int 0 1) && Bitops.eq arg2589 
            (Bits.U.of_int 13 32) && Bitops.eq arg2584 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2585 (Bits.U.of_int 0 1) && Bitops.eq arg2586 
            (Bits.U.of_int 12 32) && Bitops.eq arg2581 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2582 (Bits.U.of_int 0 1) && Bitops.eq arg2583 
            (Bits.U.of_int 10 32) && Bitops.eq arg2578 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2579 (Bits.U.of_int 0 1) && Bitops.eq arg2580 
            (Bits.U.of_int 9 32) && Bitops.eq arg2575 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2576 (Bits.U.of_int 0 1) && Bitops.eq arg2577 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 29 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2574)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2702), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2703), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2700), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2701), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2698), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2699), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2696), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2697), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2694), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2695), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2692), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2693), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2690), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2691), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2688), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2689), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2685), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2686), _), 32); 
                    RP.Const (RP.Bits arg2687)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2682), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2683), _), 32); 
                    RP.Const (RP.Bits arg2684)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2679), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2680), _), 32); 
                    RP.Const (RP.Bits arg2681)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2676), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2677), _), 32); 
                    RP.Const (RP.Bits arg2678)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2673), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2674), _), 32); 
                    RP.Const (RP.Bits arg2675)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2670), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2671), _), 32); 
                    RP.Const (RP.Bits arg2672)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2667), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2668), _), 32); 
                    RP.Const (RP.Bits arg2669)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2664), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2665), _), 32); 
                    RP.Const (RP.Bits arg2666)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2661), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2662), _), 32); 
                    RP.Const (RP.Bits arg2663)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2658), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2659), _), 32); 
                    RP.Const (RP.Bits arg2660)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2655), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2656), _), 32); 
                    RP.Const (RP.Bits arg2657)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2652), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2653), _), 32); 
                    RP.Const (RP.Bits arg2654)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2649), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2650), _), 32); 
                    RP.Const (RP.Bits arg2651)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2646), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2647), _), 32); 
                    RP.Const (RP.Bits arg2648)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2643), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2644), _), 32); 
                    RP.Const (RP.Bits arg2645)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2640), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2641), _), 32); 
                  RP.Const (RP.Bits arg2642)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2639)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2639 13 && Bitops.eq arg2702 
            (Bits.U.of_int 8 5) && Bitops.eq arg2703 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2700 (Bits.U.of_int 9 5) && Bitops.eq arg2701 
            (Bits.U.of_int 25 5) && Bitops.eq arg2698 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2699 (Bits.U.of_int 26 5) && Bitops.eq arg2696 
            (Bits.U.of_int 11 5) && Bitops.eq arg2697 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2694 (Bits.U.of_int 12 5) && Bitops.eq arg2695 
            (Bits.U.of_int 28 5) && Bitops.eq arg2692 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2693 (Bits.U.of_int 29 5) && Bitops.eq arg2690 
            (Bits.U.of_int 14 5) && Bitops.eq arg2691 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2688 (Bits.U.of_int 15 5) && Bitops.eq arg2689 
            (Bits.U.of_int 31 5) && Bitops.eq arg2685 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2686 (Bits.U.of_int 0 1) && Bitops.eq arg2687 
            (Bits.U.of_int 8 32) && Bitops.eq arg2682 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2683 (Bits.U.of_int 0 1) && Bitops.eq arg2684 
            (Bits.U.of_int 7 32) && Bitops.eq arg2679 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2680 (Bits.U.of_int 0 1) && Bitops.eq arg2681 
            (Bits.U.of_int 6 32) && Bitops.eq arg2676 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2677 (Bits.U.of_int 0 1) && Bitops.eq arg2678 
            (Bits.U.of_int 5 32) && Bitops.eq arg2673 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2674 (Bits.U.of_int 0 1) && Bitops.eq arg2675 
            (Bits.U.of_int 4 32) && Bitops.eq arg2670 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2671 (Bits.U.of_int 0 1) && Bitops.eq arg2672 
            (Bits.U.of_int 3 32) && Bitops.eq arg2667 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2668 (Bits.U.of_int 0 1) && Bitops.eq arg2669 
            (Bits.U.of_int 2 32) && Bitops.eq arg2664 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2665 (Bits.U.of_int 0 1) && Bitops.eq arg2666 
            (Bits.U.of_int 1 32) && Bitops.eq arg2661 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2662 (Bits.U.of_int 0 1) && Bitops.eq arg2663 
            (Bits.U.of_int 16 32) && Bitops.eq arg2658 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2659 (Bits.U.of_int 0 1) && Bitops.eq arg2660 
            (Bits.U.of_int 15 32) && Bitops.eq arg2655 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2656 (Bits.U.of_int 0 1) && Bitops.eq arg2657 
            (Bits.U.of_int 14 32) && Bitops.eq arg2652 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2653 (Bits.U.of_int 0 1) && Bitops.eq arg2654 
            (Bits.U.of_int 13 32) && Bitops.eq arg2649 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2650 (Bits.U.of_int 0 1) && Bitops.eq arg2651 
            (Bits.U.of_int 11 32) && Bitops.eq arg2646 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2647 (Bits.U.of_int 0 1) && Bitops.eq arg2648 
            (Bits.U.of_int 10 32) && Bitops.eq arg2643 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2644 (Bits.U.of_int 0 1) && Bitops.eq arg2645 
            (Bits.U.of_int 9 32) && Bitops.eq arg2640 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2641 (Bits.U.of_int 0 1) && Bitops.eq arg2642 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 28 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2639)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2767), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2768), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2765), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2766), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2763), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2764), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2761), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2762), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2759), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2760), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2757), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2758), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2755), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2756), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2753), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2754), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2750), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2751), _), 32); 
                    RP.Const (RP.Bits arg2752)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2747), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2748), _), 32); 
                    RP.Const (RP.Bits arg2749)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2744), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2745), _), 32); 
                    RP.Const (RP.Bits arg2746)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2741), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2742), _), 32); 
                    RP.Const (RP.Bits arg2743)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2738), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2739), _), 32); 
                    RP.Const (RP.Bits arg2740)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2735), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2736), _), 32); 
                    RP.Const (RP.Bits arg2737)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2732), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2733), _), 32); 
                    RP.Const (RP.Bits arg2734)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2729), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2730), _), 32); 
                    RP.Const (RP.Bits arg2731)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2726), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2727), _), 32); 
                    RP.Const (RP.Bits arg2728)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2723), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2724), _), 32); 
                    RP.Const (RP.Bits arg2725)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2720), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2721), _), 32); 
                    RP.Const (RP.Bits arg2722)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2717), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2718), _), 32); 
                    RP.Const (RP.Bits arg2719)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2714), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2715), _), 32); 
                    RP.Const (RP.Bits arg2716)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2711), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2712), _), 32); 
                    RP.Const (RP.Bits arg2713)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2708), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2709), _), 32); 
                    RP.Const (RP.Bits arg2710)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2705), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2706), _), 32); 
                  RP.Const (RP.Bits arg2707)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2704)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2704 13 && Bitops.eq arg2767 
            (Bits.U.of_int 8 5) && Bitops.eq arg2768 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2765 (Bits.U.of_int 9 5) && Bitops.eq arg2766 
            (Bits.U.of_int 25 5) && Bitops.eq arg2763 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2764 (Bits.U.of_int 26 5) && Bitops.eq arg2761 
            (Bits.U.of_int 11 5) && Bitops.eq arg2762 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2759 (Bits.U.of_int 12 5) && Bitops.eq arg2760 
            (Bits.U.of_int 28 5) && Bitops.eq arg2757 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2758 (Bits.U.of_int 29 5) && Bitops.eq arg2755 
            (Bits.U.of_int 14 5) && Bitops.eq arg2756 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2753 (Bits.U.of_int 15 5) && Bitops.eq arg2754 
            (Bits.U.of_int 31 5) && Bitops.eq arg2750 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2751 (Bits.U.of_int 0 1) && Bitops.eq arg2752 
            (Bits.U.of_int 8 32) && Bitops.eq arg2747 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2748 (Bits.U.of_int 0 1) && Bitops.eq arg2749 
            (Bits.U.of_int 7 32) && Bitops.eq arg2744 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2745 (Bits.U.of_int 0 1) && Bitops.eq arg2746 
            (Bits.U.of_int 6 32) && Bitops.eq arg2741 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2742 (Bits.U.of_int 0 1) && Bitops.eq arg2743 
            (Bits.U.of_int 5 32) && Bitops.eq arg2738 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2739 (Bits.U.of_int 0 1) && Bitops.eq arg2740 
            (Bits.U.of_int 4 32) && Bitops.eq arg2735 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2736 (Bits.U.of_int 0 1) && Bitops.eq arg2737 
            (Bits.U.of_int 3 32) && Bitops.eq arg2732 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2733 (Bits.U.of_int 0 1) && Bitops.eq arg2734 
            (Bits.U.of_int 2 32) && Bitops.eq arg2729 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2730 (Bits.U.of_int 0 1) && Bitops.eq arg2731 
            (Bits.U.of_int 1 32) && Bitops.eq arg2726 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2727 (Bits.U.of_int 0 1) && Bitops.eq arg2728 
            (Bits.U.of_int 16 32) && Bitops.eq arg2723 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2724 (Bits.U.of_int 0 1) && Bitops.eq arg2725 
            (Bits.U.of_int 15 32) && Bitops.eq arg2720 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2721 (Bits.U.of_int 0 1) && Bitops.eq arg2722 
            (Bits.U.of_int 14 32) && Bitops.eq arg2717 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2718 (Bits.U.of_int 0 1) && Bitops.eq arg2719 
            (Bits.U.of_int 12 32) && Bitops.eq arg2714 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2715 (Bits.U.of_int 0 1) && Bitops.eq arg2716 
            (Bits.U.of_int 11 32) && Bitops.eq arg2711 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2712 (Bits.U.of_int 0 1) && Bitops.eq arg2713 
            (Bits.U.of_int 10 32) && Bitops.eq arg2708 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2709 (Bits.U.of_int 0 1) && Bitops.eq arg2710 
            (Bits.U.of_int 9 32) && Bitops.eq arg2705 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2706 (Bits.U.of_int 0 1) && Bitops.eq arg2707 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 27 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2704)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2832), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2833), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2830), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2831), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2828), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2829), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2826), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2827), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2824), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2825), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2822), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2823), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2820), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2821), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2818), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2819), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2815), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2816), _), 32); 
                    RP.Const (RP.Bits arg2817)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2812), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2813), _), 32); 
                    RP.Const (RP.Bits arg2814)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2809), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2810), _), 32); 
                    RP.Const (RP.Bits arg2811)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2806), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2807), _), 32); 
                    RP.Const (RP.Bits arg2808)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2803), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2804), _), 32); 
                    RP.Const (RP.Bits arg2805)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2800), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2801), _), 32); 
                    RP.Const (RP.Bits arg2802)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2797), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2798), _), 32); 
                    RP.Const (RP.Bits arg2799)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2794), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2795), _), 32); 
                    RP.Const (RP.Bits arg2796)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2791), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2792), _), 32); 
                    RP.Const (RP.Bits arg2793)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2788), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2789), _), 32); 
                    RP.Const (RP.Bits arg2790)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2785), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2786), _), 32); 
                    RP.Const (RP.Bits arg2787)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2782), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2783), _), 32); 
                    RP.Const (RP.Bits arg2784)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2779), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2780), _), 32); 
                    RP.Const (RP.Bits arg2781)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2776), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2777), _), 32); 
                    RP.Const (RP.Bits arg2778)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2773), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2774), _), 32); 
                    RP.Const (RP.Bits arg2775)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2770), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2771), _), 32); 
                  RP.Const (RP.Bits arg2772)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2769)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2769 13 && Bitops.eq arg2832 
            (Bits.U.of_int 8 5) && Bitops.eq arg2833 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2830 (Bits.U.of_int 9 5) && Bitops.eq arg2831 
            (Bits.U.of_int 25 5) && Bitops.eq arg2828 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2829 (Bits.U.of_int 26 5) && Bitops.eq arg2826 
            (Bits.U.of_int 11 5) && Bitops.eq arg2827 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2824 (Bits.U.of_int 12 5) && Bitops.eq arg2825 
            (Bits.U.of_int 28 5) && Bitops.eq arg2822 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2823 (Bits.U.of_int 29 5) && Bitops.eq arg2820 
            (Bits.U.of_int 14 5) && Bitops.eq arg2821 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2818 (Bits.U.of_int 15 5) && Bitops.eq arg2819 
            (Bits.U.of_int 31 5) && Bitops.eq arg2815 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2816 (Bits.U.of_int 0 1) && Bitops.eq arg2817 
            (Bits.U.of_int 8 32) && Bitops.eq arg2812 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2813 (Bits.U.of_int 0 1) && Bitops.eq arg2814 
            (Bits.U.of_int 7 32) && Bitops.eq arg2809 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2810 (Bits.U.of_int 0 1) && Bitops.eq arg2811 
            (Bits.U.of_int 6 32) && Bitops.eq arg2806 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2807 (Bits.U.of_int 0 1) && Bitops.eq arg2808 
            (Bits.U.of_int 5 32) && Bitops.eq arg2803 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2804 (Bits.U.of_int 0 1) && Bitops.eq arg2805 
            (Bits.U.of_int 4 32) && Bitops.eq arg2800 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2801 (Bits.U.of_int 0 1) && Bitops.eq arg2802 
            (Bits.U.of_int 3 32) && Bitops.eq arg2797 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2798 (Bits.U.of_int 0 1) && Bitops.eq arg2799 
            (Bits.U.of_int 2 32) && Bitops.eq arg2794 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2795 (Bits.U.of_int 0 1) && Bitops.eq arg2796 
            (Bits.U.of_int 1 32) && Bitops.eq arg2791 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2792 (Bits.U.of_int 0 1) && Bitops.eq arg2793 
            (Bits.U.of_int 16 32) && Bitops.eq arg2788 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2789 (Bits.U.of_int 0 1) && Bitops.eq arg2790 
            (Bits.U.of_int 15 32) && Bitops.eq arg2785 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2786 (Bits.U.of_int 0 1) && Bitops.eq arg2787 
            (Bits.U.of_int 13 32) && Bitops.eq arg2782 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2783 (Bits.U.of_int 0 1) && Bitops.eq arg2784 
            (Bits.U.of_int 12 32) && Bitops.eq arg2779 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2780 (Bits.U.of_int 0 1) && Bitops.eq arg2781 
            (Bits.U.of_int 11 32) && Bitops.eq arg2776 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2777 (Bits.U.of_int 0 1) && Bitops.eq arg2778 
            (Bits.U.of_int 10 32) && Bitops.eq arg2773 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2774 (Bits.U.of_int 0 1) && Bitops.eq arg2775 
            (Bits.U.of_int 9 32) && Bitops.eq arg2770 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2771 (Bits.U.of_int 0 1) && Bitops.eq arg2772 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 26 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2769)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2897), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2898), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2895), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2896), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2893), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2894), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2891), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2892), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2889), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2890), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2887), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2888), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2885), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2886), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2883), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2884), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2880), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2881), _), 32); 
                    RP.Const (RP.Bits arg2882)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2877), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2878), _), 32); 
                    RP.Const (RP.Bits arg2879)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2874), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2875), _), 32); 
                    RP.Const (RP.Bits arg2876)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2871), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2872), _), 32); 
                    RP.Const (RP.Bits arg2873)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2868), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2869), _), 32); 
                    RP.Const (RP.Bits arg2870)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2865), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2866), _), 32); 
                    RP.Const (RP.Bits arg2867)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2862), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2863), _), 32); 
                    RP.Const (RP.Bits arg2864)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2859), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2860), _), 32); 
                    RP.Const (RP.Bits arg2861)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2856), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2857), _), 32); 
                    RP.Const (RP.Bits arg2858)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2853), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2854), _), 32); 
                    RP.Const (RP.Bits arg2855)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2850), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2851), _), 32); 
                    RP.Const (RP.Bits arg2852)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2847), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2848), _), 32); 
                    RP.Const (RP.Bits arg2849)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2844), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2845), _), 32); 
                    RP.Const (RP.Bits arg2846)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2841), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2842), _), 32); 
                    RP.Const (RP.Bits arg2843)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2838), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2839), _), 32); 
                    RP.Const (RP.Bits arg2840)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2835), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2836), _), 32); 
                  RP.Const (RP.Bits arg2837)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2834)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2834 13 && Bitops.eq arg2897 
            (Bits.U.of_int 8 5) && Bitops.eq arg2898 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2895 (Bits.U.of_int 9 5) && Bitops.eq arg2896 
            (Bits.U.of_int 25 5) && Bitops.eq arg2893 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2894 (Bits.U.of_int 26 5) && Bitops.eq arg2891 
            (Bits.U.of_int 11 5) && Bitops.eq arg2892 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2889 (Bits.U.of_int 12 5) && Bitops.eq arg2890 
            (Bits.U.of_int 28 5) && Bitops.eq arg2887 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2888 (Bits.U.of_int 29 5) && Bitops.eq arg2885 
            (Bits.U.of_int 14 5) && Bitops.eq arg2886 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2883 (Bits.U.of_int 15 5) && Bitops.eq arg2884 
            (Bits.U.of_int 31 5) && Bitops.eq arg2880 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2881 (Bits.U.of_int 0 1) && Bitops.eq arg2882 
            (Bits.U.of_int 8 32) && Bitops.eq arg2877 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2878 (Bits.U.of_int 0 1) && Bitops.eq arg2879 
            (Bits.U.of_int 7 32) && Bitops.eq arg2874 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2875 (Bits.U.of_int 0 1) && Bitops.eq arg2876 
            (Bits.U.of_int 6 32) && Bitops.eq arg2871 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2872 (Bits.U.of_int 0 1) && Bitops.eq arg2873 
            (Bits.U.of_int 5 32) && Bitops.eq arg2868 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2869 (Bits.U.of_int 0 1) && Bitops.eq arg2870 
            (Bits.U.of_int 4 32) && Bitops.eq arg2865 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2866 (Bits.U.of_int 0 1) && Bitops.eq arg2867 
            (Bits.U.of_int 3 32) && Bitops.eq arg2862 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2863 (Bits.U.of_int 0 1) && Bitops.eq arg2864 
            (Bits.U.of_int 2 32) && Bitops.eq arg2859 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2860 (Bits.U.of_int 0 1) && Bitops.eq arg2861 
            (Bits.U.of_int 1 32) && Bitops.eq arg2856 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2857 (Bits.U.of_int 0 1) && Bitops.eq arg2858 
            (Bits.U.of_int 16 32) && Bitops.eq arg2853 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2854 (Bits.U.of_int 0 1) && Bitops.eq arg2855 
            (Bits.U.of_int 14 32) && Bitops.eq arg2850 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2851 (Bits.U.of_int 0 1) && Bitops.eq arg2852 
            (Bits.U.of_int 13 32) && Bitops.eq arg2847 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2848 (Bits.U.of_int 0 1) && Bitops.eq arg2849 
            (Bits.U.of_int 12 32) && Bitops.eq arg2844 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2845 (Bits.U.of_int 0 1) && Bitops.eq arg2846 
            (Bits.U.of_int 11 32) && Bitops.eq arg2841 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2842 (Bits.U.of_int 0 1) && Bitops.eq arg2843 
            (Bits.U.of_int 10 32) && Bitops.eq arg2838 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2839 (Bits.U.of_int 0 1) && Bitops.eq arg2840 
            (Bits.U.of_int 9 32) && Bitops.eq arg2835 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2836 (Bits.U.of_int 0 1) && Bitops.eq arg2837 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 25 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2834)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg2962), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg2963), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2960), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2961), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2958), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2959), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2956), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2957), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2954), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2955), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2952), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2953), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2950), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2951), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2948), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg2949), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2945), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2946), _), 32); 
                    RP.Const (RP.Bits arg2947)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2942), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2943), _), 32); 
                    RP.Const (RP.Bits arg2944)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2939), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2940), _), 32); 
                    RP.Const (RP.Bits arg2941)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2936), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2937), _), 32); 
                    RP.Const (RP.Bits arg2938)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2933), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2934), _), 32); 
                    RP.Const (RP.Bits arg2935)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2930), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2931), _), 32); 
                    RP.Const (RP.Bits arg2932)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2927), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2928), _), 32); 
                    RP.Const (RP.Bits arg2929)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2924), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2925), _), 32); 
                    RP.Const (RP.Bits arg2926)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2921), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2922), _), 32); 
                    RP.Const (RP.Bits arg2923)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2918), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2919), _), 32); 
                    RP.Const (RP.Bits arg2920)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2915), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2916), _), 32); 
                    RP.Const (RP.Bits arg2917)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2912), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2913), _), 32); 
                    RP.Const (RP.Bits arg2914)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2909), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2910), _), 32); 
                    RP.Const (RP.Bits arg2911)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2906), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2907), _), 32); 
                    RP.Const (RP.Bits arg2908)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2903), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2904), _), 32); 
                    RP.Const (RP.Bits arg2905)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2900), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2901), _), 32); 
                  RP.Const (RP.Bits arg2902)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2899)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2899 13 && Bitops.eq arg2962 
            (Bits.U.of_int 8 5) && Bitops.eq arg2963 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2960 (Bits.U.of_int 9 5) && Bitops.eq arg2961 
            (Bits.U.of_int 25 5) && Bitops.eq arg2958 (Bits.U.of_int 10 5) && 
          Bitops.eq arg2959 (Bits.U.of_int 26 5) && Bitops.eq arg2956 
            (Bits.U.of_int 11 5) && Bitops.eq arg2957 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2954 (Bits.U.of_int 12 5) && Bitops.eq arg2955 
            (Bits.U.of_int 28 5) && Bitops.eq arg2952 (Bits.U.of_int 13 5) && 
          Bitops.eq arg2953 (Bits.U.of_int 29 5) && Bitops.eq arg2950 
            (Bits.U.of_int 14 5) && Bitops.eq arg2951 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2948 (Bits.U.of_int 15 5) && Bitops.eq arg2949 
            (Bits.U.of_int 31 5) && Bitops.eq arg2945 (Bits.U.of_int 16 5) && 
          Bitops.eq arg2946 (Bits.U.of_int 0 1) && Bitops.eq arg2947 
            (Bits.U.of_int 8 32) && Bitops.eq arg2942 (Bits.U.of_int 17 5) && 
          Bitops.eq arg2943 (Bits.U.of_int 0 1) && Bitops.eq arg2944 
            (Bits.U.of_int 7 32) && Bitops.eq arg2939 (Bits.U.of_int 18 5) && 
          Bitops.eq arg2940 (Bits.U.of_int 0 1) && Bitops.eq arg2941 
            (Bits.U.of_int 6 32) && Bitops.eq arg2936 (Bits.U.of_int 19 5) && 
          Bitops.eq arg2937 (Bits.U.of_int 0 1) && Bitops.eq arg2938 
            (Bits.U.of_int 5 32) && Bitops.eq arg2933 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2934 (Bits.U.of_int 0 1) && Bitops.eq arg2935 
            (Bits.U.of_int 4 32) && Bitops.eq arg2930 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2931 (Bits.U.of_int 0 1) && Bitops.eq arg2932 
            (Bits.U.of_int 3 32) && Bitops.eq arg2927 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2928 (Bits.U.of_int 0 1) && Bitops.eq arg2929 
            (Bits.U.of_int 2 32) && Bitops.eq arg2924 (Bits.U.of_int 23 5) && 
          Bitops.eq arg2925 (Bits.U.of_int 0 1) && Bitops.eq arg2926 
            (Bits.U.of_int 1 32) && Bitops.eq arg2921 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2922 (Bits.U.of_int 0 1) && Bitops.eq arg2923 
            (Bits.U.of_int 15 32) && Bitops.eq arg2918 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2919 (Bits.U.of_int 0 1) && Bitops.eq arg2920 
            (Bits.U.of_int 14 32) && Bitops.eq arg2915 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2916 (Bits.U.of_int 0 1) && Bitops.eq arg2917 
            (Bits.U.of_int 13 32) && Bitops.eq arg2912 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2913 (Bits.U.of_int 0 1) && Bitops.eq arg2914 
            (Bits.U.of_int 12 32) && Bitops.eq arg2909 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2910 (Bits.U.of_int 0 1) && Bitops.eq arg2911 
            (Bits.U.of_int 11 32) && Bitops.eq arg2906 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2907 (Bits.U.of_int 0 1) && Bitops.eq arg2908 
            (Bits.U.of_int 10 32) && Bitops.eq arg2903 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2904 (Bits.U.of_int 0 1) && Bitops.eq arg2905 
            (Bits.U.of_int 9 32) && Bitops.eq arg2900 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2901 (Bits.U.of_int 0 1) && Bitops.eq arg2902 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 24 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2899)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3027), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3028), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3025), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3026), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3023), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3024), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3021), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3022), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3019), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3020), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3017), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3018), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3015), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3016), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3013), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3014), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3010), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3011), _), 32); 
                    RP.Const (RP.Bits arg3012)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3007), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3008), _), 32); 
                    RP.Const (RP.Bits arg3009)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3004), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3005), _), 32); 
                    RP.Const (RP.Bits arg3006)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3001), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3002), _), 32); 
                    RP.Const (RP.Bits arg3003)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2998), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2999), _), 32); 
                    RP.Const (RP.Bits arg3000)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2995), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2996), _), 32); 
                    RP.Const (RP.Bits arg2997)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2992), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2993), _), 32); 
                    RP.Const (RP.Bits arg2994)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2989), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2990), _), 32); 
                    RP.Const (RP.Bits arg2991)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2986), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2987), _), 32); 
                    RP.Const (RP.Bits arg2988)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2983), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2984), _), 32); 
                    RP.Const (RP.Bits arg2985)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2980), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2981), _), 32); 
                    RP.Const (RP.Bits arg2982)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2977), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2978), _), 32); 
                    RP.Const (RP.Bits arg2979)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2974), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2975), _), 32); 
                    RP.Const (RP.Bits arg2976)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2971), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2972), _), 32); 
                    RP.Const (RP.Bits arg2973)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2968), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg2969), _), 32); 
                    RP.Const (RP.Bits arg2970)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg2965), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg2966), _), 32); 
                  RP.Const (RP.Bits arg2967)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg2964)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg2964 13 && Bitops.eq arg3027 
            (Bits.U.of_int 8 5) && Bitops.eq arg3028 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3025 (Bits.U.of_int 9 5) && Bitops.eq arg3026 
            (Bits.U.of_int 25 5) && Bitops.eq arg3023 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3024 (Bits.U.of_int 26 5) && Bitops.eq arg3021 
            (Bits.U.of_int 11 5) && Bitops.eq arg3022 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3019 (Bits.U.of_int 12 5) && Bitops.eq arg3020 
            (Bits.U.of_int 28 5) && Bitops.eq arg3017 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3018 (Bits.U.of_int 29 5) && Bitops.eq arg3015 
            (Bits.U.of_int 14 5) && Bitops.eq arg3016 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3013 (Bits.U.of_int 15 5) && Bitops.eq arg3014 
            (Bits.U.of_int 31 5) && Bitops.eq arg3010 (Bits.U.of_int 16 5) && 
          Bitops.eq arg3011 (Bits.U.of_int 0 1) && Bitops.eq arg3012 
            (Bits.U.of_int 8 32) && Bitops.eq arg3007 (Bits.U.of_int 17 5) && 
          Bitops.eq arg3008 (Bits.U.of_int 0 1) && Bitops.eq arg3009 
            (Bits.U.of_int 7 32) && Bitops.eq arg3004 (Bits.U.of_int 18 5) && 
          Bitops.eq arg3005 (Bits.U.of_int 0 1) && Bitops.eq arg3006 
            (Bits.U.of_int 6 32) && Bitops.eq arg3001 (Bits.U.of_int 19 5) && 
          Bitops.eq arg3002 (Bits.U.of_int 0 1) && Bitops.eq arg3003 
            (Bits.U.of_int 5 32) && Bitops.eq arg2998 (Bits.U.of_int 20 5) && 
          Bitops.eq arg2999 (Bits.U.of_int 0 1) && Bitops.eq arg3000 
            (Bits.U.of_int 4 32) && Bitops.eq arg2995 (Bits.U.of_int 21 5) && 
          Bitops.eq arg2996 (Bits.U.of_int 0 1) && Bitops.eq arg2997 
            (Bits.U.of_int 3 32) && Bitops.eq arg2992 (Bits.U.of_int 22 5) && 
          Bitops.eq arg2993 (Bits.U.of_int 0 1) && Bitops.eq arg2994 
            (Bits.U.of_int 2 32) && Bitops.eq arg2989 (Bits.U.of_int 24 5) && 
          Bitops.eq arg2990 (Bits.U.of_int 0 1) && Bitops.eq arg2991 
            (Bits.U.of_int 16 32) && Bitops.eq arg2986 (Bits.U.of_int 25 5) && 
          Bitops.eq arg2987 (Bits.U.of_int 0 1) && Bitops.eq arg2988 
            (Bits.U.of_int 15 32) && Bitops.eq arg2983 (Bits.U.of_int 26 5) && 
          Bitops.eq arg2984 (Bits.U.of_int 0 1) && Bitops.eq arg2985 
            (Bits.U.of_int 14 32) && Bitops.eq arg2980 (Bits.U.of_int 27 5) && 
          Bitops.eq arg2981 (Bits.U.of_int 0 1) && Bitops.eq arg2982 
            (Bits.U.of_int 13 32) && Bitops.eq arg2977 (Bits.U.of_int 28 5) && 
          Bitops.eq arg2978 (Bits.U.of_int 0 1) && Bitops.eq arg2979 
            (Bits.U.of_int 12 32) && Bitops.eq arg2974 (Bits.U.of_int 29 5) && 
          Bitops.eq arg2975 (Bits.U.of_int 0 1) && Bitops.eq arg2976 
            (Bits.U.of_int 11 32) && Bitops.eq arg2971 (Bits.U.of_int 30 5) && 
          Bitops.eq arg2972 (Bits.U.of_int 0 1) && Bitops.eq arg2973 
            (Bits.U.of_int 10 32) && Bitops.eq arg2968 (Bits.U.of_int 31 5) && 
          Bitops.eq arg2969 (Bits.U.of_int 0 1) && Bitops.eq arg2970 
            (Bits.U.of_int 9 32) && Bitops.eq arg2965 (Bits.U.of_int 0 1) && 
          Bitops.eq arg2966 (Bits.U.of_int 0 1) && Bitops.eq arg2967 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 23 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg2964)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3092), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3093), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3090), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3091), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3088), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3089), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3086), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3087), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3084), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3085), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3082), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3083), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3080), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3081), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3078), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3079), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3075), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3076), _), 32); 
                    RP.Const (RP.Bits arg3077)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3072), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3073), _), 32); 
                    RP.Const (RP.Bits arg3074)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3069), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3070), _), 32); 
                    RP.Const (RP.Bits arg3071)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3066), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3067), _), 32); 
                    RP.Const (RP.Bits arg3068)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3063), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3064), _), 32); 
                    RP.Const (RP.Bits arg3065)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3060), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3061), _), 32); 
                    RP.Const (RP.Bits arg3062)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3057), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3058), _), 32); 
                    RP.Const (RP.Bits arg3059)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3054), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3055), _), 32); 
                    RP.Const (RP.Bits arg3056)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3051), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3052), _), 32); 
                    RP.Const (RP.Bits arg3053)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3048), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3049), _), 32); 
                    RP.Const (RP.Bits arg3050)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3045), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3046), _), 32); 
                    RP.Const (RP.Bits arg3047)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3042), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3043), _), 32); 
                    RP.Const (RP.Bits arg3044)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3039), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3040), _), 32); 
                    RP.Const (RP.Bits arg3041)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3036), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3037), _), 32); 
                    RP.Const (RP.Bits arg3038)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3033), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3034), _), 32); 
                    RP.Const (RP.Bits arg3035)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3030), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3031), _), 32); 
                  RP.Const (RP.Bits arg3032)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3029)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3029 13 && Bitops.eq arg3092 
            (Bits.U.of_int 8 5) && Bitops.eq arg3093 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3090 (Bits.U.of_int 9 5) && Bitops.eq arg3091 
            (Bits.U.of_int 25 5) && Bitops.eq arg3088 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3089 (Bits.U.of_int 26 5) && Bitops.eq arg3086 
            (Bits.U.of_int 11 5) && Bitops.eq arg3087 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3084 (Bits.U.of_int 12 5) && Bitops.eq arg3085 
            (Bits.U.of_int 28 5) && Bitops.eq arg3082 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3083 (Bits.U.of_int 29 5) && Bitops.eq arg3080 
            (Bits.U.of_int 14 5) && Bitops.eq arg3081 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3078 (Bits.U.of_int 15 5) && Bitops.eq arg3079 
            (Bits.U.of_int 31 5) && Bitops.eq arg3075 (Bits.U.of_int 16 5) && 
          Bitops.eq arg3076 (Bits.U.of_int 0 1) && Bitops.eq arg3077 
            (Bits.U.of_int 8 32) && Bitops.eq arg3072 (Bits.U.of_int 17 5) && 
          Bitops.eq arg3073 (Bits.U.of_int 0 1) && Bitops.eq arg3074 
            (Bits.U.of_int 7 32) && Bitops.eq arg3069 (Bits.U.of_int 18 5) && 
          Bitops.eq arg3070 (Bits.U.of_int 0 1) && Bitops.eq arg3071 
            (Bits.U.of_int 6 32) && Bitops.eq arg3066 (Bits.U.of_int 19 5) && 
          Bitops.eq arg3067 (Bits.U.of_int 0 1) && Bitops.eq arg3068 
            (Bits.U.of_int 5 32) && Bitops.eq arg3063 (Bits.U.of_int 20 5) && 
          Bitops.eq arg3064 (Bits.U.of_int 0 1) && Bitops.eq arg3065 
            (Bits.U.of_int 4 32) && Bitops.eq arg3060 (Bits.U.of_int 21 5) && 
          Bitops.eq arg3061 (Bits.U.of_int 0 1) && Bitops.eq arg3062 
            (Bits.U.of_int 3 32) && Bitops.eq arg3057 (Bits.U.of_int 23 5) && 
          Bitops.eq arg3058 (Bits.U.of_int 0 1) && Bitops.eq arg3059 
            (Bits.U.of_int 1 32) && Bitops.eq arg3054 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3055 (Bits.U.of_int 0 1) && Bitops.eq arg3056 
            (Bits.U.of_int 16 32) && Bitops.eq arg3051 (Bits.U.of_int 25 5) && 
          Bitops.eq arg3052 (Bits.U.of_int 0 1) && Bitops.eq arg3053 
            (Bits.U.of_int 15 32) && Bitops.eq arg3048 (Bits.U.of_int 26 5) && 
          Bitops.eq arg3049 (Bits.U.of_int 0 1) && Bitops.eq arg3050 
            (Bits.U.of_int 14 32) && Bitops.eq arg3045 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3046 (Bits.U.of_int 0 1) && Bitops.eq arg3047 
            (Bits.U.of_int 13 32) && Bitops.eq arg3042 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3043 (Bits.U.of_int 0 1) && Bitops.eq arg3044 
            (Bits.U.of_int 12 32) && Bitops.eq arg3039 (Bits.U.of_int 29 5) && 
          Bitops.eq arg3040 (Bits.U.of_int 0 1) && Bitops.eq arg3041 
            (Bits.U.of_int 11 32) && Bitops.eq arg3036 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3037 (Bits.U.of_int 0 1) && Bitops.eq arg3038 
            (Bits.U.of_int 10 32) && Bitops.eq arg3033 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3034 (Bits.U.of_int 0 1) && Bitops.eq arg3035 
            (Bits.U.of_int 9 32) && Bitops.eq arg3030 (Bits.U.of_int 0 1) && 
          Bitops.eq arg3031 (Bits.U.of_int 0 1) && Bitops.eq arg3032 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 22 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3029)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3157), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3158), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3155), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3156), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3153), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3154), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3151), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3152), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3149), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3150), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3147), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3148), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3145), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3146), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3143), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3144), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3140), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3141), _), 32); 
                    RP.Const (RP.Bits arg3142)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3137), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3138), _), 32); 
                    RP.Const (RP.Bits arg3139)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3134), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3135), _), 32); 
                    RP.Const (RP.Bits arg3136)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3131), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3132), _), 32); 
                    RP.Const (RP.Bits arg3133)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3128), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3129), _), 32); 
                    RP.Const (RP.Bits arg3130)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3125), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3126), _), 32); 
                    RP.Const (RP.Bits arg3127)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3122), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3123), _), 32); 
                    RP.Const (RP.Bits arg3124)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3119), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3120), _), 32); 
                    RP.Const (RP.Bits arg3121)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3116), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3117), _), 32); 
                    RP.Const (RP.Bits arg3118)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3113), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3114), _), 32); 
                    RP.Const (RP.Bits arg3115)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3110), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3111), _), 32); 
                    RP.Const (RP.Bits arg3112)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3107), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3108), _), 32); 
                    RP.Const (RP.Bits arg3109)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3104), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3105), _), 32); 
                    RP.Const (RP.Bits arg3106)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3101), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3102), _), 32); 
                    RP.Const (RP.Bits arg3103)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3098), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3099), _), 32); 
                    RP.Const (RP.Bits arg3100)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3095), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3096), _), 32); 
                  RP.Const (RP.Bits arg3097)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3094)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3094 13 && Bitops.eq arg3157 
            (Bits.U.of_int 8 5) && Bitops.eq arg3158 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3155 (Bits.U.of_int 9 5) && Bitops.eq arg3156 
            (Bits.U.of_int 25 5) && Bitops.eq arg3153 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3154 (Bits.U.of_int 26 5) && Bitops.eq arg3151 
            (Bits.U.of_int 11 5) && Bitops.eq arg3152 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3149 (Bits.U.of_int 12 5) && Bitops.eq arg3150 
            (Bits.U.of_int 28 5) && Bitops.eq arg3147 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3148 (Bits.U.of_int 29 5) && Bitops.eq arg3145 
            (Bits.U.of_int 14 5) && Bitops.eq arg3146 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3143 (Bits.U.of_int 15 5) && Bitops.eq arg3144 
            (Bits.U.of_int 31 5) && Bitops.eq arg3140 (Bits.U.of_int 16 5) && 
          Bitops.eq arg3141 (Bits.U.of_int 0 1) && Bitops.eq arg3142 
            (Bits.U.of_int 8 32) && Bitops.eq arg3137 (Bits.U.of_int 17 5) && 
          Bitops.eq arg3138 (Bits.U.of_int 0 1) && Bitops.eq arg3139 
            (Bits.U.of_int 7 32) && Bitops.eq arg3134 (Bits.U.of_int 18 5) && 
          Bitops.eq arg3135 (Bits.U.of_int 0 1) && Bitops.eq arg3136 
            (Bits.U.of_int 6 32) && Bitops.eq arg3131 (Bits.U.of_int 19 5) && 
          Bitops.eq arg3132 (Bits.U.of_int 0 1) && Bitops.eq arg3133 
            (Bits.U.of_int 5 32) && Bitops.eq arg3128 (Bits.U.of_int 20 5) && 
          Bitops.eq arg3129 (Bits.U.of_int 0 1) && Bitops.eq arg3130 
            (Bits.U.of_int 4 32) && Bitops.eq arg3125 (Bits.U.of_int 22 5) && 
          Bitops.eq arg3126 (Bits.U.of_int 0 1) && Bitops.eq arg3127 
            (Bits.U.of_int 2 32) && Bitops.eq arg3122 (Bits.U.of_int 23 5) && 
          Bitops.eq arg3123 (Bits.U.of_int 0 1) && Bitops.eq arg3124 
            (Bits.U.of_int 1 32) && Bitops.eq arg3119 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3120 (Bits.U.of_int 0 1) && Bitops.eq arg3121 
            (Bits.U.of_int 16 32) && Bitops.eq arg3116 (Bits.U.of_int 25 5) && 
          Bitops.eq arg3117 (Bits.U.of_int 0 1) && Bitops.eq arg3118 
            (Bits.U.of_int 15 32) && Bitops.eq arg3113 (Bits.U.of_int 26 5) && 
          Bitops.eq arg3114 (Bits.U.of_int 0 1) && Bitops.eq arg3115 
            (Bits.U.of_int 14 32) && Bitops.eq arg3110 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3111 (Bits.U.of_int 0 1) && Bitops.eq arg3112 
            (Bits.U.of_int 13 32) && Bitops.eq arg3107 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3108 (Bits.U.of_int 0 1) && Bitops.eq arg3109 
            (Bits.U.of_int 12 32) && Bitops.eq arg3104 (Bits.U.of_int 29 5) && 
          Bitops.eq arg3105 (Bits.U.of_int 0 1) && Bitops.eq arg3106 
            (Bits.U.of_int 11 32) && Bitops.eq arg3101 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3102 (Bits.U.of_int 0 1) && Bitops.eq arg3103 
            (Bits.U.of_int 10 32) && Bitops.eq arg3098 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3099 (Bits.U.of_int 0 1) && Bitops.eq arg3100 
            (Bits.U.of_int 9 32) && Bitops.eq arg3095 (Bits.U.of_int 0 1) && 
          Bitops.eq arg3096 (Bits.U.of_int 0 1) && Bitops.eq arg3097 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 21 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3094)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3222), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3223), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3220), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3221), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3218), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3219), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3216), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3217), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3214), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3215), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3212), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3213), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3210), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3211), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3208), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3209), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3205), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3206), _), 32); 
                    RP.Const (RP.Bits arg3207)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3202), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3203), _), 32); 
                    RP.Const (RP.Bits arg3204)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3199), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3200), _), 32); 
                    RP.Const (RP.Bits arg3201)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3196), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3197), _), 32); 
                    RP.Const (RP.Bits arg3198)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3193), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3194), _), 32); 
                    RP.Const (RP.Bits arg3195)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3190), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3191), _), 32); 
                    RP.Const (RP.Bits arg3192)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3187), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3188), _), 32); 
                    RP.Const (RP.Bits arg3189)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3184), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3185), _), 32); 
                    RP.Const (RP.Bits arg3186)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3181), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3182), _), 32); 
                    RP.Const (RP.Bits arg3183)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3178), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3179), _), 32); 
                    RP.Const (RP.Bits arg3180)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3175), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3176), _), 32); 
                    RP.Const (RP.Bits arg3177)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3172), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3173), _), 32); 
                    RP.Const (RP.Bits arg3174)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3169), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3170), _), 32); 
                    RP.Const (RP.Bits arg3171)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3166), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3167), _), 32); 
                    RP.Const (RP.Bits arg3168)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3163), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3164), _), 32); 
                    RP.Const (RP.Bits arg3165)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3160), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3161), _), 32); 
                  RP.Const (RP.Bits arg3162)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3159)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3159 13 && Bitops.eq arg3222 
            (Bits.U.of_int 8 5) && Bitops.eq arg3223 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3220 (Bits.U.of_int 9 5) && Bitops.eq arg3221 
            (Bits.U.of_int 25 5) && Bitops.eq arg3218 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3219 (Bits.U.of_int 26 5) && Bitops.eq arg3216 
            (Bits.U.of_int 11 5) && Bitops.eq arg3217 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3214 (Bits.U.of_int 12 5) && Bitops.eq arg3215 
            (Bits.U.of_int 28 5) && Bitops.eq arg3212 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3213 (Bits.U.of_int 29 5) && Bitops.eq arg3210 
            (Bits.U.of_int 14 5) && Bitops.eq arg3211 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3208 (Bits.U.of_int 15 5) && Bitops.eq arg3209 
            (Bits.U.of_int 31 5) && Bitops.eq arg3205 (Bits.U.of_int 16 5) && 
          Bitops.eq arg3206 (Bits.U.of_int 0 1) && Bitops.eq arg3207 
            (Bits.U.of_int 8 32) && Bitops.eq arg3202 (Bits.U.of_int 17 5) && 
          Bitops.eq arg3203 (Bits.U.of_int 0 1) && Bitops.eq arg3204 
            (Bits.U.of_int 7 32) && Bitops.eq arg3199 (Bits.U.of_int 18 5) && 
          Bitops.eq arg3200 (Bits.U.of_int 0 1) && Bitops.eq arg3201 
            (Bits.U.of_int 6 32) && Bitops.eq arg3196 (Bits.U.of_int 19 5) && 
          Bitops.eq arg3197 (Bits.U.of_int 0 1) && Bitops.eq arg3198 
            (Bits.U.of_int 5 32) && Bitops.eq arg3193 (Bits.U.of_int 21 5) && 
          Bitops.eq arg3194 (Bits.U.of_int 0 1) && Bitops.eq arg3195 
            (Bits.U.of_int 3 32) && Bitops.eq arg3190 (Bits.U.of_int 22 5) && 
          Bitops.eq arg3191 (Bits.U.of_int 0 1) && Bitops.eq arg3192 
            (Bits.U.of_int 2 32) && Bitops.eq arg3187 (Bits.U.of_int 23 5) && 
          Bitops.eq arg3188 (Bits.U.of_int 0 1) && Bitops.eq arg3189 
            (Bits.U.of_int 1 32) && Bitops.eq arg3184 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3185 (Bits.U.of_int 0 1) && Bitops.eq arg3186 
            (Bits.U.of_int 16 32) && Bitops.eq arg3181 (Bits.U.of_int 25 5) && 
          Bitops.eq arg3182 (Bits.U.of_int 0 1) && Bitops.eq arg3183 
            (Bits.U.of_int 15 32) && Bitops.eq arg3178 (Bits.U.of_int 26 5) && 
          Bitops.eq arg3179 (Bits.U.of_int 0 1) && Bitops.eq arg3180 
            (Bits.U.of_int 14 32) && Bitops.eq arg3175 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3176 (Bits.U.of_int 0 1) && Bitops.eq arg3177 
            (Bits.U.of_int 13 32) && Bitops.eq arg3172 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3173 (Bits.U.of_int 0 1) && Bitops.eq arg3174 
            (Bits.U.of_int 12 32) && Bitops.eq arg3169 (Bits.U.of_int 29 5) && 
          Bitops.eq arg3170 (Bits.U.of_int 0 1) && Bitops.eq arg3171 
            (Bits.U.of_int 11 32) && Bitops.eq arg3166 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3167 (Bits.U.of_int 0 1) && Bitops.eq arg3168 
            (Bits.U.of_int 10 32) && Bitops.eq arg3163 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3164 (Bits.U.of_int 0 1) && Bitops.eq arg3165 
            (Bits.U.of_int 9 32) && Bitops.eq arg3160 (Bits.U.of_int 0 1) && 
          Bitops.eq arg3161 (Bits.U.of_int 0 1) && Bitops.eq arg3162 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 20 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3159)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3287), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3288), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3285), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3286), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3283), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3284), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3281), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3282), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3279), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3280), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3277), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3278), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3275), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3276), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3273), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3274), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3270), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3271), _), 32); 
                    RP.Const (RP.Bits arg3272)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3267), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3268), _), 32); 
                    RP.Const (RP.Bits arg3269)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3264), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3265), _), 32); 
                    RP.Const (RP.Bits arg3266)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3261), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3262), _), 32); 
                    RP.Const (RP.Bits arg3263)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3258), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3259), _), 32); 
                    RP.Const (RP.Bits arg3260)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3255), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3256), _), 32); 
                    RP.Const (RP.Bits arg3257)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3252), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3253), _), 32); 
                    RP.Const (RP.Bits arg3254)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3249), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3250), _), 32); 
                    RP.Const (RP.Bits arg3251)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3246), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3247), _), 32); 
                    RP.Const (RP.Bits arg3248)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3243), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3244), _), 32); 
                    RP.Const (RP.Bits arg3245)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3240), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3241), _), 32); 
                    RP.Const (RP.Bits arg3242)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3237), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3238), _), 32); 
                    RP.Const (RP.Bits arg3239)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3234), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3235), _), 32); 
                    RP.Const (RP.Bits arg3236)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3231), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3232), _), 32); 
                    RP.Const (RP.Bits arg3233)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3228), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3229), _), 32); 
                    RP.Const (RP.Bits arg3230)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3225), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3226), _), 32); 
                  RP.Const (RP.Bits arg3227)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3224)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3224 13 && Bitops.eq arg3287 
            (Bits.U.of_int 8 5) && Bitops.eq arg3288 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3285 (Bits.U.of_int 9 5) && Bitops.eq arg3286 
            (Bits.U.of_int 25 5) && Bitops.eq arg3283 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3284 (Bits.U.of_int 26 5) && Bitops.eq arg3281 
            (Bits.U.of_int 11 5) && Bitops.eq arg3282 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3279 (Bits.U.of_int 12 5) && Bitops.eq arg3280 
            (Bits.U.of_int 28 5) && Bitops.eq arg3277 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3278 (Bits.U.of_int 29 5) && Bitops.eq arg3275 
            (Bits.U.of_int 14 5) && Bitops.eq arg3276 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3273 (Bits.U.of_int 15 5) && Bitops.eq arg3274 
            (Bits.U.of_int 31 5) && Bitops.eq arg3270 (Bits.U.of_int 16 5) && 
          Bitops.eq arg3271 (Bits.U.of_int 0 1) && Bitops.eq arg3272 
            (Bits.U.of_int 8 32) && Bitops.eq arg3267 (Bits.U.of_int 17 5) && 
          Bitops.eq arg3268 (Bits.U.of_int 0 1) && Bitops.eq arg3269 
            (Bits.U.of_int 7 32) && Bitops.eq arg3264 (Bits.U.of_int 18 5) && 
          Bitops.eq arg3265 (Bits.U.of_int 0 1) && Bitops.eq arg3266 
            (Bits.U.of_int 6 32) && Bitops.eq arg3261 (Bits.U.of_int 20 5) && 
          Bitops.eq arg3262 (Bits.U.of_int 0 1) && Bitops.eq arg3263 
            (Bits.U.of_int 4 32) && Bitops.eq arg3258 (Bits.U.of_int 21 5) && 
          Bitops.eq arg3259 (Bits.U.of_int 0 1) && Bitops.eq arg3260 
            (Bits.U.of_int 3 32) && Bitops.eq arg3255 (Bits.U.of_int 22 5) && 
          Bitops.eq arg3256 (Bits.U.of_int 0 1) && Bitops.eq arg3257 
            (Bits.U.of_int 2 32) && Bitops.eq arg3252 (Bits.U.of_int 23 5) && 
          Bitops.eq arg3253 (Bits.U.of_int 0 1) && Bitops.eq arg3254 
            (Bits.U.of_int 1 32) && Bitops.eq arg3249 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3250 (Bits.U.of_int 0 1) && Bitops.eq arg3251 
            (Bits.U.of_int 16 32) && Bitops.eq arg3246 (Bits.U.of_int 25 5) && 
          Bitops.eq arg3247 (Bits.U.of_int 0 1) && Bitops.eq arg3248 
            (Bits.U.of_int 15 32) && Bitops.eq arg3243 (Bits.U.of_int 26 5) && 
          Bitops.eq arg3244 (Bits.U.of_int 0 1) && Bitops.eq arg3245 
            (Bits.U.of_int 14 32) && Bitops.eq arg3240 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3241 (Bits.U.of_int 0 1) && Bitops.eq arg3242 
            (Bits.U.of_int 13 32) && Bitops.eq arg3237 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3238 (Bits.U.of_int 0 1) && Bitops.eq arg3239 
            (Bits.U.of_int 12 32) && Bitops.eq arg3234 (Bits.U.of_int 29 5) && 
          Bitops.eq arg3235 (Bits.U.of_int 0 1) && Bitops.eq arg3236 
            (Bits.U.of_int 11 32) && Bitops.eq arg3231 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3232 (Bits.U.of_int 0 1) && Bitops.eq arg3233 
            (Bits.U.of_int 10 32) && Bitops.eq arg3228 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3229 (Bits.U.of_int 0 1) && Bitops.eq arg3230 
            (Bits.U.of_int 9 32) && Bitops.eq arg3225 (Bits.U.of_int 0 1) && 
          Bitops.eq arg3226 (Bits.U.of_int 0 1) && Bitops.eq arg3227 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 19 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3224)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3352), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3353), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3350), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3351), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3348), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3349), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3346), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3347), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3344), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3345), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3342), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3343), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3340), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3341), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3338), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3339), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3335), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3336), _), 32); 
                    RP.Const (RP.Bits arg3337)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3332), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3333), _), 32); 
                    RP.Const (RP.Bits arg3334)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3329), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3330), _), 32); 
                    RP.Const (RP.Bits arg3331)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3326), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3327), _), 32); 
                    RP.Const (RP.Bits arg3328)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3323), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3324), _), 32); 
                    RP.Const (RP.Bits arg3325)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3320), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3321), _), 32); 
                    RP.Const (RP.Bits arg3322)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3317), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3318), _), 32); 
                    RP.Const (RP.Bits arg3319)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3314), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3315), _), 32); 
                    RP.Const (RP.Bits arg3316)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3311), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3312), _), 32); 
                    RP.Const (RP.Bits arg3313)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3308), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3309), _), 32); 
                    RP.Const (RP.Bits arg3310)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3305), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3306), _), 32); 
                    RP.Const (RP.Bits arg3307)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3302), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3303), _), 32); 
                    RP.Const (RP.Bits arg3304)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3299), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3300), _), 32); 
                    RP.Const (RP.Bits arg3301)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3296), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3297), _), 32); 
                    RP.Const (RP.Bits arg3298)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3293), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3294), _), 32); 
                    RP.Const (RP.Bits arg3295)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3290), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3291), _), 32); 
                  RP.Const (RP.Bits arg3292)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3289)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3289 13 && Bitops.eq arg3352 
            (Bits.U.of_int 8 5) && Bitops.eq arg3353 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3350 (Bits.U.of_int 9 5) && Bitops.eq arg3351 
            (Bits.U.of_int 25 5) && Bitops.eq arg3348 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3349 (Bits.U.of_int 26 5) && Bitops.eq arg3346 
            (Bits.U.of_int 11 5) && Bitops.eq arg3347 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3344 (Bits.U.of_int 12 5) && Bitops.eq arg3345 
            (Bits.U.of_int 28 5) && Bitops.eq arg3342 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3343 (Bits.U.of_int 29 5) && Bitops.eq arg3340 
            (Bits.U.of_int 14 5) && Bitops.eq arg3341 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3338 (Bits.U.of_int 15 5) && Bitops.eq arg3339 
            (Bits.U.of_int 31 5) && Bitops.eq arg3335 (Bits.U.of_int 16 5) && 
          Bitops.eq arg3336 (Bits.U.of_int 0 1) && Bitops.eq arg3337 
            (Bits.U.of_int 8 32) && Bitops.eq arg3332 (Bits.U.of_int 17 5) && 
          Bitops.eq arg3333 (Bits.U.of_int 0 1) && Bitops.eq arg3334 
            (Bits.U.of_int 7 32) && Bitops.eq arg3329 (Bits.U.of_int 19 5) && 
          Bitops.eq arg3330 (Bits.U.of_int 0 1) && Bitops.eq arg3331 
            (Bits.U.of_int 5 32) && Bitops.eq arg3326 (Bits.U.of_int 20 5) && 
          Bitops.eq arg3327 (Bits.U.of_int 0 1) && Bitops.eq arg3328 
            (Bits.U.of_int 4 32) && Bitops.eq arg3323 (Bits.U.of_int 21 5) && 
          Bitops.eq arg3324 (Bits.U.of_int 0 1) && Bitops.eq arg3325 
            (Bits.U.of_int 3 32) && Bitops.eq arg3320 (Bits.U.of_int 22 5) && 
          Bitops.eq arg3321 (Bits.U.of_int 0 1) && Bitops.eq arg3322 
            (Bits.U.of_int 2 32) && Bitops.eq arg3317 (Bits.U.of_int 23 5) && 
          Bitops.eq arg3318 (Bits.U.of_int 0 1) && Bitops.eq arg3319 
            (Bits.U.of_int 1 32) && Bitops.eq arg3314 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3315 (Bits.U.of_int 0 1) && Bitops.eq arg3316 
            (Bits.U.of_int 16 32) && Bitops.eq arg3311 (Bits.U.of_int 25 5) && 
          Bitops.eq arg3312 (Bits.U.of_int 0 1) && Bitops.eq arg3313 
            (Bits.U.of_int 15 32) && Bitops.eq arg3308 (Bits.U.of_int 26 5) && 
          Bitops.eq arg3309 (Bits.U.of_int 0 1) && Bitops.eq arg3310 
            (Bits.U.of_int 14 32) && Bitops.eq arg3305 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3306 (Bits.U.of_int 0 1) && Bitops.eq arg3307 
            (Bits.U.of_int 13 32) && Bitops.eq arg3302 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3303 (Bits.U.of_int 0 1) && Bitops.eq arg3304 
            (Bits.U.of_int 12 32) && Bitops.eq arg3299 (Bits.U.of_int 29 5) && 
          Bitops.eq arg3300 (Bits.U.of_int 0 1) && Bitops.eq arg3301 
            (Bits.U.of_int 11 32) && Bitops.eq arg3296 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3297 (Bits.U.of_int 0 1) && Bitops.eq arg3298 
            (Bits.U.of_int 10 32) && Bitops.eq arg3293 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3294 (Bits.U.of_int 0 1) && Bitops.eq arg3295 
            (Bits.U.of_int 9 32) && Bitops.eq arg3290 (Bits.U.of_int 0 1) && 
          Bitops.eq arg3291 (Bits.U.of_int 0 1) && Bitops.eq arg3292 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 18 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3289)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3417), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3418), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3415), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3416), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3413), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3414), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3411), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3412), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3409), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3410), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3407), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3408), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3405), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3406), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3403), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3404), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3400), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3401), _), 32); 
                    RP.Const (RP.Bits arg3402)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3397), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3398), _), 32); 
                    RP.Const (RP.Bits arg3399)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3394), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3395), _), 32); 
                    RP.Const (RP.Bits arg3396)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3391), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3392), _), 32); 
                    RP.Const (RP.Bits arg3393)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3388), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3389), _), 32); 
                    RP.Const (RP.Bits arg3390)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3385), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3386), _), 32); 
                    RP.Const (RP.Bits arg3387)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3382), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3383), _), 32); 
                    RP.Const (RP.Bits arg3384)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3379), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3380), _), 32); 
                    RP.Const (RP.Bits arg3381)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3376), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3377), _), 32); 
                    RP.Const (RP.Bits arg3378)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3373), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3374), _), 32); 
                    RP.Const (RP.Bits arg3375)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3370), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3371), _), 32); 
                    RP.Const (RP.Bits arg3372)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3367), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3368), _), 32); 
                    RP.Const (RP.Bits arg3369)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3364), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3365), _), 32); 
                    RP.Const (RP.Bits arg3366)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3361), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3362), _), 32); 
                    RP.Const (RP.Bits arg3363)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3358), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3359), _), 32); 
                    RP.Const (RP.Bits arg3360)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3355), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3356), _), 32); 
                  RP.Const (RP.Bits arg3357)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3354)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3354 13 && Bitops.eq arg3417 
            (Bits.U.of_int 8 5) && Bitops.eq arg3418 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3415 (Bits.U.of_int 9 5) && Bitops.eq arg3416 
            (Bits.U.of_int 25 5) && Bitops.eq arg3413 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3414 (Bits.U.of_int 26 5) && Bitops.eq arg3411 
            (Bits.U.of_int 11 5) && Bitops.eq arg3412 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3409 (Bits.U.of_int 12 5) && Bitops.eq arg3410 
            (Bits.U.of_int 28 5) && Bitops.eq arg3407 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3408 (Bits.U.of_int 29 5) && Bitops.eq arg3405 
            (Bits.U.of_int 14 5) && Bitops.eq arg3406 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3403 (Bits.U.of_int 15 5) && Bitops.eq arg3404 
            (Bits.U.of_int 31 5) && Bitops.eq arg3400 (Bits.U.of_int 16 5) && 
          Bitops.eq arg3401 (Bits.U.of_int 0 1) && Bitops.eq arg3402 
            (Bits.U.of_int 8 32) && Bitops.eq arg3397 (Bits.U.of_int 18 5) && 
          Bitops.eq arg3398 (Bits.U.of_int 0 1) && Bitops.eq arg3399 
            (Bits.U.of_int 6 32) && Bitops.eq arg3394 (Bits.U.of_int 19 5) && 
          Bitops.eq arg3395 (Bits.U.of_int 0 1) && Bitops.eq arg3396 
            (Bits.U.of_int 5 32) && Bitops.eq arg3391 (Bits.U.of_int 20 5) && 
          Bitops.eq arg3392 (Bits.U.of_int 0 1) && Bitops.eq arg3393 
            (Bits.U.of_int 4 32) && Bitops.eq arg3388 (Bits.U.of_int 21 5) && 
          Bitops.eq arg3389 (Bits.U.of_int 0 1) && Bitops.eq arg3390 
            (Bits.U.of_int 3 32) && Bitops.eq arg3385 (Bits.U.of_int 22 5) && 
          Bitops.eq arg3386 (Bits.U.of_int 0 1) && Bitops.eq arg3387 
            (Bits.U.of_int 2 32) && Bitops.eq arg3382 (Bits.U.of_int 23 5) && 
          Bitops.eq arg3383 (Bits.U.of_int 0 1) && Bitops.eq arg3384 
            (Bits.U.of_int 1 32) && Bitops.eq arg3379 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3380 (Bits.U.of_int 0 1) && Bitops.eq arg3381 
            (Bits.U.of_int 16 32) && Bitops.eq arg3376 (Bits.U.of_int 25 5) && 
          Bitops.eq arg3377 (Bits.U.of_int 0 1) && Bitops.eq arg3378 
            (Bits.U.of_int 15 32) && Bitops.eq arg3373 (Bits.U.of_int 26 5) && 
          Bitops.eq arg3374 (Bits.U.of_int 0 1) && Bitops.eq arg3375 
            (Bits.U.of_int 14 32) && Bitops.eq arg3370 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3371 (Bits.U.of_int 0 1) && Bitops.eq arg3372 
            (Bits.U.of_int 13 32) && Bitops.eq arg3367 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3368 (Bits.U.of_int 0 1) && Bitops.eq arg3369 
            (Bits.U.of_int 12 32) && Bitops.eq arg3364 (Bits.U.of_int 29 5) && 
          Bitops.eq arg3365 (Bits.U.of_int 0 1) && Bitops.eq arg3366 
            (Bits.U.of_int 11 32) && Bitops.eq arg3361 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3362 (Bits.U.of_int 0 1) && Bitops.eq arg3363 
            (Bits.U.of_int 10 32) && Bitops.eq arg3358 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3359 (Bits.U.of_int 0 1) && Bitops.eq arg3360 
            (Bits.U.of_int 9 32) && Bitops.eq arg3355 (Bits.U.of_int 0 1) && 
          Bitops.eq arg3356 (Bits.U.of_int 0 1) && Bitops.eq arg3357 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 17 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3354)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3482), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3483), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3480), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3481), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3478), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3479), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3476), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3477), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3474), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3475), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3472), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3473), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3470), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3471), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3468), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3469), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3465), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3466), _), 32); 
                    RP.Const (RP.Bits arg3467)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3462), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3463), _), 32); 
                    RP.Const (RP.Bits arg3464)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3459), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3460), _), 32); 
                    RP.Const (RP.Bits arg3461)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3456), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3457), _), 32); 
                    RP.Const (RP.Bits arg3458)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3453), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3454), _), 32); 
                    RP.Const (RP.Bits arg3455)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3450), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3451), _), 32); 
                    RP.Const (RP.Bits arg3452)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3447), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3448), _), 32); 
                    RP.Const (RP.Bits arg3449)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3444), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3445), _), 32); 
                    RP.Const (RP.Bits arg3446)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3441), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3442), _), 32); 
                    RP.Const (RP.Bits arg3443)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3438), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3439), _), 32); 
                    RP.Const (RP.Bits arg3440)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3435), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3436), _), 32); 
                    RP.Const (RP.Bits arg3437)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3432), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3433), _), 32); 
                    RP.Const (RP.Bits arg3434)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3429), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3430), _), 32); 
                    RP.Const (RP.Bits arg3431)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3426), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3427), _), 32); 
                    RP.Const (RP.Bits arg3428)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3423), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3424), _), 32); 
                    RP.Const (RP.Bits arg3425)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3420), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3421), _), 32); 
                  RP.Const (RP.Bits arg3422)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3419)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3419 13 && Bitops.eq arg3482 
            (Bits.U.of_int 8 5) && Bitops.eq arg3483 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3480 (Bits.U.of_int 9 5) && Bitops.eq arg3481 
            (Bits.U.of_int 25 5) && Bitops.eq arg3478 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3479 (Bits.U.of_int 26 5) && Bitops.eq arg3476 
            (Bits.U.of_int 11 5) && Bitops.eq arg3477 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3474 (Bits.U.of_int 12 5) && Bitops.eq arg3475 
            (Bits.U.of_int 28 5) && Bitops.eq arg3472 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3473 (Bits.U.of_int 29 5) && Bitops.eq arg3470 
            (Bits.U.of_int 14 5) && Bitops.eq arg3471 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3468 (Bits.U.of_int 15 5) && Bitops.eq arg3469 
            (Bits.U.of_int 31 5) && Bitops.eq arg3465 (Bits.U.of_int 17 5) && 
          Bitops.eq arg3466 (Bits.U.of_int 0 1) && Bitops.eq arg3467 
            (Bits.U.of_int 7 32) && Bitops.eq arg3462 (Bits.U.of_int 18 5) && 
          Bitops.eq arg3463 (Bits.U.of_int 0 1) && Bitops.eq arg3464 
            (Bits.U.of_int 6 32) && Bitops.eq arg3459 (Bits.U.of_int 19 5) && 
          Bitops.eq arg3460 (Bits.U.of_int 0 1) && Bitops.eq arg3461 
            (Bits.U.of_int 5 32) && Bitops.eq arg3456 (Bits.U.of_int 20 5) && 
          Bitops.eq arg3457 (Bits.U.of_int 0 1) && Bitops.eq arg3458 
            (Bits.U.of_int 4 32) && Bitops.eq arg3453 (Bits.U.of_int 21 5) && 
          Bitops.eq arg3454 (Bits.U.of_int 0 1) && Bitops.eq arg3455 
            (Bits.U.of_int 3 32) && Bitops.eq arg3450 (Bits.U.of_int 22 5) && 
          Bitops.eq arg3451 (Bits.U.of_int 0 1) && Bitops.eq arg3452 
            (Bits.U.of_int 2 32) && Bitops.eq arg3447 (Bits.U.of_int 23 5) && 
          Bitops.eq arg3448 (Bits.U.of_int 0 1) && Bitops.eq arg3449 
            (Bits.U.of_int 1 32) && Bitops.eq arg3444 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3445 (Bits.U.of_int 0 1) && Bitops.eq arg3446 
            (Bits.U.of_int 16 32) && Bitops.eq arg3441 (Bits.U.of_int 25 5) && 
          Bitops.eq arg3442 (Bits.U.of_int 0 1) && Bitops.eq arg3443 
            (Bits.U.of_int 15 32) && Bitops.eq arg3438 (Bits.U.of_int 26 5) && 
          Bitops.eq arg3439 (Bits.U.of_int 0 1) && Bitops.eq arg3440 
            (Bits.U.of_int 14 32) && Bitops.eq arg3435 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3436 (Bits.U.of_int 0 1) && Bitops.eq arg3437 
            (Bits.U.of_int 13 32) && Bitops.eq arg3432 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3433 (Bits.U.of_int 0 1) && Bitops.eq arg3434 
            (Bits.U.of_int 12 32) && Bitops.eq arg3429 (Bits.U.of_int 29 5) && 
          Bitops.eq arg3430 (Bits.U.of_int 0 1) && Bitops.eq arg3431 
            (Bits.U.of_int 11 32) && Bitops.eq arg3426 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3427 (Bits.U.of_int 0 1) && Bitops.eq arg3428 
            (Bits.U.of_int 10 32) && Bitops.eq arg3423 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3424 (Bits.U.of_int 0 1) && Bitops.eq arg3425 
            (Bits.U.of_int 9 32) && Bitops.eq arg3420 (Bits.U.of_int 0 1) && 
          Bitops.eq arg3421 (Bits.U.of_int 0 1) && Bitops.eq arg3422 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 16 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3419)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3548), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3549), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3546), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3547), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3544), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3545), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3542), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3543), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3540), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3541), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3538), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3539), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3536), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3537), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3533), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3534), _), 32); 
                    RP.Const (RP.Bits arg3535)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3530), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3531), _), 32); 
                    RP.Const (RP.Bits arg3532)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3527), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3528), _), 32); 
                    RP.Const (RP.Bits arg3529)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3524), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3525), _), 32); 
                    RP.Const (RP.Bits arg3526)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3521), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3522), _), 32); 
                    RP.Const (RP.Bits arg3523)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3518), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3519), _), 32); 
                    RP.Const (RP.Bits arg3520)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3515), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3516), _), 32); 
                    RP.Const (RP.Bits arg3517)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3512), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3513), _), 32); 
                    RP.Const (RP.Bits arg3514)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3509), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3510), _), 32); 
                    RP.Const (RP.Bits arg3511)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3506), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3507), _), 32); 
                    RP.Const (RP.Bits arg3508)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3503), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3504), _), 32); 
                    RP.Const (RP.Bits arg3505)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3500), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3501), _), 32); 
                    RP.Const (RP.Bits arg3502)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3497), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3498), _), 32); 
                    RP.Const (RP.Bits arg3499)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3494), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3495), _), 32); 
                    RP.Const (RP.Bits arg3496)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3491), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3492), _), 32); 
                    RP.Const (RP.Bits arg3493)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3488), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3489), _), 32); 
                    RP.Const (RP.Bits arg3490)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3485), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3486), _), 32); 
                  RP.Const (RP.Bits arg3487)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3484)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3484 13 && Bitops.eq arg3548 
            (Bits.U.of_int 8 5) && Bitops.eq arg3549 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3546 (Bits.U.of_int 9 5) && Bitops.eq arg3547 
            (Bits.U.of_int 25 5) && Bitops.eq arg3544 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3545 (Bits.U.of_int 26 5) && Bitops.eq arg3542 
            (Bits.U.of_int 11 5) && Bitops.eq arg3543 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3540 (Bits.U.of_int 12 5) && Bitops.eq arg3541 
            (Bits.U.of_int 28 5) && Bitops.eq arg3538 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3539 (Bits.U.of_int 29 5) && Bitops.eq arg3536 
            (Bits.U.of_int 14 5) && Bitops.eq arg3537 (Bits.U.of_int 30 5) && 
          Bitops.eq arg3533 (Bits.U.of_int 16 5) && Bitops.eq arg3534 
            (Bits.U.of_int 0 1) && Bitops.eq arg3535 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3530 (Bits.U.of_int 17 5) && Bitops.eq arg3531 
            (Bits.U.of_int 0 1) && Bitops.eq arg3532 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3527 (Bits.U.of_int 18 5) && Bitops.eq arg3528 
            (Bits.U.of_int 0 1) && Bitops.eq arg3529 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3524 (Bits.U.of_int 19 5) && Bitops.eq arg3525 
            (Bits.U.of_int 0 1) && Bitops.eq arg3526 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3521 (Bits.U.of_int 20 5) && Bitops.eq arg3522 
            (Bits.U.of_int 0 1) && Bitops.eq arg3523 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3518 (Bits.U.of_int 21 5) && Bitops.eq arg3519 
            (Bits.U.of_int 0 1) && Bitops.eq arg3520 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3515 (Bits.U.of_int 22 5) && Bitops.eq arg3516 
            (Bits.U.of_int 0 1) && Bitops.eq arg3517 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3512 (Bits.U.of_int 23 5) && Bitops.eq arg3513 
            (Bits.U.of_int 0 1) && Bitops.eq arg3514 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3509 (Bits.U.of_int 24 5) && Bitops.eq arg3510 
            (Bits.U.of_int 0 1) && Bitops.eq arg3511 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3506 (Bits.U.of_int 25 5) && Bitops.eq arg3507 
            (Bits.U.of_int 0 1) && Bitops.eq arg3508 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3503 (Bits.U.of_int 26 5) && Bitops.eq arg3504 
            (Bits.U.of_int 0 1) && Bitops.eq arg3505 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3500 (Bits.U.of_int 27 5) && Bitops.eq arg3501 
            (Bits.U.of_int 0 1) && Bitops.eq arg3502 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3497 (Bits.U.of_int 28 5) && Bitops.eq arg3498 
            (Bits.U.of_int 0 1) && Bitops.eq arg3499 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3494 (Bits.U.of_int 29 5) && Bitops.eq arg3495 
            (Bits.U.of_int 0 1) && Bitops.eq arg3496 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3491 (Bits.U.of_int 30 5) && Bitops.eq arg3492 
            (Bits.U.of_int 0 1) && Bitops.eq arg3493 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3488 (Bits.U.of_int 31 5) && Bitops.eq arg3489 
            (Bits.U.of_int 0 1) && Bitops.eq arg3490 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3485 (Bits.U.of_int 0 1) && Bitops.eq arg3486 
            (Bits.U.of_int 0 1) && Bitops.eq arg3487 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 15 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3484)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3614), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3615), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3612), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3613), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3610), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3611), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3608), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3609), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3606), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3607), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3604), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3605), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3602), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3603), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3599), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3600), _), 32); 
                    RP.Const (RP.Bits arg3601)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3596), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3597), _), 32); 
                    RP.Const (RP.Bits arg3598)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3593), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3594), _), 32); 
                    RP.Const (RP.Bits arg3595)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3590), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3591), _), 32); 
                    RP.Const (RP.Bits arg3592)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3587), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3588), _), 32); 
                    RP.Const (RP.Bits arg3589)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3584), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3585), _), 32); 
                    RP.Const (RP.Bits arg3586)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3581), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3582), _), 32); 
                    RP.Const (RP.Bits arg3583)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3578), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3579), _), 32); 
                    RP.Const (RP.Bits arg3580)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3575), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3576), _), 32); 
                    RP.Const (RP.Bits arg3577)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3572), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3573), _), 32); 
                    RP.Const (RP.Bits arg3574)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3569), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3570), _), 32); 
                    RP.Const (RP.Bits arg3571)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3566), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3567), _), 32); 
                    RP.Const (RP.Bits arg3568)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3563), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3564), _), 32); 
                    RP.Const (RP.Bits arg3565)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3560), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3561), _), 32); 
                    RP.Const (RP.Bits arg3562)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3557), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3558), _), 32); 
                    RP.Const (RP.Bits arg3559)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3554), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3555), _), 32); 
                    RP.Const (RP.Bits arg3556)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3551), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3552), _), 32); 
                  RP.Const (RP.Bits arg3553)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3550)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3550 13 && Bitops.eq arg3614 
            (Bits.U.of_int 8 5) && Bitops.eq arg3615 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3612 (Bits.U.of_int 9 5) && Bitops.eq arg3613 
            (Bits.U.of_int 25 5) && Bitops.eq arg3610 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3611 (Bits.U.of_int 26 5) && Bitops.eq arg3608 
            (Bits.U.of_int 11 5) && Bitops.eq arg3609 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3606 (Bits.U.of_int 12 5) && Bitops.eq arg3607 
            (Bits.U.of_int 28 5) && Bitops.eq arg3604 (Bits.U.of_int 13 5) && 
          Bitops.eq arg3605 (Bits.U.of_int 29 5) && Bitops.eq arg3602 
            (Bits.U.of_int 15 5) && Bitops.eq arg3603 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3599 (Bits.U.of_int 16 5) && Bitops.eq arg3600 
            (Bits.U.of_int 0 1) && Bitops.eq arg3601 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3596 (Bits.U.of_int 17 5) && Bitops.eq arg3597 
            (Bits.U.of_int 0 1) && Bitops.eq arg3598 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3593 (Bits.U.of_int 18 5) && Bitops.eq arg3594 
            (Bits.U.of_int 0 1) && Bitops.eq arg3595 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3590 (Bits.U.of_int 19 5) && Bitops.eq arg3591 
            (Bits.U.of_int 0 1) && Bitops.eq arg3592 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3587 (Bits.U.of_int 20 5) && Bitops.eq arg3588 
            (Bits.U.of_int 0 1) && Bitops.eq arg3589 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3584 (Bits.U.of_int 21 5) && Bitops.eq arg3585 
            (Bits.U.of_int 0 1) && Bitops.eq arg3586 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3581 (Bits.U.of_int 22 5) && Bitops.eq arg3582 
            (Bits.U.of_int 0 1) && Bitops.eq arg3583 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3578 (Bits.U.of_int 23 5) && Bitops.eq arg3579 
            (Bits.U.of_int 0 1) && Bitops.eq arg3580 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3575 (Bits.U.of_int 24 5) && Bitops.eq arg3576 
            (Bits.U.of_int 0 1) && Bitops.eq arg3577 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3572 (Bits.U.of_int 25 5) && Bitops.eq arg3573 
            (Bits.U.of_int 0 1) && Bitops.eq arg3574 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3569 (Bits.U.of_int 26 5) && Bitops.eq arg3570 
            (Bits.U.of_int 0 1) && Bitops.eq arg3571 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3566 (Bits.U.of_int 27 5) && Bitops.eq arg3567 
            (Bits.U.of_int 0 1) && Bitops.eq arg3568 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3563 (Bits.U.of_int 28 5) && Bitops.eq arg3564 
            (Bits.U.of_int 0 1) && Bitops.eq arg3565 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3560 (Bits.U.of_int 29 5) && Bitops.eq arg3561 
            (Bits.U.of_int 0 1) && Bitops.eq arg3562 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3557 (Bits.U.of_int 30 5) && Bitops.eq arg3558 
            (Bits.U.of_int 0 1) && Bitops.eq arg3559 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3554 (Bits.U.of_int 31 5) && Bitops.eq arg3555 
            (Bits.U.of_int 0 1) && Bitops.eq arg3556 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3551 (Bits.U.of_int 0 1) && Bitops.eq arg3552 
            (Bits.U.of_int 0 1) && Bitops.eq arg3553 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 14 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3550)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3680), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3681), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3678), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3679), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3676), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3677), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3674), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3675), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3672), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3673), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3670), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3671), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3668), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3669), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3665), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3666), _), 32); 
                    RP.Const (RP.Bits arg3667)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3662), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3663), _), 32); 
                    RP.Const (RP.Bits arg3664)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3659), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3660), _), 32); 
                    RP.Const (RP.Bits arg3661)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3656), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3657), _), 32); 
                    RP.Const (RP.Bits arg3658)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3653), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3654), _), 32); 
                    RP.Const (RP.Bits arg3655)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3650), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3651), _), 32); 
                    RP.Const (RP.Bits arg3652)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3647), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3648), _), 32); 
                    RP.Const (RP.Bits arg3649)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3644), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3645), _), 32); 
                    RP.Const (RP.Bits arg3646)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3641), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3642), _), 32); 
                    RP.Const (RP.Bits arg3643)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3638), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3639), _), 32); 
                    RP.Const (RP.Bits arg3640)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3635), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3636), _), 32); 
                    RP.Const (RP.Bits arg3637)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3632), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3633), _), 32); 
                    RP.Const (RP.Bits arg3634)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3629), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3630), _), 32); 
                    RP.Const (RP.Bits arg3631)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3626), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3627), _), 32); 
                    RP.Const (RP.Bits arg3628)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3623), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3624), _), 32); 
                    RP.Const (RP.Bits arg3625)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3620), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3621), _), 32); 
                    RP.Const (RP.Bits arg3622)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3617), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3618), _), 32); 
                  RP.Const (RP.Bits arg3619)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3616)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3616 13 && Bitops.eq arg3680 
            (Bits.U.of_int 8 5) && Bitops.eq arg3681 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3678 (Bits.U.of_int 9 5) && Bitops.eq arg3679 
            (Bits.U.of_int 25 5) && Bitops.eq arg3676 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3677 (Bits.U.of_int 26 5) && Bitops.eq arg3674 
            (Bits.U.of_int 11 5) && Bitops.eq arg3675 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3672 (Bits.U.of_int 12 5) && Bitops.eq arg3673 
            (Bits.U.of_int 28 5) && Bitops.eq arg3670 (Bits.U.of_int 14 5) && 
          Bitops.eq arg3671 (Bits.U.of_int 30 5) && Bitops.eq arg3668 
            (Bits.U.of_int 15 5) && Bitops.eq arg3669 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3665 (Bits.U.of_int 16 5) && Bitops.eq arg3666 
            (Bits.U.of_int 0 1) && Bitops.eq arg3667 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3662 (Bits.U.of_int 17 5) && Bitops.eq arg3663 
            (Bits.U.of_int 0 1) && Bitops.eq arg3664 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3659 (Bits.U.of_int 18 5) && Bitops.eq arg3660 
            (Bits.U.of_int 0 1) && Bitops.eq arg3661 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3656 (Bits.U.of_int 19 5) && Bitops.eq arg3657 
            (Bits.U.of_int 0 1) && Bitops.eq arg3658 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3653 (Bits.U.of_int 20 5) && Bitops.eq arg3654 
            (Bits.U.of_int 0 1) && Bitops.eq arg3655 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3650 (Bits.U.of_int 21 5) && Bitops.eq arg3651 
            (Bits.U.of_int 0 1) && Bitops.eq arg3652 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3647 (Bits.U.of_int 22 5) && Bitops.eq arg3648 
            (Bits.U.of_int 0 1) && Bitops.eq arg3649 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3644 (Bits.U.of_int 23 5) && Bitops.eq arg3645 
            (Bits.U.of_int 0 1) && Bitops.eq arg3646 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3641 (Bits.U.of_int 24 5) && Bitops.eq arg3642 
            (Bits.U.of_int 0 1) && Bitops.eq arg3643 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3638 (Bits.U.of_int 25 5) && Bitops.eq arg3639 
            (Bits.U.of_int 0 1) && Bitops.eq arg3640 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3635 (Bits.U.of_int 26 5) && Bitops.eq arg3636 
            (Bits.U.of_int 0 1) && Bitops.eq arg3637 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3632 (Bits.U.of_int 27 5) && Bitops.eq arg3633 
            (Bits.U.of_int 0 1) && Bitops.eq arg3634 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3629 (Bits.U.of_int 28 5) && Bitops.eq arg3630 
            (Bits.U.of_int 0 1) && Bitops.eq arg3631 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3626 (Bits.U.of_int 29 5) && Bitops.eq arg3627 
            (Bits.U.of_int 0 1) && Bitops.eq arg3628 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3623 (Bits.U.of_int 30 5) && Bitops.eq arg3624 
            (Bits.U.of_int 0 1) && Bitops.eq arg3625 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3620 (Bits.U.of_int 31 5) && Bitops.eq arg3621 
            (Bits.U.of_int 0 1) && Bitops.eq arg3622 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3617 (Bits.U.of_int 0 1) && Bitops.eq arg3618 
            (Bits.U.of_int 0 1) && Bitops.eq arg3619 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 13 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3616)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3746), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3747), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3744), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3745), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3742), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3743), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3740), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3741), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3738), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3739), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3736), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3737), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3734), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3735), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3731), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3732), _), 32); 
                    RP.Const (RP.Bits arg3733)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3728), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3729), _), 32); 
                    RP.Const (RP.Bits arg3730)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3725), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3726), _), 32); 
                    RP.Const (RP.Bits arg3727)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3722), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3723), _), 32); 
                    RP.Const (RP.Bits arg3724)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3719), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3720), _), 32); 
                    RP.Const (RP.Bits arg3721)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3716), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3717), _), 32); 
                    RP.Const (RP.Bits arg3718)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3713), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3714), _), 32); 
                    RP.Const (RP.Bits arg3715)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3710), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3711), _), 32); 
                    RP.Const (RP.Bits arg3712)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3707), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3708), _), 32); 
                    RP.Const (RP.Bits arg3709)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3704), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3705), _), 32); 
                    RP.Const (RP.Bits arg3706)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3701), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3702), _), 32); 
                    RP.Const (RP.Bits arg3703)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3698), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3699), _), 32); 
                    RP.Const (RP.Bits arg3700)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3695), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3696), _), 32); 
                    RP.Const (RP.Bits arg3697)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3692), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3693), _), 32); 
                    RP.Const (RP.Bits arg3694)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3689), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3690), _), 32); 
                    RP.Const (RP.Bits arg3691)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3686), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3687), _), 32); 
                    RP.Const (RP.Bits arg3688)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3683), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3684), _), 32); 
                  RP.Const (RP.Bits arg3685)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3682)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3682 13 && Bitops.eq arg3746 
            (Bits.U.of_int 8 5) && Bitops.eq arg3747 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3744 (Bits.U.of_int 9 5) && Bitops.eq arg3745 
            (Bits.U.of_int 25 5) && Bitops.eq arg3742 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3743 (Bits.U.of_int 26 5) && Bitops.eq arg3740 
            (Bits.U.of_int 11 5) && Bitops.eq arg3741 (Bits.U.of_int 27 5) && 
          Bitops.eq arg3738 (Bits.U.of_int 13 5) && Bitops.eq arg3739 
            (Bits.U.of_int 29 5) && Bitops.eq arg3736 (Bits.U.of_int 14 5) && 
          Bitops.eq arg3737 (Bits.U.of_int 30 5) && Bitops.eq arg3734 
            (Bits.U.of_int 15 5) && Bitops.eq arg3735 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3731 (Bits.U.of_int 16 5) && Bitops.eq arg3732 
            (Bits.U.of_int 0 1) && Bitops.eq arg3733 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3728 (Bits.U.of_int 17 5) && Bitops.eq arg3729 
            (Bits.U.of_int 0 1) && Bitops.eq arg3730 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3725 (Bits.U.of_int 18 5) && Bitops.eq arg3726 
            (Bits.U.of_int 0 1) && Bitops.eq arg3727 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3722 (Bits.U.of_int 19 5) && Bitops.eq arg3723 
            (Bits.U.of_int 0 1) && Bitops.eq arg3724 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3719 (Bits.U.of_int 20 5) && Bitops.eq arg3720 
            (Bits.U.of_int 0 1) && Bitops.eq arg3721 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3716 (Bits.U.of_int 21 5) && Bitops.eq arg3717 
            (Bits.U.of_int 0 1) && Bitops.eq arg3718 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3713 (Bits.U.of_int 22 5) && Bitops.eq arg3714 
            (Bits.U.of_int 0 1) && Bitops.eq arg3715 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3710 (Bits.U.of_int 23 5) && Bitops.eq arg3711 
            (Bits.U.of_int 0 1) && Bitops.eq arg3712 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3707 (Bits.U.of_int 24 5) && Bitops.eq arg3708 
            (Bits.U.of_int 0 1) && Bitops.eq arg3709 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3704 (Bits.U.of_int 25 5) && Bitops.eq arg3705 
            (Bits.U.of_int 0 1) && Bitops.eq arg3706 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3701 (Bits.U.of_int 26 5) && Bitops.eq arg3702 
            (Bits.U.of_int 0 1) && Bitops.eq arg3703 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3698 (Bits.U.of_int 27 5) && Bitops.eq arg3699 
            (Bits.U.of_int 0 1) && Bitops.eq arg3700 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3695 (Bits.U.of_int 28 5) && Bitops.eq arg3696 
            (Bits.U.of_int 0 1) && Bitops.eq arg3697 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3692 (Bits.U.of_int 29 5) && Bitops.eq arg3693 
            (Bits.U.of_int 0 1) && Bitops.eq arg3694 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3689 (Bits.U.of_int 30 5) && Bitops.eq arg3690 
            (Bits.U.of_int 0 1) && Bitops.eq arg3691 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3686 (Bits.U.of_int 31 5) && Bitops.eq arg3687 
            (Bits.U.of_int 0 1) && Bitops.eq arg3688 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3683 (Bits.U.of_int 0 1) && Bitops.eq arg3684 
            (Bits.U.of_int 0 1) && Bitops.eq arg3685 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 12 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3682)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3812), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3813), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3810), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3811), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3808), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3809), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3806), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3807), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3804), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3805), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3802), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3803), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3800), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3801), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3797), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3798), _), 32); 
                    RP.Const (RP.Bits arg3799)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3794), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3795), _), 32); 
                    RP.Const (RP.Bits arg3796)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3791), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3792), _), 32); 
                    RP.Const (RP.Bits arg3793)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3788), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3789), _), 32); 
                    RP.Const (RP.Bits arg3790)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3785), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3786), _), 32); 
                    RP.Const (RP.Bits arg3787)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3782), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3783), _), 32); 
                    RP.Const (RP.Bits arg3784)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3779), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3780), _), 32); 
                    RP.Const (RP.Bits arg3781)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3776), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3777), _), 32); 
                    RP.Const (RP.Bits arg3778)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3773), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3774), _), 32); 
                    RP.Const (RP.Bits arg3775)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3770), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3771), _), 32); 
                    RP.Const (RP.Bits arg3772)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3767), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3768), _), 32); 
                    RP.Const (RP.Bits arg3769)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3764), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3765), _), 32); 
                    RP.Const (RP.Bits arg3766)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3761), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3762), _), 32); 
                    RP.Const (RP.Bits arg3763)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3758), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3759), _), 32); 
                    RP.Const (RP.Bits arg3760)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3755), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3756), _), 32); 
                    RP.Const (RP.Bits arg3757)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3752), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3753), _), 32); 
                    RP.Const (RP.Bits arg3754)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3749), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3750), _), 32); 
                  RP.Const (RP.Bits arg3751)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3748)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3748 13 && Bitops.eq arg3812 
            (Bits.U.of_int 8 5) && Bitops.eq arg3813 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3810 (Bits.U.of_int 9 5) && Bitops.eq arg3811 
            (Bits.U.of_int 25 5) && Bitops.eq arg3808 (Bits.U.of_int 10 5) && 
          Bitops.eq arg3809 (Bits.U.of_int 26 5) && Bitops.eq arg3806 
            (Bits.U.of_int 12 5) && Bitops.eq arg3807 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3804 (Bits.U.of_int 13 5) && Bitops.eq arg3805 
            (Bits.U.of_int 29 5) && Bitops.eq arg3802 (Bits.U.of_int 14 5) && 
          Bitops.eq arg3803 (Bits.U.of_int 30 5) && Bitops.eq arg3800 
            (Bits.U.of_int 15 5) && Bitops.eq arg3801 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3797 (Bits.U.of_int 16 5) && Bitops.eq arg3798 
            (Bits.U.of_int 0 1) && Bitops.eq arg3799 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3794 (Bits.U.of_int 17 5) && Bitops.eq arg3795 
            (Bits.U.of_int 0 1) && Bitops.eq arg3796 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3791 (Bits.U.of_int 18 5) && Bitops.eq arg3792 
            (Bits.U.of_int 0 1) && Bitops.eq arg3793 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3788 (Bits.U.of_int 19 5) && Bitops.eq arg3789 
            (Bits.U.of_int 0 1) && Bitops.eq arg3790 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3785 (Bits.U.of_int 20 5) && Bitops.eq arg3786 
            (Bits.U.of_int 0 1) && Bitops.eq arg3787 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3782 (Bits.U.of_int 21 5) && Bitops.eq arg3783 
            (Bits.U.of_int 0 1) && Bitops.eq arg3784 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3779 (Bits.U.of_int 22 5) && Bitops.eq arg3780 
            (Bits.U.of_int 0 1) && Bitops.eq arg3781 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3776 (Bits.U.of_int 23 5) && Bitops.eq arg3777 
            (Bits.U.of_int 0 1) && Bitops.eq arg3778 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3773 (Bits.U.of_int 24 5) && Bitops.eq arg3774 
            (Bits.U.of_int 0 1) && Bitops.eq arg3775 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3770 (Bits.U.of_int 25 5) && Bitops.eq arg3771 
            (Bits.U.of_int 0 1) && Bitops.eq arg3772 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3767 (Bits.U.of_int 26 5) && Bitops.eq arg3768 
            (Bits.U.of_int 0 1) && Bitops.eq arg3769 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3764 (Bits.U.of_int 27 5) && Bitops.eq arg3765 
            (Bits.U.of_int 0 1) && Bitops.eq arg3766 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3761 (Bits.U.of_int 28 5) && Bitops.eq arg3762 
            (Bits.U.of_int 0 1) && Bitops.eq arg3763 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3758 (Bits.U.of_int 29 5) && Bitops.eq arg3759 
            (Bits.U.of_int 0 1) && Bitops.eq arg3760 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3755 (Bits.U.of_int 30 5) && Bitops.eq arg3756 
            (Bits.U.of_int 0 1) && Bitops.eq arg3757 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3752 (Bits.U.of_int 31 5) && Bitops.eq arg3753 
            (Bits.U.of_int 0 1) && Bitops.eq arg3754 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3749 (Bits.U.of_int 0 1) && Bitops.eq arg3750 
            (Bits.U.of_int 0 1) && Bitops.eq arg3751 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 11 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3748)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3878), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3879), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3876), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3877), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3874), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3875), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3872), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3873), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3870), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3871), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3868), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3869), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3866), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3867), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3863), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3864), _), 32); 
                    RP.Const (RP.Bits arg3865)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3860), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3861), _), 32); 
                    RP.Const (RP.Bits arg3862)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3857), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3858), _), 32); 
                    RP.Const (RP.Bits arg3859)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3854), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3855), _), 32); 
                    RP.Const (RP.Bits arg3856)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3851), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3852), _), 32); 
                    RP.Const (RP.Bits arg3853)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3848), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3849), _), 32); 
                    RP.Const (RP.Bits arg3850)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3845), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3846), _), 32); 
                    RP.Const (RP.Bits arg3847)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3842), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3843), _), 32); 
                    RP.Const (RP.Bits arg3844)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3839), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3840), _), 32); 
                    RP.Const (RP.Bits arg3841)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3836), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3837), _), 32); 
                    RP.Const (RP.Bits arg3838)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3833), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3834), _), 32); 
                    RP.Const (RP.Bits arg3835)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3830), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3831), _), 32); 
                    RP.Const (RP.Bits arg3832)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3827), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3828), _), 32); 
                    RP.Const (RP.Bits arg3829)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3824), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3825), _), 32); 
                    RP.Const (RP.Bits arg3826)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3821), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3822), _), 32); 
                    RP.Const (RP.Bits arg3823)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3818), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3819), _), 32); 
                    RP.Const (RP.Bits arg3820)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3815), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3816), _), 32); 
                  RP.Const (RP.Bits arg3817)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3814)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3814 13 && Bitops.eq arg3878 
            (Bits.U.of_int 8 5) && Bitops.eq arg3879 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3876 (Bits.U.of_int 9 5) && Bitops.eq arg3877 
            (Bits.U.of_int 25 5) && Bitops.eq arg3874 (Bits.U.of_int 11 5) && 
          Bitops.eq arg3875 (Bits.U.of_int 27 5) && Bitops.eq arg3872 
            (Bits.U.of_int 12 5) && Bitops.eq arg3873 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3870 (Bits.U.of_int 13 5) && Bitops.eq arg3871 
            (Bits.U.of_int 29 5) && Bitops.eq arg3868 (Bits.U.of_int 14 5) && 
          Bitops.eq arg3869 (Bits.U.of_int 30 5) && Bitops.eq arg3866 
            (Bits.U.of_int 15 5) && Bitops.eq arg3867 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3863 (Bits.U.of_int 16 5) && Bitops.eq arg3864 
            (Bits.U.of_int 0 1) && Bitops.eq arg3865 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3860 (Bits.U.of_int 17 5) && Bitops.eq arg3861 
            (Bits.U.of_int 0 1) && Bitops.eq arg3862 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3857 (Bits.U.of_int 18 5) && Bitops.eq arg3858 
            (Bits.U.of_int 0 1) && Bitops.eq arg3859 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3854 (Bits.U.of_int 19 5) && Bitops.eq arg3855 
            (Bits.U.of_int 0 1) && Bitops.eq arg3856 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3851 (Bits.U.of_int 20 5) && Bitops.eq arg3852 
            (Bits.U.of_int 0 1) && Bitops.eq arg3853 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3848 (Bits.U.of_int 21 5) && Bitops.eq arg3849 
            (Bits.U.of_int 0 1) && Bitops.eq arg3850 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3845 (Bits.U.of_int 22 5) && Bitops.eq arg3846 
            (Bits.U.of_int 0 1) && Bitops.eq arg3847 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3842 (Bits.U.of_int 23 5) && Bitops.eq arg3843 
            (Bits.U.of_int 0 1) && Bitops.eq arg3844 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3839 (Bits.U.of_int 24 5) && Bitops.eq arg3840 
            (Bits.U.of_int 0 1) && Bitops.eq arg3841 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3836 (Bits.U.of_int 25 5) && Bitops.eq arg3837 
            (Bits.U.of_int 0 1) && Bitops.eq arg3838 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3833 (Bits.U.of_int 26 5) && Bitops.eq arg3834 
            (Bits.U.of_int 0 1) && Bitops.eq arg3835 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3830 (Bits.U.of_int 27 5) && Bitops.eq arg3831 
            (Bits.U.of_int 0 1) && Bitops.eq arg3832 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3827 (Bits.U.of_int 28 5) && Bitops.eq arg3828 
            (Bits.U.of_int 0 1) && Bitops.eq arg3829 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3824 (Bits.U.of_int 29 5) && Bitops.eq arg3825 
            (Bits.U.of_int 0 1) && Bitops.eq arg3826 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3821 (Bits.U.of_int 30 5) && Bitops.eq arg3822 
            (Bits.U.of_int 0 1) && Bitops.eq arg3823 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3818 (Bits.U.of_int 31 5) && Bitops.eq arg3819 
            (Bits.U.of_int 0 1) && Bitops.eq arg3820 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3815 (Bits.U.of_int 0 1) && Bitops.eq arg3816 
            (Bits.U.of_int 0 1) && Bitops.eq arg3817 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 10 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3814)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg3944), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg3945), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3942), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3943), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3940), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3941), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3938), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3939), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3936), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3937), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3934), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3935), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3932), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3933), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3929), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3930), _), 32); 
                    RP.Const (RP.Bits arg3931)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3926), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3927), _), 32); 
                    RP.Const (RP.Bits arg3928)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3923), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3924), _), 32); 
                    RP.Const (RP.Bits arg3925)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3920), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3921), _), 32); 
                    RP.Const (RP.Bits arg3922)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3917), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3918), _), 32); 
                    RP.Const (RP.Bits arg3919)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3914), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3915), _), 32); 
                    RP.Const (RP.Bits arg3916)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3911), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3912), _), 32); 
                    RP.Const (RP.Bits arg3913)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3908), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3909), _), 32); 
                    RP.Const (RP.Bits arg3910)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3905), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3906), _), 32); 
                    RP.Const (RP.Bits arg3907)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3902), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3903), _), 32); 
                    RP.Const (RP.Bits arg3904)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3899), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3900), _), 32); 
                    RP.Const (RP.Bits arg3901)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3896), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3897), _), 32); 
                    RP.Const (RP.Bits arg3898)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3893), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3894), _), 32); 
                    RP.Const (RP.Bits arg3895)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3890), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3891), _), 32); 
                    RP.Const (RP.Bits arg3892)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3887), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3888), _), 32); 
                    RP.Const (RP.Bits arg3889)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3884), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3885), _), 32); 
                    RP.Const (RP.Bits arg3886)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3881), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3882), _), 32); 
                  RP.Const (RP.Bits arg3883)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3880)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3880 13 && Bitops.eq arg3944 
            (Bits.U.of_int 8 5) && Bitops.eq arg3945 (Bits.U.of_int 24 5) && 
          Bitops.eq arg3942 (Bits.U.of_int 10 5) && Bitops.eq arg3943 
            (Bits.U.of_int 26 5) && Bitops.eq arg3940 (Bits.U.of_int 11 5) && 
          Bitops.eq arg3941 (Bits.U.of_int 27 5) && Bitops.eq arg3938 
            (Bits.U.of_int 12 5) && Bitops.eq arg3939 (Bits.U.of_int 28 5) && 
          Bitops.eq arg3936 (Bits.U.of_int 13 5) && Bitops.eq arg3937 
            (Bits.U.of_int 29 5) && Bitops.eq arg3934 (Bits.U.of_int 14 5) && 
          Bitops.eq arg3935 (Bits.U.of_int 30 5) && Bitops.eq arg3932 
            (Bits.U.of_int 15 5) && Bitops.eq arg3933 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3929 (Bits.U.of_int 16 5) && Bitops.eq arg3930 
            (Bits.U.of_int 0 1) && Bitops.eq arg3931 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3926 (Bits.U.of_int 17 5) && Bitops.eq arg3927 
            (Bits.U.of_int 0 1) && Bitops.eq arg3928 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3923 (Bits.U.of_int 18 5) && Bitops.eq arg3924 
            (Bits.U.of_int 0 1) && Bitops.eq arg3925 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3920 (Bits.U.of_int 19 5) && Bitops.eq arg3921 
            (Bits.U.of_int 0 1) && Bitops.eq arg3922 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3917 (Bits.U.of_int 20 5) && Bitops.eq arg3918 
            (Bits.U.of_int 0 1) && Bitops.eq arg3919 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3914 (Bits.U.of_int 21 5) && Bitops.eq arg3915 
            (Bits.U.of_int 0 1) && Bitops.eq arg3916 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3911 (Bits.U.of_int 22 5) && Bitops.eq arg3912 
            (Bits.U.of_int 0 1) && Bitops.eq arg3913 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3908 (Bits.U.of_int 23 5) && Bitops.eq arg3909 
            (Bits.U.of_int 0 1) && Bitops.eq arg3910 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3905 (Bits.U.of_int 24 5) && Bitops.eq arg3906 
            (Bits.U.of_int 0 1) && Bitops.eq arg3907 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3902 (Bits.U.of_int 25 5) && Bitops.eq arg3903 
            (Bits.U.of_int 0 1) && Bitops.eq arg3904 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3899 (Bits.U.of_int 26 5) && Bitops.eq arg3900 
            (Bits.U.of_int 0 1) && Bitops.eq arg3901 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3896 (Bits.U.of_int 27 5) && Bitops.eq arg3897 
            (Bits.U.of_int 0 1) && Bitops.eq arg3898 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3893 (Bits.U.of_int 28 5) && Bitops.eq arg3894 
            (Bits.U.of_int 0 1) && Bitops.eq arg3895 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3890 (Bits.U.of_int 29 5) && Bitops.eq arg3891 
            (Bits.U.of_int 0 1) && Bitops.eq arg3892 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3887 (Bits.U.of_int 30 5) && Bitops.eq arg3888 
            (Bits.U.of_int 0 1) && Bitops.eq arg3889 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3884 (Bits.U.of_int 31 5) && Bitops.eq arg3885 
            (Bits.U.of_int 0 1) && Bitops.eq arg3886 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3881 (Bits.U.of_int 0 1) && Bitops.eq arg3882 
            (Bits.U.of_int 0 1) && Bitops.eq arg3883 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 9 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3880)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4010), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4011), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4008), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4009), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4006), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4007), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4004), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4005), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4002), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4003), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4000), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4001), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3998), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg3999), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3995), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3996), _), 32); 
                    RP.Const (RP.Bits arg3997)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3992), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3993), _), 32); 
                    RP.Const (RP.Bits arg3994)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3989), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3990), _), 32); 
                    RP.Const (RP.Bits arg3991)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3986), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3987), _), 32); 
                    RP.Const (RP.Bits arg3988)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3983), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3984), _), 32); 
                    RP.Const (RP.Bits arg3985)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3980), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3981), _), 32); 
                    RP.Const (RP.Bits arg3982)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3977), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3978), _), 32); 
                    RP.Const (RP.Bits arg3979)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3974), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3975), _), 32); 
                    RP.Const (RP.Bits arg3976)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3971), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3972), _), 32); 
                    RP.Const (RP.Bits arg3973)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3968), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3969), _), 32); 
                    RP.Const (RP.Bits arg3970)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3965), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3966), _), 32); 
                    RP.Const (RP.Bits arg3967)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3962), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3963), _), 32); 
                    RP.Const (RP.Bits arg3964)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3959), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3960), _), 32); 
                    RP.Const (RP.Bits arg3961)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3956), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3957), _), 32); 
                    RP.Const (RP.Bits arg3958)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3953), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3954), _), 32); 
                    RP.Const (RP.Bits arg3955)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3950), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg3951), _), 32); 
                    RP.Const (RP.Bits arg3952)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg3947), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg3948), _), 32); 
                  RP.Const (RP.Bits arg3949)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg3946)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg3946 13 && Bitops.eq arg4010 
            (Bits.U.of_int 9 5) && Bitops.eq arg4011 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4008 (Bits.U.of_int 10 5) && Bitops.eq arg4009 
            (Bits.U.of_int 26 5) && Bitops.eq arg4006 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4007 (Bits.U.of_int 27 5) && Bitops.eq arg4004 
            (Bits.U.of_int 12 5) && Bitops.eq arg4005 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4002 (Bits.U.of_int 13 5) && Bitops.eq arg4003 
            (Bits.U.of_int 29 5) && Bitops.eq arg4000 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4001 (Bits.U.of_int 30 5) && Bitops.eq arg3998 
            (Bits.U.of_int 15 5) && Bitops.eq arg3999 (Bits.U.of_int 31 5) && 
          Bitops.eq arg3995 (Bits.U.of_int 16 5) && Bitops.eq arg3996 
            (Bits.U.of_int 0 1) && Bitops.eq arg3997 (Bits.U.of_int 8 32) && 
          Bitops.eq arg3992 (Bits.U.of_int 17 5) && Bitops.eq arg3993 
            (Bits.U.of_int 0 1) && Bitops.eq arg3994 (Bits.U.of_int 7 32) && 
          Bitops.eq arg3989 (Bits.U.of_int 18 5) && Bitops.eq arg3990 
            (Bits.U.of_int 0 1) && Bitops.eq arg3991 (Bits.U.of_int 6 32) && 
          Bitops.eq arg3986 (Bits.U.of_int 19 5) && Bitops.eq arg3987 
            (Bits.U.of_int 0 1) && Bitops.eq arg3988 (Bits.U.of_int 5 32) && 
          Bitops.eq arg3983 (Bits.U.of_int 20 5) && Bitops.eq arg3984 
            (Bits.U.of_int 0 1) && Bitops.eq arg3985 (Bits.U.of_int 4 32) && 
          Bitops.eq arg3980 (Bits.U.of_int 21 5) && Bitops.eq arg3981 
            (Bits.U.of_int 0 1) && Bitops.eq arg3982 (Bits.U.of_int 3 32) && 
          Bitops.eq arg3977 (Bits.U.of_int 22 5) && Bitops.eq arg3978 
            (Bits.U.of_int 0 1) && Bitops.eq arg3979 (Bits.U.of_int 2 32) && 
          Bitops.eq arg3974 (Bits.U.of_int 23 5) && Bitops.eq arg3975 
            (Bits.U.of_int 0 1) && Bitops.eq arg3976 (Bits.U.of_int 1 32) && 
          Bitops.eq arg3971 (Bits.U.of_int 24 5) && Bitops.eq arg3972 
            (Bits.U.of_int 0 1) && Bitops.eq arg3973 (Bits.U.of_int 16 32) && 
          Bitops.eq arg3968 (Bits.U.of_int 25 5) && Bitops.eq arg3969 
            (Bits.U.of_int 0 1) && Bitops.eq arg3970 (Bits.U.of_int 15 32) && 
          Bitops.eq arg3965 (Bits.U.of_int 26 5) && Bitops.eq arg3966 
            (Bits.U.of_int 0 1) && Bitops.eq arg3967 (Bits.U.of_int 14 32) && 
          Bitops.eq arg3962 (Bits.U.of_int 27 5) && Bitops.eq arg3963 
            (Bits.U.of_int 0 1) && Bitops.eq arg3964 (Bits.U.of_int 13 32) && 
          Bitops.eq arg3959 (Bits.U.of_int 28 5) && Bitops.eq arg3960 
            (Bits.U.of_int 0 1) && Bitops.eq arg3961 (Bits.U.of_int 12 32) && 
          Bitops.eq arg3956 (Bits.U.of_int 29 5) && Bitops.eq arg3957 
            (Bits.U.of_int 0 1) && Bitops.eq arg3958 (Bits.U.of_int 11 32) && 
          Bitops.eq arg3953 (Bits.U.of_int 30 5) && Bitops.eq arg3954 
            (Bits.U.of_int 0 1) && Bitops.eq arg3955 (Bits.U.of_int 10 32) && 
          Bitops.eq arg3950 (Bits.U.of_int 31 5) && Bitops.eq arg3951 
            (Bits.U.of_int 0 1) && Bitops.eq arg3952 (Bits.U.of_int 9 32) && 
          Bitops.eq arg3947 (Bits.U.of_int 0 1) && Bitops.eq arg3948 
            (Bits.U.of_int 0 1) && Bitops.eq arg3949 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 8 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.imode 
                 (Bits.U.to_native arg3946)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4077), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4078), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4075), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4076), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4073), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4074), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4071), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4072), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4069), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4070), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4067), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4068), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4065), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4066), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4063), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4064), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4060), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4061), _), 32); 
                    RP.Const (RP.Bits arg4062)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4057), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4058), _), 32); 
                    RP.Const (RP.Bits arg4059)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4054), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4055), _), 32); 
                    RP.Const (RP.Bits arg4056)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4051), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4052), _), 32); 
                    RP.Const (RP.Bits arg4053)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4048), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4049), _), 32); 
                    RP.Const (RP.Bits arg4050)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4045), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4046), _), 32); 
                    RP.Const (RP.Bits arg4047)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4042), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4043), _), 32); 
                    RP.Const (RP.Bits arg4044)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4039), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4040), _), 32); 
                    RP.Const (RP.Bits arg4041)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4036), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4037), _), 32); 
                    RP.Const (RP.Bits arg4038)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4033), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4034), _), 32); 
                    RP.Const (RP.Bits arg4035)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4030), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4031), _), 32); 
                    RP.Const (RP.Bits arg4032)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4027), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4028), _), 32); 
                    RP.Const (RP.Bits arg4029)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4024), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4025), _), 32); 
                    RP.Const (RP.Bits arg4026)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4021), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4022), _), 32); 
                    RP.Const (RP.Bits arg4023)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4018), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4019), _), 32); 
                    RP.Const (RP.Bits arg4020)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4015), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4016), _), 32); 
                    RP.Const (RP.Bits arg4017)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4012), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4013), _), 32); 
                  RP.Const (RP.Bits arg4014)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4077 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4078 (Bits.U.of_int 24 5) && Bitops.eq arg4075 
            (Bits.U.of_int 9 5) && Bitops.eq arg4076 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4073 (Bits.U.of_int 10 5) && Bitops.eq arg4074 
            (Bits.U.of_int 26 5) && Bitops.eq arg4071 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4072 (Bits.U.of_int 27 5) && Bitops.eq arg4069 
            (Bits.U.of_int 12 5) && Bitops.eq arg4070 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4067 (Bits.U.of_int 13 5) && Bitops.eq arg4068 
            (Bits.U.of_int 29 5) && Bitops.eq arg4065 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4066 (Bits.U.of_int 30 5) && Bitops.eq arg4063 
            (Bits.U.of_int 15 5) && Bitops.eq arg4064 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4060 (Bits.U.of_int 16 5) && Bitops.eq arg4061 
            (Bits.U.of_int 0 1) && Bitops.eq arg4062 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4057 (Bits.U.of_int 17 5) && Bitops.eq arg4058 
            (Bits.U.of_int 0 1) && Bitops.eq arg4059 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4054 (Bits.U.of_int 18 5) && Bitops.eq arg4055 
            (Bits.U.of_int 0 1) && Bitops.eq arg4056 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4051 (Bits.U.of_int 19 5) && Bitops.eq arg4052 
            (Bits.U.of_int 0 1) && Bitops.eq arg4053 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4048 (Bits.U.of_int 20 5) && Bitops.eq arg4049 
            (Bits.U.of_int 0 1) && Bitops.eq arg4050 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4045 (Bits.U.of_int 21 5) && Bitops.eq arg4046 
            (Bits.U.of_int 0 1) && Bitops.eq arg4047 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4042 (Bits.U.of_int 22 5) && Bitops.eq arg4043 
            (Bits.U.of_int 0 1) && Bitops.eq arg4044 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4039 (Bits.U.of_int 23 5) && Bitops.eq arg4040 
            (Bits.U.of_int 0 1) && Bitops.eq arg4041 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4036 (Bits.U.of_int 24 5) && Bitops.eq arg4037 
            (Bits.U.of_int 0 1) && Bitops.eq arg4038 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4033 (Bits.U.of_int 25 5) && Bitops.eq arg4034 
            (Bits.U.of_int 0 1) && Bitops.eq arg4035 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4030 (Bits.U.of_int 26 5) && Bitops.eq arg4031 
            (Bits.U.of_int 0 1) && Bitops.eq arg4032 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4027 (Bits.U.of_int 27 5) && Bitops.eq arg4028 
            (Bits.U.of_int 0 1) && Bitops.eq arg4029 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4024 (Bits.U.of_int 28 5) && Bitops.eq arg4025 
            (Bits.U.of_int 0 1) && Bitops.eq arg4026 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4021 (Bits.U.of_int 29 5) && Bitops.eq arg4022 
            (Bits.U.of_int 0 1) && Bitops.eq arg4023 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4018 (Bits.U.of_int 30 5) && Bitops.eq arg4019 
            (Bits.U.of_int 0 1) && Bitops.eq arg4020 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4015 (Bits.U.of_int 31 5) && Bitops.eq arg4016 
            (Bits.U.of_int 0 1) && Bitops.eq arg4017 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4012 (Bits.U.of_int 0 1) && Bitops.eq arg4013 
            (Bits.U.of_int 0 1) && Bitops.eq arg4014 (Bits.U.of_int 16 32) && 
          Bitops.ne rd (Bits.U.of_int 8 5) && Bitops.ne rd (Bits.U.of_int 9 5) && 
          Bitops.ne rd (Bits.U.of_int 10 5) && Bitops.ne rd (Bits.U.of_int 11 5) && 
          Bitops.ne rd (Bits.U.of_int 12 5) && Bitops.ne rd (Bits.U.of_int 13 5) && 
          Bitops.ne rd (Bits.U.of_int 14 5) && Bitops.ne rd (Bits.U.of_int 15 5) && 
          Bitops.ne rd (Bits.U.of_int 16 5) && Bitops.ne rd (Bits.U.of_int 17 5) && 
          Bitops.ne rd (Bits.U.of_int 18 5) && Bitops.ne rd (Bits.U.of_int 19 5) && 
          Bitops.ne rd (Bits.U.of_int 20 5) && Bitops.ne rd (Bits.U.of_int 21 5) && 
          Bitops.ne rd (Bits.U.of_int 22 5) && Bitops.ne rd (Bits.U.of_int 23 5) && 
          Bitops.ne rd (Bits.U.of_int 24 5) && Bitops.ne rd (Bits.U.of_int 25 5) && 
          Bitops.ne rd (Bits.U.of_int 26 5) && Bitops.ne rd (Bits.U.of_int 27 5) && 
          Bitops.ne rd (Bits.U.of_int 28 5) && Bitops.ne rd (Bits.U.of_int 29 5) && 
          Bitops.ne rd (Bits.U.of_int 30 5) && Bitops.ne rd 
            (Bits.U.of_int 31 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4141), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4142), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4139), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4140), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4137), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4138), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4135), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4136), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4133), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4134), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4131), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4132), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4129), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4130), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4127), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4128), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4124), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4125), _), 32); 
                    RP.Const (RP.Bits arg4126)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4121), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4122), _), 32); 
                    RP.Const (RP.Bits arg4123)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4118), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4119), _), 32); 
                    RP.Const (RP.Bits arg4120)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4115), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4116), _), 32); 
                    RP.Const (RP.Bits arg4117)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4112), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4113), _), 32); 
                    RP.Const (RP.Bits arg4114)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4109), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4110), _), 32); 
                    RP.Const (RP.Bits arg4111)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4106), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4107), _), 32); 
                    RP.Const (RP.Bits arg4108)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4103), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4104), _), 32); 
                    RP.Const (RP.Bits arg4105)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4100), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4101), _), 32); 
                    RP.Const (RP.Bits arg4102)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4097), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4098), _), 32); 
                    RP.Const (RP.Bits arg4099)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4094), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4095), _), 32); 
                    RP.Const (RP.Bits arg4096)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4091), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4092), _), 32); 
                    RP.Const (RP.Bits arg4093)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4088), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4089), _), 32); 
                    RP.Const (RP.Bits arg4090)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4085), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4086), _), 32); 
                    RP.Const (RP.Bits arg4087)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4082), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4083), _), 32); 
                    RP.Const (RP.Bits arg4084)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4079), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4080), _), 32); 
                  RP.Const (RP.Bits arg4081)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4141 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4142 (Bits.U.of_int 24 5) && Bitops.eq arg4139 
            (Bits.U.of_int 9 5) && Bitops.eq arg4140 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4137 (Bits.U.of_int 10 5) && Bitops.eq arg4138 
            (Bits.U.of_int 26 5) && Bitops.eq arg4135 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4136 (Bits.U.of_int 27 5) && Bitops.eq arg4133 
            (Bits.U.of_int 12 5) && Bitops.eq arg4134 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4131 (Bits.U.of_int 13 5) && Bitops.eq arg4132 
            (Bits.U.of_int 29 5) && Bitops.eq arg4129 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4130 (Bits.U.of_int 30 5) && Bitops.eq arg4127 
            (Bits.U.of_int 15 5) && Bitops.eq arg4128 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4124 (Bits.U.of_int 16 5) && Bitops.eq arg4125 
            (Bits.U.of_int 0 1) && Bitops.eq arg4126 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4121 (Bits.U.of_int 17 5) && Bitops.eq arg4122 
            (Bits.U.of_int 0 1) && Bitops.eq arg4123 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4118 (Bits.U.of_int 18 5) && Bitops.eq arg4119 
            (Bits.U.of_int 0 1) && Bitops.eq arg4120 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4115 (Bits.U.of_int 19 5) && Bitops.eq arg4116 
            (Bits.U.of_int 0 1) && Bitops.eq arg4117 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4112 (Bits.U.of_int 20 5) && Bitops.eq arg4113 
            (Bits.U.of_int 0 1) && Bitops.eq arg4114 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4109 (Bits.U.of_int 21 5) && Bitops.eq arg4110 
            (Bits.U.of_int 0 1) && Bitops.eq arg4111 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4106 (Bits.U.of_int 22 5) && Bitops.eq arg4107 
            (Bits.U.of_int 0 1) && Bitops.eq arg4108 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4103 (Bits.U.of_int 23 5) && Bitops.eq arg4104 
            (Bits.U.of_int 0 1) && Bitops.eq arg4105 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4100 (Bits.U.of_int 24 5) && Bitops.eq arg4101 
            (Bits.U.of_int 0 1) && Bitops.eq arg4102 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4097 (Bits.U.of_int 25 5) && Bitops.eq arg4098 
            (Bits.U.of_int 0 1) && Bitops.eq arg4099 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4094 (Bits.U.of_int 26 5) && Bitops.eq arg4095 
            (Bits.U.of_int 0 1) && Bitops.eq arg4096 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4091 (Bits.U.of_int 27 5) && Bitops.eq arg4092 
            (Bits.U.of_int 0 1) && Bitops.eq arg4093 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4088 (Bits.U.of_int 28 5) && Bitops.eq arg4089 
            (Bits.U.of_int 0 1) && Bitops.eq arg4090 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4085 (Bits.U.of_int 29 5) && Bitops.eq arg4086 
            (Bits.U.of_int 0 1) && Bitops.eq arg4087 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4082 (Bits.U.of_int 30 5) && Bitops.eq arg4083 
            (Bits.U.of_int 0 1) && Bitops.eq arg4084 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4079 (Bits.U.of_int 0 1) && Bitops.eq arg4080 
            (Bits.U.of_int 0 1) && Bitops.eq arg4081 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 31 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4205), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4206), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4203), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4204), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4201), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4202), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4199), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4200), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4197), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4198), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4195), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4196), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4193), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4194), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4191), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4192), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4188), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4189), _), 32); 
                    RP.Const (RP.Bits arg4190)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4185), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4186), _), 32); 
                    RP.Const (RP.Bits arg4187)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4182), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4183), _), 32); 
                    RP.Const (RP.Bits arg4184)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4179), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4180), _), 32); 
                    RP.Const (RP.Bits arg4181)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4176), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4177), _), 32); 
                    RP.Const (RP.Bits arg4178)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4173), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4174), _), 32); 
                    RP.Const (RP.Bits arg4175)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4170), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4171), _), 32); 
                    RP.Const (RP.Bits arg4172)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4167), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4168), _), 32); 
                    RP.Const (RP.Bits arg4169)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4164), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4165), _), 32); 
                    RP.Const (RP.Bits arg4166)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4161), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4162), _), 32); 
                    RP.Const (RP.Bits arg4163)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4158), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4159), _), 32); 
                    RP.Const (RP.Bits arg4160)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4155), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4156), _), 32); 
                    RP.Const (RP.Bits arg4157)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4152), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4153), _), 32); 
                    RP.Const (RP.Bits arg4154)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4149), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4150), _), 32); 
                    RP.Const (RP.Bits arg4151)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4146), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4147), _), 32); 
                    RP.Const (RP.Bits arg4148)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4143), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4144), _), 32); 
                  RP.Const (RP.Bits arg4145)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4205 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4206 (Bits.U.of_int 24 5) && Bitops.eq arg4203 
            (Bits.U.of_int 9 5) && Bitops.eq arg4204 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4201 (Bits.U.of_int 10 5) && Bitops.eq arg4202 
            (Bits.U.of_int 26 5) && Bitops.eq arg4199 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4200 (Bits.U.of_int 27 5) && Bitops.eq arg4197 
            (Bits.U.of_int 12 5) && Bitops.eq arg4198 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4195 (Bits.U.of_int 13 5) && Bitops.eq arg4196 
            (Bits.U.of_int 29 5) && Bitops.eq arg4193 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4194 (Bits.U.of_int 30 5) && Bitops.eq arg4191 
            (Bits.U.of_int 15 5) && Bitops.eq arg4192 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4188 (Bits.U.of_int 16 5) && Bitops.eq arg4189 
            (Bits.U.of_int 0 1) && Bitops.eq arg4190 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4185 (Bits.U.of_int 17 5) && Bitops.eq arg4186 
            (Bits.U.of_int 0 1) && Bitops.eq arg4187 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4182 (Bits.U.of_int 18 5) && Bitops.eq arg4183 
            (Bits.U.of_int 0 1) && Bitops.eq arg4184 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4179 (Bits.U.of_int 19 5) && Bitops.eq arg4180 
            (Bits.U.of_int 0 1) && Bitops.eq arg4181 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4176 (Bits.U.of_int 20 5) && Bitops.eq arg4177 
            (Bits.U.of_int 0 1) && Bitops.eq arg4178 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4173 (Bits.U.of_int 21 5) && Bitops.eq arg4174 
            (Bits.U.of_int 0 1) && Bitops.eq arg4175 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4170 (Bits.U.of_int 22 5) && Bitops.eq arg4171 
            (Bits.U.of_int 0 1) && Bitops.eq arg4172 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4167 (Bits.U.of_int 23 5) && Bitops.eq arg4168 
            (Bits.U.of_int 0 1) && Bitops.eq arg4169 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4164 (Bits.U.of_int 24 5) && Bitops.eq arg4165 
            (Bits.U.of_int 0 1) && Bitops.eq arg4166 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4161 (Bits.U.of_int 25 5) && Bitops.eq arg4162 
            (Bits.U.of_int 0 1) && Bitops.eq arg4163 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4158 (Bits.U.of_int 26 5) && Bitops.eq arg4159 
            (Bits.U.of_int 0 1) && Bitops.eq arg4160 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4155 (Bits.U.of_int 27 5) && Bitops.eq arg4156 
            (Bits.U.of_int 0 1) && Bitops.eq arg4157 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4152 (Bits.U.of_int 28 5) && Bitops.eq arg4153 
            (Bits.U.of_int 0 1) && Bitops.eq arg4154 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4149 (Bits.U.of_int 29 5) && Bitops.eq arg4150 
            (Bits.U.of_int 0 1) && Bitops.eq arg4151 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4146 (Bits.U.of_int 31 5) && Bitops.eq arg4147 
            (Bits.U.of_int 0 1) && Bitops.eq arg4148 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4143 (Bits.U.of_int 0 1) && Bitops.eq arg4144 
            (Bits.U.of_int 0 1) && Bitops.eq arg4145 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 30 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4269), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4270), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4267), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4268), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4265), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4266), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4263), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4264), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4261), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4262), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4259), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4260), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4257), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4258), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4255), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4256), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4252), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4253), _), 32); 
                    RP.Const (RP.Bits arg4254)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4249), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4250), _), 32); 
                    RP.Const (RP.Bits arg4251)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4246), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4247), _), 32); 
                    RP.Const (RP.Bits arg4248)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4243), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4244), _), 32); 
                    RP.Const (RP.Bits arg4245)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4240), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4241), _), 32); 
                    RP.Const (RP.Bits arg4242)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4237), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4238), _), 32); 
                    RP.Const (RP.Bits arg4239)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4234), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4235), _), 32); 
                    RP.Const (RP.Bits arg4236)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4231), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4232), _), 32); 
                    RP.Const (RP.Bits arg4233)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4228), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4229), _), 32); 
                    RP.Const (RP.Bits arg4230)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4225), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4226), _), 32); 
                    RP.Const (RP.Bits arg4227)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4222), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4223), _), 32); 
                    RP.Const (RP.Bits arg4224)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4219), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4220), _), 32); 
                    RP.Const (RP.Bits arg4221)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4216), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4217), _), 32); 
                    RP.Const (RP.Bits arg4218)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4213), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4214), _), 32); 
                    RP.Const (RP.Bits arg4215)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4210), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4211), _), 32); 
                    RP.Const (RP.Bits arg4212)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4207), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4208), _), 32); 
                  RP.Const (RP.Bits arg4209)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4269 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4270 (Bits.U.of_int 24 5) && Bitops.eq arg4267 
            (Bits.U.of_int 9 5) && Bitops.eq arg4268 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4265 (Bits.U.of_int 10 5) && Bitops.eq arg4266 
            (Bits.U.of_int 26 5) && Bitops.eq arg4263 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4264 (Bits.U.of_int 27 5) && Bitops.eq arg4261 
            (Bits.U.of_int 12 5) && Bitops.eq arg4262 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4259 (Bits.U.of_int 13 5) && Bitops.eq arg4260 
            (Bits.U.of_int 29 5) && Bitops.eq arg4257 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4258 (Bits.U.of_int 30 5) && Bitops.eq arg4255 
            (Bits.U.of_int 15 5) && Bitops.eq arg4256 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4252 (Bits.U.of_int 16 5) && Bitops.eq arg4253 
            (Bits.U.of_int 0 1) && Bitops.eq arg4254 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4249 (Bits.U.of_int 17 5) && Bitops.eq arg4250 
            (Bits.U.of_int 0 1) && Bitops.eq arg4251 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4246 (Bits.U.of_int 18 5) && Bitops.eq arg4247 
            (Bits.U.of_int 0 1) && Bitops.eq arg4248 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4243 (Bits.U.of_int 19 5) && Bitops.eq arg4244 
            (Bits.U.of_int 0 1) && Bitops.eq arg4245 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4240 (Bits.U.of_int 20 5) && Bitops.eq arg4241 
            (Bits.U.of_int 0 1) && Bitops.eq arg4242 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4237 (Bits.U.of_int 21 5) && Bitops.eq arg4238 
            (Bits.U.of_int 0 1) && Bitops.eq arg4239 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4234 (Bits.U.of_int 22 5) && Bitops.eq arg4235 
            (Bits.U.of_int 0 1) && Bitops.eq arg4236 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4231 (Bits.U.of_int 23 5) && Bitops.eq arg4232 
            (Bits.U.of_int 0 1) && Bitops.eq arg4233 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4228 (Bits.U.of_int 24 5) && Bitops.eq arg4229 
            (Bits.U.of_int 0 1) && Bitops.eq arg4230 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4225 (Bits.U.of_int 25 5) && Bitops.eq arg4226 
            (Bits.U.of_int 0 1) && Bitops.eq arg4227 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4222 (Bits.U.of_int 26 5) && Bitops.eq arg4223 
            (Bits.U.of_int 0 1) && Bitops.eq arg4224 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4219 (Bits.U.of_int 27 5) && Bitops.eq arg4220 
            (Bits.U.of_int 0 1) && Bitops.eq arg4221 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4216 (Bits.U.of_int 28 5) && Bitops.eq arg4217 
            (Bits.U.of_int 0 1) && Bitops.eq arg4218 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4213 (Bits.U.of_int 30 5) && Bitops.eq arg4214 
            (Bits.U.of_int 0 1) && Bitops.eq arg4215 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4210 (Bits.U.of_int 31 5) && Bitops.eq arg4211 
            (Bits.U.of_int 0 1) && Bitops.eq arg4212 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4207 (Bits.U.of_int 0 1) && Bitops.eq arg4208 
            (Bits.U.of_int 0 1) && Bitops.eq arg4209 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 29 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4333), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4334), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4331), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4332), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4329), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4330), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4327), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4328), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4325), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4326), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4323), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4324), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4321), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4322), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4319), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4320), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4316), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4317), _), 32); 
                    RP.Const (RP.Bits arg4318)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4313), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4314), _), 32); 
                    RP.Const (RP.Bits arg4315)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4310), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4311), _), 32); 
                    RP.Const (RP.Bits arg4312)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4307), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4308), _), 32); 
                    RP.Const (RP.Bits arg4309)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4304), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4305), _), 32); 
                    RP.Const (RP.Bits arg4306)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4301), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4302), _), 32); 
                    RP.Const (RP.Bits arg4303)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4298), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4299), _), 32); 
                    RP.Const (RP.Bits arg4300)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4295), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4296), _), 32); 
                    RP.Const (RP.Bits arg4297)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4292), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4293), _), 32); 
                    RP.Const (RP.Bits arg4294)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4289), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4290), _), 32); 
                    RP.Const (RP.Bits arg4291)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4286), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4287), _), 32); 
                    RP.Const (RP.Bits arg4288)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4283), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4284), _), 32); 
                    RP.Const (RP.Bits arg4285)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4280), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4281), _), 32); 
                    RP.Const (RP.Bits arg4282)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4277), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4278), _), 32); 
                    RP.Const (RP.Bits arg4279)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4274), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4275), _), 32); 
                    RP.Const (RP.Bits arg4276)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4271), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4272), _), 32); 
                  RP.Const (RP.Bits arg4273)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4333 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4334 (Bits.U.of_int 24 5) && Bitops.eq arg4331 
            (Bits.U.of_int 9 5) && Bitops.eq arg4332 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4329 (Bits.U.of_int 10 5) && Bitops.eq arg4330 
            (Bits.U.of_int 26 5) && Bitops.eq arg4327 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4328 (Bits.U.of_int 27 5) && Bitops.eq arg4325 
            (Bits.U.of_int 12 5) && Bitops.eq arg4326 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4323 (Bits.U.of_int 13 5) && Bitops.eq arg4324 
            (Bits.U.of_int 29 5) && Bitops.eq arg4321 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4322 (Bits.U.of_int 30 5) && Bitops.eq arg4319 
            (Bits.U.of_int 15 5) && Bitops.eq arg4320 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4316 (Bits.U.of_int 16 5) && Bitops.eq arg4317 
            (Bits.U.of_int 0 1) && Bitops.eq arg4318 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4313 (Bits.U.of_int 17 5) && Bitops.eq arg4314 
            (Bits.U.of_int 0 1) && Bitops.eq arg4315 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4310 (Bits.U.of_int 18 5) && Bitops.eq arg4311 
            (Bits.U.of_int 0 1) && Bitops.eq arg4312 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4307 (Bits.U.of_int 19 5) && Bitops.eq arg4308 
            (Bits.U.of_int 0 1) && Bitops.eq arg4309 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4304 (Bits.U.of_int 20 5) && Bitops.eq arg4305 
            (Bits.U.of_int 0 1) && Bitops.eq arg4306 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4301 (Bits.U.of_int 21 5) && Bitops.eq arg4302 
            (Bits.U.of_int 0 1) && Bitops.eq arg4303 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4298 (Bits.U.of_int 22 5) && Bitops.eq arg4299 
            (Bits.U.of_int 0 1) && Bitops.eq arg4300 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4295 (Bits.U.of_int 23 5) && Bitops.eq arg4296 
            (Bits.U.of_int 0 1) && Bitops.eq arg4297 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4292 (Bits.U.of_int 24 5) && Bitops.eq arg4293 
            (Bits.U.of_int 0 1) && Bitops.eq arg4294 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4289 (Bits.U.of_int 25 5) && Bitops.eq arg4290 
            (Bits.U.of_int 0 1) && Bitops.eq arg4291 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4286 (Bits.U.of_int 26 5) && Bitops.eq arg4287 
            (Bits.U.of_int 0 1) && Bitops.eq arg4288 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4283 (Bits.U.of_int 27 5) && Bitops.eq arg4284 
            (Bits.U.of_int 0 1) && Bitops.eq arg4285 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4280 (Bits.U.of_int 29 5) && Bitops.eq arg4281 
            (Bits.U.of_int 0 1) && Bitops.eq arg4282 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4277 (Bits.U.of_int 30 5) && Bitops.eq arg4278 
            (Bits.U.of_int 0 1) && Bitops.eq arg4279 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4274 (Bits.U.of_int 31 5) && Bitops.eq arg4275 
            (Bits.U.of_int 0 1) && Bitops.eq arg4276 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4271 (Bits.U.of_int 0 1) && Bitops.eq arg4272 
            (Bits.U.of_int 0 1) && Bitops.eq arg4273 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 28 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4397), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4398), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4395), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4396), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4393), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4394), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4391), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4392), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4389), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4390), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4387), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4388), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4385), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4386), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4383), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4384), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4380), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4381), _), 32); 
                    RP.Const (RP.Bits arg4382)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4377), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4378), _), 32); 
                    RP.Const (RP.Bits arg4379)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4374), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4375), _), 32); 
                    RP.Const (RP.Bits arg4376)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4371), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4372), _), 32); 
                    RP.Const (RP.Bits arg4373)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4368), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4369), _), 32); 
                    RP.Const (RP.Bits arg4370)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4365), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4366), _), 32); 
                    RP.Const (RP.Bits arg4367)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4362), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4363), _), 32); 
                    RP.Const (RP.Bits arg4364)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4359), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4360), _), 32); 
                    RP.Const (RP.Bits arg4361)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4356), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4357), _), 32); 
                    RP.Const (RP.Bits arg4358)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4353), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4354), _), 32); 
                    RP.Const (RP.Bits arg4355)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4350), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4351), _), 32); 
                    RP.Const (RP.Bits arg4352)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4347), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4348), _), 32); 
                    RP.Const (RP.Bits arg4349)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4344), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4345), _), 32); 
                    RP.Const (RP.Bits arg4346)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4341), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4342), _), 32); 
                    RP.Const (RP.Bits arg4343)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4338), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4339), _), 32); 
                    RP.Const (RP.Bits arg4340)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4335), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4336), _), 32); 
                  RP.Const (RP.Bits arg4337)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4397 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4398 (Bits.U.of_int 24 5) && Bitops.eq arg4395 
            (Bits.U.of_int 9 5) && Bitops.eq arg4396 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4393 (Bits.U.of_int 10 5) && Bitops.eq arg4394 
            (Bits.U.of_int 26 5) && Bitops.eq arg4391 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4392 (Bits.U.of_int 27 5) && Bitops.eq arg4389 
            (Bits.U.of_int 12 5) && Bitops.eq arg4390 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4387 (Bits.U.of_int 13 5) && Bitops.eq arg4388 
            (Bits.U.of_int 29 5) && Bitops.eq arg4385 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4386 (Bits.U.of_int 30 5) && Bitops.eq arg4383 
            (Bits.U.of_int 15 5) && Bitops.eq arg4384 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4380 (Bits.U.of_int 16 5) && Bitops.eq arg4381 
            (Bits.U.of_int 0 1) && Bitops.eq arg4382 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4377 (Bits.U.of_int 17 5) && Bitops.eq arg4378 
            (Bits.U.of_int 0 1) && Bitops.eq arg4379 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4374 (Bits.U.of_int 18 5) && Bitops.eq arg4375 
            (Bits.U.of_int 0 1) && Bitops.eq arg4376 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4371 (Bits.U.of_int 19 5) && Bitops.eq arg4372 
            (Bits.U.of_int 0 1) && Bitops.eq arg4373 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4368 (Bits.U.of_int 20 5) && Bitops.eq arg4369 
            (Bits.U.of_int 0 1) && Bitops.eq arg4370 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4365 (Bits.U.of_int 21 5) && Bitops.eq arg4366 
            (Bits.U.of_int 0 1) && Bitops.eq arg4367 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4362 (Bits.U.of_int 22 5) && Bitops.eq arg4363 
            (Bits.U.of_int 0 1) && Bitops.eq arg4364 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4359 (Bits.U.of_int 23 5) && Bitops.eq arg4360 
            (Bits.U.of_int 0 1) && Bitops.eq arg4361 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4356 (Bits.U.of_int 24 5) && Bitops.eq arg4357 
            (Bits.U.of_int 0 1) && Bitops.eq arg4358 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4353 (Bits.U.of_int 25 5) && Bitops.eq arg4354 
            (Bits.U.of_int 0 1) && Bitops.eq arg4355 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4350 (Bits.U.of_int 26 5) && Bitops.eq arg4351 
            (Bits.U.of_int 0 1) && Bitops.eq arg4352 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4347 (Bits.U.of_int 28 5) && Bitops.eq arg4348 
            (Bits.U.of_int 0 1) && Bitops.eq arg4349 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4344 (Bits.U.of_int 29 5) && Bitops.eq arg4345 
            (Bits.U.of_int 0 1) && Bitops.eq arg4346 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4341 (Bits.U.of_int 30 5) && Bitops.eq arg4342 
            (Bits.U.of_int 0 1) && Bitops.eq arg4343 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4338 (Bits.U.of_int 31 5) && Bitops.eq arg4339 
            (Bits.U.of_int 0 1) && Bitops.eq arg4340 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4335 (Bits.U.of_int 0 1) && Bitops.eq arg4336 
            (Bits.U.of_int 0 1) && Bitops.eq arg4337 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 27 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4461), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4462), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4459), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4460), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4457), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4458), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4455), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4456), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4453), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4454), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4451), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4452), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4449), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4450), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4447), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4448), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4444), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4445), _), 32); 
                    RP.Const (RP.Bits arg4446)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4441), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4442), _), 32); 
                    RP.Const (RP.Bits arg4443)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4438), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4439), _), 32); 
                    RP.Const (RP.Bits arg4440)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4435), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4436), _), 32); 
                    RP.Const (RP.Bits arg4437)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4432), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4433), _), 32); 
                    RP.Const (RP.Bits arg4434)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4429), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4430), _), 32); 
                    RP.Const (RP.Bits arg4431)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4426), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4427), _), 32); 
                    RP.Const (RP.Bits arg4428)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4423), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4424), _), 32); 
                    RP.Const (RP.Bits arg4425)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4420), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4421), _), 32); 
                    RP.Const (RP.Bits arg4422)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4417), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4418), _), 32); 
                    RP.Const (RP.Bits arg4419)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4414), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4415), _), 32); 
                    RP.Const (RP.Bits arg4416)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4411), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4412), _), 32); 
                    RP.Const (RP.Bits arg4413)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4408), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4409), _), 32); 
                    RP.Const (RP.Bits arg4410)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4405), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4406), _), 32); 
                    RP.Const (RP.Bits arg4407)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4402), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4403), _), 32); 
                    RP.Const (RP.Bits arg4404)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4399), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4400), _), 32); 
                  RP.Const (RP.Bits arg4401)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4461 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4462 (Bits.U.of_int 24 5) && Bitops.eq arg4459 
            (Bits.U.of_int 9 5) && Bitops.eq arg4460 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4457 (Bits.U.of_int 10 5) && Bitops.eq arg4458 
            (Bits.U.of_int 26 5) && Bitops.eq arg4455 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4456 (Bits.U.of_int 27 5) && Bitops.eq arg4453 
            (Bits.U.of_int 12 5) && Bitops.eq arg4454 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4451 (Bits.U.of_int 13 5) && Bitops.eq arg4452 
            (Bits.U.of_int 29 5) && Bitops.eq arg4449 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4450 (Bits.U.of_int 30 5) && Bitops.eq arg4447 
            (Bits.U.of_int 15 5) && Bitops.eq arg4448 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4444 (Bits.U.of_int 16 5) && Bitops.eq arg4445 
            (Bits.U.of_int 0 1) && Bitops.eq arg4446 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4441 (Bits.U.of_int 17 5) && Bitops.eq arg4442 
            (Bits.U.of_int 0 1) && Bitops.eq arg4443 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4438 (Bits.U.of_int 18 5) && Bitops.eq arg4439 
            (Bits.U.of_int 0 1) && Bitops.eq arg4440 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4435 (Bits.U.of_int 19 5) && Bitops.eq arg4436 
            (Bits.U.of_int 0 1) && Bitops.eq arg4437 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4432 (Bits.U.of_int 20 5) && Bitops.eq arg4433 
            (Bits.U.of_int 0 1) && Bitops.eq arg4434 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4429 (Bits.U.of_int 21 5) && Bitops.eq arg4430 
            (Bits.U.of_int 0 1) && Bitops.eq arg4431 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4426 (Bits.U.of_int 22 5) && Bitops.eq arg4427 
            (Bits.U.of_int 0 1) && Bitops.eq arg4428 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4423 (Bits.U.of_int 23 5) && Bitops.eq arg4424 
            (Bits.U.of_int 0 1) && Bitops.eq arg4425 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4420 (Bits.U.of_int 24 5) && Bitops.eq arg4421 
            (Bits.U.of_int 0 1) && Bitops.eq arg4422 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4417 (Bits.U.of_int 25 5) && Bitops.eq arg4418 
            (Bits.U.of_int 0 1) && Bitops.eq arg4419 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4414 (Bits.U.of_int 27 5) && Bitops.eq arg4415 
            (Bits.U.of_int 0 1) && Bitops.eq arg4416 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4411 (Bits.U.of_int 28 5) && Bitops.eq arg4412 
            (Bits.U.of_int 0 1) && Bitops.eq arg4413 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4408 (Bits.U.of_int 29 5) && Bitops.eq arg4409 
            (Bits.U.of_int 0 1) && Bitops.eq arg4410 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4405 (Bits.U.of_int 30 5) && Bitops.eq arg4406 
            (Bits.U.of_int 0 1) && Bitops.eq arg4407 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4402 (Bits.U.of_int 31 5) && Bitops.eq arg4403 
            (Bits.U.of_int 0 1) && Bitops.eq arg4404 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4399 (Bits.U.of_int 0 1) && Bitops.eq arg4400 
            (Bits.U.of_int 0 1) && Bitops.eq arg4401 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 26 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4525), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4526), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4523), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4524), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4521), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4522), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4519), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4520), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4517), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4518), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4515), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4516), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4513), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4514), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4511), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4512), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4508), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4509), _), 32); 
                    RP.Const (RP.Bits arg4510)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4505), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4506), _), 32); 
                    RP.Const (RP.Bits arg4507)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4502), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4503), _), 32); 
                    RP.Const (RP.Bits arg4504)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4499), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4500), _), 32); 
                    RP.Const (RP.Bits arg4501)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4496), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4497), _), 32); 
                    RP.Const (RP.Bits arg4498)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4493), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4494), _), 32); 
                    RP.Const (RP.Bits arg4495)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4490), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4491), _), 32); 
                    RP.Const (RP.Bits arg4492)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4487), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4488), _), 32); 
                    RP.Const (RP.Bits arg4489)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4484), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4485), _), 32); 
                    RP.Const (RP.Bits arg4486)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4481), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4482), _), 32); 
                    RP.Const (RP.Bits arg4483)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4478), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4479), _), 32); 
                    RP.Const (RP.Bits arg4480)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4475), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4476), _), 32); 
                    RP.Const (RP.Bits arg4477)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4472), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4473), _), 32); 
                    RP.Const (RP.Bits arg4474)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4469), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4470), _), 32); 
                    RP.Const (RP.Bits arg4471)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4466), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4467), _), 32); 
                    RP.Const (RP.Bits arg4468)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4463), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4464), _), 32); 
                  RP.Const (RP.Bits arg4465)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4525 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4526 (Bits.U.of_int 24 5) && Bitops.eq arg4523 
            (Bits.U.of_int 9 5) && Bitops.eq arg4524 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4521 (Bits.U.of_int 10 5) && Bitops.eq arg4522 
            (Bits.U.of_int 26 5) && Bitops.eq arg4519 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4520 (Bits.U.of_int 27 5) && Bitops.eq arg4517 
            (Bits.U.of_int 12 5) && Bitops.eq arg4518 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4515 (Bits.U.of_int 13 5) && Bitops.eq arg4516 
            (Bits.U.of_int 29 5) && Bitops.eq arg4513 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4514 (Bits.U.of_int 30 5) && Bitops.eq arg4511 
            (Bits.U.of_int 15 5) && Bitops.eq arg4512 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4508 (Bits.U.of_int 16 5) && Bitops.eq arg4509 
            (Bits.U.of_int 0 1) && Bitops.eq arg4510 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4505 (Bits.U.of_int 17 5) && Bitops.eq arg4506 
            (Bits.U.of_int 0 1) && Bitops.eq arg4507 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4502 (Bits.U.of_int 18 5) && Bitops.eq arg4503 
            (Bits.U.of_int 0 1) && Bitops.eq arg4504 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4499 (Bits.U.of_int 19 5) && Bitops.eq arg4500 
            (Bits.U.of_int 0 1) && Bitops.eq arg4501 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4496 (Bits.U.of_int 20 5) && Bitops.eq arg4497 
            (Bits.U.of_int 0 1) && Bitops.eq arg4498 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4493 (Bits.U.of_int 21 5) && Bitops.eq arg4494 
            (Bits.U.of_int 0 1) && Bitops.eq arg4495 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4490 (Bits.U.of_int 22 5) && Bitops.eq arg4491 
            (Bits.U.of_int 0 1) && Bitops.eq arg4492 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4487 (Bits.U.of_int 23 5) && Bitops.eq arg4488 
            (Bits.U.of_int 0 1) && Bitops.eq arg4489 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4484 (Bits.U.of_int 24 5) && Bitops.eq arg4485 
            (Bits.U.of_int 0 1) && Bitops.eq arg4486 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4481 (Bits.U.of_int 26 5) && Bitops.eq arg4482 
            (Bits.U.of_int 0 1) && Bitops.eq arg4483 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4478 (Bits.U.of_int 27 5) && Bitops.eq arg4479 
            (Bits.U.of_int 0 1) && Bitops.eq arg4480 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4475 (Bits.U.of_int 28 5) && Bitops.eq arg4476 
            (Bits.U.of_int 0 1) && Bitops.eq arg4477 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4472 (Bits.U.of_int 29 5) && Bitops.eq arg4473 
            (Bits.U.of_int 0 1) && Bitops.eq arg4474 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4469 (Bits.U.of_int 30 5) && Bitops.eq arg4470 
            (Bits.U.of_int 0 1) && Bitops.eq arg4471 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4466 (Bits.U.of_int 31 5) && Bitops.eq arg4467 
            (Bits.U.of_int 0 1) && Bitops.eq arg4468 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4463 (Bits.U.of_int 0 1) && Bitops.eq arg4464 
            (Bits.U.of_int 0 1) && Bitops.eq arg4465 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 25 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4589), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4590), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4587), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4588), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4585), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4586), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4583), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4584), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4581), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4582), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4579), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4580), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4577), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4578), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4575), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4576), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4572), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4573), _), 32); 
                    RP.Const (RP.Bits arg4574)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4569), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4570), _), 32); 
                    RP.Const (RP.Bits arg4571)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4566), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4567), _), 32); 
                    RP.Const (RP.Bits arg4568)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4563), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4564), _), 32); 
                    RP.Const (RP.Bits arg4565)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4560), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4561), _), 32); 
                    RP.Const (RP.Bits arg4562)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4557), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4558), _), 32); 
                    RP.Const (RP.Bits arg4559)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4554), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4555), _), 32); 
                    RP.Const (RP.Bits arg4556)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4551), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4552), _), 32); 
                    RP.Const (RP.Bits arg4553)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4548), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4549), _), 32); 
                    RP.Const (RP.Bits arg4550)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4545), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4546), _), 32); 
                    RP.Const (RP.Bits arg4547)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4542), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4543), _), 32); 
                    RP.Const (RP.Bits arg4544)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4539), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4540), _), 32); 
                    RP.Const (RP.Bits arg4541)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4536), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4537), _), 32); 
                    RP.Const (RP.Bits arg4538)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4533), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4534), _), 32); 
                    RP.Const (RP.Bits arg4535)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4530), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4531), _), 32); 
                    RP.Const (RP.Bits arg4532)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4527), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4528), _), 32); 
                  RP.Const (RP.Bits arg4529)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4589 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4590 (Bits.U.of_int 24 5) && Bitops.eq arg4587 
            (Bits.U.of_int 9 5) && Bitops.eq arg4588 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4585 (Bits.U.of_int 10 5) && Bitops.eq arg4586 
            (Bits.U.of_int 26 5) && Bitops.eq arg4583 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4584 (Bits.U.of_int 27 5) && Bitops.eq arg4581 
            (Bits.U.of_int 12 5) && Bitops.eq arg4582 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4579 (Bits.U.of_int 13 5) && Bitops.eq arg4580 
            (Bits.U.of_int 29 5) && Bitops.eq arg4577 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4578 (Bits.U.of_int 30 5) && Bitops.eq arg4575 
            (Bits.U.of_int 15 5) && Bitops.eq arg4576 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4572 (Bits.U.of_int 16 5) && Bitops.eq arg4573 
            (Bits.U.of_int 0 1) && Bitops.eq arg4574 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4569 (Bits.U.of_int 17 5) && Bitops.eq arg4570 
            (Bits.U.of_int 0 1) && Bitops.eq arg4571 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4566 (Bits.U.of_int 18 5) && Bitops.eq arg4567 
            (Bits.U.of_int 0 1) && Bitops.eq arg4568 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4563 (Bits.U.of_int 19 5) && Bitops.eq arg4564 
            (Bits.U.of_int 0 1) && Bitops.eq arg4565 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4560 (Bits.U.of_int 20 5) && Bitops.eq arg4561 
            (Bits.U.of_int 0 1) && Bitops.eq arg4562 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4557 (Bits.U.of_int 21 5) && Bitops.eq arg4558 
            (Bits.U.of_int 0 1) && Bitops.eq arg4559 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4554 (Bits.U.of_int 22 5) && Bitops.eq arg4555 
            (Bits.U.of_int 0 1) && Bitops.eq arg4556 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4551 (Bits.U.of_int 23 5) && Bitops.eq arg4552 
            (Bits.U.of_int 0 1) && Bitops.eq arg4553 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4548 (Bits.U.of_int 25 5) && Bitops.eq arg4549 
            (Bits.U.of_int 0 1) && Bitops.eq arg4550 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4545 (Bits.U.of_int 26 5) && Bitops.eq arg4546 
            (Bits.U.of_int 0 1) && Bitops.eq arg4547 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4542 (Bits.U.of_int 27 5) && Bitops.eq arg4543 
            (Bits.U.of_int 0 1) && Bitops.eq arg4544 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4539 (Bits.U.of_int 28 5) && Bitops.eq arg4540 
            (Bits.U.of_int 0 1) && Bitops.eq arg4541 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4536 (Bits.U.of_int 29 5) && Bitops.eq arg4537 
            (Bits.U.of_int 0 1) && Bitops.eq arg4538 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4533 (Bits.U.of_int 30 5) && Bitops.eq arg4534 
            (Bits.U.of_int 0 1) && Bitops.eq arg4535 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4530 (Bits.U.of_int 31 5) && Bitops.eq arg4531 
            (Bits.U.of_int 0 1) && Bitops.eq arg4532 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4527 (Bits.U.of_int 0 1) && Bitops.eq arg4528 
            (Bits.U.of_int 0 1) && Bitops.eq arg4529 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 24 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4653), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4654), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4651), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4652), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4649), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4650), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4647), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4648), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4645), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4646), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4643), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4644), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4641), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4642), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4639), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4640), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4636), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4637), _), 32); 
                    RP.Const (RP.Bits arg4638)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4633), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4634), _), 32); 
                    RP.Const (RP.Bits arg4635)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4630), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4631), _), 32); 
                    RP.Const (RP.Bits arg4632)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4627), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4628), _), 32); 
                    RP.Const (RP.Bits arg4629)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4624), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4625), _), 32); 
                    RP.Const (RP.Bits arg4626)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4621), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4622), _), 32); 
                    RP.Const (RP.Bits arg4623)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4618), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4619), _), 32); 
                    RP.Const (RP.Bits arg4620)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4615), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4616), _), 32); 
                    RP.Const (RP.Bits arg4617)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4612), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4613), _), 32); 
                    RP.Const (RP.Bits arg4614)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4609), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4610), _), 32); 
                    RP.Const (RP.Bits arg4611)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4606), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4607), _), 32); 
                    RP.Const (RP.Bits arg4608)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4603), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4604), _), 32); 
                    RP.Const (RP.Bits arg4605)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4600), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4601), _), 32); 
                    RP.Const (RP.Bits arg4602)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4597), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4598), _), 32); 
                    RP.Const (RP.Bits arg4599)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4594), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4595), _), 32); 
                    RP.Const (RP.Bits arg4596)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4591), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4592), _), 32); 
                  RP.Const (RP.Bits arg4593)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4653 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4654 (Bits.U.of_int 24 5) && Bitops.eq arg4651 
            (Bits.U.of_int 9 5) && Bitops.eq arg4652 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4649 (Bits.U.of_int 10 5) && Bitops.eq arg4650 
            (Bits.U.of_int 26 5) && Bitops.eq arg4647 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4648 (Bits.U.of_int 27 5) && Bitops.eq arg4645 
            (Bits.U.of_int 12 5) && Bitops.eq arg4646 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4643 (Bits.U.of_int 13 5) && Bitops.eq arg4644 
            (Bits.U.of_int 29 5) && Bitops.eq arg4641 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4642 (Bits.U.of_int 30 5) && Bitops.eq arg4639 
            (Bits.U.of_int 15 5) && Bitops.eq arg4640 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4636 (Bits.U.of_int 16 5) && Bitops.eq arg4637 
            (Bits.U.of_int 0 1) && Bitops.eq arg4638 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4633 (Bits.U.of_int 17 5) && Bitops.eq arg4634 
            (Bits.U.of_int 0 1) && Bitops.eq arg4635 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4630 (Bits.U.of_int 18 5) && Bitops.eq arg4631 
            (Bits.U.of_int 0 1) && Bitops.eq arg4632 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4627 (Bits.U.of_int 19 5) && Bitops.eq arg4628 
            (Bits.U.of_int 0 1) && Bitops.eq arg4629 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4624 (Bits.U.of_int 20 5) && Bitops.eq arg4625 
            (Bits.U.of_int 0 1) && Bitops.eq arg4626 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4621 (Bits.U.of_int 21 5) && Bitops.eq arg4622 
            (Bits.U.of_int 0 1) && Bitops.eq arg4623 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4618 (Bits.U.of_int 22 5) && Bitops.eq arg4619 
            (Bits.U.of_int 0 1) && Bitops.eq arg4620 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4615 (Bits.U.of_int 24 5) && Bitops.eq arg4616 
            (Bits.U.of_int 0 1) && Bitops.eq arg4617 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4612 (Bits.U.of_int 25 5) && Bitops.eq arg4613 
            (Bits.U.of_int 0 1) && Bitops.eq arg4614 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4609 (Bits.U.of_int 26 5) && Bitops.eq arg4610 
            (Bits.U.of_int 0 1) && Bitops.eq arg4611 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4606 (Bits.U.of_int 27 5) && Bitops.eq arg4607 
            (Bits.U.of_int 0 1) && Bitops.eq arg4608 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4603 (Bits.U.of_int 28 5) && Bitops.eq arg4604 
            (Bits.U.of_int 0 1) && Bitops.eq arg4605 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4600 (Bits.U.of_int 29 5) && Bitops.eq arg4601 
            (Bits.U.of_int 0 1) && Bitops.eq arg4602 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4597 (Bits.U.of_int 30 5) && Bitops.eq arg4598 
            (Bits.U.of_int 0 1) && Bitops.eq arg4599 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4594 (Bits.U.of_int 31 5) && Bitops.eq arg4595 
            (Bits.U.of_int 0 1) && Bitops.eq arg4596 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4591 (Bits.U.of_int 0 1) && Bitops.eq arg4592 
            (Bits.U.of_int 0 1) && Bitops.eq arg4593 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 23 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4717), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4718), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4715), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4716), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4713), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4714), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4711), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4712), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4709), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4710), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4707), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4708), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4705), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4706), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4703), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4704), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4700), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4701), _), 32); 
                    RP.Const (RP.Bits arg4702)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4697), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4698), _), 32); 
                    RP.Const (RP.Bits arg4699)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4694), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4695), _), 32); 
                    RP.Const (RP.Bits arg4696)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4691), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4692), _), 32); 
                    RP.Const (RP.Bits arg4693)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4688), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4689), _), 32); 
                    RP.Const (RP.Bits arg4690)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4685), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4686), _), 32); 
                    RP.Const (RP.Bits arg4687)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4682), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4683), _), 32); 
                    RP.Const (RP.Bits arg4684)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4679), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4680), _), 32); 
                    RP.Const (RP.Bits arg4681)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4676), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4677), _), 32); 
                    RP.Const (RP.Bits arg4678)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4673), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4674), _), 32); 
                    RP.Const (RP.Bits arg4675)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4670), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4671), _), 32); 
                    RP.Const (RP.Bits arg4672)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4667), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4668), _), 32); 
                    RP.Const (RP.Bits arg4669)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4664), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4665), _), 32); 
                    RP.Const (RP.Bits arg4666)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4661), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4662), _), 32); 
                    RP.Const (RP.Bits arg4663)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4658), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4659), _), 32); 
                    RP.Const (RP.Bits arg4660)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4655), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4656), _), 32); 
                  RP.Const (RP.Bits arg4657)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4717 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4718 (Bits.U.of_int 24 5) && Bitops.eq arg4715 
            (Bits.U.of_int 9 5) && Bitops.eq arg4716 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4713 (Bits.U.of_int 10 5) && Bitops.eq arg4714 
            (Bits.U.of_int 26 5) && Bitops.eq arg4711 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4712 (Bits.U.of_int 27 5) && Bitops.eq arg4709 
            (Bits.U.of_int 12 5) && Bitops.eq arg4710 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4707 (Bits.U.of_int 13 5) && Bitops.eq arg4708 
            (Bits.U.of_int 29 5) && Bitops.eq arg4705 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4706 (Bits.U.of_int 30 5) && Bitops.eq arg4703 
            (Bits.U.of_int 15 5) && Bitops.eq arg4704 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4700 (Bits.U.of_int 16 5) && Bitops.eq arg4701 
            (Bits.U.of_int 0 1) && Bitops.eq arg4702 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4697 (Bits.U.of_int 17 5) && Bitops.eq arg4698 
            (Bits.U.of_int 0 1) && Bitops.eq arg4699 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4694 (Bits.U.of_int 18 5) && Bitops.eq arg4695 
            (Bits.U.of_int 0 1) && Bitops.eq arg4696 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4691 (Bits.U.of_int 19 5) && Bitops.eq arg4692 
            (Bits.U.of_int 0 1) && Bitops.eq arg4693 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4688 (Bits.U.of_int 20 5) && Bitops.eq arg4689 
            (Bits.U.of_int 0 1) && Bitops.eq arg4690 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4685 (Bits.U.of_int 21 5) && Bitops.eq arg4686 
            (Bits.U.of_int 0 1) && Bitops.eq arg4687 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4682 (Bits.U.of_int 23 5) && Bitops.eq arg4683 
            (Bits.U.of_int 0 1) && Bitops.eq arg4684 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4679 (Bits.U.of_int 24 5) && Bitops.eq arg4680 
            (Bits.U.of_int 0 1) && Bitops.eq arg4681 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4676 (Bits.U.of_int 25 5) && Bitops.eq arg4677 
            (Bits.U.of_int 0 1) && Bitops.eq arg4678 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4673 (Bits.U.of_int 26 5) && Bitops.eq arg4674 
            (Bits.U.of_int 0 1) && Bitops.eq arg4675 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4670 (Bits.U.of_int 27 5) && Bitops.eq arg4671 
            (Bits.U.of_int 0 1) && Bitops.eq arg4672 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4667 (Bits.U.of_int 28 5) && Bitops.eq arg4668 
            (Bits.U.of_int 0 1) && Bitops.eq arg4669 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4664 (Bits.U.of_int 29 5) && Bitops.eq arg4665 
            (Bits.U.of_int 0 1) && Bitops.eq arg4666 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4661 (Bits.U.of_int 30 5) && Bitops.eq arg4662 
            (Bits.U.of_int 0 1) && Bitops.eq arg4663 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4658 (Bits.U.of_int 31 5) && Bitops.eq arg4659 
            (Bits.U.of_int 0 1) && Bitops.eq arg4660 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4655 (Bits.U.of_int 0 1) && Bitops.eq arg4656 
            (Bits.U.of_int 0 1) && Bitops.eq arg4657 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 22 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4781), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4782), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4779), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4780), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4777), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4778), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4775), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4776), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4773), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4774), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4771), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4772), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4769), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4770), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4767), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4768), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4764), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4765), _), 32); 
                    RP.Const (RP.Bits arg4766)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4761), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4762), _), 32); 
                    RP.Const (RP.Bits arg4763)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4758), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4759), _), 32); 
                    RP.Const (RP.Bits arg4760)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4755), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4756), _), 32); 
                    RP.Const (RP.Bits arg4757)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4752), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4753), _), 32); 
                    RP.Const (RP.Bits arg4754)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4749), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4750), _), 32); 
                    RP.Const (RP.Bits arg4751)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4746), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4747), _), 32); 
                    RP.Const (RP.Bits arg4748)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4743), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4744), _), 32); 
                    RP.Const (RP.Bits arg4745)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4740), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4741), _), 32); 
                    RP.Const (RP.Bits arg4742)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4737), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4738), _), 32); 
                    RP.Const (RP.Bits arg4739)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4734), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4735), _), 32); 
                    RP.Const (RP.Bits arg4736)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4731), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4732), _), 32); 
                    RP.Const (RP.Bits arg4733)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4728), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4729), _), 32); 
                    RP.Const (RP.Bits arg4730)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4725), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4726), _), 32); 
                    RP.Const (RP.Bits arg4727)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4722), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4723), _), 32); 
                    RP.Const (RP.Bits arg4724)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4719), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4720), _), 32); 
                  RP.Const (RP.Bits arg4721)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4781 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4782 (Bits.U.of_int 24 5) && Bitops.eq arg4779 
            (Bits.U.of_int 9 5) && Bitops.eq arg4780 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4777 (Bits.U.of_int 10 5) && Bitops.eq arg4778 
            (Bits.U.of_int 26 5) && Bitops.eq arg4775 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4776 (Bits.U.of_int 27 5) && Bitops.eq arg4773 
            (Bits.U.of_int 12 5) && Bitops.eq arg4774 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4771 (Bits.U.of_int 13 5) && Bitops.eq arg4772 
            (Bits.U.of_int 29 5) && Bitops.eq arg4769 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4770 (Bits.U.of_int 30 5) && Bitops.eq arg4767 
            (Bits.U.of_int 15 5) && Bitops.eq arg4768 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4764 (Bits.U.of_int 16 5) && Bitops.eq arg4765 
            (Bits.U.of_int 0 1) && Bitops.eq arg4766 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4761 (Bits.U.of_int 17 5) && Bitops.eq arg4762 
            (Bits.U.of_int 0 1) && Bitops.eq arg4763 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4758 (Bits.U.of_int 18 5) && Bitops.eq arg4759 
            (Bits.U.of_int 0 1) && Bitops.eq arg4760 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4755 (Bits.U.of_int 19 5) && Bitops.eq arg4756 
            (Bits.U.of_int 0 1) && Bitops.eq arg4757 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4752 (Bits.U.of_int 20 5) && Bitops.eq arg4753 
            (Bits.U.of_int 0 1) && Bitops.eq arg4754 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4749 (Bits.U.of_int 22 5) && Bitops.eq arg4750 
            (Bits.U.of_int 0 1) && Bitops.eq arg4751 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4746 (Bits.U.of_int 23 5) && Bitops.eq arg4747 
            (Bits.U.of_int 0 1) && Bitops.eq arg4748 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4743 (Bits.U.of_int 24 5) && Bitops.eq arg4744 
            (Bits.U.of_int 0 1) && Bitops.eq arg4745 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4740 (Bits.U.of_int 25 5) && Bitops.eq arg4741 
            (Bits.U.of_int 0 1) && Bitops.eq arg4742 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4737 (Bits.U.of_int 26 5) && Bitops.eq arg4738 
            (Bits.U.of_int 0 1) && Bitops.eq arg4739 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4734 (Bits.U.of_int 27 5) && Bitops.eq arg4735 
            (Bits.U.of_int 0 1) && Bitops.eq arg4736 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4731 (Bits.U.of_int 28 5) && Bitops.eq arg4732 
            (Bits.U.of_int 0 1) && Bitops.eq arg4733 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4728 (Bits.U.of_int 29 5) && Bitops.eq arg4729 
            (Bits.U.of_int 0 1) && Bitops.eq arg4730 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4725 (Bits.U.of_int 30 5) && Bitops.eq arg4726 
            (Bits.U.of_int 0 1) && Bitops.eq arg4727 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4722 (Bits.U.of_int 31 5) && Bitops.eq arg4723 
            (Bits.U.of_int 0 1) && Bitops.eq arg4724 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4719 (Bits.U.of_int 0 1) && Bitops.eq arg4720 
            (Bits.U.of_int 0 1) && Bitops.eq arg4721 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 21 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4845), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4846), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4843), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4844), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4841), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4842), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4839), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4840), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4837), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4838), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4835), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4836), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4833), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4834), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4831), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4832), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4828), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4829), _), 32); 
                    RP.Const (RP.Bits arg4830)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4825), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4826), _), 32); 
                    RP.Const (RP.Bits arg4827)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4822), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4823), _), 32); 
                    RP.Const (RP.Bits arg4824)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4819), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4820), _), 32); 
                    RP.Const (RP.Bits arg4821)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4816), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4817), _), 32); 
                    RP.Const (RP.Bits arg4818)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4813), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4814), _), 32); 
                    RP.Const (RP.Bits arg4815)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4810), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4811), _), 32); 
                    RP.Const (RP.Bits arg4812)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4807), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4808), _), 32); 
                    RP.Const (RP.Bits arg4809)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4804), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4805), _), 32); 
                    RP.Const (RP.Bits arg4806)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4801), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4802), _), 32); 
                    RP.Const (RP.Bits arg4803)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4798), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4799), _), 32); 
                    RP.Const (RP.Bits arg4800)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4795), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4796), _), 32); 
                    RP.Const (RP.Bits arg4797)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4792), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4793), _), 32); 
                    RP.Const (RP.Bits arg4794)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4789), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4790), _), 32); 
                    RP.Const (RP.Bits arg4791)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4786), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4787), _), 32); 
                    RP.Const (RP.Bits arg4788)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4783), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4784), _), 32); 
                  RP.Const (RP.Bits arg4785)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4845 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4846 (Bits.U.of_int 24 5) && Bitops.eq arg4843 
            (Bits.U.of_int 9 5) && Bitops.eq arg4844 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4841 (Bits.U.of_int 10 5) && Bitops.eq arg4842 
            (Bits.U.of_int 26 5) && Bitops.eq arg4839 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4840 (Bits.U.of_int 27 5) && Bitops.eq arg4837 
            (Bits.U.of_int 12 5) && Bitops.eq arg4838 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4835 (Bits.U.of_int 13 5) && Bitops.eq arg4836 
            (Bits.U.of_int 29 5) && Bitops.eq arg4833 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4834 (Bits.U.of_int 30 5) && Bitops.eq arg4831 
            (Bits.U.of_int 15 5) && Bitops.eq arg4832 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4828 (Bits.U.of_int 16 5) && Bitops.eq arg4829 
            (Bits.U.of_int 0 1) && Bitops.eq arg4830 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4825 (Bits.U.of_int 17 5) && Bitops.eq arg4826 
            (Bits.U.of_int 0 1) && Bitops.eq arg4827 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4822 (Bits.U.of_int 18 5) && Bitops.eq arg4823 
            (Bits.U.of_int 0 1) && Bitops.eq arg4824 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4819 (Bits.U.of_int 19 5) && Bitops.eq arg4820 
            (Bits.U.of_int 0 1) && Bitops.eq arg4821 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4816 (Bits.U.of_int 21 5) && Bitops.eq arg4817 
            (Bits.U.of_int 0 1) && Bitops.eq arg4818 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4813 (Bits.U.of_int 22 5) && Bitops.eq arg4814 
            (Bits.U.of_int 0 1) && Bitops.eq arg4815 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4810 (Bits.U.of_int 23 5) && Bitops.eq arg4811 
            (Bits.U.of_int 0 1) && Bitops.eq arg4812 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4807 (Bits.U.of_int 24 5) && Bitops.eq arg4808 
            (Bits.U.of_int 0 1) && Bitops.eq arg4809 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4804 (Bits.U.of_int 25 5) && Bitops.eq arg4805 
            (Bits.U.of_int 0 1) && Bitops.eq arg4806 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4801 (Bits.U.of_int 26 5) && Bitops.eq arg4802 
            (Bits.U.of_int 0 1) && Bitops.eq arg4803 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4798 (Bits.U.of_int 27 5) && Bitops.eq arg4799 
            (Bits.U.of_int 0 1) && Bitops.eq arg4800 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4795 (Bits.U.of_int 28 5) && Bitops.eq arg4796 
            (Bits.U.of_int 0 1) && Bitops.eq arg4797 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4792 (Bits.U.of_int 29 5) && Bitops.eq arg4793 
            (Bits.U.of_int 0 1) && Bitops.eq arg4794 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4789 (Bits.U.of_int 30 5) && Bitops.eq arg4790 
            (Bits.U.of_int 0 1) && Bitops.eq arg4791 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4786 (Bits.U.of_int 31 5) && Bitops.eq arg4787 
            (Bits.U.of_int 0 1) && Bitops.eq arg4788 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4783 (Bits.U.of_int 0 1) && Bitops.eq arg4784 
            (Bits.U.of_int 0 1) && Bitops.eq arg4785 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 20 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4909), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4910), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4907), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4908), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4905), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4906), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4903), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4904), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4901), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4902), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4899), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4900), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4897), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4898), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4895), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4896), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4892), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4893), _), 32); 
                    RP.Const (RP.Bits arg4894)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4889), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4890), _), 32); 
                    RP.Const (RP.Bits arg4891)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4886), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4887), _), 32); 
                    RP.Const (RP.Bits arg4888)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4883), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4884), _), 32); 
                    RP.Const (RP.Bits arg4885)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4880), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4881), _), 32); 
                    RP.Const (RP.Bits arg4882)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4877), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4878), _), 32); 
                    RP.Const (RP.Bits arg4879)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4874), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4875), _), 32); 
                    RP.Const (RP.Bits arg4876)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4871), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4872), _), 32); 
                    RP.Const (RP.Bits arg4873)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4868), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4869), _), 32); 
                    RP.Const (RP.Bits arg4870)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4865), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4866), _), 32); 
                    RP.Const (RP.Bits arg4867)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4862), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4863), _), 32); 
                    RP.Const (RP.Bits arg4864)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4859), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4860), _), 32); 
                    RP.Const (RP.Bits arg4861)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4856), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4857), _), 32); 
                    RP.Const (RP.Bits arg4858)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4853), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4854), _), 32); 
                    RP.Const (RP.Bits arg4855)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4850), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4851), _), 32); 
                    RP.Const (RP.Bits arg4852)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4847), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4848), _), 32); 
                  RP.Const (RP.Bits arg4849)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4909 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4910 (Bits.U.of_int 24 5) && Bitops.eq arg4907 
            (Bits.U.of_int 9 5) && Bitops.eq arg4908 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4905 (Bits.U.of_int 10 5) && Bitops.eq arg4906 
            (Bits.U.of_int 26 5) && Bitops.eq arg4903 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4904 (Bits.U.of_int 27 5) && Bitops.eq arg4901 
            (Bits.U.of_int 12 5) && Bitops.eq arg4902 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4899 (Bits.U.of_int 13 5) && Bitops.eq arg4900 
            (Bits.U.of_int 29 5) && Bitops.eq arg4897 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4898 (Bits.U.of_int 30 5) && Bitops.eq arg4895 
            (Bits.U.of_int 15 5) && Bitops.eq arg4896 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4892 (Bits.U.of_int 16 5) && Bitops.eq arg4893 
            (Bits.U.of_int 0 1) && Bitops.eq arg4894 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4889 (Bits.U.of_int 17 5) && Bitops.eq arg4890 
            (Bits.U.of_int 0 1) && Bitops.eq arg4891 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4886 (Bits.U.of_int 18 5) && Bitops.eq arg4887 
            (Bits.U.of_int 0 1) && Bitops.eq arg4888 (Bits.U.of_int 6 32) && 
          Bitops.eq arg4883 (Bits.U.of_int 20 5) && Bitops.eq arg4884 
            (Bits.U.of_int 0 1) && Bitops.eq arg4885 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4880 (Bits.U.of_int 21 5) && Bitops.eq arg4881 
            (Bits.U.of_int 0 1) && Bitops.eq arg4882 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4877 (Bits.U.of_int 22 5) && Bitops.eq arg4878 
            (Bits.U.of_int 0 1) && Bitops.eq arg4879 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4874 (Bits.U.of_int 23 5) && Bitops.eq arg4875 
            (Bits.U.of_int 0 1) && Bitops.eq arg4876 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4871 (Bits.U.of_int 24 5) && Bitops.eq arg4872 
            (Bits.U.of_int 0 1) && Bitops.eq arg4873 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4868 (Bits.U.of_int 25 5) && Bitops.eq arg4869 
            (Bits.U.of_int 0 1) && Bitops.eq arg4870 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4865 (Bits.U.of_int 26 5) && Bitops.eq arg4866 
            (Bits.U.of_int 0 1) && Bitops.eq arg4867 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4862 (Bits.U.of_int 27 5) && Bitops.eq arg4863 
            (Bits.U.of_int 0 1) && Bitops.eq arg4864 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4859 (Bits.U.of_int 28 5) && Bitops.eq arg4860 
            (Bits.U.of_int 0 1) && Bitops.eq arg4861 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4856 (Bits.U.of_int 29 5) && Bitops.eq arg4857 
            (Bits.U.of_int 0 1) && Bitops.eq arg4858 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4853 (Bits.U.of_int 30 5) && Bitops.eq arg4854 
            (Bits.U.of_int 0 1) && Bitops.eq arg4855 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4850 (Bits.U.of_int 31 5) && Bitops.eq arg4851 
            (Bits.U.of_int 0 1) && Bitops.eq arg4852 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4847 (Bits.U.of_int 0 1) && Bitops.eq arg4848 
            (Bits.U.of_int 0 1) && Bitops.eq arg4849 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 19 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg4973), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg4974), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4971), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4972), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4969), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4970), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4967), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4968), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4965), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4966), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4963), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4964), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4961), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4962), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4959), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg4960), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4956), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4957), _), 32); 
                    RP.Const (RP.Bits arg4958)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4953), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4954), _), 32); 
                    RP.Const (RP.Bits arg4955)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4950), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4951), _), 32); 
                    RP.Const (RP.Bits arg4952)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4947), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4948), _), 32); 
                    RP.Const (RP.Bits arg4949)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4944), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4945), _), 32); 
                    RP.Const (RP.Bits arg4946)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4941), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4942), _), 32); 
                    RP.Const (RP.Bits arg4943)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4938), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4939), _), 32); 
                    RP.Const (RP.Bits arg4940)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4935), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4936), _), 32); 
                    RP.Const (RP.Bits arg4937)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4932), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4933), _), 32); 
                    RP.Const (RP.Bits arg4934)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4929), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4930), _), 32); 
                    RP.Const (RP.Bits arg4931)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4926), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4927), _), 32); 
                    RP.Const (RP.Bits arg4928)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4923), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4924), _), 32); 
                    RP.Const (RP.Bits arg4925)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4920), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4921), _), 32); 
                    RP.Const (RP.Bits arg4922)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4917), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4918), _), 32); 
                    RP.Const (RP.Bits arg4919)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4914), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4915), _), 32); 
                    RP.Const (RP.Bits arg4916)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4911), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4912), _), 32); 
                  RP.Const (RP.Bits arg4913)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg4973 (Bits.U.of_int 8 5) && Bitops.eq 
            arg4974 (Bits.U.of_int 24 5) && Bitops.eq arg4971 
            (Bits.U.of_int 9 5) && Bitops.eq arg4972 (Bits.U.of_int 25 5) && 
          Bitops.eq arg4969 (Bits.U.of_int 10 5) && Bitops.eq arg4970 
            (Bits.U.of_int 26 5) && Bitops.eq arg4967 (Bits.U.of_int 11 5) && 
          Bitops.eq arg4968 (Bits.U.of_int 27 5) && Bitops.eq arg4965 
            (Bits.U.of_int 12 5) && Bitops.eq arg4966 (Bits.U.of_int 28 5) && 
          Bitops.eq arg4963 (Bits.U.of_int 13 5) && Bitops.eq arg4964 
            (Bits.U.of_int 29 5) && Bitops.eq arg4961 (Bits.U.of_int 14 5) && 
          Bitops.eq arg4962 (Bits.U.of_int 30 5) && Bitops.eq arg4959 
            (Bits.U.of_int 15 5) && Bitops.eq arg4960 (Bits.U.of_int 31 5) && 
          Bitops.eq arg4956 (Bits.U.of_int 16 5) && Bitops.eq arg4957 
            (Bits.U.of_int 0 1) && Bitops.eq arg4958 (Bits.U.of_int 8 32) && 
          Bitops.eq arg4953 (Bits.U.of_int 17 5) && Bitops.eq arg4954 
            (Bits.U.of_int 0 1) && Bitops.eq arg4955 (Bits.U.of_int 7 32) && 
          Bitops.eq arg4950 (Bits.U.of_int 19 5) && Bitops.eq arg4951 
            (Bits.U.of_int 0 1) && Bitops.eq arg4952 (Bits.U.of_int 5 32) && 
          Bitops.eq arg4947 (Bits.U.of_int 20 5) && Bitops.eq arg4948 
            (Bits.U.of_int 0 1) && Bitops.eq arg4949 (Bits.U.of_int 4 32) && 
          Bitops.eq arg4944 (Bits.U.of_int 21 5) && Bitops.eq arg4945 
            (Bits.U.of_int 0 1) && Bitops.eq arg4946 (Bits.U.of_int 3 32) && 
          Bitops.eq arg4941 (Bits.U.of_int 22 5) && Bitops.eq arg4942 
            (Bits.U.of_int 0 1) && Bitops.eq arg4943 (Bits.U.of_int 2 32) && 
          Bitops.eq arg4938 (Bits.U.of_int 23 5) && Bitops.eq arg4939 
            (Bits.U.of_int 0 1) && Bitops.eq arg4940 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4935 (Bits.U.of_int 24 5) && Bitops.eq arg4936 
            (Bits.U.of_int 0 1) && Bitops.eq arg4937 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4932 (Bits.U.of_int 25 5) && Bitops.eq arg4933 
            (Bits.U.of_int 0 1) && Bitops.eq arg4934 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4929 (Bits.U.of_int 26 5) && Bitops.eq arg4930 
            (Bits.U.of_int 0 1) && Bitops.eq arg4931 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4926 (Bits.U.of_int 27 5) && Bitops.eq arg4927 
            (Bits.U.of_int 0 1) && Bitops.eq arg4928 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4923 (Bits.U.of_int 28 5) && Bitops.eq arg4924 
            (Bits.U.of_int 0 1) && Bitops.eq arg4925 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4920 (Bits.U.of_int 29 5) && Bitops.eq arg4921 
            (Bits.U.of_int 0 1) && Bitops.eq arg4922 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4917 (Bits.U.of_int 30 5) && Bitops.eq arg4918 
            (Bits.U.of_int 0 1) && Bitops.eq arg4919 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4914 (Bits.U.of_int 31 5) && Bitops.eq arg4915 
            (Bits.U.of_int 0 1) && Bitops.eq arg4916 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4911 (Bits.U.of_int 0 1) && Bitops.eq arg4912 
            (Bits.U.of_int 0 1) && Bitops.eq arg4913 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 18 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5037), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5038), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5035), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5036), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5033), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5034), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5031), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5032), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5029), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5030), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5027), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5028), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5025), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5026), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5023), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5024), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5020), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5021), _), 32); 
                    RP.Const (RP.Bits arg5022)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5017), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5018), _), 32); 
                    RP.Const (RP.Bits arg5019)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5014), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5015), _), 32); 
                    RP.Const (RP.Bits arg5016)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5011), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5012), _), 32); 
                    RP.Const (RP.Bits arg5013)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5008), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5009), _), 32); 
                    RP.Const (RP.Bits arg5010)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5005), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5006), _), 32); 
                    RP.Const (RP.Bits arg5007)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5002), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5003), _), 32); 
                    RP.Const (RP.Bits arg5004)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4999), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5000), _), 32); 
                    RP.Const (RP.Bits arg5001)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4996), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4997), _), 32); 
                    RP.Const (RP.Bits arg4998)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4993), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4994), _), 32); 
                    RP.Const (RP.Bits arg4995)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4990), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4991), _), 32); 
                    RP.Const (RP.Bits arg4992)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4987), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4988), _), 32); 
                    RP.Const (RP.Bits arg4989)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4984), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4985), _), 32); 
                    RP.Const (RP.Bits arg4986)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4981), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4982), _), 32); 
                    RP.Const (RP.Bits arg4983)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4978), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg4979), _), 32); 
                    RP.Const (RP.Bits arg4980)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg4975), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg4976), _), 32); 
                  RP.Const (RP.Bits arg4977)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5037 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5038 (Bits.U.of_int 24 5) && Bitops.eq arg5035 
            (Bits.U.of_int 9 5) && Bitops.eq arg5036 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5033 (Bits.U.of_int 10 5) && Bitops.eq arg5034 
            (Bits.U.of_int 26 5) && Bitops.eq arg5031 (Bits.U.of_int 11 5) && 
          Bitops.eq arg5032 (Bits.U.of_int 27 5) && Bitops.eq arg5029 
            (Bits.U.of_int 12 5) && Bitops.eq arg5030 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5027 (Bits.U.of_int 13 5) && Bitops.eq arg5028 
            (Bits.U.of_int 29 5) && Bitops.eq arg5025 (Bits.U.of_int 14 5) && 
          Bitops.eq arg5026 (Bits.U.of_int 30 5) && Bitops.eq arg5023 
            (Bits.U.of_int 15 5) && Bitops.eq arg5024 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5020 (Bits.U.of_int 16 5) && Bitops.eq arg5021 
            (Bits.U.of_int 0 1) && Bitops.eq arg5022 (Bits.U.of_int 8 32) && 
          Bitops.eq arg5017 (Bits.U.of_int 18 5) && Bitops.eq arg5018 
            (Bits.U.of_int 0 1) && Bitops.eq arg5019 (Bits.U.of_int 6 32) && 
          Bitops.eq arg5014 (Bits.U.of_int 19 5) && Bitops.eq arg5015 
            (Bits.U.of_int 0 1) && Bitops.eq arg5016 (Bits.U.of_int 5 32) && 
          Bitops.eq arg5011 (Bits.U.of_int 20 5) && Bitops.eq arg5012 
            (Bits.U.of_int 0 1) && Bitops.eq arg5013 (Bits.U.of_int 4 32) && 
          Bitops.eq arg5008 (Bits.U.of_int 21 5) && Bitops.eq arg5009 
            (Bits.U.of_int 0 1) && Bitops.eq arg5010 (Bits.U.of_int 3 32) && 
          Bitops.eq arg5005 (Bits.U.of_int 22 5) && Bitops.eq arg5006 
            (Bits.U.of_int 0 1) && Bitops.eq arg5007 (Bits.U.of_int 2 32) && 
          Bitops.eq arg5002 (Bits.U.of_int 23 5) && Bitops.eq arg5003 
            (Bits.U.of_int 0 1) && Bitops.eq arg5004 (Bits.U.of_int 1 32) && 
          Bitops.eq arg4999 (Bits.U.of_int 24 5) && Bitops.eq arg5000 
            (Bits.U.of_int 0 1) && Bitops.eq arg5001 (Bits.U.of_int 16 32) && 
          Bitops.eq arg4996 (Bits.U.of_int 25 5) && Bitops.eq arg4997 
            (Bits.U.of_int 0 1) && Bitops.eq arg4998 (Bits.U.of_int 15 32) && 
          Bitops.eq arg4993 (Bits.U.of_int 26 5) && Bitops.eq arg4994 
            (Bits.U.of_int 0 1) && Bitops.eq arg4995 (Bits.U.of_int 14 32) && 
          Bitops.eq arg4990 (Bits.U.of_int 27 5) && Bitops.eq arg4991 
            (Bits.U.of_int 0 1) && Bitops.eq arg4992 (Bits.U.of_int 13 32) && 
          Bitops.eq arg4987 (Bits.U.of_int 28 5) && Bitops.eq arg4988 
            (Bits.U.of_int 0 1) && Bitops.eq arg4989 (Bits.U.of_int 12 32) && 
          Bitops.eq arg4984 (Bits.U.of_int 29 5) && Bitops.eq arg4985 
            (Bits.U.of_int 0 1) && Bitops.eq arg4986 (Bits.U.of_int 11 32) && 
          Bitops.eq arg4981 (Bits.U.of_int 30 5) && Bitops.eq arg4982 
            (Bits.U.of_int 0 1) && Bitops.eq arg4983 (Bits.U.of_int 10 32) && 
          Bitops.eq arg4978 (Bits.U.of_int 31 5) && Bitops.eq arg4979 
            (Bits.U.of_int 0 1) && Bitops.eq arg4980 (Bits.U.of_int 9 32) && 
          Bitops.eq arg4975 (Bits.U.of_int 0 1) && Bitops.eq arg4976 
            (Bits.U.of_int 0 1) && Bitops.eq arg4977 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 17 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5101), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5102), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5099), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5100), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5097), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5098), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5095), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5096), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5093), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5094), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5091), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5092), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5089), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5090), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5087), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5088), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5084), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5085), _), 32); 
                    RP.Const (RP.Bits arg5086)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5081), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5082), _), 32); 
                    RP.Const (RP.Bits arg5083)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5078), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5079), _), 32); 
                    RP.Const (RP.Bits arg5080)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5075), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5076), _), 32); 
                    RP.Const (RP.Bits arg5077)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5072), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5073), _), 32); 
                    RP.Const (RP.Bits arg5074)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5069), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5070), _), 32); 
                    RP.Const (RP.Bits arg5071)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5066), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5067), _), 32); 
                    RP.Const (RP.Bits arg5068)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5063), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5064), _), 32); 
                    RP.Const (RP.Bits arg5065)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5060), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5061), _), 32); 
                    RP.Const (RP.Bits arg5062)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5057), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5058), _), 32); 
                    RP.Const (RP.Bits arg5059)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5054), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5055), _), 32); 
                    RP.Const (RP.Bits arg5056)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5051), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5052), _), 32); 
                    RP.Const (RP.Bits arg5053)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5048), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5049), _), 32); 
                    RP.Const (RP.Bits arg5050)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5045), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5046), _), 32); 
                    RP.Const (RP.Bits arg5047)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5042), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5043), _), 32); 
                    RP.Const (RP.Bits arg5044)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5039), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5040), _), 32); 
                  RP.Const (RP.Bits arg5041)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5101 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5102 (Bits.U.of_int 24 5) && Bitops.eq arg5099 
            (Bits.U.of_int 9 5) && Bitops.eq arg5100 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5097 (Bits.U.of_int 10 5) && Bitops.eq arg5098 
            (Bits.U.of_int 26 5) && Bitops.eq arg5095 (Bits.U.of_int 11 5) && 
          Bitops.eq arg5096 (Bits.U.of_int 27 5) && Bitops.eq arg5093 
            (Bits.U.of_int 12 5) && Bitops.eq arg5094 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5091 (Bits.U.of_int 13 5) && Bitops.eq arg5092 
            (Bits.U.of_int 29 5) && Bitops.eq arg5089 (Bits.U.of_int 14 5) && 
          Bitops.eq arg5090 (Bits.U.of_int 30 5) && Bitops.eq arg5087 
            (Bits.U.of_int 15 5) && Bitops.eq arg5088 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5084 (Bits.U.of_int 17 5) && Bitops.eq arg5085 
            (Bits.U.of_int 0 1) && Bitops.eq arg5086 (Bits.U.of_int 7 32) && 
          Bitops.eq arg5081 (Bits.U.of_int 18 5) && Bitops.eq arg5082 
            (Bits.U.of_int 0 1) && Bitops.eq arg5083 (Bits.U.of_int 6 32) && 
          Bitops.eq arg5078 (Bits.U.of_int 19 5) && Bitops.eq arg5079 
            (Bits.U.of_int 0 1) && Bitops.eq arg5080 (Bits.U.of_int 5 32) && 
          Bitops.eq arg5075 (Bits.U.of_int 20 5) && Bitops.eq arg5076 
            (Bits.U.of_int 0 1) && Bitops.eq arg5077 (Bits.U.of_int 4 32) && 
          Bitops.eq arg5072 (Bits.U.of_int 21 5) && Bitops.eq arg5073 
            (Bits.U.of_int 0 1) && Bitops.eq arg5074 (Bits.U.of_int 3 32) && 
          Bitops.eq arg5069 (Bits.U.of_int 22 5) && Bitops.eq arg5070 
            (Bits.U.of_int 0 1) && Bitops.eq arg5071 (Bits.U.of_int 2 32) && 
          Bitops.eq arg5066 (Bits.U.of_int 23 5) && Bitops.eq arg5067 
            (Bits.U.of_int 0 1) && Bitops.eq arg5068 (Bits.U.of_int 1 32) && 
          Bitops.eq arg5063 (Bits.U.of_int 24 5) && Bitops.eq arg5064 
            (Bits.U.of_int 0 1) && Bitops.eq arg5065 (Bits.U.of_int 16 32) && 
          Bitops.eq arg5060 (Bits.U.of_int 25 5) && Bitops.eq arg5061 
            (Bits.U.of_int 0 1) && Bitops.eq arg5062 (Bits.U.of_int 15 32) && 
          Bitops.eq arg5057 (Bits.U.of_int 26 5) && Bitops.eq arg5058 
            (Bits.U.of_int 0 1) && Bitops.eq arg5059 (Bits.U.of_int 14 32) && 
          Bitops.eq arg5054 (Bits.U.of_int 27 5) && Bitops.eq arg5055 
            (Bits.U.of_int 0 1) && Bitops.eq arg5056 (Bits.U.of_int 13 32) && 
          Bitops.eq arg5051 (Bits.U.of_int 28 5) && Bitops.eq arg5052 
            (Bits.U.of_int 0 1) && Bitops.eq arg5053 (Bits.U.of_int 12 32) && 
          Bitops.eq arg5048 (Bits.U.of_int 29 5) && Bitops.eq arg5049 
            (Bits.U.of_int 0 1) && Bitops.eq arg5050 (Bits.U.of_int 11 32) && 
          Bitops.eq arg5045 (Bits.U.of_int 30 5) && Bitops.eq arg5046 
            (Bits.U.of_int 0 1) && Bitops.eq arg5047 (Bits.U.of_int 10 32) && 
          Bitops.eq arg5042 (Bits.U.of_int 31 5) && Bitops.eq arg5043 
            (Bits.U.of_int 0 1) && Bitops.eq arg5044 (Bits.U.of_int 9 32) && 
          Bitops.eq arg5039 (Bits.U.of_int 0 1) && Bitops.eq arg5040 
            (Bits.U.of_int 0 1) && Bitops.eq arg5041 (Bits.U.of_int 16 32) && 
          Bitops.eq rd (Bits.U.of_int 16 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5166), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5167), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5164), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5165), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5162), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5163), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5160), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5161), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5158), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5159), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5156), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5157), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5154), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5155), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5151), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5152), _), 32); 
                    RP.Const (RP.Bits arg5153)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5148), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5149), _), 32); 
                    RP.Const (RP.Bits arg5150)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5145), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5146), _), 32); 
                    RP.Const (RP.Bits arg5147)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5142), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5143), _), 32); 
                    RP.Const (RP.Bits arg5144)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5139), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5140), _), 32); 
                    RP.Const (RP.Bits arg5141)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5136), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5137), _), 32); 
                    RP.Const (RP.Bits arg5138)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5133), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5134), _), 32); 
                    RP.Const (RP.Bits arg5135)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5130), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5131), _), 32); 
                    RP.Const (RP.Bits arg5132)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5127), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5128), _), 32); 
                    RP.Const (RP.Bits arg5129)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5124), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5125), _), 32); 
                    RP.Const (RP.Bits arg5126)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5121), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5122), _), 32); 
                    RP.Const (RP.Bits arg5123)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5118), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5119), _), 32); 
                    RP.Const (RP.Bits arg5120)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5115), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5116), _), 32); 
                    RP.Const (RP.Bits arg5117)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5112), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5113), _), 32); 
                    RP.Const (RP.Bits arg5114)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5109), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5110), _), 32); 
                    RP.Const (RP.Bits arg5111)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5106), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5107), _), 32); 
                    RP.Const (RP.Bits arg5108)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5103), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5104), _), 32); 
                  RP.Const (RP.Bits arg5105)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5166 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5167 (Bits.U.of_int 24 5) && Bitops.eq arg5164 (Bits.U.of_int 9 
              5) && Bitops.eq arg5165 (Bits.U.of_int 25 5) && Bitops.eq arg5162 
            (Bits.U.of_int 10 5) && Bitops.eq arg5163 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5160 (Bits.U.of_int 11 5) && Bitops.eq arg5161 
            (Bits.U.of_int 27 5) && Bitops.eq arg5158 (Bits.U.of_int 12 5) && 
          Bitops.eq arg5159 (Bits.U.of_int 28 5) && Bitops.eq arg5156 
            (Bits.U.of_int 13 5) && Bitops.eq arg5157 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5154 (Bits.U.of_int 14 5) && Bitops.eq arg5155 
            (Bits.U.of_int 30 5) && Bitops.eq arg5151 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5152 (Bits.U.of_int 0 1) && Bitops.eq arg5153 
            (Bits.U.of_int 8 32) && Bitops.eq arg5148 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5149 (Bits.U.of_int 0 1) && Bitops.eq arg5150 
            (Bits.U.of_int 7 32) && Bitops.eq arg5145 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5146 (Bits.U.of_int 0 1) && Bitops.eq arg5147 
            (Bits.U.of_int 6 32) && Bitops.eq arg5142 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5143 (Bits.U.of_int 0 1) && Bitops.eq arg5144 
            (Bits.U.of_int 5 32) && Bitops.eq arg5139 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5140 (Bits.U.of_int 0 1) && Bitops.eq arg5141 
            (Bits.U.of_int 4 32) && Bitops.eq arg5136 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5137 (Bits.U.of_int 0 1) && Bitops.eq arg5138 
            (Bits.U.of_int 3 32) && Bitops.eq arg5133 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5134 (Bits.U.of_int 0 1) && Bitops.eq arg5135 
            (Bits.U.of_int 2 32) && Bitops.eq arg5130 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5131 (Bits.U.of_int 0 1) && Bitops.eq arg5132 
            (Bits.U.of_int 1 32) && Bitops.eq arg5127 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5128 (Bits.U.of_int 0 1) && Bitops.eq arg5129 
            (Bits.U.of_int 16 32) && Bitops.eq arg5124 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5125 (Bits.U.of_int 0 1) && Bitops.eq arg5126 
            (Bits.U.of_int 15 32) && Bitops.eq arg5121 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5122 (Bits.U.of_int 0 1) && Bitops.eq arg5123 
            (Bits.U.of_int 14 32) && Bitops.eq arg5118 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5119 (Bits.U.of_int 0 1) && Bitops.eq arg5120 
            (Bits.U.of_int 13 32) && Bitops.eq arg5115 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5116 (Bits.U.of_int 0 1) && Bitops.eq arg5117 
            (Bits.U.of_int 12 32) && Bitops.eq arg5112 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5113 (Bits.U.of_int 0 1) && Bitops.eq arg5114 
            (Bits.U.of_int 11 32) && Bitops.eq arg5109 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5110 (Bits.U.of_int 0 1) && Bitops.eq arg5111 
            (Bits.U.of_int 10 32) && Bitops.eq arg5106 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5107 (Bits.U.of_int 0 1) && Bitops.eq arg5108 
            (Bits.U.of_int 9 32) && Bitops.eq arg5103 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5104 (Bits.U.of_int 0 1) && Bitops.eq arg5105 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 15 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5231), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5232), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5229), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5230), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5227), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5228), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5225), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5226), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5223), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5224), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5221), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5222), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5219), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5220), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5216), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5217), _), 32); 
                    RP.Const (RP.Bits arg5218)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5213), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5214), _), 32); 
                    RP.Const (RP.Bits arg5215)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5210), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5211), _), 32); 
                    RP.Const (RP.Bits arg5212)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5207), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5208), _), 32); 
                    RP.Const (RP.Bits arg5209)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5204), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5205), _), 32); 
                    RP.Const (RP.Bits arg5206)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5201), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5202), _), 32); 
                    RP.Const (RP.Bits arg5203)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5198), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5199), _), 32); 
                    RP.Const (RP.Bits arg5200)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5195), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5196), _), 32); 
                    RP.Const (RP.Bits arg5197)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5192), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5193), _), 32); 
                    RP.Const (RP.Bits arg5194)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5189), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5190), _), 32); 
                    RP.Const (RP.Bits arg5191)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5186), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5187), _), 32); 
                    RP.Const (RP.Bits arg5188)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5183), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5184), _), 32); 
                    RP.Const (RP.Bits arg5185)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5180), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5181), _), 32); 
                    RP.Const (RP.Bits arg5182)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5177), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5178), _), 32); 
                    RP.Const (RP.Bits arg5179)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5174), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5175), _), 32); 
                    RP.Const (RP.Bits arg5176)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5171), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5172), _), 32); 
                    RP.Const (RP.Bits arg5173)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5168), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5169), _), 32); 
                  RP.Const (RP.Bits arg5170)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5231 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5232 (Bits.U.of_int 24 5) && Bitops.eq arg5229 (Bits.U.of_int 9 
              5) && Bitops.eq arg5230 (Bits.U.of_int 25 5) && Bitops.eq arg5227 
            (Bits.U.of_int 10 5) && Bitops.eq arg5228 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5225 (Bits.U.of_int 11 5) && Bitops.eq arg5226 
            (Bits.U.of_int 27 5) && Bitops.eq arg5223 (Bits.U.of_int 12 5) && 
          Bitops.eq arg5224 (Bits.U.of_int 28 5) && Bitops.eq arg5221 
            (Bits.U.of_int 13 5) && Bitops.eq arg5222 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5219 (Bits.U.of_int 15 5) && Bitops.eq arg5220 
            (Bits.U.of_int 31 5) && Bitops.eq arg5216 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5217 (Bits.U.of_int 0 1) && Bitops.eq arg5218 
            (Bits.U.of_int 8 32) && Bitops.eq arg5213 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5214 (Bits.U.of_int 0 1) && Bitops.eq arg5215 
            (Bits.U.of_int 7 32) && Bitops.eq arg5210 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5211 (Bits.U.of_int 0 1) && Bitops.eq arg5212 
            (Bits.U.of_int 6 32) && Bitops.eq arg5207 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5208 (Bits.U.of_int 0 1) && Bitops.eq arg5209 
            (Bits.U.of_int 5 32) && Bitops.eq arg5204 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5205 (Bits.U.of_int 0 1) && Bitops.eq arg5206 
            (Bits.U.of_int 4 32) && Bitops.eq arg5201 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5202 (Bits.U.of_int 0 1) && Bitops.eq arg5203 
            (Bits.U.of_int 3 32) && Bitops.eq arg5198 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5199 (Bits.U.of_int 0 1) && Bitops.eq arg5200 
            (Bits.U.of_int 2 32) && Bitops.eq arg5195 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5196 (Bits.U.of_int 0 1) && Bitops.eq arg5197 
            (Bits.U.of_int 1 32) && Bitops.eq arg5192 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5193 (Bits.U.of_int 0 1) && Bitops.eq arg5194 
            (Bits.U.of_int 16 32) && Bitops.eq arg5189 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5190 (Bits.U.of_int 0 1) && Bitops.eq arg5191 
            (Bits.U.of_int 15 32) && Bitops.eq arg5186 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5187 (Bits.U.of_int 0 1) && Bitops.eq arg5188 
            (Bits.U.of_int 14 32) && Bitops.eq arg5183 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5184 (Bits.U.of_int 0 1) && Bitops.eq arg5185 
            (Bits.U.of_int 13 32) && Bitops.eq arg5180 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5181 (Bits.U.of_int 0 1) && Bitops.eq arg5182 
            (Bits.U.of_int 12 32) && Bitops.eq arg5177 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5178 (Bits.U.of_int 0 1) && Bitops.eq arg5179 
            (Bits.U.of_int 11 32) && Bitops.eq arg5174 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5175 (Bits.U.of_int 0 1) && Bitops.eq arg5176 
            (Bits.U.of_int 10 32) && Bitops.eq arg5171 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5172 (Bits.U.of_int 0 1) && Bitops.eq arg5173 
            (Bits.U.of_int 9 32) && Bitops.eq arg5168 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5169 (Bits.U.of_int 0 1) && Bitops.eq arg5170 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 14 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5296), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5297), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5294), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5295), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5292), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5293), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5290), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5291), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5288), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5289), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5286), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5287), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5284), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5285), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5281), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5282), _), 32); 
                    RP.Const (RP.Bits arg5283)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5278), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5279), _), 32); 
                    RP.Const (RP.Bits arg5280)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5275), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5276), _), 32); 
                    RP.Const (RP.Bits arg5277)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5272), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5273), _), 32); 
                    RP.Const (RP.Bits arg5274)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5269), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5270), _), 32); 
                    RP.Const (RP.Bits arg5271)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5266), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5267), _), 32); 
                    RP.Const (RP.Bits arg5268)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5263), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5264), _), 32); 
                    RP.Const (RP.Bits arg5265)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5260), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5261), _), 32); 
                    RP.Const (RP.Bits arg5262)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5257), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5258), _), 32); 
                    RP.Const (RP.Bits arg5259)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5254), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5255), _), 32); 
                    RP.Const (RP.Bits arg5256)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5251), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5252), _), 32); 
                    RP.Const (RP.Bits arg5253)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5248), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5249), _), 32); 
                    RP.Const (RP.Bits arg5250)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5245), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5246), _), 32); 
                    RP.Const (RP.Bits arg5247)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5242), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5243), _), 32); 
                    RP.Const (RP.Bits arg5244)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5239), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5240), _), 32); 
                    RP.Const (RP.Bits arg5241)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5236), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5237), _), 32); 
                    RP.Const (RP.Bits arg5238)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5233), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5234), _), 32); 
                  RP.Const (RP.Bits arg5235)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5296 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5297 (Bits.U.of_int 24 5) && Bitops.eq arg5294 (Bits.U.of_int 9 
              5) && Bitops.eq arg5295 (Bits.U.of_int 25 5) && Bitops.eq arg5292 
            (Bits.U.of_int 10 5) && Bitops.eq arg5293 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5290 (Bits.U.of_int 11 5) && Bitops.eq arg5291 
            (Bits.U.of_int 27 5) && Bitops.eq arg5288 (Bits.U.of_int 12 5) && 
          Bitops.eq arg5289 (Bits.U.of_int 28 5) && Bitops.eq arg5286 
            (Bits.U.of_int 14 5) && Bitops.eq arg5287 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5284 (Bits.U.of_int 15 5) && Bitops.eq arg5285 
            (Bits.U.of_int 31 5) && Bitops.eq arg5281 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5282 (Bits.U.of_int 0 1) && Bitops.eq arg5283 
            (Bits.U.of_int 8 32) && Bitops.eq arg5278 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5279 (Bits.U.of_int 0 1) && Bitops.eq arg5280 
            (Bits.U.of_int 7 32) && Bitops.eq arg5275 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5276 (Bits.U.of_int 0 1) && Bitops.eq arg5277 
            (Bits.U.of_int 6 32) && Bitops.eq arg5272 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5273 (Bits.U.of_int 0 1) && Bitops.eq arg5274 
            (Bits.U.of_int 5 32) && Bitops.eq arg5269 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5270 (Bits.U.of_int 0 1) && Bitops.eq arg5271 
            (Bits.U.of_int 4 32) && Bitops.eq arg5266 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5267 (Bits.U.of_int 0 1) && Bitops.eq arg5268 
            (Bits.U.of_int 3 32) && Bitops.eq arg5263 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5264 (Bits.U.of_int 0 1) && Bitops.eq arg5265 
            (Bits.U.of_int 2 32) && Bitops.eq arg5260 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5261 (Bits.U.of_int 0 1) && Bitops.eq arg5262 
            (Bits.U.of_int 1 32) && Bitops.eq arg5257 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5258 (Bits.U.of_int 0 1) && Bitops.eq arg5259 
            (Bits.U.of_int 16 32) && Bitops.eq arg5254 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5255 (Bits.U.of_int 0 1) && Bitops.eq arg5256 
            (Bits.U.of_int 15 32) && Bitops.eq arg5251 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5252 (Bits.U.of_int 0 1) && Bitops.eq arg5253 
            (Bits.U.of_int 14 32) && Bitops.eq arg5248 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5249 (Bits.U.of_int 0 1) && Bitops.eq arg5250 
            (Bits.U.of_int 13 32) && Bitops.eq arg5245 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5246 (Bits.U.of_int 0 1) && Bitops.eq arg5247 
            (Bits.U.of_int 12 32) && Bitops.eq arg5242 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5243 (Bits.U.of_int 0 1) && Bitops.eq arg5244 
            (Bits.U.of_int 11 32) && Bitops.eq arg5239 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5240 (Bits.U.of_int 0 1) && Bitops.eq arg5241 
            (Bits.U.of_int 10 32) && Bitops.eq arg5236 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5237 (Bits.U.of_int 0 1) && Bitops.eq arg5238 
            (Bits.U.of_int 9 32) && Bitops.eq arg5233 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5234 (Bits.U.of_int 0 1) && Bitops.eq arg5235 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 13 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5361), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5362), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5359), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5360), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5357), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5358), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5355), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5356), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5353), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5354), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5351), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5352), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5349), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5350), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5346), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5347), _), 32); 
                    RP.Const (RP.Bits arg5348)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5343), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5344), _), 32); 
                    RP.Const (RP.Bits arg5345)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5340), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5341), _), 32); 
                    RP.Const (RP.Bits arg5342)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5337), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5338), _), 32); 
                    RP.Const (RP.Bits arg5339)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5334), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5335), _), 32); 
                    RP.Const (RP.Bits arg5336)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5331), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5332), _), 32); 
                    RP.Const (RP.Bits arg5333)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5328), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5329), _), 32); 
                    RP.Const (RP.Bits arg5330)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5325), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5326), _), 32); 
                    RP.Const (RP.Bits arg5327)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5322), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5323), _), 32); 
                    RP.Const (RP.Bits arg5324)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5319), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5320), _), 32); 
                    RP.Const (RP.Bits arg5321)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5316), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5317), _), 32); 
                    RP.Const (RP.Bits arg5318)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5313), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5314), _), 32); 
                    RP.Const (RP.Bits arg5315)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5310), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5311), _), 32); 
                    RP.Const (RP.Bits arg5312)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5307), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5308), _), 32); 
                    RP.Const (RP.Bits arg5309)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5304), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5305), _), 32); 
                    RP.Const (RP.Bits arg5306)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5301), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5302), _), 32); 
                    RP.Const (RP.Bits arg5303)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5298), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5299), _), 32); 
                  RP.Const (RP.Bits arg5300)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5361 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5362 (Bits.U.of_int 24 5) && Bitops.eq arg5359 (Bits.U.of_int 9 
              5) && Bitops.eq arg5360 (Bits.U.of_int 25 5) && Bitops.eq arg5357 
            (Bits.U.of_int 10 5) && Bitops.eq arg5358 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5355 (Bits.U.of_int 11 5) && Bitops.eq arg5356 
            (Bits.U.of_int 27 5) && Bitops.eq arg5353 (Bits.U.of_int 13 5) && 
          Bitops.eq arg5354 (Bits.U.of_int 29 5) && Bitops.eq arg5351 
            (Bits.U.of_int 14 5) && Bitops.eq arg5352 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5349 (Bits.U.of_int 15 5) && Bitops.eq arg5350 
            (Bits.U.of_int 31 5) && Bitops.eq arg5346 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5347 (Bits.U.of_int 0 1) && Bitops.eq arg5348 
            (Bits.U.of_int 8 32) && Bitops.eq arg5343 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5344 (Bits.U.of_int 0 1) && Bitops.eq arg5345 
            (Bits.U.of_int 7 32) && Bitops.eq arg5340 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5341 (Bits.U.of_int 0 1) && Bitops.eq arg5342 
            (Bits.U.of_int 6 32) && Bitops.eq arg5337 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5338 (Bits.U.of_int 0 1) && Bitops.eq arg5339 
            (Bits.U.of_int 5 32) && Bitops.eq arg5334 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5335 (Bits.U.of_int 0 1) && Bitops.eq arg5336 
            (Bits.U.of_int 4 32) && Bitops.eq arg5331 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5332 (Bits.U.of_int 0 1) && Bitops.eq arg5333 
            (Bits.U.of_int 3 32) && Bitops.eq arg5328 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5329 (Bits.U.of_int 0 1) && Bitops.eq arg5330 
            (Bits.U.of_int 2 32) && Bitops.eq arg5325 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5326 (Bits.U.of_int 0 1) && Bitops.eq arg5327 
            (Bits.U.of_int 1 32) && Bitops.eq arg5322 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5323 (Bits.U.of_int 0 1) && Bitops.eq arg5324 
            (Bits.U.of_int 16 32) && Bitops.eq arg5319 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5320 (Bits.U.of_int 0 1) && Bitops.eq arg5321 
            (Bits.U.of_int 15 32) && Bitops.eq arg5316 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5317 (Bits.U.of_int 0 1) && Bitops.eq arg5318 
            (Bits.U.of_int 14 32) && Bitops.eq arg5313 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5314 (Bits.U.of_int 0 1) && Bitops.eq arg5315 
            (Bits.U.of_int 13 32) && Bitops.eq arg5310 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5311 (Bits.U.of_int 0 1) && Bitops.eq arg5312 
            (Bits.U.of_int 12 32) && Bitops.eq arg5307 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5308 (Bits.U.of_int 0 1) && Bitops.eq arg5309 
            (Bits.U.of_int 11 32) && Bitops.eq arg5304 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5305 (Bits.U.of_int 0 1) && Bitops.eq arg5306 
            (Bits.U.of_int 10 32) && Bitops.eq arg5301 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5302 (Bits.U.of_int 0 1) && Bitops.eq arg5303 
            (Bits.U.of_int 9 32) && Bitops.eq arg5298 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5299 (Bits.U.of_int 0 1) && Bitops.eq arg5300 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 12 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5426), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5427), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5424), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5425), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5422), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5423), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5420), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5421), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5418), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5419), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5416), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5417), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5414), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5415), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5411), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5412), _), 32); 
                    RP.Const (RP.Bits arg5413)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5408), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5409), _), 32); 
                    RP.Const (RP.Bits arg5410)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5405), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5406), _), 32); 
                    RP.Const (RP.Bits arg5407)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5402), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5403), _), 32); 
                    RP.Const (RP.Bits arg5404)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5399), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5400), _), 32); 
                    RP.Const (RP.Bits arg5401)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5396), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5397), _), 32); 
                    RP.Const (RP.Bits arg5398)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5393), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5394), _), 32); 
                    RP.Const (RP.Bits arg5395)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5390), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5391), _), 32); 
                    RP.Const (RP.Bits arg5392)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5387), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5388), _), 32); 
                    RP.Const (RP.Bits arg5389)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5384), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5385), _), 32); 
                    RP.Const (RP.Bits arg5386)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5381), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5382), _), 32); 
                    RP.Const (RP.Bits arg5383)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5378), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5379), _), 32); 
                    RP.Const (RP.Bits arg5380)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5375), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5376), _), 32); 
                    RP.Const (RP.Bits arg5377)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5372), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5373), _), 32); 
                    RP.Const (RP.Bits arg5374)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5369), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5370), _), 32); 
                    RP.Const (RP.Bits arg5371)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5366), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5367), _), 32); 
                    RP.Const (RP.Bits arg5368)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5363), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5364), _), 32); 
                  RP.Const (RP.Bits arg5365)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5426 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5427 (Bits.U.of_int 24 5) && Bitops.eq arg5424 (Bits.U.of_int 9 
              5) && Bitops.eq arg5425 (Bits.U.of_int 25 5) && Bitops.eq arg5422 
            (Bits.U.of_int 10 5) && Bitops.eq arg5423 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5420 (Bits.U.of_int 12 5) && Bitops.eq arg5421 
            (Bits.U.of_int 28 5) && Bitops.eq arg5418 (Bits.U.of_int 13 5) && 
          Bitops.eq arg5419 (Bits.U.of_int 29 5) && Bitops.eq arg5416 
            (Bits.U.of_int 14 5) && Bitops.eq arg5417 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5414 (Bits.U.of_int 15 5) && Bitops.eq arg5415 
            (Bits.U.of_int 31 5) && Bitops.eq arg5411 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5412 (Bits.U.of_int 0 1) && Bitops.eq arg5413 
            (Bits.U.of_int 8 32) && Bitops.eq arg5408 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5409 (Bits.U.of_int 0 1) && Bitops.eq arg5410 
            (Bits.U.of_int 7 32) && Bitops.eq arg5405 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5406 (Bits.U.of_int 0 1) && Bitops.eq arg5407 
            (Bits.U.of_int 6 32) && Bitops.eq arg5402 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5403 (Bits.U.of_int 0 1) && Bitops.eq arg5404 
            (Bits.U.of_int 5 32) && Bitops.eq arg5399 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5400 (Bits.U.of_int 0 1) && Bitops.eq arg5401 
            (Bits.U.of_int 4 32) && Bitops.eq arg5396 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5397 (Bits.U.of_int 0 1) && Bitops.eq arg5398 
            (Bits.U.of_int 3 32) && Bitops.eq arg5393 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5394 (Bits.U.of_int 0 1) && Bitops.eq arg5395 
            (Bits.U.of_int 2 32) && Bitops.eq arg5390 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5391 (Bits.U.of_int 0 1) && Bitops.eq arg5392 
            (Bits.U.of_int 1 32) && Bitops.eq arg5387 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5388 (Bits.U.of_int 0 1) && Bitops.eq arg5389 
            (Bits.U.of_int 16 32) && Bitops.eq arg5384 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5385 (Bits.U.of_int 0 1) && Bitops.eq arg5386 
            (Bits.U.of_int 15 32) && Bitops.eq arg5381 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5382 (Bits.U.of_int 0 1) && Bitops.eq arg5383 
            (Bits.U.of_int 14 32) && Bitops.eq arg5378 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5379 (Bits.U.of_int 0 1) && Bitops.eq arg5380 
            (Bits.U.of_int 13 32) && Bitops.eq arg5375 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5376 (Bits.U.of_int 0 1) && Bitops.eq arg5377 
            (Bits.U.of_int 12 32) && Bitops.eq arg5372 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5373 (Bits.U.of_int 0 1) && Bitops.eq arg5374 
            (Bits.U.of_int 11 32) && Bitops.eq arg5369 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5370 (Bits.U.of_int 0 1) && Bitops.eq arg5371 
            (Bits.U.of_int 10 32) && Bitops.eq arg5366 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5367 (Bits.U.of_int 0 1) && Bitops.eq arg5368 
            (Bits.U.of_int 9 32) && Bitops.eq arg5363 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5364 (Bits.U.of_int 0 1) && Bitops.eq arg5365 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 11 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5491), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5492), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5489), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5490), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5487), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5488), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5485), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5486), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5483), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5484), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5481), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5482), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5479), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5480), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5476), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5477), _), 32); 
                    RP.Const (RP.Bits arg5478)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5473), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5474), _), 32); 
                    RP.Const (RP.Bits arg5475)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5470), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5471), _), 32); 
                    RP.Const (RP.Bits arg5472)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5467), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5468), _), 32); 
                    RP.Const (RP.Bits arg5469)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5464), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5465), _), 32); 
                    RP.Const (RP.Bits arg5466)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5461), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5462), _), 32); 
                    RP.Const (RP.Bits arg5463)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5458), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5459), _), 32); 
                    RP.Const (RP.Bits arg5460)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5455), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5456), _), 32); 
                    RP.Const (RP.Bits arg5457)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5452), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5453), _), 32); 
                    RP.Const (RP.Bits arg5454)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5449), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5450), _), 32); 
                    RP.Const (RP.Bits arg5451)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5446), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5447), _), 32); 
                    RP.Const (RP.Bits arg5448)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5443), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5444), _), 32); 
                    RP.Const (RP.Bits arg5445)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5440), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5441), _), 32); 
                    RP.Const (RP.Bits arg5442)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5437), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5438), _), 32); 
                    RP.Const (RP.Bits arg5439)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5434), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5435), _), 32); 
                    RP.Const (RP.Bits arg5436)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5431), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5432), _), 32); 
                    RP.Const (RP.Bits arg5433)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5428), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5429), _), 32); 
                  RP.Const (RP.Bits arg5430)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5491 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5492 (Bits.U.of_int 24 5) && Bitops.eq arg5489 (Bits.U.of_int 9 
              5) && Bitops.eq arg5490 (Bits.U.of_int 25 5) && Bitops.eq arg5487 
            (Bits.U.of_int 11 5) && Bitops.eq arg5488 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5485 (Bits.U.of_int 12 5) && Bitops.eq arg5486 
            (Bits.U.of_int 28 5) && Bitops.eq arg5483 (Bits.U.of_int 13 5) && 
          Bitops.eq arg5484 (Bits.U.of_int 29 5) && Bitops.eq arg5481 
            (Bits.U.of_int 14 5) && Bitops.eq arg5482 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5479 (Bits.U.of_int 15 5) && Bitops.eq arg5480 
            (Bits.U.of_int 31 5) && Bitops.eq arg5476 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5477 (Bits.U.of_int 0 1) && Bitops.eq arg5478 
            (Bits.U.of_int 8 32) && Bitops.eq arg5473 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5474 (Bits.U.of_int 0 1) && Bitops.eq arg5475 
            (Bits.U.of_int 7 32) && Bitops.eq arg5470 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5471 (Bits.U.of_int 0 1) && Bitops.eq arg5472 
            (Bits.U.of_int 6 32) && Bitops.eq arg5467 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5468 (Bits.U.of_int 0 1) && Bitops.eq arg5469 
            (Bits.U.of_int 5 32) && Bitops.eq arg5464 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5465 (Bits.U.of_int 0 1) && Bitops.eq arg5466 
            (Bits.U.of_int 4 32) && Bitops.eq arg5461 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5462 (Bits.U.of_int 0 1) && Bitops.eq arg5463 
            (Bits.U.of_int 3 32) && Bitops.eq arg5458 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5459 (Bits.U.of_int 0 1) && Bitops.eq arg5460 
            (Bits.U.of_int 2 32) && Bitops.eq arg5455 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5456 (Bits.U.of_int 0 1) && Bitops.eq arg5457 
            (Bits.U.of_int 1 32) && Bitops.eq arg5452 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5453 (Bits.U.of_int 0 1) && Bitops.eq arg5454 
            (Bits.U.of_int 16 32) && Bitops.eq arg5449 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5450 (Bits.U.of_int 0 1) && Bitops.eq arg5451 
            (Bits.U.of_int 15 32) && Bitops.eq arg5446 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5447 (Bits.U.of_int 0 1) && Bitops.eq arg5448 
            (Bits.U.of_int 14 32) && Bitops.eq arg5443 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5444 (Bits.U.of_int 0 1) && Bitops.eq arg5445 
            (Bits.U.of_int 13 32) && Bitops.eq arg5440 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5441 (Bits.U.of_int 0 1) && Bitops.eq arg5442 
            (Bits.U.of_int 12 32) && Bitops.eq arg5437 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5438 (Bits.U.of_int 0 1) && Bitops.eq arg5439 
            (Bits.U.of_int 11 32) && Bitops.eq arg5434 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5435 (Bits.U.of_int 0 1) && Bitops.eq arg5436 
            (Bits.U.of_int 10 32) && Bitops.eq arg5431 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5432 (Bits.U.of_int 0 1) && Bitops.eq arg5433 
            (Bits.U.of_int 9 32) && Bitops.eq arg5428 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5429 (Bits.U.of_int 0 1) && Bitops.eq arg5430 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 10 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5556), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5557), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5554), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5555), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5552), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5553), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5550), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5551), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5548), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5549), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5546), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5547), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5544), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5545), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5541), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5542), _), 32); 
                    RP.Const (RP.Bits arg5543)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5538), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5539), _), 32); 
                    RP.Const (RP.Bits arg5540)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5535), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5536), _), 32); 
                    RP.Const (RP.Bits arg5537)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5532), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5533), _), 32); 
                    RP.Const (RP.Bits arg5534)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5529), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5530), _), 32); 
                    RP.Const (RP.Bits arg5531)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5526), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5527), _), 32); 
                    RP.Const (RP.Bits arg5528)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5523), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5524), _), 32); 
                    RP.Const (RP.Bits arg5525)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5520), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5521), _), 32); 
                    RP.Const (RP.Bits arg5522)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5517), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5518), _), 32); 
                    RP.Const (RP.Bits arg5519)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5514), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5515), _), 32); 
                    RP.Const (RP.Bits arg5516)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5511), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5512), _), 32); 
                    RP.Const (RP.Bits arg5513)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5508), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5509), _), 32); 
                    RP.Const (RP.Bits arg5510)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5505), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5506), _), 32); 
                    RP.Const (RP.Bits arg5507)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5502), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5503), _), 32); 
                    RP.Const (RP.Bits arg5504)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5499), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5500), _), 32); 
                    RP.Const (RP.Bits arg5501)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5496), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5497), _), 32); 
                    RP.Const (RP.Bits arg5498)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5493), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5494), _), 32); 
                  RP.Const (RP.Bits arg5495)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5556 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5557 (Bits.U.of_int 24 5) && Bitops.eq arg5554 (Bits.U.of_int 10 
              5) && Bitops.eq arg5555 (Bits.U.of_int 26 5) && Bitops.eq arg5552 
            (Bits.U.of_int 11 5) && Bitops.eq arg5553 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5550 (Bits.U.of_int 12 5) && Bitops.eq arg5551 
            (Bits.U.of_int 28 5) && Bitops.eq arg5548 (Bits.U.of_int 13 5) && 
          Bitops.eq arg5549 (Bits.U.of_int 29 5) && Bitops.eq arg5546 
            (Bits.U.of_int 14 5) && Bitops.eq arg5547 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5544 (Bits.U.of_int 15 5) && Bitops.eq arg5545 
            (Bits.U.of_int 31 5) && Bitops.eq arg5541 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5542 (Bits.U.of_int 0 1) && Bitops.eq arg5543 
            (Bits.U.of_int 8 32) && Bitops.eq arg5538 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5539 (Bits.U.of_int 0 1) && Bitops.eq arg5540 
            (Bits.U.of_int 7 32) && Bitops.eq arg5535 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5536 (Bits.U.of_int 0 1) && Bitops.eq arg5537 
            (Bits.U.of_int 6 32) && Bitops.eq arg5532 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5533 (Bits.U.of_int 0 1) && Bitops.eq arg5534 
            (Bits.U.of_int 5 32) && Bitops.eq arg5529 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5530 (Bits.U.of_int 0 1) && Bitops.eq arg5531 
            (Bits.U.of_int 4 32) && Bitops.eq arg5526 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5527 (Bits.U.of_int 0 1) && Bitops.eq arg5528 
            (Bits.U.of_int 3 32) && Bitops.eq arg5523 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5524 (Bits.U.of_int 0 1) && Bitops.eq arg5525 
            (Bits.U.of_int 2 32) && Bitops.eq arg5520 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5521 (Bits.U.of_int 0 1) && Bitops.eq arg5522 
            (Bits.U.of_int 1 32) && Bitops.eq arg5517 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5518 (Bits.U.of_int 0 1) && Bitops.eq arg5519 
            (Bits.U.of_int 16 32) && Bitops.eq arg5514 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5515 (Bits.U.of_int 0 1) && Bitops.eq arg5516 
            (Bits.U.of_int 15 32) && Bitops.eq arg5511 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5512 (Bits.U.of_int 0 1) && Bitops.eq arg5513 
            (Bits.U.of_int 14 32) && Bitops.eq arg5508 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5509 (Bits.U.of_int 0 1) && Bitops.eq arg5510 
            (Bits.U.of_int 13 32) && Bitops.eq arg5505 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5506 (Bits.U.of_int 0 1) && Bitops.eq arg5507 
            (Bits.U.of_int 12 32) && Bitops.eq arg5502 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5503 (Bits.U.of_int 0 1) && Bitops.eq arg5504 
            (Bits.U.of_int 11 32) && Bitops.eq arg5499 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5500 (Bits.U.of_int 0 1) && Bitops.eq arg5501 
            (Bits.U.of_int 10 32) && Bitops.eq arg5496 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5497 (Bits.U.of_int 0 1) && Bitops.eq arg5498 
            (Bits.U.of_int 9 32) && Bitops.eq arg5493 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5494 (Bits.U.of_int 0 1) && Bitops.eq arg5495 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 9 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5621), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5622), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5619), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5620), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5617), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5618), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5615), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5616), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5613), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5614), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5611), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5612), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5609), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5610), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5606), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5607), _), 32); 
                    RP.Const (RP.Bits arg5608)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5603), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5604), _), 32); 
                    RP.Const (RP.Bits arg5605)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5600), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5601), _), 32); 
                    RP.Const (RP.Bits arg5602)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5597), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5598), _), 32); 
                    RP.Const (RP.Bits arg5599)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5594), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5595), _), 32); 
                    RP.Const (RP.Bits arg5596)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5591), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5592), _), 32); 
                    RP.Const (RP.Bits arg5593)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5588), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5589), _), 32); 
                    RP.Const (RP.Bits arg5590)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5585), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5586), _), 32); 
                    RP.Const (RP.Bits arg5587)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5582), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5583), _), 32); 
                    RP.Const (RP.Bits arg5584)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5579), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5580), _), 32); 
                    RP.Const (RP.Bits arg5581)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5576), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5577), _), 32); 
                    RP.Const (RP.Bits arg5578)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5573), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5574), _), 32); 
                    RP.Const (RP.Bits arg5575)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5570), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5571), _), 32); 
                    RP.Const (RP.Bits arg5572)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5567), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5568), _), 32); 
                    RP.Const (RP.Bits arg5569)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5564), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5565), _), 32); 
                    RP.Const (RP.Bits arg5566)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5561), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5562), _), 32); 
                    RP.Const (RP.Bits arg5563)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5558), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5559), _), 32); 
                  RP.Const (RP.Bits arg5560)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Cell ('r', Rtl.Identity, 32, RP.Const (RP.Bits rd), 
              _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                  32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5621 (Bits.U.of_int 9 5) && Bitops.eq 
            arg5622 (Bits.U.of_int 25 5) && Bitops.eq arg5619 (Bits.U.of_int 10 
              5) && Bitops.eq arg5620 (Bits.U.of_int 26 5) && Bitops.eq arg5617 
            (Bits.U.of_int 11 5) && Bitops.eq arg5618 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5615 (Bits.U.of_int 12 5) && Bitops.eq arg5616 
            (Bits.U.of_int 28 5) && Bitops.eq arg5613 (Bits.U.of_int 13 5) && 
          Bitops.eq arg5614 (Bits.U.of_int 29 5) && Bitops.eq arg5611 
            (Bits.U.of_int 14 5) && Bitops.eq arg5612 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5609 (Bits.U.of_int 15 5) && Bitops.eq arg5610 
            (Bits.U.of_int 31 5) && Bitops.eq arg5606 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5607 (Bits.U.of_int 0 1) && Bitops.eq arg5608 
            (Bits.U.of_int 8 32) && Bitops.eq arg5603 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5604 (Bits.U.of_int 0 1) && Bitops.eq arg5605 
            (Bits.U.of_int 7 32) && Bitops.eq arg5600 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5601 (Bits.U.of_int 0 1) && Bitops.eq arg5602 
            (Bits.U.of_int 6 32) && Bitops.eq arg5597 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5598 (Bits.U.of_int 0 1) && Bitops.eq arg5599 
            (Bits.U.of_int 5 32) && Bitops.eq arg5594 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5595 (Bits.U.of_int 0 1) && Bitops.eq arg5596 
            (Bits.U.of_int 4 32) && Bitops.eq arg5591 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5592 (Bits.U.of_int 0 1) && Bitops.eq arg5593 
            (Bits.U.of_int 3 32) && Bitops.eq arg5588 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5589 (Bits.U.of_int 0 1) && Bitops.eq arg5590 
            (Bits.U.of_int 2 32) && Bitops.eq arg5585 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5586 (Bits.U.of_int 0 1) && Bitops.eq arg5587 
            (Bits.U.of_int 1 32) && Bitops.eq arg5582 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5583 (Bits.U.of_int 0 1) && Bitops.eq arg5584 
            (Bits.U.of_int 16 32) && Bitops.eq arg5579 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5580 (Bits.U.of_int 0 1) && Bitops.eq arg5581 
            (Bits.U.of_int 15 32) && Bitops.eq arg5576 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5577 (Bits.U.of_int 0 1) && Bitops.eq arg5578 
            (Bits.U.of_int 14 32) && Bitops.eq arg5573 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5574 (Bits.U.of_int 0 1) && Bitops.eq arg5575 
            (Bits.U.of_int 13 32) && Bitops.eq arg5570 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5571 (Bits.U.of_int 0 1) && Bitops.eq arg5572 
            (Bits.U.of_int 12 32) && Bitops.eq arg5567 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5568 (Bits.U.of_int 0 1) && Bitops.eq arg5569 
            (Bits.U.of_int 11 32) && Bitops.eq arg5564 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5565 (Bits.U.of_int 0 1) && Bitops.eq arg5566 
            (Bits.U.of_int 10 32) && Bitops.eq arg5561 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5562 (Bits.U.of_int 0 1) && Bitops.eq arg5563 
            (Bits.U.of_int 9 32) && Bitops.eq arg5558 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5559 (Bits.U.of_int 0 1) && Bitops.eq arg5560 
            (Bits.U.of_int 16 32) && Bitops.eq rd (Bits.U.of_int 8 5)) -> 
           Instruction.restore (Bits.U.to_native rs1) (Instruction.rmode 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool false), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5623), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5623 (Bits.U.of_int 5 3)) -> 
           Instruction.bn (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.Fetch (RP.Slice (1, 22, 
            RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5624), _)), 
            1); RP.Const (RP.Bits arg5625)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5626), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5624 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5625 (Bits.U.of_int 0 1) && Bitops.eq arg5626 
            (Bits.U.of_int 5 3)) -> Instruction.be (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), 
          [RP.App (("or", [1]), [RP.Fetch (RP.Slice (1, 22, RP.Cell ('i', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5629), _)), 1); 
              RP.App (("xor", [1]), [RP.Fetch (RP.Slice (1, 23, RP.Cell ('i', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5628), _)), 1); 
                  RP.Fetch (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                      RP.Const (RP.Bits arg5627), _)), 1)])]); 
            RP.Const (RP.Bits arg5630)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5631), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5629 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5628 (Bits.U.of_int 0 3) && Bitops.eq arg5627 (Bits.U.of_int 0 3) && 
          Bitops.eq arg5630 (Bits.U.of_int 0 1) && Bitops.eq arg5631 
            (Bits.U.of_int 5 3)) -> Instruction.ble (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.App (("xor", [1]), 
            [RP.Fetch (RP.Slice (1, 23, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5633), _)), 1); RP.Fetch (RP.Slice (1, 21, 
                RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5632), 
                  _)), 1)]); RP.Const (RP.Bits arg5634)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5635), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5633 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5632 (Bits.U.of_int 0 3) && Bitops.eq arg5634 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5635 (Bits.U.of_int 5 3)) -> 
           Instruction.bl (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.App (("or", [1]), 
            [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5637), _)), 1); RP.Fetch (RP.Slice (1, 22, 
                RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5636), 
                  _)), 1)]); RP.Const (RP.Bits arg5638)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5639), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5637 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5636 (Bits.U.of_int 0 3) && Bitops.eq arg5638 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5639 (Bits.U.of_int 5 3)) -> 
           Instruction.bleu (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.Fetch (RP.Slice (1, 20, 
            RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5640), _)), 
            1); RP.Const (RP.Bits arg5641)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5642), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5640 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5641 (Bits.U.of_int 0 1) && Bitops.eq arg5642 
            (Bits.U.of_int 5 3)) -> Instruction.bcs (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.Fetch (RP.Slice (1, 23, 
            RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5643), _)), 
            1); RP.Const (RP.Bits arg5644)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5645), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5643 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5644 (Bits.U.of_int 0 1) && Bitops.eq arg5645 
            (Bits.U.of_int 5 3)) -> Instruction.bneg (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.Fetch (RP.Slice (1, 21, 
            RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5646), _)), 
            1); RP.Const (RP.Bits arg5647)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5648), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5646 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5647 (Bits.U.of_int 0 1) && Bitops.eq arg5648 
            (Bits.U.of_int 5 3)) -> Instruction.bvs (Bits.U.to_native target) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5649), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5649 (Bits.U.of_int 5 3)) -> 
           Instruction.ba (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), 
          [RP.App (("com", [1]), [RP.Fetch (RP.Slice (1, 22, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5650), _)), 
              1)]); RP.Const (RP.Bits arg5651)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5652), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5650 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5651 (Bits.U.of_int 0 1) && Bitops.eq arg5652 
            (Bits.U.of_int 5 3)) -> Instruction.bne (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.App (("com", [1]), 
            [RP.App (("or", [1]), [RP.Fetch (RP.Slice (1, 22, RP.Cell ('i', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5655), _)), 1); 
                RP.App (("xor", [1]), [RP.Fetch (RP.Slice (1, 23, RP.Cell ('i', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg5654), _)), 1); 
                    RP.Fetch (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg5653), _)), 1)])])]); 
            RP.Const (RP.Bits arg5656)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5657), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5655 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5654 (Bits.U.of_int 0 3) && Bitops.eq arg5653 (Bits.U.of_int 0 3) && 
          Bitops.eq arg5656 (Bits.U.of_int 0 1) && Bitops.eq arg5657 
            (Bits.U.of_int 5 3)) -> Instruction.bg (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.App (("com", [1]), 
            [RP.App (("xor", [1]), [RP.Fetch (RP.Slice (1, 23, RP.Cell ('i', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5659), _)), 1); 
                RP.Fetch (RP.Slice (1, 21, RP.Cell ('i', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5658), _)), 1)])]); 
            RP.Const (RP.Bits arg5660)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5661), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5659 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5658 (Bits.U.of_int 0 3) && Bitops.eq arg5660 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5661 (Bits.U.of_int 5 3)) -> 
           Instruction.bge (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), [RP.App (("com", [1]), 
            [RP.App (("or", [1]), [RP.Fetch (RP.Slice (1, 20, RP.Cell ('i', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5663), _)), 1); 
                RP.Fetch (RP.Slice (1, 22, RP.Cell ('i', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5662), _)), 1)])]); 
            RP.Const (RP.Bits arg5664)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5665), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5663 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5662 (Bits.U.of_int 0 3) && Bitops.eq arg5664 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5665 (Bits.U.of_int 5 3)) -> 
           Instruction.bgu (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), 
          [RP.App (("com", [1]), [RP.Fetch (RP.Slice (1, 20, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5666), _)), 
              1)]); RP.Const (RP.Bits arg5667)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5668), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5666 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5667 (Bits.U.of_int 0 1) && Bitops.eq arg5668 
            (Bits.U.of_int 5 3)) -> Instruction.bcc (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), 
          [RP.App (("com", [1]), [RP.Fetch (RP.Slice (1, 23, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5669), _)), 
              1)]); RP.Const (RP.Bits arg5670)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5671), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5669 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5670 (Bits.U.of_int 0 1) && Bitops.eq arg5671 
            (Bits.U.of_int 5 3)) -> Instruction.bpos (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("ne", [1]), 
          [RP.App (("com", [1]), [RP.Fetch (RP.Slice (1, 21, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5672), _)), 
              1)]); RP.Const (RP.Bits arg5673)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5674), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5672 (Bits.U.of_int 0 3) && Bitops.eq 
            arg5673 (Bits.U.of_int 0 1) && Bitops.eq arg5674 
            (Bits.U.of_int 5 3)) -> Instruction.bvc (Bits.U.to_native target) 
      | RP.Rtl [] -> Instruction.fbn (Nativeint.of_int 0) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5687), _)), 
            2); RP.Const (RP.Bits arg5688)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5689), 
            _), RP.Const (RP.Bits target''), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5684), _)), 2); 
                RP.Const (RP.Bits arg5685)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5682), _)), 2); 
                  RP.Const (RP.Bits arg5683)])]), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5686), _), 
              RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5679), _)), 2); 
                RP.Const (RP.Bits arg5680)]); RP.App (("conjoin", []), 
                [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg5677), _)), 2); 
                    RP.Const (RP.Bits arg5678)]); RP.App (("eq", [2]), 
                    [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg5675), _)), 2); 
                      RP.Const (RP.Bits arg5676)])])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5681), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target'' && target = target' && (Bitops.eq 
            arg5687 (Bits.U.of_int 0 1) && Bitops.eq arg5688 Bitops.float_lt && 
          Bitops.eq arg5689 (Bits.U.of_int 5 3) && Bitops.eq arg5684 
            (Bits.U.of_int 0 1) && Bitops.eq arg5685 Bitops.float_lt && Bitops.eq 
            arg5682 (Bits.U.of_int 0 1) && Bitops.eq arg5683 Bitops.float_gt && 
          Bitops.eq arg5686 (Bits.U.of_int 5 3) && Bitops.eq arg5679 
            (Bits.U.of_int 0 1) && Bitops.eq arg5680 Bitops.float_lt && Bitops.eq 
            arg5677 (Bits.U.of_int 0 1) && Bitops.eq arg5678 Bitops.float_gt && 
          Bitops.eq arg5675 (Bits.U.of_int 0 1) && Bitops.eq arg5676 
            Bitops.unordered && Bitops.eq arg5681 (Bits.U.of_int 5 3))) -> 
           Instruction.fbne (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5695), _)), 
            2); RP.Const (RP.Bits arg5696)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5697), 
            _), RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5692), _)), 2); 
                RP.Const (RP.Bits arg5693)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5690), _)), 2); 
                  RP.Const (RP.Bits arg5691)])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5694), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target' && (Bitops.eq arg5695 
            (Bits.U.of_int 0 1) && Bitops.eq arg5696 Bitops.float_lt && Bitops.eq 
            arg5697 (Bits.U.of_int 5 3) && Bitops.eq arg5692 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5693 Bitops.float_lt && Bitops.eq arg5690 (Bits.U.of_int 0 
              1) && Bitops.eq arg5691 Bitops.float_gt && Bitops.eq arg5694 
            (Bits.U.of_int 5 3))) -> Instruction.fblg (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5703), _)), 
            2); RP.Const (RP.Bits arg5704)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5705), 
            _), RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5700), _)), 2); 
                RP.Const (RP.Bits arg5701)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5698), _)), 2); 
                  RP.Const (RP.Bits arg5699)])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5702), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target' && (Bitops.eq arg5703 
            (Bits.U.of_int 0 1) && Bitops.eq arg5704 Bitops.float_lt && Bitops.eq 
            arg5705 (Bits.U.of_int 5 3) && Bitops.eq arg5700 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5701 Bitops.float_lt && Bitops.eq arg5698 (Bits.U.of_int 0 
              1) && Bitops.eq arg5699 Bitops.unordered && Bitops.eq arg5702 
            (Bits.U.of_int 5 3))) -> Instruction.fbul (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5706), _)), 
            2); RP.Const (RP.Bits arg5707)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5708), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5706 (Bits.U.of_int 0 1) && Bitops.eq 
            arg5707 Bitops.float_lt && Bitops.eq arg5708 (Bits.U.of_int 5 3)) -> 
           Instruction.fbl (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5714), _)), 
            2); RP.Const (RP.Bits arg5715)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5716), 
            _), RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5711), _)), 2); 
                RP.Const (RP.Bits arg5712)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5709), _)), 2); 
                  RP.Const (RP.Bits arg5710)])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5713), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target' && (Bitops.eq arg5714 
            (Bits.U.of_int 0 1) && Bitops.eq arg5715 Bitops.float_gt && Bitops.eq 
            arg5716 (Bits.U.of_int 5 3) && Bitops.eq arg5711 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5712 Bitops.float_gt && Bitops.eq arg5709 (Bits.U.of_int 0 
              1) && Bitops.eq arg5710 Bitops.unordered && Bitops.eq arg5713 
            (Bits.U.of_int 5 3))) -> Instruction.fbug (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5717), _)), 
            2); RP.Const (RP.Bits arg5718)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5719), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5717 (Bits.U.of_int 0 1) && Bitops.eq 
            arg5718 Bitops.float_gt && Bitops.eq arg5719 (Bits.U.of_int 5 3)) -> 
           Instruction.fbg (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5720), _)), 
            2); RP.Const (RP.Bits arg5721)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5722), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5720 (Bits.U.of_int 0 1) && Bitops.eq 
            arg5721 Bitops.unordered && Bitops.eq arg5722 (Bits.U.of_int 5 3)) -> 
           Instruction.fbu (Bits.U.to_native target) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5723), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5723 (Bits.U.of_int 5 3)) -> 
           Instruction.fba (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5724), _)), 
            2); RP.Const (RP.Bits arg5725)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5726), 
            _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (Bitops.eq arg5724 (Bits.U.of_int 0 1) && Bitops.eq 
            arg5725 Bitops.float_eq && Bitops.eq arg5726 (Bits.U.of_int 5 3)) -> 
           Instruction.fbe (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5732), _)), 
            2); RP.Const (RP.Bits arg5733)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5734), 
            _), RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5729), _)), 2); 
                RP.Const (RP.Bits arg5730)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5727), _)), 2); 
                  RP.Const (RP.Bits arg5728)])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5731), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target' && (Bitops.eq arg5732 
            (Bits.U.of_int 0 1) && Bitops.eq arg5733 Bitops.float_eq && Bitops.eq 
            arg5734 (Bits.U.of_int 5 3) && Bitops.eq arg5729 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5730 Bitops.float_eq && Bitops.eq arg5727 (Bits.U.of_int 0 
              1) && Bitops.eq arg5728 Bitops.unordered && Bitops.eq arg5731 
            (Bits.U.of_int 5 3))) -> Instruction.fbue (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5740), _)), 
            2); RP.Const (RP.Bits arg5741)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5742), 
            _), RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5737), _)), 2); 
                RP.Const (RP.Bits arg5738)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5735), _)), 2); 
                  RP.Const (RP.Bits arg5736)])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5739), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target' && (Bitops.eq arg5740 
            (Bits.U.of_int 0 1) && Bitops.eq arg5741 Bitops.float_eq && Bitops.eq 
            arg5742 (Bits.U.of_int 5 3) && Bitops.eq arg5737 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5738 Bitops.float_eq && Bitops.eq arg5735 (Bits.U.of_int 0 
              1) && Bitops.eq arg5736 Bitops.float_gt && Bitops.eq arg5739 
            (Bits.U.of_int 5 3))) -> Instruction.fbge (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5755), _)), 
            2); RP.Const (RP.Bits arg5756)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5757), 
            _), RP.Const (RP.Bits target''), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5752), _)), 2); 
                RP.Const (RP.Bits arg5753)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5750), _)), 2); 
                  RP.Const (RP.Bits arg5751)])]), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5754), _), 
              RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5747), _)), 2); 
                RP.Const (RP.Bits arg5748)]); RP.App (("conjoin", []), 
                [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg5745), _)), 2); 
                    RP.Const (RP.Bits arg5746)]); RP.App (("eq", [2]), 
                    [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg5743), _)), 2); 
                      RP.Const (RP.Bits arg5744)])])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5749), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target'' && target = target' && (Bitops.eq 
            arg5755 (Bits.U.of_int 0 1) && Bitops.eq arg5756 Bitops.float_eq && 
          Bitops.eq arg5757 (Bits.U.of_int 5 3) && Bitops.eq arg5752 
            (Bits.U.of_int 0 1) && Bitops.eq arg5753 Bitops.float_eq && Bitops.eq 
            arg5750 (Bits.U.of_int 0 1) && Bitops.eq arg5751 Bitops.float_gt && 
          Bitops.eq arg5754 (Bits.U.of_int 5 3) && Bitops.eq arg5747 
            (Bits.U.of_int 0 1) && Bitops.eq arg5748 Bitops.float_eq && Bitops.eq 
            arg5745 (Bits.U.of_int 0 1) && Bitops.eq arg5746 Bitops.float_gt && 
          Bitops.eq arg5743 (Bits.U.of_int 0 1) && Bitops.eq arg5744 
            Bitops.unordered && Bitops.eq arg5749 (Bits.U.of_int 5 3))) -> 
           Instruction.fbuge (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5763), _)), 
            2); RP.Const (RP.Bits arg5764)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5765), 
            _), RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5760), _)), 2); 
                RP.Const (RP.Bits arg5761)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5758), _)), 2); 
                  RP.Const (RP.Bits arg5759)])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5762), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target' && (Bitops.eq arg5763 
            (Bits.U.of_int 0 1) && Bitops.eq arg5764 Bitops.float_eq && Bitops.eq 
            arg5765 (Bits.U.of_int 5 3) && Bitops.eq arg5760 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5761 Bitops.float_eq && Bitops.eq arg5758 (Bits.U.of_int 0 
              1) && Bitops.eq arg5759 Bitops.float_lt && Bitops.eq arg5762 
            (Bits.U.of_int 5 3))) -> Instruction.fble (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5778), _)), 
            2); RP.Const (RP.Bits arg5779)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5780), 
            _), RP.Const (RP.Bits target''), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5775), _)), 2); 
                RP.Const (RP.Bits arg5776)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5773), _)), 2); 
                  RP.Const (RP.Bits arg5774)])]), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5777), _), 
              RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5770), _)), 2); 
                RP.Const (RP.Bits arg5771)]); RP.App (("conjoin", []), 
                [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg5768), _)), 2); 
                    RP.Const (RP.Bits arg5769)]); RP.App (("eq", [2]), 
                    [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg5766), _)), 2); 
                      RP.Const (RP.Bits arg5767)])])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5772), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target'' && target = target' && (Bitops.eq 
            arg5778 (Bits.U.of_int 0 1) && Bitops.eq arg5779 Bitops.float_eq && 
          Bitops.eq arg5780 (Bits.U.of_int 5 3) && Bitops.eq arg5775 
            (Bits.U.of_int 0 1) && Bitops.eq arg5776 Bitops.float_eq && Bitops.eq 
            arg5773 (Bits.U.of_int 0 1) && Bitops.eq arg5774 Bitops.float_lt && 
          Bitops.eq arg5777 (Bits.U.of_int 5 3) && Bitops.eq arg5770 
            (Bits.U.of_int 0 1) && Bitops.eq arg5771 Bitops.float_eq && Bitops.eq 
            arg5768 (Bits.U.of_int 0 1) && Bitops.eq arg5769 Bitops.float_lt && 
          Bitops.eq arg5766 (Bits.U.of_int 0 1) && Bitops.eq arg5767 
            Bitops.unordered && Bitops.eq arg5772 (Bits.U.of_int 5 3))) -> 
           Instruction.fbule (Bits.U.to_native target) 
      | RP.Rtl [(RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5793), _)), 
            2); RP.Const (RP.Bits arg5794)]), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5795), 
            _), RP.Const (RP.Bits target''), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5790), _)), 2); 
                RP.Const (RP.Bits arg5791)]); 
              RP.App (("eq", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                    Rtl.Identity, 32, RP.Const (RP.Bits arg5788), _)), 2); 
                  RP.Const (RP.Bits arg5789)])]), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5792), _), 
              RP.Const (RP.Bits target'), 32)); (RP.App (("conjoin", []), 
            [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                  Rtl.Identity, 32, RP.Const (RP.Bits arg5785), _)), 2); 
                RP.Const (RP.Bits arg5786)]); RP.App (("conjoin", []), 
                [RP.App (("ne", [2]), [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', 
                      Rtl.Identity, 32, RP.Const (RP.Bits arg5783), _)), 2); 
                    RP.Const (RP.Bits arg5784)]); RP.App (("eq", [2]), 
                    [RP.Fetch (RP.Slice (2, 10, RP.Cell ('F', Rtl.Identity, 32, 
                        RP.Const (RP.Bits arg5781), _)), 2); 
                      RP.Const (RP.Bits arg5782)])])]), 
            RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5787), _), RP.Const (RP.Bits target), 32))] 
      when Base.to_bool (target = target'' && target = target' && 
          (Bitops.eq arg5793 (Bits.U.of_int 0 1) && Bitops.eq arg5794 
            Bitops.float_eq && Bitops.eq arg5795 (Bits.U.of_int 5 3) && Bitops.eq 
            arg5790 (Bits.U.of_int 0 1) && Bitops.eq arg5791 Bitops.float_eq && 
          Bitops.eq arg5788 (Bits.U.of_int 0 1) && Bitops.eq arg5789 
            Bitops.float_lt && Bitops.eq arg5792 (Bits.U.of_int 5 3) && Bitops.eq 
            arg5785 (Bits.U.of_int 0 1) && Bitops.eq arg5786 Bitops.float_eq && 
          Bitops.eq arg5783 (Bits.U.of_int 0 1) && Bitops.eq arg5784 
            Bitops.float_lt && Bitops.eq arg5781 (Bits.U.of_int 0 1) && Bitops.eq 
            arg5782 Bitops.float_gt && Bitops.eq arg5787 (Bits.U.of_int 5 3))) -> 
           Instruction.fbo (Bits.U.to_native target) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
          32, RP.Const (RP.Bits fd), _), RP.Fetch (RP.Cell ('f', Rtl.Identity, 
            32, RP.Const (RP.Bits fs2), _), 32), 32))] -> 
           Instruction.fmovs (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
          RP.App (("fneg", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
              RP.Const (RP.Bits fs2), _), 32)]), 32))] -> 
           Instruction.fnegs (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
          RP.App (("fabs", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
              RP.Const (RP.Bits fs2), _), 32)]), 32))] -> 
           Instruction.fabss (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("fsqrt", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5796), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5796 (Bits.U.of_int 0 1)) -> 
           Instruction.fsqrts (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("fsqrt", [64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs2), _), 64); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5797), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5797 (Bits.U.of_int 0 1)) -> 
           Instruction.fsqrtd (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("fsqrt", [128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs2), _), 128); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5798), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5798 (Bits.U.of_int 0 1)) -> 
           Instruction.fsqrtq (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("i2f", [32; 32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5799), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5799 (Bits.U.of_int 0 1)) -> 
           Instruction.fitos (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            32, RP.Const (RP.Bits fd), _), RP.App (("f2i", [32; 32]), 
              [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs2), _), 32); RP.Const (RP.Bits arg5800)]), 
            32))] 
      when Base.to_bool (Bitops.eq arg5800 Bitops.round_zero) -> 
           Instruction.fstoi (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("i2f", [32; 64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5801), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5801 (Bits.U.of_int 0 1)) -> 
           Instruction.fitod (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("f2f", [32; 64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5802), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5802 (Bits.U.of_int 0 1)) -> 
           Instruction.fstod (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("i2f", [32; 128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5803), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5803 (Bits.U.of_int 0 1)) -> 
           Instruction.fitoq (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("f2f", [32; 128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5804), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5804 (Bits.U.of_int 0 1)) -> 
           Instruction.fstoq (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            32, RP.Const (RP.Bits fd), _), RP.App (("f2i", [64; 32]), 
              [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs2), _), 64); RP.Const (RP.Bits arg5805)]), 
            32))] 
      when Base.to_bool (Bitops.eq arg5805 Bitops.round_zero) -> 
           Instruction.fdtoi (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("f2f", [64; 32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                64, RP.Const (RP.Bits fs2), _), 64); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5806), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5806 (Bits.U.of_int 0 1)) -> 
           Instruction.fdtos (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("f2f", [128; 32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs2), _), 128); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5807), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5807 (Bits.U.of_int 0 1)) -> 
           Instruction.fqtos (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('f', Rtl.Identity, 
            32, RP.Const (RP.Bits fd), _), RP.App (("f2i", [128; 32]), 
              [RP.Fetch (RP.Cell ('f', Rtl.Identity, 128, 
                RP.Const (RP.Bits fs2), _), 128); RP.Const (RP.Bits arg5808)]), 
            32))] 
      when Base.to_bool (Bitops.eq arg5808 Bitops.round_zero) -> 
           Instruction.fqtoi (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("f2f", [128; 64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs2), _), 128); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5809), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5809 (Bits.U.of_int 0 1)) -> 
           Instruction.fqtod (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("f2f", [64; 128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                64, RP.Const (RP.Bits fs2), _), 64); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5810), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5810 (Bits.U.of_int 0 1)) -> 
           Instruction.fdtoq (Bits.U.to_native fs2) (Bits.U.to_native fd) 
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("fadd", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs1), _), 32); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5811), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5811 (Bits.U.of_int 0 1)) -> 
           Instruction.fadds (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("fsub", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs1), _), 32); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5812), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5812 (Bits.U.of_int 0 1)) -> 
           Instruction.fsubs (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("fmul", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs1), _), 32); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5813), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5813 (Bits.U.of_int 0 1)) -> 
           Instruction.fmuls (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 32, RP.Const (RP.Bits fd), _), 
            RP.App (("fdiv", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs1), _), 32); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 32, RP.Const (RP.Bits fs2), _), 32); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5814), _)), 2)]), 32))] 
      when Base.to_bool (Bitops.eq arg5814 (Bits.U.of_int 0 1)) -> 
           Instruction.fdivs (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("fadd", [64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs1), _), 64); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 64, RP.Const (RP.Bits fs2), _), 64); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5815), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5815 (Bits.U.of_int 0 1)) -> 
           Instruction.faddd (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("fsub", [64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs1), _), 64); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 64, RP.Const (RP.Bits fs2), _), 64); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5816), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5816 (Bits.U.of_int 0 1)) -> 
           Instruction.fsubd (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("fmul", [64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs1), _), 64); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 64, RP.Const (RP.Bits fs2), _), 64); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5817), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5817 (Bits.U.of_int 0 1)) -> 
           Instruction.fmuld (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
            RP.App (("fdiv", [64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs1), _), 64); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 64, RP.Const (RP.Bits fs2), _), 64); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5818), _)), 2)]), 64))] 
      when Base.to_bool (Bitops.eq arg5818 (Bits.U.of_int 0 1)) -> 
           Instruction.fdivd (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("fadd", [128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs1), _), 128); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 128, RP.Const (RP.Bits fs2), _), 128); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5819), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5819 (Bits.U.of_int 0 1)) -> 
           Instruction.faddq (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("fsub", [128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs1), _), 128); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 128, RP.Const (RP.Bits fs2), _), 128); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5820), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5820 (Bits.U.of_int 0 1)) -> 
           Instruction.fsubq (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("fmul", [128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs1), _), 128); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 128, RP.Const (RP.Bits fs2), _), 128); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5821), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5821 (Bits.U.of_int 0 1)) -> 
           Instruction.fmulq (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
            RP.App (("fdiv", [128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs1), _), 128); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 128, RP.Const (RP.Bits fs2), _), 128); 
                RP.Fetch (RP.Slice (2, 30, RP.Cell ('F', Rtl.Identity, 32, 
                    RP.Const (RP.Bits arg5822), _)), 2)]), 128))] 
      when Base.to_bool (Bitops.eq arg5822 (Bits.U.of_int 0 1)) -> 
           Instruction.fdivq (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('f', Rtl.Identity, 64, RP.Const (RP.Bits fd), _), 
          RP.App (("fmulx", [32; 64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
              32, RP.Const (RP.Bits fs1), _), 32); RP.Fetch (RP.Cell ('f', 
                Rtl.Identity, 32, RP.Const (RP.Bits fs2), _), 32)]), 64))] -> 
           Instruction.fsmuld (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
        RP.Store (RP.Cell ('f', Rtl.Identity, 128, RP.Const (RP.Bits fd), _), 
          RP.App (("fmulx", [64; 128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
              64, RP.Const (RP.Bits fs1), _), 64); RP.Fetch (RP.Cell ('f', 
                Rtl.Identity, 64, RP.Const (RP.Bits fs2), _), 64)]), 128))] -> 
           Instruction.fdmulq (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
             (Bits.U.to_native fd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5823), _)), 
            RP.App (("fcmp", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs1), _), 32); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 32, RP.Const (RP.Bits fs2), _), 32)]), 2))] 
      when Base.to_bool (Bitops.eq arg5823 (Bits.U.of_int 0 1)) -> 
           Instruction.fcmps (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5824), _)), 
            RP.App (("fcmp", [32]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 32, 
                RP.Const (RP.Bits fs1), _), 32); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 32, RP.Const (RP.Bits fs2), _), 32)]), 2))] 
      when Base.to_bool (Bitops.eq arg5824 (Bits.U.of_int 0 1)) -> 
           Instruction.fcmpes (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5825), _)), 
            RP.App (("fcmp", [64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs1), _), 64); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 64, RP.Const (RP.Bits fs2), _), 64)]), 2))] 
      when Base.to_bool (Bitops.eq arg5825 (Bits.U.of_int 0 1)) -> 
           Instruction.fcmpd (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5826), _)), 
            RP.App (("fcmp", [64]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 64, 
                RP.Const (RP.Bits fs1), _), 64); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 64, RP.Const (RP.Bits fs2), _), 64)]), 2))] 
      when Base.to_bool (Bitops.eq arg5826 (Bits.U.of_int 0 1)) -> 
           Instruction.fcmped (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5827), _)), 
            RP.App (("fcmp", [128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs1), _), 128); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 128, RP.Const (RP.Bits fs2), _), 128)]), 2))] 
      when Base.to_bool (Bitops.eq arg5827 (Bits.U.of_int 0 1)) -> 
           Instruction.fcmpq (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Slice (2, 10, 
            RP.Cell ('F', Rtl.Identity, 32, RP.Const (RP.Bits arg5828), _)), 
            RP.App (("fcmp", [128]), [RP.Fetch (RP.Cell ('f', Rtl.Identity, 
                128, RP.Const (RP.Bits fs1), _), 128); RP.Fetch (RP.Cell ('f', 
                  Rtl.Identity, 128, RP.Const (RP.Bits fs2), _), 128)]), 2))] 
      when Base.to_bool (Bitops.eq arg5828 (Bits.U.of_int 0 1)) -> 
           Instruction.fcmpeq (Bits.U.to_native fs1) (Bits.U.to_native fs2) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5830), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg5831)]), 
            32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', Rtl.Identity, 
                32, RP.Const (RP.Bits arg5829), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg5831 13 && Bitops.eq arg5830 
            (Bits.U.of_int 5 3) && Bitops.eq arg5829 (Bits.U.of_int 4 3)) -> 
           Instruction.jmpl (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.imode (Bits.U.to_native arg5831))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5833), 
            _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', Rtl.Identity, 
                32, RP.Const (RP.Bits arg5832), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg5833 (Bits.U.of_int 5 3) && Bitops.eq 
            arg5832 (Bits.U.of_int 4 3)) -> 
           Instruction.jmpl (Instruction.generala (Bits.U.to_native rs1) 
                 (Instruction.rmode (Bits.U.to_native rs2))) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5835), _), 
            RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                RP.Const (RP.Bits rs1), _), 32); RP.Const (RP.Bits arg5836)]), 
            32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', Rtl.Identity, 
                32, RP.Const (RP.Bits arg5834), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg5836 13 && Bitops.eq arg5835 
            (Bits.U.of_int 5 3) && Bitops.eq arg5834 (Bits.U.of_int 4 3)) -> 
           Instruction.jmpl (Instruction.dispa (Bits.U.to_native rs1) 
                 (Bits.U.to_native arg5836)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5838), 
            _), RP.Const (RP.Bits arg5839), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', Rtl.Identity, 
                32, RP.Const (RP.Bits arg5837), _), 32), 32))] 
      when Base.to_bool (Bitops.fits_signed arg5839 13 && Bitops.eq arg5838 
            (Bits.U.of_int 5 3) && Bitops.eq arg5837 (Bits.U.of_int 4 3)) -> 
           Instruction.jmpl (Instruction.absolutea (Bits.U.to_native arg5839)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), 
          RP.Store (RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5841), 
            _), RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 
                32, RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                  Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', Rtl.Identity, 
                32, RP.Const (RP.Bits arg5840), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg5841 (Bits.U.of_int 5 3) && Bitops.eq 
            arg5840 (Bits.U.of_int 4 3)) -> 
           Instruction.jmpl (Instruction.indexa (Bits.U.to_native rs1) 
                 (Bits.U.to_native rs2)) (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5843), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits rd), _), RP.Fetch (RP.Cell ('i', Rtl.Identity, 
                32, RP.Const (RP.Bits arg5842), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg5843 (Bits.U.of_int 5 3) && Bitops.eq 
            arg5842 (Bits.U.of_int 4 3)) -> 
           Instruction.jmpl (Instruction.indirecta (Bits.U.to_native rs1)) 
             (Bits.U.to_native rd) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5915), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5916), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5913), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5914), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5911), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5912), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5909), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5910), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5907), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5908), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5905), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5906), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5903), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5904), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5901), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5902), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5898), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5899), _), 32); 
                    RP.Const (RP.Bits arg5900)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5895), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5896), _), 32); 
                    RP.Const (RP.Bits arg5897)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5892), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5893), _), 32); 
                    RP.Const (RP.Bits arg5894)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5889), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5890), _), 32); 
                    RP.Const (RP.Bits arg5891)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5886), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5887), _), 32); 
                    RP.Const (RP.Bits arg5888)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5883), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5884), _), 32); 
                    RP.Const (RP.Bits arg5885)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5880), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5881), _), 32); 
                    RP.Const (RP.Bits arg5882)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5877), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5878), _), 32); 
                    RP.Const (RP.Bits arg5879)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5874), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5875), _), 32); 
                    RP.Const (RP.Bits arg5876)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5871), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5872), _), 32); 
                    RP.Const (RP.Bits arg5873)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5868), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5869), _), 32); 
                    RP.Const (RP.Bits arg5870)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5865), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5866), _), 32); 
                    RP.Const (RP.Bits arg5867)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5862), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5863), _), 32); 
                    RP.Const (RP.Bits arg5864)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5859), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5860), _), 32); 
                    RP.Const (RP.Bits arg5861)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5856), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5857), _), 32); 
                    RP.Const (RP.Bits arg5858)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5853), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5854), _), 32); 
                    RP.Const (RP.Bits arg5855)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5850), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5851), _), 32); 
                  RP.Const (RP.Bits arg5852)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 7, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5848), _)), RP.Fetch (RP.Slice (1, 6, 
                RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5849), 
                  _)), 1), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 5, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5846), _)), RP.Const (RP.Bits arg5847), 
              1)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5844), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg5845)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg5845 13 && Bitops.eq arg5915 
            (Bits.U.of_int 8 5) && Bitops.eq arg5916 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5913 (Bits.U.of_int 9 5) && Bitops.eq arg5914 
            (Bits.U.of_int 25 5) && Bitops.eq arg5911 (Bits.U.of_int 10 5) && 
          Bitops.eq arg5912 (Bits.U.of_int 26 5) && Bitops.eq arg5909 
            (Bits.U.of_int 11 5) && Bitops.eq arg5910 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5907 (Bits.U.of_int 12 5) && Bitops.eq arg5908 
            (Bits.U.of_int 28 5) && Bitops.eq arg5905 (Bits.U.of_int 13 5) && 
          Bitops.eq arg5906 (Bits.U.of_int 29 5) && Bitops.eq arg5903 
            (Bits.U.of_int 14 5) && Bitops.eq arg5904 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5901 (Bits.U.of_int 15 5) && Bitops.eq arg5902 
            (Bits.U.of_int 31 5) && Bitops.eq arg5898 (Bits.U.of_int 16 5) && 
          Bitops.eq arg5899 (Bits.U.of_int 0 1) && Bitops.eq arg5900 
            (Bits.U.of_int 8 32) && Bitops.eq arg5895 (Bits.U.of_int 17 5) && 
          Bitops.eq arg5896 (Bits.U.of_int 0 1) && Bitops.eq arg5897 
            (Bits.U.of_int 7 32) && Bitops.eq arg5892 (Bits.U.of_int 18 5) && 
          Bitops.eq arg5893 (Bits.U.of_int 0 1) && Bitops.eq arg5894 
            (Bits.U.of_int 6 32) && Bitops.eq arg5889 (Bits.U.of_int 19 5) && 
          Bitops.eq arg5890 (Bits.U.of_int 0 1) && Bitops.eq arg5891 
            (Bits.U.of_int 5 32) && Bitops.eq arg5886 (Bits.U.of_int 20 5) && 
          Bitops.eq arg5887 (Bits.U.of_int 0 1) && Bitops.eq arg5888 
            (Bits.U.of_int 4 32) && Bitops.eq arg5883 (Bits.U.of_int 21 5) && 
          Bitops.eq arg5884 (Bits.U.of_int 0 1) && Bitops.eq arg5885 
            (Bits.U.of_int 3 32) && Bitops.eq arg5880 (Bits.U.of_int 22 5) && 
          Bitops.eq arg5881 (Bits.U.of_int 0 1) && Bitops.eq arg5882 
            (Bits.U.of_int 2 32) && Bitops.eq arg5877 (Bits.U.of_int 23 5) && 
          Bitops.eq arg5878 (Bits.U.of_int 0 1) && Bitops.eq arg5879 
            (Bits.U.of_int 1 32) && Bitops.eq arg5874 (Bits.U.of_int 24 5) && 
          Bitops.eq arg5875 (Bits.U.of_int 0 1) && Bitops.eq arg5876 
            (Bits.U.of_int 16 32) && Bitops.eq arg5871 (Bits.U.of_int 25 5) && 
          Bitops.eq arg5872 (Bits.U.of_int 0 1) && Bitops.eq arg5873 
            (Bits.U.of_int 15 32) && Bitops.eq arg5868 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5869 (Bits.U.of_int 0 1) && Bitops.eq arg5870 
            (Bits.U.of_int 14 32) && Bitops.eq arg5865 (Bits.U.of_int 27 5) && 
          Bitops.eq arg5866 (Bits.U.of_int 0 1) && Bitops.eq arg5867 
            (Bits.U.of_int 13 32) && Bitops.eq arg5862 (Bits.U.of_int 28 5) && 
          Bitops.eq arg5863 (Bits.U.of_int 0 1) && Bitops.eq arg5864 
            (Bits.U.of_int 12 32) && Bitops.eq arg5859 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5860 (Bits.U.of_int 0 1) && Bitops.eq arg5861 
            (Bits.U.of_int 11 32) && Bitops.eq arg5856 (Bits.U.of_int 30 5) && 
          Bitops.eq arg5857 (Bits.U.of_int 0 1) && Bitops.eq arg5858 
            (Bits.U.of_int 10 32) && Bitops.eq arg5853 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5854 (Bits.U.of_int 0 1) && Bitops.eq arg5855 (Bits.U.of_int 
              9 32) && Bitops.eq arg5850 (Bits.U.of_int 0 1) && Bitops.eq arg5851 
            (Bits.U.of_int 0 1) && Bitops.eq arg5852 (Bits.U.of_int 16 32) && 
          Bitops.eq arg5848 (Bits.U.of_int 0 3) && Bitops.eq arg5849 (Bits.U.of_int 
              0 3) && Bitops.eq arg5846 (Bits.U.of_int 0 3) && Bitops.eq arg5847 
            (Bits.U.of_int 1 1) && Bitops.eq arg5844 (Bits.U.of_int 5 3)) -> 
           Instruction.rett (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.imode (Bits.U.to_native arg5845))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg5987), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5988), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5985), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5986), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5983), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5984), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5981), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5982), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5979), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5980), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5977), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5978), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5975), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5976), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5973), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg5974), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5970), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5971), _), 32); 
                    RP.Const (RP.Bits arg5972)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5967), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5968), _), 32); 
                    RP.Const (RP.Bits arg5969)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5964), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5965), _), 32); 
                    RP.Const (RP.Bits arg5966)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5961), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5962), _), 32); 
                    RP.Const (RP.Bits arg5963)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5958), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5959), _), 32); 
                    RP.Const (RP.Bits arg5960)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5955), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5956), _), 32); 
                    RP.Const (RP.Bits arg5957)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5952), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5953), _), 32); 
                    RP.Const (RP.Bits arg5954)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5949), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5950), _), 32); 
                    RP.Const (RP.Bits arg5951)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5946), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5947), _), 32); 
                    RP.Const (RP.Bits arg5948)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5943), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5944), _), 32); 
                    RP.Const (RP.Bits arg5945)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5940), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5941), _), 32); 
                    RP.Const (RP.Bits arg5942)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5937), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5938), _), 32); 
                    RP.Const (RP.Bits arg5939)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5934), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5935), _), 32); 
                    RP.Const (RP.Bits arg5936)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5931), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5932), _), 32); 
                    RP.Const (RP.Bits arg5933)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5928), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5929), _), 32); 
                    RP.Const (RP.Bits arg5930)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5925), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5926), _), 32); 
                    RP.Const (RP.Bits arg5927)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5922), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5923), _), 32); 
                  RP.Const (RP.Bits arg5924)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 7, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5920), _)), RP.Fetch (RP.Slice (1, 6, 
                RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5921), 
                  _)), 1), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 5, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5918), _)), RP.Const (RP.Bits arg5919), 
              1)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5917), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg5987 (Bits.U.of_int 8 5) && Bitops.eq 
            arg5988 (Bits.U.of_int 24 5) && Bitops.eq arg5985 (Bits.U.of_int 9 
              5) && Bitops.eq arg5986 (Bits.U.of_int 25 5) && Bitops.eq arg5983 
            (Bits.U.of_int 10 5) && Bitops.eq arg5984 (Bits.U.of_int 26 5) && 
          Bitops.eq arg5981 (Bits.U.of_int 11 5) && Bitops.eq arg5982 
            (Bits.U.of_int 27 5) && Bitops.eq arg5979 (Bits.U.of_int 12 5) && 
          Bitops.eq arg5980 (Bits.U.of_int 28 5) && Bitops.eq arg5977 
            (Bits.U.of_int 13 5) && Bitops.eq arg5978 (Bits.U.of_int 29 5) && 
          Bitops.eq arg5975 (Bits.U.of_int 14 5) && Bitops.eq arg5976 
            (Bits.U.of_int 30 5) && Bitops.eq arg5973 (Bits.U.of_int 15 5) && 
          Bitops.eq arg5974 (Bits.U.of_int 31 5) && Bitops.eq arg5970 
            (Bits.U.of_int 16 5) && Bitops.eq arg5971 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5972 (Bits.U.of_int 8 32) && Bitops.eq arg5967 
            (Bits.U.of_int 17 5) && Bitops.eq arg5968 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5969 (Bits.U.of_int 7 32) && Bitops.eq arg5964 
            (Bits.U.of_int 18 5) && Bitops.eq arg5965 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5966 (Bits.U.of_int 6 32) && Bitops.eq arg5961 
            (Bits.U.of_int 19 5) && Bitops.eq arg5962 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5963 (Bits.U.of_int 5 32) && Bitops.eq arg5958 
            (Bits.U.of_int 20 5) && Bitops.eq arg5959 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5960 (Bits.U.of_int 4 32) && Bitops.eq arg5955 
            (Bits.U.of_int 21 5) && Bitops.eq arg5956 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5957 (Bits.U.of_int 3 32) && Bitops.eq arg5952 
            (Bits.U.of_int 22 5) && Bitops.eq arg5953 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5954 (Bits.U.of_int 2 32) && Bitops.eq arg5949 
            (Bits.U.of_int 23 5) && Bitops.eq arg5950 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5951 (Bits.U.of_int 1 32) && Bitops.eq arg5946 
            (Bits.U.of_int 24 5) && Bitops.eq arg5947 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5948 (Bits.U.of_int 16 32) && Bitops.eq arg5943 
            (Bits.U.of_int 25 5) && Bitops.eq arg5944 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5945 (Bits.U.of_int 15 32) && Bitops.eq arg5940 
            (Bits.U.of_int 26 5) && Bitops.eq arg5941 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5942 (Bits.U.of_int 14 32) && Bitops.eq arg5937 
            (Bits.U.of_int 27 5) && Bitops.eq arg5938 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5939 (Bits.U.of_int 13 32) && Bitops.eq arg5934 
            (Bits.U.of_int 28 5) && Bitops.eq arg5935 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5936 (Bits.U.of_int 12 32) && Bitops.eq arg5931 
            (Bits.U.of_int 29 5) && Bitops.eq arg5932 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5933 (Bits.U.of_int 11 32) && Bitops.eq arg5928 
            (Bits.U.of_int 30 5) && Bitops.eq arg5929 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5930 (Bits.U.of_int 10 32) && Bitops.eq arg5925 
            (Bits.U.of_int 31 5) && Bitops.eq arg5926 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5927 (Bits.U.of_int 9 32) && Bitops.eq arg5922 
            (Bits.U.of_int 0 1) && Bitops.eq arg5923 (Bits.U.of_int 0 1) && 
          Bitops.eq arg5924 (Bits.U.of_int 16 32) && Bitops.eq arg5920 
            (Bits.U.of_int 0 3) && Bitops.eq arg5921 (Bits.U.of_int 0 3) && 
          Bitops.eq arg5918 (Bits.U.of_int 0 3) && Bitops.eq arg5919 
            (Bits.U.of_int 1 1) && Bitops.eq arg5917 (Bits.U.of_int 5 3)) -> 
           Instruction.rett (Instruction.generala (Bits.U.to_native rs1) 
               (Instruction.rmode (Bits.U.to_native rs2))) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg6060), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg6061), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6058), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6059), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6056), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6057), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6054), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6055), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6052), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6053), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6050), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6051), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6048), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6049), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6046), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6047), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6043), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6044), _), 32); 
                    RP.Const (RP.Bits arg6045)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6040), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6041), _), 32); 
                    RP.Const (RP.Bits arg6042)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6037), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6038), _), 32); 
                    RP.Const (RP.Bits arg6039)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6034), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6035), _), 32); 
                    RP.Const (RP.Bits arg6036)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6031), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6032), _), 32); 
                    RP.Const (RP.Bits arg6033)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6028), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6029), _), 32); 
                    RP.Const (RP.Bits arg6030)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6025), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6026), _), 32); 
                    RP.Const (RP.Bits arg6027)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6022), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6023), _), 32); 
                    RP.Const (RP.Bits arg6024)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6019), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6020), _), 32); 
                    RP.Const (RP.Bits arg6021)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6016), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6017), _), 32); 
                    RP.Const (RP.Bits arg6018)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6013), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6014), _), 32); 
                    RP.Const (RP.Bits arg6015)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6010), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6011), _), 32); 
                    RP.Const (RP.Bits arg6012)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6007), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6008), _), 32); 
                    RP.Const (RP.Bits arg6009)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6004), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6005), _), 32); 
                    RP.Const (RP.Bits arg6006)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6001), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6002), _), 32); 
                    RP.Const (RP.Bits arg6003)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5998), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg5999), _), 32); 
                    RP.Const (RP.Bits arg6000)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg5995), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg5996), _), 32); 
                  RP.Const (RP.Bits arg5997)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 7, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5993), _)), RP.Fetch (RP.Slice (1, 6, 
                RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg5994), 
                  _)), 1), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 5, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg5991), _)), RP.Const (RP.Bits arg5992), 
              1)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg5989), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); 
                  RP.Const (RP.Bits arg5990)]), 32))] 
      when Base.to_bool (Bitops.fits_signed arg5990 13 && Bitops.eq arg6060 
            (Bits.U.of_int 8 5) && Bitops.eq arg6061 (Bits.U.of_int 24 5) && 
          Bitops.eq arg6058 (Bits.U.of_int 9 5) && Bitops.eq arg6059 
            (Bits.U.of_int 25 5) && Bitops.eq arg6056 (Bits.U.of_int 10 5) && 
          Bitops.eq arg6057 (Bits.U.of_int 26 5) && Bitops.eq arg6054 
            (Bits.U.of_int 11 5) && Bitops.eq arg6055 (Bits.U.of_int 27 5) && 
          Bitops.eq arg6052 (Bits.U.of_int 12 5) && Bitops.eq arg6053 
            (Bits.U.of_int 28 5) && Bitops.eq arg6050 (Bits.U.of_int 13 5) && 
          Bitops.eq arg6051 (Bits.U.of_int 29 5) && Bitops.eq arg6048 
            (Bits.U.of_int 14 5) && Bitops.eq arg6049 (Bits.U.of_int 30 5) && 
          Bitops.eq arg6046 (Bits.U.of_int 15 5) && Bitops.eq arg6047 
            (Bits.U.of_int 31 5) && Bitops.eq arg6043 (Bits.U.of_int 16 5) && 
          Bitops.eq arg6044 (Bits.U.of_int 0 1) && Bitops.eq arg6045 
            (Bits.U.of_int 8 32) && Bitops.eq arg6040 (Bits.U.of_int 17 5) && 
          Bitops.eq arg6041 (Bits.U.of_int 0 1) && Bitops.eq arg6042 
            (Bits.U.of_int 7 32) && Bitops.eq arg6037 (Bits.U.of_int 18 5) && 
          Bitops.eq arg6038 (Bits.U.of_int 0 1) && Bitops.eq arg6039 
            (Bits.U.of_int 6 32) && Bitops.eq arg6034 (Bits.U.of_int 19 5) && 
          Bitops.eq arg6035 (Bits.U.of_int 0 1) && Bitops.eq arg6036 
            (Bits.U.of_int 5 32) && Bitops.eq arg6031 (Bits.U.of_int 20 5) && 
          Bitops.eq arg6032 (Bits.U.of_int 0 1) && Bitops.eq arg6033 
            (Bits.U.of_int 4 32) && Bitops.eq arg6028 (Bits.U.of_int 21 5) && 
          Bitops.eq arg6029 (Bits.U.of_int 0 1) && Bitops.eq arg6030 
            (Bits.U.of_int 3 32) && Bitops.eq arg6025 (Bits.U.of_int 22 5) && 
          Bitops.eq arg6026 (Bits.U.of_int 0 1) && Bitops.eq arg6027 
            (Bits.U.of_int 2 32) && Bitops.eq arg6022 (Bits.U.of_int 23 5) && 
          Bitops.eq arg6023 (Bits.U.of_int 0 1) && Bitops.eq arg6024 
            (Bits.U.of_int 1 32) && Bitops.eq arg6019 (Bits.U.of_int 24 5) && 
          Bitops.eq arg6020 (Bits.U.of_int 0 1) && Bitops.eq arg6021 
            (Bits.U.of_int 16 32) && Bitops.eq arg6016 (Bits.U.of_int 25 5) && 
          Bitops.eq arg6017 (Bits.U.of_int 0 1) && Bitops.eq arg6018 
            (Bits.U.of_int 15 32) && Bitops.eq arg6013 (Bits.U.of_int 26 5) && 
          Bitops.eq arg6014 (Bits.U.of_int 0 1) && Bitops.eq arg6015 
            (Bits.U.of_int 14 32) && Bitops.eq arg6010 (Bits.U.of_int 27 5) && 
          Bitops.eq arg6011 (Bits.U.of_int 0 1) && Bitops.eq arg6012 
            (Bits.U.of_int 13 32) && Bitops.eq arg6007 (Bits.U.of_int 28 5) && 
          Bitops.eq arg6008 (Bits.U.of_int 0 1) && Bitops.eq arg6009 
            (Bits.U.of_int 12 32) && Bitops.eq arg6004 (Bits.U.of_int 29 5) && 
          Bitops.eq arg6005 (Bits.U.of_int 0 1) && Bitops.eq arg6006 
            (Bits.U.of_int 11 32) && Bitops.eq arg6001 (Bits.U.of_int 30 5) && 
          Bitops.eq arg6002 (Bits.U.of_int 0 1) && Bitops.eq arg6003 
            (Bits.U.of_int 10 32) && Bitops.eq arg5998 (Bits.U.of_int 31 5) && 
          Bitops.eq arg5999 (Bits.U.of_int 0 1) && Bitops.eq arg6000 (Bits.U.of_int 
              9 32) && Bitops.eq arg5995 (Bits.U.of_int 0 1) && Bitops.eq arg5996 
            (Bits.U.of_int 0 1) && Bitops.eq arg5997 (Bits.U.of_int 16 32) && 
          Bitops.eq arg5993 (Bits.U.of_int 0 3) && Bitops.eq arg5994 (Bits.U.of_int 
              0 3) && Bitops.eq arg5991 (Bits.U.of_int 0 3) && Bitops.eq arg5992 
            (Bits.U.of_int 1 1) && Bitops.eq arg5989 (Bits.U.of_int 5 3)) -> 
           Instruction.rett (Instruction.dispa (Bits.U.to_native rs1) 
               (Bits.U.to_native arg5990)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg6133), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg6134), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6131), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6132), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6129), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6130), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6127), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6128), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6125), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6126), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6123), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6124), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6121), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6122), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6119), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6120), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6116), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6117), _), 32); 
                    RP.Const (RP.Bits arg6118)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6113), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6114), _), 32); 
                    RP.Const (RP.Bits arg6115)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6110), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6111), _), 32); 
                    RP.Const (RP.Bits arg6112)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6107), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6108), _), 32); 
                    RP.Const (RP.Bits arg6109)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6104), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6105), _), 32); 
                    RP.Const (RP.Bits arg6106)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6101), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6102), _), 32); 
                    RP.Const (RP.Bits arg6103)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6098), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6099), _), 32); 
                    RP.Const (RP.Bits arg6100)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6095), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6096), _), 32); 
                    RP.Const (RP.Bits arg6097)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6092), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6093), _), 32); 
                    RP.Const (RP.Bits arg6094)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6089), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6090), _), 32); 
                    RP.Const (RP.Bits arg6091)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6086), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6087), _), 32); 
                    RP.Const (RP.Bits arg6088)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6083), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6084), _), 32); 
                    RP.Const (RP.Bits arg6085)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6080), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6081), _), 32); 
                    RP.Const (RP.Bits arg6082)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6077), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6078), _), 32); 
                    RP.Const (RP.Bits arg6079)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6074), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6075), _), 32); 
                    RP.Const (RP.Bits arg6076)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6071), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6072), _), 32); 
                    RP.Const (RP.Bits arg6073)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6068), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg6069), _), 32); 
                  RP.Const (RP.Bits arg6070)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 7, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg6066), _)), 
              RP.Fetch (RP.Slice (1, 6, RP.Cell ('i', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg6067), _)), 1), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 5, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg6064), _)), 
              RP.Const (RP.Bits arg6065), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6062), _), RP.Const (RP.Bits arg6063), 32))] 
      when Base.to_bool (Bitops.fits_signed arg6063 13 && Bitops.eq arg6133 
            (Bits.U.of_int 8 5) && Bitops.eq arg6134 (Bits.U.of_int 24 5) && 
          Bitops.eq arg6131 (Bits.U.of_int 9 5) && Bitops.eq arg6132 
            (Bits.U.of_int 25 5) && Bitops.eq arg6129 (Bits.U.of_int 10 5) && 
          Bitops.eq arg6130 (Bits.U.of_int 26 5) && Bitops.eq arg6127 
            (Bits.U.of_int 11 5) && Bitops.eq arg6128 (Bits.U.of_int 27 5) && 
          Bitops.eq arg6125 (Bits.U.of_int 12 5) && Bitops.eq arg6126 
            (Bits.U.of_int 28 5) && Bitops.eq arg6123 (Bits.U.of_int 13 5) && 
          Bitops.eq arg6124 (Bits.U.of_int 29 5) && Bitops.eq arg6121 
            (Bits.U.of_int 14 5) && Bitops.eq arg6122 (Bits.U.of_int 30 5) && 
          Bitops.eq arg6119 (Bits.U.of_int 15 5) && Bitops.eq arg6120 
            (Bits.U.of_int 31 5) && Bitops.eq arg6116 (Bits.U.of_int 16 5) && 
          Bitops.eq arg6117 (Bits.U.of_int 0 1) && Bitops.eq arg6118 
            (Bits.U.of_int 8 32) && Bitops.eq arg6113 (Bits.U.of_int 17 5) && 
          Bitops.eq arg6114 (Bits.U.of_int 0 1) && Bitops.eq arg6115 
            (Bits.U.of_int 7 32) && Bitops.eq arg6110 (Bits.U.of_int 18 5) && 
          Bitops.eq arg6111 (Bits.U.of_int 0 1) && Bitops.eq arg6112 
            (Bits.U.of_int 6 32) && Bitops.eq arg6107 (Bits.U.of_int 19 5) && 
          Bitops.eq arg6108 (Bits.U.of_int 0 1) && Bitops.eq arg6109 
            (Bits.U.of_int 5 32) && Bitops.eq arg6104 (Bits.U.of_int 20 5) && 
          Bitops.eq arg6105 (Bits.U.of_int 0 1) && Bitops.eq arg6106 
            (Bits.U.of_int 4 32) && Bitops.eq arg6101 (Bits.U.of_int 21 5) && 
          Bitops.eq arg6102 (Bits.U.of_int 0 1) && Bitops.eq arg6103 
            (Bits.U.of_int 3 32) && Bitops.eq arg6098 (Bits.U.of_int 22 5) && 
          Bitops.eq arg6099 (Bits.U.of_int 0 1) && Bitops.eq arg6100 
            (Bits.U.of_int 2 32) && Bitops.eq arg6095 (Bits.U.of_int 23 5) && 
          Bitops.eq arg6096 (Bits.U.of_int 0 1) && Bitops.eq arg6097 
            (Bits.U.of_int 1 32) && Bitops.eq arg6092 (Bits.U.of_int 24 5) && 
          Bitops.eq arg6093 (Bits.U.of_int 0 1) && Bitops.eq arg6094 
            (Bits.U.of_int 16 32) && Bitops.eq arg6089 (Bits.U.of_int 25 5) && 
          Bitops.eq arg6090 (Bits.U.of_int 0 1) && Bitops.eq arg6091 
            (Bits.U.of_int 15 32) && Bitops.eq arg6086 (Bits.U.of_int 26 5) && 
          Bitops.eq arg6087 (Bits.U.of_int 0 1) && Bitops.eq arg6088 
            (Bits.U.of_int 14 32) && Bitops.eq arg6083 (Bits.U.of_int 27 5) && 
          Bitops.eq arg6084 (Bits.U.of_int 0 1) && Bitops.eq arg6085 
            (Bits.U.of_int 13 32) && Bitops.eq arg6080 (Bits.U.of_int 28 5) && 
          Bitops.eq arg6081 (Bits.U.of_int 0 1) && Bitops.eq arg6082 
            (Bits.U.of_int 12 32) && Bitops.eq arg6077 (Bits.U.of_int 29 5) && 
          Bitops.eq arg6078 (Bits.U.of_int 0 1) && Bitops.eq arg6079 
            (Bits.U.of_int 11 32) && Bitops.eq arg6074 (Bits.U.of_int 30 5) && 
          Bitops.eq arg6075 (Bits.U.of_int 0 1) && Bitops.eq arg6076 
            (Bits.U.of_int 10 32) && Bitops.eq arg6071 (Bits.U.of_int 31 5) && 
          Bitops.eq arg6072 (Bits.U.of_int 0 1) && Bitops.eq arg6073 (Bits.U.of_int 
              9 32) && Bitops.eq arg6068 (Bits.U.of_int 0 1) && Bitops.eq arg6069 
            (Bits.U.of_int 0 1) && Bitops.eq arg6070 (Bits.U.of_int 16 32) && 
          Bitops.eq arg6066 (Bits.U.of_int 0 3) && Bitops.eq arg6067 (Bits.U.of_int 
              0 3) && Bitops.eq arg6064 (Bits.U.of_int 0 3) && Bitops.eq arg6065 
            (Bits.U.of_int 1 1) && Bitops.eq arg6062 (Bits.U.of_int 5 3)) -> 
           Instruction.rett (Instruction.absolutea (Bits.U.to_native arg6063)) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg6205), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg6206), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6203), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6204), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6201), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6202), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6199), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6200), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6197), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6198), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6195), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6196), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6193), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6194), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6191), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6192), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6188), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6189), _), 32); 
                    RP.Const (RP.Bits arg6190)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6185), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6186), _), 32); 
                    RP.Const (RP.Bits arg6187)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6182), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6183), _), 32); 
                    RP.Const (RP.Bits arg6184)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6179), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6180), _), 32); 
                    RP.Const (RP.Bits arg6181)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6176), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6177), _), 32); 
                    RP.Const (RP.Bits arg6178)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6173), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6174), _), 32); 
                    RP.Const (RP.Bits arg6175)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6170), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6171), _), 32); 
                    RP.Const (RP.Bits arg6172)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6167), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6168), _), 32); 
                    RP.Const (RP.Bits arg6169)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6164), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6165), _), 32); 
                    RP.Const (RP.Bits arg6166)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6161), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6162), _), 32); 
                    RP.Const (RP.Bits arg6163)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6158), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6159), _), 32); 
                    RP.Const (RP.Bits arg6160)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6155), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6156), _), 32); 
                    RP.Const (RP.Bits arg6157)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6152), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6153), _), 32); 
                    RP.Const (RP.Bits arg6154)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6149), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6150), _), 32); 
                    RP.Const (RP.Bits arg6151)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6146), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6147), _), 32); 
                    RP.Const (RP.Bits arg6148)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6143), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6144), _), 32); 
                    RP.Const (RP.Bits arg6145)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6140), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg6141), _), 32); 
                  RP.Const (RP.Bits arg6142)]), 32)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 7, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg6138), _)), RP.Fetch (RP.Slice (1, 6, 
                RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg6139), 
                  _)), 1), 1)); (RP.Const (RP.Bool true), 
            RP.Store (RP.Slice (1, 5, RP.Cell ('i', Rtl.Identity, 32, 
                RP.Const (RP.Bits arg6136), _)), RP.Const (RP.Bits arg6137), 
              1)); (RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg6135), _), 
              RP.App (("add", [32]), [RP.Fetch (RP.Cell ('r', Rtl.Identity, 32, 
                  RP.Const (RP.Bits rs1), _), 32); RP.Fetch (RP.Cell ('r', 
                    Rtl.Identity, 32, RP.Const (RP.Bits rs2), _), 32)]), 32))] 
      when Base.to_bool (Bitops.eq arg6205 (Bits.U.of_int 8 5) && Bitops.eq 
            arg6206 (Bits.U.of_int 24 5) && Bitops.eq arg6203 (Bits.U.of_int 9 
              5) && Bitops.eq arg6204 (Bits.U.of_int 25 5) && Bitops.eq arg6201 
            (Bits.U.of_int 10 5) && Bitops.eq arg6202 (Bits.U.of_int 26 5) && 
          Bitops.eq arg6199 (Bits.U.of_int 11 5) && Bitops.eq arg6200 
            (Bits.U.of_int 27 5) && Bitops.eq arg6197 (Bits.U.of_int 12 5) && 
          Bitops.eq arg6198 (Bits.U.of_int 28 5) && Bitops.eq arg6195 
            (Bits.U.of_int 13 5) && Bitops.eq arg6196 (Bits.U.of_int 29 5) && 
          Bitops.eq arg6193 (Bits.U.of_int 14 5) && Bitops.eq arg6194 
            (Bits.U.of_int 30 5) && Bitops.eq arg6191 (Bits.U.of_int 15 5) && 
          Bitops.eq arg6192 (Bits.U.of_int 31 5) && Bitops.eq arg6188 
            (Bits.U.of_int 16 5) && Bitops.eq arg6189 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6190 (Bits.U.of_int 8 32) && Bitops.eq arg6185 
            (Bits.U.of_int 17 5) && Bitops.eq arg6186 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6187 (Bits.U.of_int 7 32) && Bitops.eq arg6182 
            (Bits.U.of_int 18 5) && Bitops.eq arg6183 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6184 (Bits.U.of_int 6 32) && Bitops.eq arg6179 
            (Bits.U.of_int 19 5) && Bitops.eq arg6180 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6181 (Bits.U.of_int 5 32) && Bitops.eq arg6176 
            (Bits.U.of_int 20 5) && Bitops.eq arg6177 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6178 (Bits.U.of_int 4 32) && Bitops.eq arg6173 
            (Bits.U.of_int 21 5) && Bitops.eq arg6174 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6175 (Bits.U.of_int 3 32) && Bitops.eq arg6170 
            (Bits.U.of_int 22 5) && Bitops.eq arg6171 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6172 (Bits.U.of_int 2 32) && Bitops.eq arg6167 
            (Bits.U.of_int 23 5) && Bitops.eq arg6168 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6169 (Bits.U.of_int 1 32) && Bitops.eq arg6164 
            (Bits.U.of_int 24 5) && Bitops.eq arg6165 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6166 (Bits.U.of_int 16 32) && Bitops.eq arg6161 
            (Bits.U.of_int 25 5) && Bitops.eq arg6162 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6163 (Bits.U.of_int 15 32) && Bitops.eq arg6158 
            (Bits.U.of_int 26 5) && Bitops.eq arg6159 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6160 (Bits.U.of_int 14 32) && Bitops.eq arg6155 
            (Bits.U.of_int 27 5) && Bitops.eq arg6156 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6157 (Bits.U.of_int 13 32) && Bitops.eq arg6152 
            (Bits.U.of_int 28 5) && Bitops.eq arg6153 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6154 (Bits.U.of_int 12 32) && Bitops.eq arg6149 
            (Bits.U.of_int 29 5) && Bitops.eq arg6150 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6151 (Bits.U.of_int 11 32) && Bitops.eq arg6146 
            (Bits.U.of_int 30 5) && Bitops.eq arg6147 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6148 (Bits.U.of_int 10 32) && Bitops.eq arg6143 
            (Bits.U.of_int 31 5) && Bitops.eq arg6144 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6145 (Bits.U.of_int 9 32) && Bitops.eq arg6140 
            (Bits.U.of_int 0 1) && Bitops.eq arg6141 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6142 (Bits.U.of_int 16 32) && Bitops.eq arg6138 
            (Bits.U.of_int 0 3) && Bitops.eq arg6139 (Bits.U.of_int 0 3) && 
          Bitops.eq arg6136 (Bits.U.of_int 0 3) && Bitops.eq arg6137 
            (Bits.U.of_int 1 1) && Bitops.eq arg6135 (Bits.U.of_int 5 3)) -> 
           Instruction.rett (Instruction.indexa (Bits.U.to_native rs1) 
               (Bits.U.to_native rs2)) 
      
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits arg6277), _), RP.Fetch (RP.Cell ('r', 
              Rtl.Identity, 32, RP.Const (RP.Bits arg6278), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6275), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6276), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6273), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6274), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6271), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6272), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6269), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6270), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6267), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6268), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6265), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6266), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6263), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits arg6264), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6260), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6261), _), 32); 
                    RP.Const (RP.Bits arg6262)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6257), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6258), _), 32); 
                    RP.Const (RP.Bits arg6259)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6254), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6255), _), 32); 
                    RP.Const (RP.Bits arg6256)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6251), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6252), _), 32); 
                    RP.Const (RP.Bits arg6253)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6248), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6249), _), 32); 
                    RP.Const (RP.Bits arg6250)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6245), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6246), _), 32); 
                    RP.Const (RP.Bits arg6247)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6242), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6243), _), 32); 
                    RP.Const (RP.Bits arg6244)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6239), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6240), _), 32); 
                    RP.Const (RP.Bits arg6241)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6236), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6237), _), 32); 
                    RP.Const (RP.Bits arg6238)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6233), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6234), _), 32); 
                    RP.Const (RP.Bits arg6235)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6230), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6231), _), 32); 
                    RP.Const (RP.Bits arg6232)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6227), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6228), _), 32); 
                    RP.Const (RP.Bits arg6229)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6224), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6225), _), 32); 
                    RP.Const (RP.Bits arg6226)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6221), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6222), _), 32); 
                    RP.Const (RP.Bits arg6223)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6218), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6219), _), 32); 
                    RP.Const (RP.Bits arg6220)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6215), _), 
              RP.Fetch (RP.Cell ('w', Rtl.Identity, 32, 
                RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 
                    32, RP.Const (RP.Bits arg6216), _), 32); 
                    RP.Const (RP.Bits arg6217)]), _), 32), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('W', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6212), _), 
              RP.App (("sub", [32]), [RP.Fetch (RP.Cell ('W', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg6213), _), 32); 
                  RP.Const (RP.Bits arg6214)]), 32)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 7, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg6210), _)), 
              RP.Fetch (RP.Slice (1, 6, RP.Cell ('i', Rtl.Identity, 32, 
                  RP.Const (RP.Bits arg6211), _)), 1), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Slice (1, 5, 
              RP.Cell ('i', Rtl.Identity, 32, RP.Const (RP.Bits arg6208), _)), 
              RP.Const (RP.Bits arg6209), 1)); 
          (RP.Const (RP.Bool true), RP.Store (RP.Cell ('i', Rtl.Identity, 32, 
              RP.Const (RP.Bits arg6207), _), RP.Fetch (RP.Cell ('r', 
                Rtl.Identity, 32, RP.Const (RP.Bits rs1), _), 32), 32))] 
      when Base.to_bool (Bitops.eq arg6277 (Bits.U.of_int 8 5) && Bitops.eq 
            arg6278 (Bits.U.of_int 24 5) && Bitops.eq arg6275 (Bits.U.of_int 9 
              5) && Bitops.eq arg6276 (Bits.U.of_int 25 5) && Bitops.eq arg6273 
            (Bits.U.of_int 10 5) && Bitops.eq arg6274 (Bits.U.of_int 26 5) && 
          Bitops.eq arg6271 (Bits.U.of_int 11 5) && Bitops.eq arg6272 
            (Bits.U.of_int 27 5) && Bitops.eq arg6269 (Bits.U.of_int 12 5) && 
          Bitops.eq arg6270 (Bits.U.of_int 28 5) && Bitops.eq arg6267 
            (Bits.U.of_int 13 5) && Bitops.eq arg6268 (Bits.U.of_int 29 5) && 
          Bitops.eq arg6265 (Bits.U.of_int 14 5) && Bitops.eq arg6266 
            (Bits.U.of_int 30 5) && Bitops.eq arg6263 (Bits.U.of_int 15 5) && 
          Bitops.eq arg6264 (Bits.U.of_int 31 5) && Bitops.eq arg6260 
            (Bits.U.of_int 16 5) && Bitops.eq arg6261 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6262 (Bits.U.of_int 8 32) && Bitops.eq arg6257 
            (Bits.U.of_int 17 5) && Bitops.eq arg6258 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6259 (Bits.U.of_int 7 32) && Bitops.eq arg6254 
            (Bits.U.of_int 18 5) && Bitops.eq arg6255 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6256 (Bits.U.of_int 6 32) && Bitops.eq arg6251 
            (Bits.U.of_int 19 5) && Bitops.eq arg6252 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6253 (Bits.U.of_int 5 32) && Bitops.eq arg6248 
            (Bits.U.of_int 20 5) && Bitops.eq arg6249 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6250 (Bits.U.of_int 4 32) && Bitops.eq arg6245 
            (Bits.U.of_int 21 5) && Bitops.eq arg6246 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6247 (Bits.U.of_int 3 32) && Bitops.eq arg6242 
            (Bits.U.of_int 22 5) && Bitops.eq arg6243 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6244 (Bits.U.of_int 2 32) && Bitops.eq arg6239 
            (Bits.U.of_int 23 5) && Bitops.eq arg6240 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6241 (Bits.U.of_int 1 32) && Bitops.eq arg6236 
            (Bits.U.of_int 24 5) && Bitops.eq arg6237 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6238 (Bits.U.of_int 16 32) && Bitops.eq arg6233 
            (Bits.U.of_int 25 5) && Bitops.eq arg6234 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6235 (Bits.U.of_int 15 32) && Bitops.eq arg6230 
            (Bits.U.of_int 26 5) && Bitops.eq arg6231 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6232 (Bits.U.of_int 14 32) && Bitops.eq arg6227 
            (Bits.U.of_int 27 5) && Bitops.eq arg6228 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6229 (Bits.U.of_int 13 32) && Bitops.eq arg6224 
            (Bits.U.of_int 28 5) && Bitops.eq arg6225 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6226 (Bits.U.of_int 12 32) && Bitops.eq arg6221 
            (Bits.U.of_int 29 5) && Bitops.eq arg6222 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6223 (Bits.U.of_int 11 32) && Bitops.eq arg6218 
            (Bits.U.of_int 30 5) && Bitops.eq arg6219 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6220 (Bits.U.of_int 10 32) && Bitops.eq arg6215 
            (Bits.U.of_int 31 5) && Bitops.eq arg6216 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6217 (Bits.U.of_int 9 32) && Bitops.eq arg6212 
            (Bits.U.of_int 0 1) && Bitops.eq arg6213 (Bits.U.of_int 0 1) && 
          Bitops.eq arg6214 (Bits.U.of_int 16 32) && Bitops.eq arg6210 
            (Bits.U.of_int 0 3) && Bitops.eq arg6211 (Bits.U.of_int 0 3) && 
          Bitops.eq arg6208 (Bits.U.of_int 0 3) && Bitops.eq arg6209 
            (Bits.U.of_int 1 1) && Bitops.eq arg6207 (Bits.U.of_int 5 3)) -> 
           Instruction.rett (Instruction.indirecta (Bits.U.to_native rs1)) 
      | RP.Rtl [] -> 
           Instruction.tn (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tn (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tn 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tn (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tn 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tn (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.te (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.te (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.te 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.te (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.te 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.te (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tle (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tle (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tle 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tle (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tle 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tle (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tl (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tl (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tl 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tl (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tl 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tl (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tleu (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tleu (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tleu 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tleu (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tleu 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tleu (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tcs (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tcs (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tcs 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tcs (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tcs 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tcs (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tneg (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tneg (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tneg 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tneg (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tneg 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tneg (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tvs (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tvs (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tvs 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tvs (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tvs 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tvs (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.ta (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.ta (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.ta 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.ta (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.ta 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.ta (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tne (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tne (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tne 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tne (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tne 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tne (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tg (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tg (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tg 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tg (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tg 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tg (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tge (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tge (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tge 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tge (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tge 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tge (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tgu (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tgu (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tgu 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tgu (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tgu 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tgu (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tcc (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tcc (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tcc 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tcc (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tcc 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tcc (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tpos (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tpos (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tpos 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tpos (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tpos 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tpos (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tvc (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.imode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tvc (Instruction.generala (Nativeint.of_int 0) 
               (Instruction.rmode (Nativeint.of_int 0))) 
      
      | RP.Rtl [] -> 
           Instruction.tvc 
             (Instruction.dispa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tvc (Instruction.absolutea (Nativeint.of_int 0)) 
      | RP.Rtl [] -> 
           Instruction.tvc 
             (Instruction.indexa (Nativeint.of_int 0) (Nativeint.of_int 0)) 
      
      | RP.Rtl [] -> 
           Instruction.tvc (Instruction.indirecta (Nativeint.of_int 0)) 
      | RP.Rtl [] -> Instruction.unimp (Nativeint.of_int 0) 
      | RP.Rtl [(RP.Const (RP.Bool true), RP.Store (RP.Cell ('r', Rtl.Identity, 
            32, RP.Const (RP.Bits rd), _), RP.Const (RP.Bits arg6279), 32))] 
      when false -> 
           Sledlib.fail 
             ["could not solve equations: arg6279 = bitInsert(0, n, 0)"] 
      
      ) 
  end


